package com.bocloud.cop.booter.controller;

import com.bocloud.cop.entity.ScriptGroup;
import com.bocloud.cop.intf.ScriptGroupService;
import com.bocloud.cop.model.ScriptGroupBean;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 脚本分组控制器
 *
 * @author weiwei
 */
@RestController
@RequestMapping("/v1/scripts/groups")
@Tag(name = "脚本分组管理")
public class ScriptGroupController {

    @Autowired
    private ScriptGroupService scriptGroupService;

    /**
     * 获取脚本分组列表
     *
     * @param parentId
     * @return 数据结果集
     */
    @GetMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "脚本分组列表")
    public GeneralResult<List<ScriptGroup>>
    list(@RequestParam(value = Common.PARENTID, required = false) Long parentId) {
        return scriptGroupService.list(parentId);
    }

    /**
     * 添加脚本分组
     *
     * @param params
     * @param context
     * @return 添加结果
     */
    @PostMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "脚本分组创建")
    public GeneralResult<Void> create(@RequestBody ScriptGroup params,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        params.setCreatorId(context.getTarget());
        params.setMenderId(context.getTarget());
        return scriptGroupService.create(params);
    }

    /**
     * 修改脚本分组
     *
     * @param params
     * @param id
     * @param context
     * @return
     */
    @PutMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "脚本分组修改")
    public GeneralResult<Void> modify(@RequestBody ScriptGroupBean params, @PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return scriptGroupService.modify(params, context.getTarget());
    }

    /**
     * 移除脚本分组
     *
     * @param id
     * @param context
     * @return 操作结果
     */
    @DeleteMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "脚本分组删除")
    public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return scriptGroupService.remove(id, context.getTarget());
    }

    /**
     * 查看脚本分组详细信息
     *
     * @param id
     * @return 操作结果
     */
    @GetMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "脚本分组详情")
    public GeneralResult<ScriptGroup> detail(@PathVariable(value = Common.ID) Long id) {
        return scriptGroupService.detail(id);
    }

}
