package com.bocloud.cop.booter.controller;

import com.bocloud.cop.entity.Script;
import com.bocloud.cop.entity.ScriptContent;
import com.bocloud.cop.intf.ScriptContentService;
import com.bocloud.cop.intf.ScriptVersionService;
import com.bocloud.cop.model.ScriptBean;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 脚本版本类Controller
 *
 * @author wangshuai
 * @since 2011.01.11
 */
@RestController
@RequestMapping("/v1/scripts/versions")
@Tag(name = "脚本版本管理")
public class ScriptVersionController {
	@Autowired
	private ScriptVersionService scriptVersionService;
	@Autowired
	private ScriptContentService scriptContentService;

	/**
	 * 脚本版本新增
	 *
	 * @param bean
	 * @param context
	 * @return
	 */
	@PostMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本版本创建")
	public GeneralResult<Void> create(@RequestBody ScriptBean bean, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return scriptVersionService.create(bean, context.getTarget());
	}

	/**
	 * 脚本版本修改
	 *
	 * @param bean
	 * @param id
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本版本修改")
	public GeneralResult<Void> modify(@RequestBody ScriptBean bean, @PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return scriptVersionService.modify(bean, context.getTarget());
	}

	/**
	 * 脚本版本删除
	 *
	 * @param id
	 * @param context
	 * @return
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本版本删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return scriptVersionService.remove(id, context.getTarget());
	}

	/**
	 * 脚本版本详情
	 *
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本版本详情")
	public GeneralResult<Script> detail(@PathVariable(value = Common.ID) Long id) {
		return scriptVersionService.detail(id);
	}

	/**
	 * 脚本版本对比
	 *
	 * @param id
	 * @return
	 */
	@PatchMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本版本操作")
	public GeneralResult<List<ScriptContent>> compare(@PathVariable(value = Common.ID) Long id) {
		return scriptVersionService.compare(id);
	}

	/**
	 * 脚本内容查询
	 *
	 * @param id
	 * @param pager
	 * @return
	 */
	@GetMapping(value = "/{id}/contents")
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本内容列表")
	public GeneralResult<GridBean<ScriptContent>> contents(@PathVariable(value = Common.ID) Long id, Pager pager) {
		return scriptContentService.list(id, pager);
	}

	/**
	 * 脚本内容查询
	 *
	 * @param id
	 * @param pager
	 * @return
	 */
	@GetMapping(value = "/{id}/contents/simple")
	@Operation(tags = { "CMC", "CSC" }, summary = "脚本内容列表（简单）")
	public GeneralResult<GridBean<SimpleBean>> simple(@PathVariable(value = Common.ID) Long id, Pager pager) {
		return scriptContentService.simple(pager);
	}

}
