package com.bocloud.cop.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudExchange;
import com.bocloud.cop.entity.*;
import com.bocloud.cop.enums.*;
import com.bocloud.cop.intf.TaskInstanceExportService;
import com.bocloud.cop.intf.TaskInstanceService;
import com.bocloud.cop.intf.TaskNodeService;
import com.bocloud.cop.intf.network.TaskWorkerService;
import com.bocloud.cop.model.Job;
import com.bocloud.cop.model.Target;
import com.bocloud.cop.model.TaskActionMessage;
import com.bocloud.cop.model.params.DeployGaiaArgs;
import com.bocloud.cop.model.params.DeployGaiaParams;
import com.bocloud.cop.model.params.TaskInstanceActiveParam;
import com.bocloud.cop.model.result.TaskInstanceExecResult;
import com.bocloud.cop.model.result.TaskInstanceExportResult;
import com.bocloud.cop.service.utils.JobScheduler;
import com.bocloud.cop.service.utils.StorageConfig;
import com.bocloud.cop.service.utils.TaskInstanceFileHelper;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.Binding.DestinationType;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * 作业实例控制器
 *
 * @author weiwei
 */
@Slf4j
@RestController
@RequestMapping("/v1/tasks/instances")
@Tag(name = "作业实例管理")
public class TaskInstanceController {

	@Autowired
	private TaskInstanceService taskInstanceService;
	@Autowired
	private TaskNodeService taskNodeService;
	@Autowired
	private JobScheduler jobScheduler;
	@Autowired
	private RabbitAdmin rabbitAdmin;
	@Autowired
	private AmqpTemplate amqpTemplate;
	@Autowired
	private StorageConfig storageConfig;
	@Autowired
	private TaskInstanceExportService taskInstanceExportService;
	@Autowired
	private TaskWorkerService taskWorkerService;
	@Autowired
	private StringRedisTemplate redisTemplate;

	/**
	 * 执行历史分析导出
	 *
	 * @param pager
	 * @param request
	 * @param response
	 * @param context
	 * @return
	 */
	@GetMapping(value = "/failure/records/download")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例分析导出excel")
	public GeneralResult download(Pager pager, HttpServletRequest request, HttpServletResponse response,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		if (null == pager) {
			pager = new Pager();
		}
		pager.setPage(1);
		pager.setRows(Integer.MAX_VALUE);
		GeneralResult<GridBean<TaskFailRecord>> result = taskInstanceService.listFailRecord(pager);
		if (result.isFailed()) {
			return new GeneralResult<>(false, "查询数据失败");
		}
		GeneralResult<Map<String, Object>> dataResult = TaskInstanceFileHelper.download(result.getData().getRows());
		if (dataResult.isSuccess()) {
			String content;
			try {
				Map<String, Object> dataMap = dataResult.getData();
				String fileName = dataMap.get("fileName").toString();
				response.setContentType("application/vnd.ms-excel");
				final String userAgent = request.getHeader("user-agent").toLowerCase();
				if (userAgent.contains("trident") || userAgent.contains("edge")) {
					fileName = URLEncoder.encode(fileName, "UTF-8");
				} else {
					fileName = new String(fileName.getBytes("utf-8"), "iso-8859-1");
				}
				response.setCharacterEncoding("utf-8");
				response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
				OutputStream outputStream = response.getOutputStream();
				content = (String) dataMap.get("data");
				if (content != null) {
					byte[] bytes = Base64.getDecoder().decode(content);
					outputStream.write(bytes);
					outputStream.close();
				}
			} catch (Exception e) {
				return new GeneralResult<>(false, "内部错误");
			}
		}
		return dataResult;
	}

	/**
	 * 执行历史分析导出pdf
	 *
	 * @param pager
	 * @param request
	 * @param response
	 * @param context
	 * @return
	 */
	@GetMapping(value = "/failure/records/download/pdf")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例分析导出pdf")
	public GeneralResult pdfDownload(Pager pager, HttpServletRequest request, HttpServletResponse response,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		if (null == pager) {
			pager = new Pager();
		}
		pager.setPage(1);
		pager.setRows(Integer.MAX_VALUE);
		GeneralResult<GridBean<TaskFailRecord>> result = taskInstanceService.listFailRecord(pager);
		if (result.isSuccess()) {
			try {
				// 构建pdf
				String fileName = "执行历史分析" + System.currentTimeMillis() + ".pdf";
				byte[] bytes = TaskInstanceFileHelper.generatePdf(result.getData().getRows());
				response.setContentType("application/pdf");
				final String userAgent = request.getHeader("user-agent").toLowerCase();
				if (userAgent.contains("trident") || userAgent.contains("edge")) {
					fileName = URLEncoder.encode(fileName, "UTF-8");
				} else {
					fileName = new String(fileName.getBytes("utf-8"), "iso-8859-1");
				}
				response.setCharacterEncoding("utf-8");
				response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
				OutputStream outputStream = response.getOutputStream();
				outputStream.write(bytes);
				outputStream.close();
			} catch (Exception e) {
				return new GeneralResult<>(false, "内部错误");
			}
		}
		return result;
	}

	/**
	 * 获取作业失败记录
	 *
	 * @param pager 分页参数
	 * @return 数据结果集
	 */
	@GetMapping("/failure/records")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例失败列表")
	public GeneralResult<GridBean<TaskFailRecord>> listFailRecord(Pager pager,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskInstanceService.listFailRecord(pager);
	}

	/**
	 * 作业实例执行链路
	 *
	 * @param pager
	 * @param context
	 * @return
	 */
	@GetMapping(value = "/links")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例执行链路")
	public GeneralResult<GridBean<TaskInstance>> links(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskInstanceService.links(pager, context);
	}

	/**
	 * 获取作业记录列表
	 *
	 * @param pager
	 * @param context
	 * @return 数据结果集
	 */
	@GetMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例列表")
	public GeneralResult<GridBean<TaskInstance>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskInstanceService.list(pager, context);
	}

	/**
	 * 查看作业详情
	 *
	 * @param id
	 * @return 操作结果
	 */
	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例详情")
	public GeneralResult<TaskInstance> detail(@PathVariable(value = Common.ID) Long id) {
		return taskInstanceService.detail(id);
	}

	/**
	 * 任务实例日志查看
	 *
	 * @param id 定时任务ID
	 * @return 操作结果
	 */
	@GetMapping(value = "/{id}/log")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例日志")
	public GeneralResult<TaskInstance> log(@PathVariable(value = Common.ID) Long id) {
		return taskInstanceService.log(id);
	}

	/**
	 * 移除作业实例
	 *
	 * @param id
	 * @param context
	 * @return 操作结果
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskInstanceService.remove(id, context.getTarget());
	}

	/**
	 * 查看作业详情·
	 *
	 * @param id
	 * @return 操作结果
	 */
	@GetMapping(value = "/nodes/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业节点详细")
	public GeneralResult<TaskNode> nodeDetail(@PathVariable(value = Common.ID) Long id) {
		return taskNodeService.detail(id);
	}

	@PatchMapping(value = "/{id}/result")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例结果")
	public GeneralResult<List<TaskInstanceExecResult>> result(@PathVariable(value = Common.ID) Long id) {
		return taskInstanceService.result(id);
	}

	@PatchMapping(value = "/{id}/active")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例继续")
	public GeneralResult<Void> active(@PathVariable(value = Common.ID) Long id,
			@RequestBody TaskInstanceActiveParam params) {
		TaskInstance instance = this.taskInstanceService.get(id);
		if (Status.Inst.RUNNING.name().equals(instance.getStatus())) {
			return new GeneralResult<>(false, "作业正在执行");
		}
		if (null != params.getNodeId()) {
			TaskNode node = this.taskNodeService.get(params.getNodeId());
			if (null == node.getGmtFinish()) {
				return new GeneralResult<>(false, "节点作业没有完成，无法继续");
			}
		}
		Result result = handleTaskAction(params.getUuid(), params.getNodeId(), TaskAction.ACTIVE);
		if (result.isSuccess()) {
			return new GeneralResult<>(true, "节点作业继续成功");
		}
		return new GeneralResult<>(false, "节点作业继续失败");
	}

	@PatchMapping(value = "/{id}/suspend")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例暂停")
	public GeneralResult<Void> suspend(@PathVariable(value = Common.ID) Long id,
			@RequestBody TaskInstanceActiveParam params) {
		GeneralResult<?> checkResult = taskInstanceService.handlerCheck(id, TaskAction.SUSPEND);
		if (checkResult.isFailed()) {
			return new GeneralResult<>(false, "暂停失败：" + checkResult.getMessage());
		}
		Result result = handleTaskAction(params.getUuid(), null, TaskAction.SUSPEND);
		if (null != result && result.isSuccess()) {
			return new GeneralResult<>(true, "作业暂停成功");
		}
		return new GeneralResult<>(false, "作业暂停失败");

	}

	@PatchMapping(value = "/{id}/cancel")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例取消")
	public GeneralResult<Void> cancel(@PathVariable(value = Common.ID) Long id,
			@RequestBody TaskInstanceActiveParam params) {
		GeneralResult<?> checkResult = taskInstanceService.handlerCheck(id, TaskAction.CANCEL);
		if (checkResult.isFailed()) {
			return new GeneralResult<>(false, "取消失败：" + checkResult.getMessage());
		}
		Result result = handleTaskAction(params.getUuid(), null, TaskAction.CANCEL);
		if (null != result && result.isSuccess()) {
			return new GeneralResult<>(true, "作业取消成功");
		}
		return new GeneralResult<>(false, "作业取消失败");

	}

	@PatchMapping(value = "/{id}/start")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例启动")
	public GeneralResult<Long> start(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		TaskInstance instance = this.taskInstanceService.get(id);
		if (!Status.Inst.CREATED.name().equals(instance.getStatus())) {
			return new GeneralResult<>(false, "作业未处于未执行状态");
		}
		instance.setContext(context);
		Job job = new Job();
		job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(instance));
		job.setParam(TaskParam.TASKTYPE.name(), Catalog.Task.STANDARD.getValue());
		job.setTaskId(instance.getUuid());
		job.setParam(Common.NAME, instance.getName());
		boolean result = jobScheduler.insertTask(job);
		if (result) {
			return new GeneralResult<>(true, id, "作业启动成功");
		}
		return new GeneralResult<>(false, "作业启动失败");
	}

	@PatchMapping(value = "/{id}/recover")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例恢复")
	public GeneralResult<Long> recover(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		Task task = taskInstanceService.recover(id, context);
		if (null == task) {
			return new GeneralResult<>(false, "创建恢复作业失败");
		}
		TaskInstance instance = taskInstanceService.initInstance(task, ExecuteMode.WEB.name(), context);
		if (null == instance) {
			return new GeneralResult<>(false, "执行失败：原因可能是作业节点分类参数格式错误");
		}
		instance.setContext(context);
		Job job = buildJob(instance, Catalog.Task.STANDARD.getValue());
		boolean flag = jobScheduler.insertTask(job);
		return flag ? new GeneralResult<>(true, instance.getId(), "作业启动成功") : new GeneralResult<>(false, "作业启动失败");
	}

	private GeneralResult<?> handleTaskAction(String taskId, Long nodeId, TaskAction action) {
		String queue = UUID.randomUUID().toString();
		String exchange = BoCloudExchange.Default.DIRECT.getExchange();
		rabbitAdmin.declareQueue(new Queue(queue, false, false, false));
		rabbitAdmin.declareBinding(new Binding(queue, DestinationType.QUEUE, exchange, queue, null));
		TaskActionMessage actionMessage = new TaskActionMessage(taskId, nodeId, queue, action);
		String taskActionExchange = BoCloudExchange.Default.TOPIC.getExchange();
		amqpTemplate.convertAndSend(taskActionExchange, "topic.task.action.#", actionMessage);
		Message message = amqpTemplate.receive(queue, 60 * 1_000L);
		try {
			if (null == message) {
				return new GeneralResult<>(false, "操作超时！");
			} else {
				String content = new String(message.getBody());
				return JSONObject.parseObject(content, GeneralResult.class);
			}
		} catch (Exception e) {
			log.error("handleTaskAction {} exception !", action, e);
			return new GeneralResult<>(false, "操作失败");
		} finally {
			rabbitAdmin.deleteQueue(queue);
		}
	}

	/**
	 * 作业实例导出
	 *
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/{id}/export")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例导出")
	public void export(@PathVariable(value = Common.ID) Long id, @RequestParam(required = false) String filetype,
			HttpServletRequest request, HttpServletResponse response) {

		GeneralResult<TaskInstanceExportResult> result = taskInstanceExportService.export(id, filetype);
		if (result.isFailed()) {
			return;
		}
		String data = JSONObject.toJSONString(result.getData());
		JSONObject object = JSONObject.parseObject(data);
		response.setCharacterEncoding(StandardCharsets.UTF_8.toString());
		String ext = "";
		if ("pdf".equals(filetype)) {
			ext = ".pdf";
			response.setContentType("application/pdf");
		} else if ("excel".equals(filetype)) {
			ext = ".xls";
			response.setContentType("application/vnd.ms-excel");
		} else if ("word".equals(filetype)) {
			ext = ".docx";
			response.setContentType("application/vnd.openxmlformats-officedocument.wordprocessingml.document");
		}
		try {
			String fileName = "作业-" + object.getString("name") + "_" + System.currentTimeMillis() + ext;
			final String userAgent = request.getHeader("user-agent").toLowerCase();
			if (userAgent.contains("trident") || userAgent.contains("edge")) {
				fileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.toString());
			} else {
				fileName = new String(fileName.getBytes(StandardCharsets.UTF_8), "iso-8859-1");
			}
			response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
			OutputStream os = response.getOutputStream();
			String fileStr = object.getString("file");
			byte[] decode = Base64.getDecoder().decode(fileStr);
			os.write(decode);
			os.close();
			os.flush();
		} catch (Exception e) {
			log.error("Download fail:", e);
		}
	}

	private Job buildJob(TaskInstance instance, String type) {
		Job job = new Job();
		job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(instance));
		job.setParam(TaskParam.TASKTYPE.name(), type);
		job.setTaskId(instance.getUuid());
		job.setParam(Common.NAME, instance.getName());
		return job;
	}

	/**
	 * 检查文件是否存在
	 *
	 * @param path
	 * @param response
	 * @return
	 */
	@GetMapping(value = "/files/check")
	@Operation(tags = { "CMC", "CSC" }, summary = "文件是否存在")
	public GeneralResult<Void> check(@RequestParam(value = "path", required = true) String path,
			HttpServletResponse response) {
		String filepath = this.storageConfig.getPath() + path;
		log.info("检查文件是否存在:{}", filepath);
		File file = new File(filepath);
		if (file.exists()) {
			return new GeneralResult<>(true, "文件检查成功!");
		}
		return new GeneralResult<>(false, "文件检查失败，文件可能不存在!");
	}

	/**
	 * 下载备份文件的方法
	 */
	@GetMapping(value = "/files/download")
	@Operation(tags = { "CMC", "CSC" }, summary = "备份文件下载")
	public void download(@RequestParam(value = "path", required = true) String path, HttpServletRequest request,
			HttpServletResponse response) {
		response.setCharacterEncoding(StandardCharsets.UTF_8.toString());
		response.setContentType("multipart/form-data");
		try {
			String fileName = new String(path.substring(path.lastIndexOf(File.separator) + 1).getBytes());
			final String userAgent = request.getHeader("user-agent").toLowerCase();
			if (userAgent.contains("trident") || userAgent.contains("edge")) {
				fileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.toString());
			} else {
				fileName = new String(fileName.getBytes(StandardCharsets.UTF_8), "iso-8859-1");
			}
			response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
		} catch (UnsupportedEncodingException e1) {
			log.error("set code failure:", e1);
		}
		InputStream inputStream = null;
		try {
			String filepath = this.storageConfig.getPath() + path;
			File file = new File(filepath);
			OutputStream os = response.getOutputStream();
			inputStream = new FileInputStream(file);
			byte[] b = new byte[2048];
			int length;
			while ((length = inputStream.read(b)) != -1) {
				os.write(b, 0, length);
			}
			inputStream.close();
			os.close();
			os.flush();
		} catch (IOException e) {
			log.error("Download fail:", e);
		} finally {
			try {
				if (null != inputStream) {
					inputStream.close();
				}
			} catch (IOException e) {
				log.error("关闭输入流失败", e);
			}
		}
	}

	/**
	 * 查看作业详情·
	 *
	 * @param id
	 * @return 操作结果
	 */
	@GetMapping(value = "/nodes/{id}/logs")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例节点日志")
	public GeneralResult<?> getNodeLog(@PathVariable(value = Common.ID) Long id,
			@RequestParam(value = "priority", required = false) Integer priority,
			@RequestParam(value = "end", required = false) Boolean end) {
		return taskNodeService.getLog(id, priority, end);
	}

	@DeleteMapping(value = "/{id}/cancel")
	@Operation(tags = { "CMC", "CSC" }, summary = "未执行作业实例取消")
	public GeneralResult<Void> cancel(@PathVariable(value = Common.ID) Long id) {
		return taskInstanceService.cancel(id);
	}

	@DeleteMapping(value = "/cancel")
	@Operation(tags = { "CMC", "CSC" }, summary = "未执行作业实例批量取消")
	public GeneralResult<Void> cancels(@RequestBody List<Long> ids) {
		return taskInstanceService.cancel(ids);
	}

	@PatchMapping(value = "/{id}/restart")
	@Operation(tags = { "CMC", "CSC" }, summary = "失败重新启动作业实例")
	public GeneralResult<Long> restart(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		GeneralResult<TaskInstance> result = taskInstanceService.initFailedStatus(id, context);
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		TaskInstance instance = result.getData();
		instance.setContext(context);
		Job job = new Job();
		job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(instance));
		job.setParam(TaskParam.TASKTYPE.name(), Catalog.Task.STANDARD.getValue());
		job.setTaskId(instance.getUuid());
		job.setParam(Common.NAME, instance.getName());
		boolean flag = jobScheduler.insertTask(job);
		if (flag) {
			return new GeneralResult<>(true, id, "作业启动成功");
		}
		return new GeneralResult<>(false, "作业启动失败");
	}

	/**
	 * 任务节点跳过
	 */
	@PatchMapping(value = "/nodes/{id}/skip")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例失败节点人工跳过")
	public GeneralResult<?> skip(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskNodeService.skip(id, context);
	}

	@PatchMapping(value = "/{id}/execute")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例失败人工介入执行")
	public GeneralResult<?> execute(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		GeneralResult<TaskInstance> result = this.taskInstanceService.executeCheck(id, context);
		if (result.isFailed()) {
			return result;
		}
		TaskInstance instance = result.getData();
		instance.setContext(context);
		Job job = new Job();
		job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(instance));
		job.setParam(TaskParam.TASKTYPE.name(), Catalog.Task.STANDARD.getValue());
		job.setTaskId(instance.getUuid());
		job.setParam(Common.NAME, instance.getName());
		boolean success = jobScheduler.insertTask(job);
		if (success) {
			return new GeneralResult<>(true, "失败任务人工介入下发成功");
		}
		return new GeneralResult<>(false, "失败任务人工介入下发失败");
	}

	@PostMapping("/deploy/gaia")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例创建执行（gaia扩容）")
	public GeneralResult<?> deployGaia(@RequestBody DeployGaiaParams params,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		DeployGaiaArgs args = params.getArgs();
		if (null == args) {
			return new GeneralResult<>(false, "gaia扩容异常，参数异常");
		}
		args.setType("add");
		long startTime = System.currentTimeMillis();
		GeneralResult<Task> taskResult = taskInstanceService.taskDeployGaia(params);
		if (taskResult.isFailed()) {
			log.warn("请求作业构建失败，失败原因为：" + taskResult.getMessage());
			return taskResult;
		}
		log.info("生成任务task用时：" + (System.currentTimeMillis() - startTime));
		Task task = taskResult.getData();

		// 创建任务实例
		TaskInstance instance = taskInstanceService.initInstance(task, ExecuteMode.WEB.name(), context);
		if (null == instance) {
			log.warn("用户请求作业执行失败，失败原因为：原因可能是作业节点分类参数格式错误");
			return new GeneralResult<>(false, "执行失败：原因可能是作业节点分类参数格式错误");
		}
		log.info("生成实例instance用时：" + (System.currentTimeMillis() - startTime));
		Job job = new Job();
		job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(instance));
		job.setParam(TaskParam.TASKTYPE.name(), Catalog.Task.STANDARD.getValue());
		job.setTaskId(instance.getUuid());
		job.setParam(Common.NAME, instance.getName());
		boolean success = jobScheduler.insertTask(job);
		if (success) {
			return new GeneralResult<>(true, instance.getUuid(), "gaia扩容任务下发成功");
		}
		return new GeneralResult<>(false, "gaia扩容任务下发失败");
	}

	@DeleteMapping("/{workerId}/gaia")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业实例创建执行（gaia缩容）")
	public GeneralResult<?> removeGaia(@PathVariable(value = "workerId") Long workerId,
			@RequestBody DeployGaiaParams params, @Value(Common.REQ_CONTEXT) RequestContext context) {

		GeneralResult<?> checkResult = deleteGaiaCheck(workerId, params);
		if (checkResult.isFailed()) {
			return checkResult;
		}
		DeployGaiaArgs args = params.getArgs();
		if (null == args) {
			args = new DeployGaiaArgs();
		}
		args.setType("delete");
		long startTime = System.currentTimeMillis();
		GeneralResult<Task> taskResult = taskInstanceService.taskDeployGaia(params);
		if (taskResult.isFailed()) {
			log.warn("请求作业构建失败，失败原因为：" + taskResult.getMessage());
			return taskResult;
		}
		log.info("生成任务task用时：" + (System.currentTimeMillis() - startTime));
		Task task = taskResult.getData();

		// 创建任务实例
		TaskInstance instance = taskInstanceService.initInstance(task, ExecuteMode.WEB.name(), context);
		if (null == instance) {
			log.warn("用户请求作业执行失败，失败原因为：原因可能是作业节点分类参数格式错误");
			return new GeneralResult<>(false, "执行失败：原因可能是作业节点分类参数格式错误");
		}
		log.info("生成实例instance用时：" + (System.currentTimeMillis() - startTime));
		Job job = new Job();
		job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(instance));
		job.setParam(TaskParam.TASKTYPE.name(), Catalog.Task.STANDARD.getValue());
		job.setTaskId(instance.getUuid());
		job.setParam(Common.NAME, instance.getName());
		boolean success = jobScheduler.insertTask(job);
		if (success) {
			taskWorkerService.updateRemark(workerId, instance.getUuid(), context.getTarget());
			return new GeneralResult<>(true, instance.getUuid(), "gaia缩容任务下发成功");
		}
		return new GeneralResult<>(false, "gaia缩容任务下发失败");
	}

	private GeneralResult<?> deleteGaiaCheck(Long workerId, DeployGaiaParams params) {
		try {
			// 先检查worker是否排除，然后任务数是否为0
			GeneralResult<TaskWorker> result = taskWorkerService.detail(workerId);
			if (result.isFailed()) {
				return new GeneralResult<>(false, "worker不存在");
			}
			TaskWorker worker = result.getData();
			if (!TaskWorker.Category.Ansible.equals(worker.getCategory())) {
				return new GeneralResult<>(false, "worker不存在");
			}
			if (null != worker.getGroupId() && worker.getGroupId() != 0) {
				if (null == worker.getIsExclude() || !worker.getIsExclude()) {
					return new GeneralResult<>(false, "请先排除worker");
				}
			}
			String value = redisTemplate.opsForValue().get("workmetric_" + worker.getInstance());
			Integer tasks = StringUtils.isEmpty(value) ? 0 : Integer.valueOf(value);
			if (tasks > 0) {
				return new GeneralResult<>(false, "当前worker存在正在执行的任务，不能删除，请稍后重试！");
			}
			Target target = params.getTarget();
			if (null == target) {
				return new GeneralResult<>(false, "目标主机不存在");
			}
			if (CollectionUtils.isEmpty(target.getData())) {
				return new GeneralResult<>(false, "目标主机不存在");
			}
			if (target.getData().size() > 1) {
				return new GeneralResult<>(false, "目标主机数异常");
			}
			if (!worker.getInstance().split(":")[0].equalsIgnoreCase(target.getData().get(0).getHost())) {
				return new GeneralResult<>(false, "目标主机与当前worker不匹配");
			}
		} catch (Exception e) {
			log.error("delete worker check error", e);
			return new GeneralResult<>(false, e.getMessage());
		}
		return new GeneralResult<>(true, "删除worker校验成功");
	}

}
