package com.bocloud.cop.booter.controller.backup;

import com.bocloud.cop.entity.backup.TaskBackup;
import com.bocloud.cop.intf.backup.TaskBackupService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.event.ApplicationReadyEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.web.bind.annotation.*;

/**
 * @author huliwei
 * @date 2022/10/18
 */
@RestController
@RequestMapping("/v1/backup")
@Tag(name = "备份任务管理")
public class TaskBackupController implements ApplicationListener<ApplicationReadyEvent> {

	@Autowired
	private TaskBackupService taskBackupService;

	/**
	 * 获取备份任务列表
	 *
	 * @param pager
	 * @param context
	 * @return 数据结果集
	 */
	@GetMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "备份任务列表")
	public GeneralResult<GridBean<TaskBackup>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskBackupService.list(pager, context);
	}

	/**
	 * 新增备份任务
	 *
	 * @param
	 * @param context
	 * @return 添加结果
	 */
	@PostMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "备份任务创建")
	public GeneralResult<Void> create(@RequestBody TaskBackup taskBackUp,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskBackupService.create(taskBackUp, context);
	}

	/**
	 * 备份任务删除
	 *
	 * @param id      备份任务表id
	 * @param context 系统信息
	 * @return
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "备份任务删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskBackupService.remove(id, context.getTarget());
	}

	/**
	 * 定时任务操作-启动/暂停
	 *
	 * @param action
	 * @param id
	 * @param requestContext
	 * @return
	 */
	@PatchMapping(value = "/{id}/{action}/schedule")
	@Operation(summary = "定时任务操作-启动/暂停")
	public GeneralResult<Void> operate(@PathVariable(value = "action") String action,
			@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
		return taskBackupService.operate(action, id, requestContext);
	}

	/**
	 * 定时备份添加任务(主服务有效)
	 *
	 * @param id 备份任务表id
	 * @return
	 */
	@GetMapping(value = "/{id}/schedule")
	@Operation(tags = { "CMC" }, summary = "定时采集添加任务(主服务有效)")
	public GeneralResult<String> addScheduleIfLeader(@PathVariable(Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskBackupService.addScheduleIfLeader(id, context);
	}

	/**
	 * 定时备份删除任务(主服务有效)
	 *
	 * @param id 备份任务表id
	 * @return
	 */
	@DeleteMapping(value = "/{id}/schedule")
	@Operation(tags = { "CMC" }, summary = "定时采集删除任务(主服务有效)")
	public GeneralResult<String> removeScheduleIfLeader(@PathVariable(Common.ID) Long id) {
		return taskBackupService.removeScheduleIfLeader(id);
	}

	@Override
	public void onApplicationEvent(ApplicationReadyEvent event) {
		taskBackupService.onApplicationReadyEvent();
	}
}
