package com.bocloud.cop.booter.controller.backup;

import com.bocloud.cop.entity.backup.TaskBackupScene;
import com.bocloud.cop.intf.backup.TaskBackupSceneService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author huliwei
 * @date 2022/10/19
 */
@RestController
@RequestMapping("/v1/backup/scene")
@Tag(name = "备份场景管理")
public class TaskBackupSceneController {

	@Autowired
	private TaskBackupSceneService taskBackupSceneService;

	/**
	 * 配置场景查询
	 *
	 * @param pager
	 * @return
	 */
	@GetMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "配置场景列表")
	public GeneralResult<GridBean<List<TaskBackupScene>>> list(Pager pager) {
		return taskBackupSceneService.list(pager);
	}

	/**
	 * 配置场景新增
	 *
	 * @param taskBackupScene 新增实例信息
	 * @param context         系统参数信息
	 * @return
	 */
	@PostMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "配置场景创建")
	public GeneralResult create(@RequestBody TaskBackupScene taskBackupScene,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskBackupSceneService.create(taskBackupScene, context);
	}

	/**
	 * 配置场景修改
	 *
	 * @param taskBackupScene 实例信息
	 * @param context         系统参数信息
	 * @return
	 */
	@PutMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "配置场景修改")
	public GeneralResult modify(@RequestBody TaskBackupScene taskBackupScene,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskBackupSceneService.modify(taskBackupScene, context);
	}

	/**
	 * 配置场景删除
	 *
	 * @param id      数据id
	 * @param context 系统参数信息
	 * @return
	 */
	@DeleteMapping("/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "配置场景删除")
	public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskBackupSceneService.remove(id, context);
	}

}
