package com.bocloud.cop.booter.controller.compare;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.bocloud.cop.entity.compare.StandardDocument;
import com.bocloud.cop.intf.compare.StandardDocumentService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;

/**
 * @Author liuyuanyuan
 * @Date 2022-10-292 13:46:31
 * @Description:
 */
@RestController
@RequestMapping("/v1/standard/documents")
@Tag(name = "标准文件管理")
public class StandardDocumentController {

	@Autowired
	private StandardDocumentService standardDocumentService;

	/**
	 * 列表查询
	 *
	 * @param pager
	 * @return
	 */
	@Operation(tags = { "CMC" }, summary = "文件列表")
	@GetMapping
	public GeneralResult<GridBean<StandardDocument>> list(Pager pager) {
		return standardDocumentService.list(pager);
	}

	/**
	 * 创建
	 *
	 * @param versionId
	 * @param remark
	 * @param context
	 * @param file
	 * @return
	 */
	@PostMapping
	@Operation(tags = { "CMC" }, summary = "文件创建")
	public GeneralResult<Void> create(@RequestParam Long versionId, @RequestParam(required = false) String remark,
			@Value(Common.REQ_CONTEXT) RequestContext context, @RequestParam("file") MultipartFile file) {
		return standardDocumentService.save(versionId, remark, file, context);
	}

	/**
	 * 修改
	 *
	 * @param param
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "文件修改")
	public GeneralResult<Void> modify(@RequestBody StandardDocument param,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return standardDocumentService.modify(param, context);
	}

	/**
	 * 删除
	 *
	 * @param id
	 * @param context
	 * @return
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "文件删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return standardDocumentService.remove(id, context.getTarget());
	}

	/**
	 * 详情
	 *
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "文件详情")
	public GeneralResult<StandardDocument> detail(@PathVariable(value = Common.ID) Long id) {
		return standardDocumentService.detail(id);
	}

}