package com.bocloud.cop.booter.controller.compare;

import com.bocloud.cop.entity.compare.StandardDocumentVersion;
import com.bocloud.cop.intf.compare.StandardDocumentVersionService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * @Author liuyuanyuan
 * @Date 2022-10-292 10:55:25
 * @Description:
 */
@RestController
@RequestMapping("/v1/standard/versions")
@Tag(name = "标准文件版本管理")
public class StandardDocumentVersionController {

	@Autowired
	private StandardDocumentVersionService standardDocumentVersionService;

	/**
	 * 列表查询
	 *
	 * @param pager
	 * @return
	 */
	@Operation(tags = { "CMC" }, summary = "版本列表")
	@GetMapping
	public GeneralResult<GridBean<StandardDocumentVersion>> list(Pager pager) {
		return standardDocumentVersionService.list(pager);
	}

	/**
	 * 创建
	 *
	 * @param param
	 * @param context
	 * @return
	 */
	@PostMapping
	@Operation(tags = { "CMC" }, summary = "版本创建")
	public GeneralResult<Void> create(@RequestBody StandardDocumentVersion param,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return standardDocumentVersionService.save(param, context);
	}

	/**
	 * 修改
	 *
	 * @param param
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "版本修改")
	public GeneralResult<Void> modify(@PathVariable(value = Common.ID) Long id,
			@RequestBody StandardDocumentVersion param, @Value(Common.REQ_CONTEXT) RequestContext context) {
		param.setId(id);
		return standardDocumentVersionService.modify(param, context);
	}

	/**
	 * 删除
	 *
	 * @param id
	 * @param context
	 * @return
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "版本删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return standardDocumentVersionService.remove(id, context.getTarget());
	}

	/**
	 * 详情
	 *
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "版本详情")
	public GeneralResult<StandardDocumentVersion> detail(@PathVariable(value = Common.ID) Long id) {
		return standardDocumentVersionService.detail(id);
	}

}