package com.bocloud.cop.booter.controller.compare;

import com.bocloud.cop.entity.compare.TaskDocumentComparison;
import com.bocloud.cop.entity.compare.TaskInstanceDocumentComparisonItem;
import com.bocloud.cop.intf.compare.TaskDocumentComparisonService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.event.ApplicationReadyEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.web.bind.annotation.*;

/**
 * @Author liuyuanyuan
 * @Date 2022-10-292 16:04:48
 * @Description:
 */
@RestController
@RequestMapping("/v1/tasks/comparisons")
@Tag(name = "文件对比任务管理")
public class TaskDocumentComparisonController implements ApplicationListener<ApplicationReadyEvent> {

	@Autowired
	private TaskDocumentComparisonService documentComparisonService;

	/**
	 * 版本文件对比明细
	 *
	 * @param id
	 * @param pager
	 * @return
	 */
	@Operation(tags = { "CMC" }, summary = "文件对比明细")
	@GetMapping("/{id}/items")
	public GeneralResult<GridBean<TaskInstanceDocumentComparisonItem>> items(@PathVariable(value = Common.ID) Long id,
			Pager pager) {
		return documentComparisonService.items(id, pager);
	}

	/**
	 * 列表查询
	 *
	 * @param pager
	 * @return
	 */
	@Operation(tags = { "CMC" }, summary = "对比任务列表")
	@GetMapping
	public GeneralResult<GridBean<TaskDocumentComparison>> list(Pager pager) {
		return documentComparisonService.list(pager);
	}

	/**
	 * 创建
	 *
	 * @param param
	 * @param context
	 * @return
	 */
	@PostMapping
	@Operation(tags = { "CMC" }, summary = "对比任务创建")
	public GeneralResult<Void> create(@RequestBody TaskDocumentComparison param,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return documentComparisonService.create(param, context);
	}

	/**
	 * 删除
	 *
	 * @param id
	 * @param context
	 * @return
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "对比任务删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return documentComparisonService.remove(id, context.getTarget());
	}

	/**
	 * 定时任务操作-启动/暂停
	 *
	 * @param action
	 * @param id
	 * @param requestContext
	 * @return
	 */
	@PatchMapping(value = "/{id}/{action}/schedule")
	@Operation(summary = "定时对比任务操作-启动/暂停")
	public GeneralResult<Void> operate(@PathVariable(value = "action") String action,
			@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
		return documentComparisonService.operate(action, id, requestContext);
	}

	/**
	 * 定时对比添加任务(主服务有效)
	 *
	 * @param id 备份任务表id
	 * @return
	 */
	@GetMapping(value = "/{id}/schedule")
	@Operation(tags = { "CMC" }, summary = "定时对比添加任务(主服务有效)")
	public GeneralResult<String> addScheduleIfLeader(@PathVariable(Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return documentComparisonService.addScheduleIfLeader(id, context);
	}

	/**
	 * 定时对比删除任务(主服务有效)
	 *
	 * @param id 备份任务表id
	 * @return
	 */
	@DeleteMapping(value = "/{id}/schedule")
	@Operation(tags = { "CMC" }, summary = "定时对比删除任务(主服务有效)")
	public GeneralResult<String> removeScheduleIfLeader(@PathVariable(Common.ID) Long id) {
		return documentComparisonService.removeScheduleIfLeader(id);
	}

	@Override
	public void onApplicationEvent(ApplicationReadyEvent event) {
		documentComparisonService.onApplicationEvent();
	}
}