package com.bocloud.cop.booter.controller.inspection;

import com.bocloud.cop.intf.inspection.ExecuteTaskService;
import com.bocloud.cop.intf.inspection.InspectService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

@RestController
@RequestMapping("/v1/inspect")
@Tag(name = "巡检管理")
public class InspectController {

	@Autowired
	private InspectService inspectService;

	@PostMapping(value = "/run")
	@Operation(tags = { "CMC" }, summary = "快速巡检")
	public GeneralResult<Void> run(@RequestBody ExecuteTaskService.RunData runData,
			@Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectService.run(runData, user);
	}

	@GetMapping(value = "/summary")
	@Operation(tags = { "CMC" }, summary = "统计信息")
	public GeneralResult<Object> firstPageData(@Value(Common.REQ_CONTEXT) RequestContext user,
			@RequestParam(value = "dataType", required = false) String type,
			@RequestParam(value = "code") String code) {
		Integer limitDay = 10;
		return inspectService.firstPageData(limitDay, user, type, code);
	}

	@GetMapping(value = "/summary/current")
	@Operation(tags = { "CMC" }, summary = "最近巡检记录")
	public GeneralResult<Map<String, Object>> firstPageDataOverview(@Value(Common.REQ_CONTEXT) RequestContext user,
			@RequestParam(value = "type") String type) {
		return inspectService.firstPageDataOverview(user, type);
	}

	@GetMapping(value = "/summary/history")
	@Operation(tags = { "CMC" }, summary = "历史巡检记录")
	public GeneralResult<Map<String, Object>> firstPageDataHistoryInfo(
			@RequestParam(value = "limitDay") Integer limitDay,
			@RequestParam(value = "category", required = false) String category,
			@RequestParam(value = "type") String type, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return inspectService.firstPageDataHistoryInfo(limitDay, category, context, type);
	}
}
