package com.bocloud.cop.booter.controller.inspection;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.inspection.InspectRecord;
import com.bocloud.cop.entity.inspection.InspectRecordItem;
import com.bocloud.cop.entity.inspection.InspectRecordWarning;
import com.bocloud.cop.intf.inspection.InspectRecordService;
import com.bocloud.cop.model.inspection.ManualInspectRecord;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/v1/inspect/records")
@Tag(name = "巡检记录管理")
public class InspectRecordController {

	@Autowired
	private InspectRecordService inspectRecordService;

	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "巡检记录详情")
	public GeneralResult<InspectRecord> get(@PathVariable(value = Common.ID) Long id) {
		return inspectRecordService.get(id);
	}

	@GetMapping
	@Operation(tags = { "CMC" }, summary = "巡检记录列表")
	public GeneralResult<GridBean<InspectRecord>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectRecordService.list(pager, user.getTarget());
	}

	@GetMapping(value = "/{id}/items")
	@Operation(tags = { "CMC" }, summary = "巡检记录执行项目列表")
	public GeneralResult<GridBean<InspectRecordItem>> listItems(Pager pager,
			@PathVariable(value = Common.ID) Long recordId) {
		return inspectRecordService.listItems(recordId, pager);
	}

	@GetMapping(value = "/{id}/warns")
	@Operation(tags = { "CMC" }, summary = "巡检记录告警项目列表")
	public GeneralResult<GridBean<InspectRecordWarning>> listWarnings(Pager pager,
			@PathVariable(value = Common.ID) Long recordId) {
		return inspectRecordService.listWarning(recordId, pager);
	}

	@GetMapping(value = "/{id}/errors")
	@Operation(tags = { "CMC" }, summary = "巡检记录错误项目列表")
	public GeneralResult<GridBean<InspectRecordItem>> listErrors(Pager pager,
			@PathVariable(value = Common.ID) Long recordId) {
		return inspectRecordService.listError(recordId, pager);
	}

	@PostMapping
	@Operation(tags = { "CMC" }, summary = "巡检记录创建")
	public GeneralResult<InspectRecord> create(@RequestBody ManualInspectRecord manualInspectRecord,
			@Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectRecordService.save(manualInspectRecord, user.getTarget());
	}

	@GetMapping(value = "/{id}/pdf")
	@Operation(tags = { "CMC" }, summary = "巡检记录导出PDF")
	public void pdf(@PathVariable("id") Long id, HttpServletResponse response) {
		inspectRecordService.pdf(id, response);
	}

	@GetMapping(value = "/{id}/excel")
	@Operation(tags = { "CMC" }, summary = "巡检记录导出EXCEL")
	public void excel(@PathVariable("id") Long id, HttpServletResponse response) {
		inspectRecordService.excel(id, response);
	}

	@GetMapping(value = "/details")
	@Operation(tags = { "CMC" }, summary = "历史详情信息")
	public GeneralResult<GridBean<JSONObject>> getDetail(Pager pager,
			@RequestParam(value = "id", required = false) Long id,
			@RequestParam(value = "type", required = false) String type,
			@Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectRecordService.getDetail(pager, id, type);
	}

}
