package com.bocloud.cop.booter.controller.inspection;

import com.bocloud.cop.entity.inspection.InspectRecordWarning;
import com.bocloud.cop.entity.inspection.InspectReportDetail;
import com.bocloud.cop.intf.inspection.InspectReportDetailService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 巡检报表详情Controller
 *
 * @author jiaohongtao
 * @version 1.0
 * @since 2020-11-03 15:09:21
 */
@RestController
@RequestMapping("/v1/inspect/reports/details")
@Tag(name = "巡检报表管理")
public class InspectReportDetailController {

	@Autowired
	private InspectReportDetailService inspectReportDetailService;

	/**
	 * 翻页查询所有
	 *
	 * @param pager 分页参数
	 */
	@GetMapping
	@Operation(tags = { "CMC" }, summary = "巡检详情列表")
	public GeneralResult<GridBean<InspectReportDetail>> list(Pager pager) {
		return inspectReportDetailService.list(pager);
	}

	/**
	 * 详情
	 *
	 * @param id id
	 */
	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "巡检详情查询")
	public GeneralResult<InspectReportDetail> detail(@PathVariable(Common.ID) Long id) {
		return inspectReportDetailService.detail(id);
	}

	/**
	 * 巡检列表详情
	 *
	 * @param id
	 * @param key
	 * @return
	 */
	@GetMapping(value = "/{id}/templates")
	@Operation(tags = { "CMC" }, summary = "巡检详情模板列表")
	public GeneralResult<List<InspectRecordWarning>> templateDetail(@PathVariable(Common.ID) Long id,
			@RequestParam(value = "key") Long key) {
		return inspectReportDetailService.detail(id, key);
	}

	@GetMapping(value = "/{id}/pdf")
	@Operation(tags = { "CMC" }, summary = "巡检记录导出PDF")
	public void pdf(@PathVariable("id") Long id, HttpServletResponse response) {
		inspectReportDetailService.pdf(id, response);
	}

	@GetMapping(value = "/{id}/excel")
	@Operation(tags = { "CMC" }, summary = "巡检记录导出EXCEL")
	public void excel(@PathVariable("id") Long id, HttpServletResponse response) {
		inspectReportDetailService.excel(id, response);
	}
}
