package com.bocloud.cop.booter.controller.inspection;

import com.bocloud.cop.entity.inspection.Inspect;
import com.bocloud.cop.intf.inspection.InspectRoutineService;
import com.bocloud.cop.model.OperationParam;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/v1/inspect/routines")
@Tag(name = "常规巡检管理")
public class InspectRoutineController {

	@Autowired
	private InspectRoutineService inspectRoutineService;

	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "常规巡检详情")
	public GeneralResult<Inspect> get(@PathVariable(value = Common.ID) Long id) {
		return inspectRoutineService.get(id);
	}

	@GetMapping
	@Operation(tags = { "CMC" }, summary = "常规巡检列表")
	public GeneralResult<GridBean<Inspect>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectRoutineService.list(pager, user.getTarget());
	}

	@PostMapping
	@Operation(tags = { "CMC" }, summary = "常规巡检创建")
	public GeneralResult<Inspect> create(@RequestBody Inspect routine, @Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectRoutineService.save(routine, user.getTarget());
	}

	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "常规巡检修改")
	public GeneralResult<Inspect> update(@RequestBody Inspect routine, @PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext user) {
		routine.setId(id);
		return inspectRoutineService.update(routine, user.getTarget());
	}

	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "常规巡检删除")
	public GeneralResult<Void> delete(@PathVariable("id") Long id, @Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectRoutineService.delete(id, user.getTarget());
	}

	@PatchMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "常规巡检操作")
	public GeneralResult<Void> run(@PathVariable("id") Long id,
			@RequestParam(value = Common.ACTION, required = false) String action,
			@RequestBody(required = false) OperationParam param, @Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectRoutineService.routineRun(id, action, param, user);
	}

	@PatchMapping(value = "/{id}/release")
	@Operation(tags = { "CMC", "CSC" }, summary = "操作常规巡检-发布")
	public GeneralResult<Void> release(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return inspectRoutineService.release(id, context.getTarget());
	}

	@PatchMapping(value = "/{id}/recall")
	@Operation(tags = { "CMC", "CSC" }, summary = "操作常规巡检-下架")
	public GeneralResult<Void> recall(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return inspectRoutineService.recall(id, context.getTarget());
	}

}
