package com.bocloud.cop.booter.controller.inspection;

import com.bocloud.cop.entity.inspection.InspectWarnLevel;
import com.bocloud.cop.intf.inspection.InspectWarnService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 巡检告警模板控制器
 *
 * @author weirongheng
 */
@RestController
@RequestMapping("/v1/inspect/warns")
@Tag(name = "巡检告警管理")
public class InspectWarnController {

	@Autowired
	private InspectWarnService inspectWarnService;

	/**
	 * 新增巡检模板
	 *
	 * @param inspectWarnLevel
	 * @param user
	 * @return
	 */
	@PostMapping
	@Operation(tags = { "CMC" }, summary = "告警创建")
	public GeneralResult<Void> create(@RequestBody InspectWarnLevel inspectWarnLevel,
			@Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectWarnService.create(inspectWarnLevel, user.getTarget());
	}

	/**
	 * 更新告警模板
	 *
	 * @param warnLevel
	 * @param user
	 * @return
	 */
	@PutMapping(value = "{id}")
	@Operation(tags = { "CMC" }, summary = "告警修改")
	public GeneralResult<Void> modify(@PathVariable Long id, @RequestBody InspectWarnLevel warnLevel,
			@Value(Common.REQ_CONTEXT) RequestContext user) {
		warnLevel.setId(id);
		return inspectWarnService.update(warnLevel, user.getTarget());
	}

	/**
	 * 删除告警模板
	 *
	 * @param id
	 * @param user
	 * @return
	 */
	@DeleteMapping(value = "{id}")
	@Operation(tags = { "CMC" }, summary = "告警删除")
	public GeneralResult<Void> remove(@PathVariable Long id, @Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectWarnService.delete(id, user.getTarget());
	}

	/**
	 * 查询告警模板
	 *
	 * @param pager
	 */
	@GetMapping
	@Operation(tags = { "CMC" }, summary = "告警列表")
	public GeneralResult<GridBean<InspectWarnLevel>> list(Pager pager) {
		return inspectWarnService.list(pager);
	}
}
