package com.bocloud.cop.booter.controller.module;

import com.bocloud.cop.entity.module.NodeModuleGroup;
import com.bocloud.cop.intf.NodeModuleGroupService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 分组控制器
 *
 * @author weiwei
 */
@RestController
@RequestMapping("/v1/tasknodes/modules/groups")
@Tag(name = "模型分组管理")
public class NodeModuleGroupController {

	@Autowired
	private NodeModuleGroupService nodeModuleGroupService;

	/**
	 * 获取分组列表
	 *
	 * @param parentId
	 * @param context
	 * @return 数据结果集
	 */
	@GetMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "分组列表")
	public GeneralResult<List<NodeModuleGroup>> list(
			@RequestParam(value = Common.PARENTID, required = false) Long parentId,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return nodeModuleGroupService.list(parentId);
	}

	/**
	 * 添加分组
	 *
	 * @param params
	 * @param context
	 * @return 添加结果
	 */
	@PostMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "分组创建")
	public GeneralResult<Void> create(@RequestBody NodeModuleGroup params,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		params.setCreatorId(context.getTarget());
		params.setMenderId(context.getTarget());
		return nodeModuleGroupService.create(params);
	}

	/**
	 * 修改分组
	 *
	 * @param params
	 * @param id
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "分组修改")
	public GeneralResult<Void> modify(@RequestBody NodeModuleGroup params, @PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return nodeModuleGroupService.modify(params, context.getTarget());
	}

	/**
	 * 移除分组
	 *
	 * @param id
	 * @param context
	 * @return 操作结果
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "分组删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return nodeModuleGroupService.remove(id, context.getTarget());
	}

	/**
	 * 查看分组详细信息
	 *
	 * @param id
	 * @return 操作结果
	 */
	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "分组详情")
	public GeneralResult<NodeModuleGroup> detail(@PathVariable(value = Common.ID) Long id) {
		return nodeModuleGroupService.detail(id);
	}

	/**
	 * 获取分组列表
	 *
	 * @param code
	 * @return 数据结果集
	 */
	@GetMapping(value = "/nodes")
	@Operation(tags = { "CMC", "CSC" }, summary = "分组节点列表")
	public GeneralResult<List<NodeModuleGroup>> nodes(@RequestParam(value = Common.CODE, required = true) String code,
			@RequestParam(value = "executeType", required = true) String executeType,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return nodeModuleGroupService.listAllNodes(code, executeType, context);
	}

}
