package com.bocloud.cop.booter.controller.network;

import com.bocloud.cop.entity.NetworkSegment;
import com.bocloud.cop.intf.network.NetworkSegmentService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/v1/network/segments")
@Tag(name = "网段管理")
public class NetworkSegmentController {

	@Autowired
	NetworkSegmentService networkSegmentService;

	/**
	 * 获取网段列表
	 *
	 * @param pager
	 * @return 数据结果集
	 */
	@GetMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "网段列表")
	public GeneralResult<GridBean<NetworkSegment>> list(Pager pager,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return networkSegmentService.list(pager);
	}

	/**
	 * 添加网段
	 *
	 * @param params
	 * @param context
	 * @return
	 */
	@PostMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "网段创建")
	public GeneralResult<Void> create(@RequestBody NetworkSegment params,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		params.setCreatorId(context.getTarget());
		params.setMenderId(context.getTarget());
		return networkSegmentService.create(params);
	}

	/**
	 * 查看网段详细信息
	 *
	 * @param id
	 * @return 操作结果
	 */
	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "网段详情")
	public GeneralResult<NetworkSegment> detail(@PathVariable(value = Common.ID) Long id) {
		return networkSegmentService.detail(id);
	}

	/**
	 * 删除网段
	 *
	 * @param id
	 * @param context
	 * @return 操作结果
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "网段删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return networkSegmentService.remove(id, context.getTarget());
	}

	/**
	 * 修改网段
	 *
	 * @param params
	 * @param id
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "网段修改")
	public GeneralResult<Void> modify(@RequestBody NetworkSegment params, @PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		params.setMenderId(context.getTarget());
		params.setId(id);
		return networkSegmentService.modify(params);
	}

	/**
	 * 查看网段详细信息
	 *
	 * @param id
	 * @return 操作结果
	 */
	@GetMapping(value = "/{id}/hosts/count")
	@Operation(tags = { "CMC", "CSC" }, summary = "网段主机数量查询")
	public GeneralResult<Integer> hostCount(@PathVariable(value = Common.ID) Long id) {
		return networkSegmentService.getHostConut(id);
	}
}
