package com.bocloud.cop.booter.controller.network;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.TaskWorker;
import com.bocloud.cop.intf.TaskHostService;
import com.bocloud.cop.intf.network.TaskWorkerService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/v1/task/workers")
@Tag(name = "worker管理")
public class TaskWorkerController {

	@Autowired
	TaskWorkerService taskWorkerService;

	@Autowired
	private TaskHostService taskHostService;

	/**
	 * worker作业排除
	 *
	 * @param object
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/exclude")
	@Operation(tags = { "CMC", "CSC" }, summary = "worker排除")
	public GeneralResult<Void> exclude(@RequestBody JSONObject object,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		List<Long> workerIds = object.getJSONArray("workerIds").toJavaList(Long.class);
		return taskWorkerService.exclude(workerIds, context);
	}

	/**
	 * worker作业排除恢复
	 *
	 * @param object
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/recovery")
	@Operation(tags = { "CMC", "CSC" }, summary = "worker恢复")
	public GeneralResult<Void> recovery(@RequestBody JSONObject object,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		List<Long> workerIds = object.getJSONArray("workerIds").toJavaList(Long.class);
		return taskWorkerService.recovery(workerIds, context);
	}

	/**
	 * 获取作业worker列表
	 *
	 * @param pager
	 * @return 数据结果集
	 */
	@GetMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "worker列表")
	public GeneralResult<GridBean<TaskWorker>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskWorkerService.list(pager);
	}

	/**
	 * 添加作业worker
	 *
	 * @param params
	 * @param context
	 * @return
	 */
	@PostMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "worker创建")
	public GeneralResult<Void> create(@RequestBody TaskWorker params,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		params.setCreatorId(context.getTarget());
		params.setMenderId(context.getTarget());
		return taskWorkerService.create(params);
	}

	/**
	 * 查看作业worker详细信息
	 *
	 * @param id
	 * @return 操作结果
	 */
	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "worker详情")
	public GeneralResult<TaskWorker> detail(@PathVariable(value = Common.ID) Long id) {
		return taskWorkerService.detail(id);
	}

	/**
	 * 删除作业worker
	 *
	 * @param id
	 * @param context
	 * @return 操作结果
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "worker删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskWorkerService.remove(id, context.getTarget());
	}

	/**
	 * 修改作业worker
	 *
	 * @param params
	 * @param id
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "worker修改")
	public GeneralResult<Void> modify(@RequestBody TaskWorker params, @PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		params.setId(id);
		params.setMenderId(context.getTarget());
		return taskWorkerService.modify(params);
	}

	/**
	 * 配置worker组
	 *
	 * @param workerId
	 * @param groupId
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/{workerId}/configure/{groupId}")
	@Operation(tags = { "CMC", "CSC" }, summary = "worker配置")
	public GeneralResult<Void> configure(@PathVariable Long workerId, @PathVariable Long groupId,
			@RequestBody JSONObject object, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskWorkerService.configure(workerId, groupId, object.getBoolean("isSuper"));
	}

	/**
	 * 同步SaltMinions
	 *
	 * @param id      salt id
	 * @param context 操作人
	 * @return 同步minions结果
	 */
	@PostMapping(value = "/salt/minion/sync/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "SaltMinions同步")
	public GeneralResult<Void> syncSaltMinions(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskHostService.syncSaltMinions(context, id);
	}
}
