package com.bocloud.cop.booter.controller.patch;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.TaskHost;
import com.bocloud.cop.entity.patch.Patch;
import com.bocloud.cop.intf.patch.PatchDeployService;
import com.bocloud.cop.intf.patch.PatchService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * @author zoucg
 * @Company: 博云 www.bocloud.com
 * @ClassName: PatchController
 * @Description:
 * @DATE: 2022/6/13
 */
@RestController
@RequestMapping("/v1/patchs")
@Tag(name = "补丁管理")
public class PatchController {

	@Autowired
	private PatchService patchService;
	@Autowired
	private PatchDeployService patchDeployService;

	@GetMapping
	@Operation(tags = { "CMC" }, summary = "补丁列表")
	public GeneralResult<GridBean<Patch>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return patchService.list(pager, context.getTarget());
	}

	@PostMapping
	@Operation(tags = { "CMC" }, summary = "补丁创建")
	public GeneralResult<Patch> create(@RequestBody Patch Patch, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return patchService.create(Patch, context.getTarget());
	}

	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "补丁删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return patchService.remove(id, context.getTarget());
	}

	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "补丁修改")
	public GeneralResult<Patch> modify(@RequestBody Patch Patch, @PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		Patch.setId(id);
		return patchService.modify(Patch, context.getTarget());
	}

	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "补丁详情")
	public GeneralResult<Patch> detail(@PathVariable(value = Common.ID) Long id) {
		return patchService.detail(id);
	}

	@PatchMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "补丁发布/下架")
	public GeneralResult<Void> updateStatus(@PathVariable(value = Common.ID) Long id, @RequestBody JSONObject params,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		String status = params.getString("status");
		return patchService.updateStatus(id, status, context.getTarget());
	}

	@GetMapping(value = "/{id}/hosts")
	@Operation(tags = { "CMC", "CSC" }, summary = "补丁部署主机列表")
	public GeneralResult<GridBean<TaskHost>> recall(Pager pager, @PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return patchDeployService.listInstallHost(pager, id, context.getTarget());
	}
}
