package com.bocloud.cop.booter.controller.patch;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.patch.PatchDeploy;
import com.bocloud.cop.enums.Catalog;
import com.bocloud.cop.enums.TaskParam;
import com.bocloud.cop.intf.patch.PatchDeployService;
import com.bocloud.cop.model.Job;
import com.bocloud.cop.service.utils.JobScheduler;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * @author renzhihui
 */
@RestController
@RequestMapping("/v1/patchs/deploy")
@Tag(name = "补丁部署管理")
public class PatchDeployController {

	@Autowired
	private PatchDeployService patchDeployService;
	@Autowired
	private JobScheduler jobScheduler;

	@GetMapping
	@Operation(tags = { "CMC" }, summary = "部署列表")
	public GeneralResult<GridBean<PatchDeploy>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return patchDeployService.list(pager, context.getTarget());
	}

	@PostMapping
	@Operation(tags = { "CMC" }, summary = "部署创建")
	public GeneralResult<PatchDeploy> create(@RequestBody PatchDeploy deployPatch,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return patchDeployService.create(deployPatch, false, context.getTarget());
	}

	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "部署修改")
	public GeneralResult<PatchDeploy> modify(@RequestBody PatchDeploy deployPatch,
			@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
		deployPatch.setId(id);
		return patchDeployService.modify(deployPatch, context.getTarget());
	}

	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "部署删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return patchDeployService.remove(id, context.getTarget());
	}

	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "部署详情")
	public GeneralResult<PatchDeploy> detail(@PathVariable(value = Common.ID) Long id) {
		return patchDeployService.detail(id);
	}

	@PatchMapping(value = "/{id}/operation")
	@Operation(tags = { "CMC" }, summary = "部署操作")
	public GeneralResult<Void> run(@PathVariable("id") Long id, @RequestBody JSONObject params,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		String action = params.getString("action");
		if ("run".equals(action)) {
			GeneralResult<TaskInstance> result = patchDeployService.run(id, context);
			if (result.isFailed()) {
				return new GeneralResult<>(false, "操作失败");
			}
			TaskInstance instance = result.getData();
			Job job = new Job();
			job.setTaskId(instance.getUuid());
			job.setParam("QUEUE", "task.job.patch.result");
			job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(instance));
			job.setParam(TaskParam.TASKTYPE.name(), Catalog.Task.STANDARD.getValue());
			job.setParam(Common.NAME, instance.getName());
			boolean flag = jobScheduler.insertTask(job);
			if (!flag) {
				return new GeneralResult<>(false, "任务下发失败");
			}
			return new GeneralResult<>(true, "操作成功");
		}
		return new GeneralResult<>(false, "不支持的操作类型");
	}

	@PostMapping(value = "/run/fast")
	@Operation(tags = { "CMC" }, summary = "部署创建(快速)")
	public GeneralResult<Void> runFast(@RequestBody PatchDeploy deployPatch,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		GeneralResult<PatchDeploy> result = patchDeployService.create(deployPatch, true, context.getTarget());
		if (result.isFailed()) {
			return new GeneralResult<>(false, "补丁部署创建失败：" + result.getMessage());
		}

		PatchDeploy data = result.getData();
		GeneralResult<TaskInstance> taskResult = patchDeployService.run(data.getId(), context);
		if (result.isFailed()) {
			return new GeneralResult<>(false, "操作失败");
		}
		TaskInstance instance = taskResult.getData();
		Job job = new Job();
		job.setTaskId(instance.getUuid());
		job.setParam("QUEUE", "task.job.patch.result");
		job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(instance));
		job.setParam(TaskParam.TASKTYPE.name(), Catalog.Task.STANDARD.getValue());
		job.setParam(Common.NAME, instance.getName());
		boolean flag = jobScheduler.insertTask(job);
		if (!flag) {
			return new GeneralResult<>(false, "任务下发失败");
		}
		return new GeneralResult<>(true, "操作成功");
	}

	@PatchMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "部署发布/下架")
	public GeneralResult<Void> updateStatus(@PathVariable(value = Common.ID) Long id, @RequestBody JSONObject params,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		String status = params.getString("status");
		return patchDeployService.updateStatus(id, status, context.getTarget());
	}

	// @GetMapping(value = "/tasks/templates")
	// @Operation(tags = {"CMC"}, summary = "查询执行模板")
	// public GeneralResult<List<TaskTemplate>> taskTemplate() {
	// return patchDeployService.listTemplates();
	// }
	//
	// @GetMapping(value = "/templates")
	// @Operation(tags = {"CMC"}, summary = "查询补丁模板")
	// public GeneralResult<List<SoftwareGroup>> template() {
	// return patchDeployService.listPatchs();
	// }
	//
	// @GetMapping(value = "/softwares")
	// @Operation(tags = {"CMC"}, summary = "查询补丁部署软件包")
	// public GeneralResult<List<Software>> softwareList(@RequestParam(value =
	// "groupId", required = false) Long
	// groupId) {
	// if (null == groupId) {
	// return new GeneralResult<>(true, "获取补丁文件成功");
	// }
	// return patchDeployService.listSoftwares(groupId);
	// }
}
