package com.bocloud.cop.booter.listener;

import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.MessageListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.enums.TaskHostType;
import com.bocloud.cop.intf.TaskHostService;
import com.megatron.common.utils.JSONTools;

import lombok.extern.slf4j.Slf4j;

/**
 * taskHost监听器
 * 作用：ims的虚拟机，宿主机变更后通过消息队列通知cop修改对应主机信息.
 * @author weiwei
 */
@Slf4j
@Component("taskHostListener")
public class TaskHostListener implements MessageListener {

    @Autowired
    private TaskHostService taskHostService;

    @Override
    public void onMessage(Message message) {
        String content = new String(message.getBody());
        processDebugLog("get taskHost sync message:{}", content);
        if (!StringUtils.hasText(content)) {
            // 消息没有内容，直接确认收到 淹没处理
            return;
        }
        JSONObject object = JSONTools.isJSONObj(content);
        if (null == object) {
            // 消息参数格式错误，直接确认收到淹没处理
            return;
        }
        // log.info("taskHost同步信息：{}", content);
        String action = object.getString("action");
        String category = object.getString("category");
        // 操作不能为空
        if (org.apache.commons.lang3.StringUtils.isBlank(action)) {
            return;
        }
        // 暂时只支持category 为PHYSICAL，LOGICAL, 平台删除DELYVID没有category字段
        if (!TaskHostType.Category.PHYSICAL.name().equals(category)
            && !TaskHostType.Category.LOGICAL.name().equals(category)
            && !TaskHostType.Action.DELYVID.name().equals(action)) {
            return;
        }
        JSONObject data = object.getJSONObject("body");
        if (null == data) {
            return;
        }
        data.put("category", category);
        taskHostService.modifyByMessage(action, data, category);
    }

    private void processDebugLog(String format, Object... objects) {
        if (log.isDebugEnabled()) {
            log.debug(format, objects);
        }
    }

}
