package com.bocloud.cop.booter.scheduler;


import com.bocloud.cop.entity.*;
import com.bocloud.cop.enums.Status;
import com.bocloud.cop.repository.*;
import com.bocloud.cop.utils.DynamicClassUtils;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Component
public class TaskDataBakScheduler {

    private static final String TASK_NODE_BAK_ = "task_node_bak_";
    private static final String TASK_NODE_RELATION_BAK_ = "task_node_relation_bak_";
    private static final String TASK_INSTANCE_HOST_BAK_ = "task_instance_host_bak_";
    private static final String TASK_INSTANCE_BAK_ = "task_instance_bak_";
    private static final String TASK_NODE_LOG_BAK_ = "task_node_log_bak_";

    private static final String TASK_NODE_BAK = "task_node_bak";
    private static final String TASK_NODE_RELATION_BAK = "task_node_relation_bak";
    private static final String TASK_INSTANCE_HOST_BAK = "task_instance_host_bak";
    private static final String TASK_INSTANCE_BAK = "task_instance_bak";
    private static final String TASK_NODE_LOG_BAK = "task_node_log_bak";

    @Value("${spring.datasource.druid.dialect:mysql}")
    private String databaseDriver;
    @Autowired
    private TaskNodeBakRepository taskNodeBakRepository;
    @Autowired
    private NodeRelationBakRepository nodeRelationBakRepository;
    @Autowired
    private TaskInstanceHostBakRepository taskInstanceHostBakRepository;
    @Autowired
    private TaskInstanceBakRepository taskInstanceBakRepository;
    @Autowired
    private TableLogRepository tableLogRepository;
    @Autowired
    private CurrentService service;
    @Autowired
    private TaskInstanceRepository taskInstanceRepository;
    @Autowired
    private TaskNodeRepository taskNodeRepository;
    @Autowired
    private TaskInstanceHostRepository taskInstanceHostRepository;
    @Autowired
    private NodeRelationRepository nodeRelationRepository;
    @Autowired
    private TaskConfigRepository taskConfigRepository;
    @Autowired
    private TaskNodeLogBakRepository taskNodeLogBakRepository;
    @Autowired
    private TaskNodeLogRepository taskNodeLogRepository;


    //每月第一天的0时1分开始创建当月的备份表
    @Scheduled(cron = "${task.instance.create-table:0 1 0 1 * ? }")
    private void createTable() {
        try {
            if (!service.isLeader()) {
                return;
            }
            TaskConfig switchConfig = taskConfigRepository.queryByCode("TaskDataBakSwitch");
            if (null == switchConfig || StringUtils.isEmpty(switchConfig.getValue()) || !switchConfig.getValue().equalsIgnoreCase("true")) {
                log.info("备份任务未开启");
                return;
            }
            String tableNameTaskNodeBak = TaskDataBakScheduler.TASK_NODE_BAK_ + this.getTableSuffix(LocalDateTime.now());
            String tableNameNodeRelationBak = TaskDataBakScheduler.TASK_NODE_RELATION_BAK_ + this.getTableSuffix(LocalDateTime.now());
            String tableNameTaskInstanceHostBak = TaskDataBakScheduler.TASK_INSTANCE_HOST_BAK_ + this.getTableSuffix(LocalDateTime.now());
            String tableNameTaskInstanceBak = TaskDataBakScheduler.TASK_INSTANCE_BAK_ + this.getTableSuffix(LocalDateTime.now());
            String tableNameTaskModeLogBak = TaskDataBakScheduler.TASK_NODE_LOG_BAK_ + this.getTableSuffix(LocalDateTime.now());
            this.createTaskInstanceBakTable("dm".equalsIgnoreCase(databaseDriver) ? tableNameTaskInstanceBak.toUpperCase():tableNameTaskInstanceBak , databaseDriver);
            this.createTaskNodeBakTable("dm".equalsIgnoreCase(databaseDriver) ? tableNameTaskNodeBak.toUpperCase():tableNameTaskNodeBak,databaseDriver);
            this.createNodeRelationBakTable("dm".equalsIgnoreCase(databaseDriver) ? tableNameNodeRelationBak.toUpperCase():tableNameNodeRelationBak,databaseDriver);
            this.createTaskInstanceHostBakTable("dm".equalsIgnoreCase(databaseDriver) ? tableNameTaskInstanceHostBak.toUpperCase():tableNameTaskInstanceHostBak,databaseDriver);
            this.createTaskNodeLogBakTable("dm".equalsIgnoreCase(databaseDriver) ? tableNameTaskModeLogBak.toUpperCase():tableNameTaskModeLogBak,databaseDriver);
        } catch (Exception e) {
            log.error("定时创建库表失败", e);
        }
    }

    @Scheduled(cron = "${task.instance.bakup:0 0 4 * * ?}")
    private void instanceDataBak() {
        if (!service.isLeader()) {
            return;
        }
        log.info("开始备份任务数据");
        try {
            int stepSize = 200;
            TaskConfig switchConfig = taskConfigRepository.queryByCode("TaskDataBakSwitch");
            if (null == switchConfig || StringUtils.isEmpty(switchConfig.getValue()) || !switchConfig.getValue().equalsIgnoreCase("true")) {
                log.info("备份任务未开启");
                return;
            }
            Integer days = null;
            TaskConfig config = taskConfigRepository.queryByCode("TaskDataBakDaysAgo");
            if (null != config && StringUtils.isNotEmpty(config.getValue())) {
                days = Integer.valueOf(config.getValue());
            }
            days = null == days ? 90 : days;
            LocalDate localDate = LocalDate.now().minusDays(days);
            LocalDateTime localDateTime = LocalDateTime.of(localDate, LocalTime.MIN);
            DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
            String formatTime = dateTimeFormatter.format(localDateTime);
            log.info("备份{}之前的数据", formatTime);
            List<Param> params = Lists.newArrayList();
            params.add(new Param(MapTools.simpleMap("gmtCreate", formatTime), Sign.LT));
            while (true) {
                List<TaskInstance> instances = this.taskInstanceRepository.list(params, 1000);
                if (ListTool.isEmpty(instances)) {
                    log.info("结束备份任务数据，没有查询到需要备份的数据");
                    return;
                }
                int size = instances.size();
                log.info("备份任务数{}，每批任务数{}", size, stepSize);
                if (size > stepSize) {
                    int num = size % stepSize == 0 ? size / stepSize : size / stepSize + 1;
                    for (int i = 0; i < num; i++) {
                        log.info("备份第{}批数据", i + 1);
                        if (i + 1 == num) {
                            batchBak(instances.subList(i * stepSize, size));
                        } else {
                            batchBak(instances.subList(i * stepSize, (i + 1) * stepSize));
                        }
                    }

                } else {
                    batchBak(instances);
                }
            }

        } catch (Exception e) {
            log.error("task instance backup fail:", e);
        }

    }

    private void batchBak(List<TaskInstance> instances) {
        try {
            //根据年和月分组
            Map<String, List<TaskInstance>> map = new HashMap<>();
            for (TaskInstance instance : instances) {
                LocalDateTime localDateTime = LocalDateTime.ofInstant(instance.getGmtCreate().toInstant(), ZoneId.systemDefault());
                String tableSuffix = this.getTableSuffix(localDateTime);
                if (map.containsKey(tableSuffix)) {
                    List<TaskInstance> taskInstanceList = map.get(tableSuffix);
                    taskInstanceList.add(instance);
                } else {
                    List<TaskInstance> taskInstances = new ArrayList<>();
                    taskInstances.add(instance);
                    map.put(tableSuffix, taskInstances);
                }
            }
            for (String tableSuffix : map.keySet()) {
                List<TaskInstance> taskInstances = map.get(tableSuffix);
                List<Long> instanceIds = taskInstances.stream().map(TaskInstance::getId).collect(Collectors.toList());
                List<TaskInstanceBak> instanceBaks = Lists.newArrayList();
                for (TaskInstance instance : taskInstances) {
                    TaskInstanceBak bak = new TaskInstanceBak();
                    BeanUtils.copyProperties(instance, bak);
                    instanceBaks.add(bak);
                }
                if (!instanceBaks.isEmpty()) {
                    String mainTableName = TaskDataBakScheduler.TASK_INSTANCE_BAK;
                    String tableName = TaskDataBakScheduler.TASK_INSTANCE_BAK_ + tableSuffix;
                    if("dm".equalsIgnoreCase(databaseDriver)){
                        tableName = tableName.toUpperCase();
                    }
                    GeneralResult<Void> taskInstanceBakTable = this.createTaskInstanceBakTable(tableName,databaseDriver);
                    if (taskInstanceBakTable.isSuccess()) {
                        this.batchSaveTaskInstanceBak(instanceBaks, tableName, mainTableName, instanceBaks.size());
                        taskInstanceRepository.delete(instanceIds);
                    }
                }
                List<TaskNode> nodes = taskNodeRepository.listByInstances(instanceIds);
                List<TaskNodeBak> nodeBaks = Lists.newArrayList();
                for (TaskNode node : nodes) {
                    TaskNodeBak bak = new TaskNodeBak();
                    BeanUtils.copyProperties(node, bak);
                    nodeBaks.add(bak);
                }
                if (!nodeBaks.isEmpty()) {
                    String mainTableName = TaskDataBakScheduler.TASK_NODE_BAK;
                    String tableName = TaskDataBakScheduler.TASK_NODE_BAK_ + tableSuffix;
                    if("dm".equalsIgnoreCase(databaseDriver)){
                        tableName = tableName.toUpperCase();
                    }
                    GeneralResult<Void> taskNodeBakTable = this.createTaskNodeBakTable(tableName,databaseDriver);
                    if (taskNodeBakTable.isSuccess()) {
                        this.batchSaveTaskNodeBak(nodeBaks, tableName, mainTableName, nodeBaks.size());
                        taskNodeRepository.deleteByInstanceIds(instanceIds);
                    }
                }
                List<NodeRelation> relations = nodeRelationRepository.listByInstances(instanceIds);
                List<NodeRelationBak> relationBaks = Lists.newArrayList();
                for (NodeRelation relation : relations) {
                    NodeRelationBak bak = new NodeRelationBak();
                    BeanUtils.copyProperties(relation, bak);
                    relationBaks.add(bak);
                }
                if (!relationBaks.isEmpty()) {
                    String mainTableName = TaskDataBakScheduler.TASK_NODE_RELATION_BAK;
                    String tableName = TaskDataBakScheduler.TASK_NODE_RELATION_BAK_ + tableSuffix;
                    if("dm".equalsIgnoreCase(databaseDriver)){
                        tableName = tableName.toUpperCase();
                    }
                    GeneralResult<Void> nodeRelationBakTable = this.createNodeRelationBakTable(tableName,databaseDriver);
                    if (nodeRelationBakTable.isSuccess()) {
                        this.batchSaveNodeRelationBak(relationBaks, tableName, mainTableName, relationBaks.size());
                        nodeRelationRepository.deleteByInstanceIds(instanceIds);
                    }
                }
                List<TaskInstanceHost> hosts = taskInstanceHostRepository.listByInstances(instanceIds);
                List<TaskInstanceHostBak> hostBaks = Lists.newArrayList();
                for (TaskInstanceHost host : hosts) {
                    TaskInstanceHostBak bak = new TaskInstanceHostBak();
                    BeanUtils.copyProperties(host, bak);
                    hostBaks.add(bak);
                }
                if (!hostBaks.isEmpty()) {
                    String mainTableName = TaskDataBakScheduler.TASK_INSTANCE_HOST_BAK;
                    String tableName = TaskDataBakScheduler.TASK_INSTANCE_HOST_BAK_ + tableSuffix;
                    if("dm".equalsIgnoreCase(databaseDriver)){
                        tableName = tableName.toUpperCase();
                    }
                    GeneralResult<Void> taskInstanceHostBakTable = this.createTaskInstanceHostBakTable(tableName,databaseDriver);
                    if (taskInstanceHostBakTable.isSuccess()) {
                        this.batchSaveTaskInstanceHostBak(hostBaks, tableName, mainTableName, hostBaks.size());
                        taskInstanceHostRepository.deleteByInstanceIds(instanceIds);
                    }
                }
                List<TaskNodeLog> logs = taskNodeLogRepository.listByInstances(instanceIds);
                List<TaskNodeLogBak> logBaks = Lists.newArrayList();
                for (TaskNodeLog log : logs) {
                    TaskNodeLogBak bak = new TaskNodeLogBak();
                    BeanUtils.copyProperties(log, bak);
                    logBaks.add(bak);
                }
                if (!logBaks.isEmpty()) {
                    String mainTableName = TaskDataBakScheduler.TASK_NODE_LOG_BAK;
                    String tableName = TaskDataBakScheduler.TASK_NODE_LOG_BAK_ + tableSuffix;
                    if("dm".equalsIgnoreCase(databaseDriver)){
                        tableName = tableName.toUpperCase();
                    }
                    GeneralResult<Void> taskNodeLogBakTable = this.createTaskNodeLogBakTable(tableName,databaseDriver);
                    if (taskNodeLogBakTable.isSuccess()) {
                        this.batchSaveTaskNodeLogBak(logBaks, tableName, mainTableName, logBaks.size());
                        taskNodeLogRepository.deleteByInstanceIds(instanceIds);
                    }
                }
            }
        } catch (Exception e) {
            log.error("任务备份异常", e);
        }
    }


    //批量新增
    public void batchSaveTaskNodeBak(List<TaskNodeBak> taskNodes, String tableName, String mainTableName, Integer batchSize) {
        synchronized (TaskNode.class) {
            DynamicClassUtils.changeTableAnnotationValue(TaskNodeBak.class, tableName);
            taskNodeBakRepository.batchSave(taskNodes, batchSize, false);
            //还原操作
            DynamicClassUtils.changeTableAnnotationValue(TaskNodeBak.class, mainTableName);
        }
    }

    //批量新增
    public void batchSaveNodeRelationBak(List<NodeRelationBak> nodeRelations, String tableName, String mainTableName, Integer batchSize) {
        synchronized (NodeRelation.class) {
            DynamicClassUtils.changeTableAnnotationValue(NodeRelationBak.class, tableName);
            nodeRelationBakRepository.batchSave(nodeRelations, batchSize, false);
            //还原操作
            DynamicClassUtils.changeTableAnnotationValue(NodeRelationBak.class, mainTableName);
        }
    }

    //批量新增
    public void batchSaveTaskInstanceHostBak(List<TaskInstanceHostBak> taskInstanceHosts, String tableName, String mainTableName, Integer batchSize) {
        synchronized (TaskInstanceHost.class) {
            DynamicClassUtils.changeTableAnnotationValue(TaskInstanceHostBak.class, tableName);
            taskInstanceHostBakRepository.batchSave(taskInstanceHosts, batchSize, false);
            //还原操作
            DynamicClassUtils.changeTableAnnotationValue(TaskInstanceHostBak.class, mainTableName);
        }

    }

    //批量新增
    public void batchSaveTaskInstanceBak(List<TaskInstanceBak> taskInstances, String tableName, String mainTableName, Integer batchSize) {
        synchronized (TaskInstance.class) {
            //更改Table注解值
            DynamicClassUtils.changeTableAnnotationValue(TaskInstanceBak.class, tableName);
            taskInstanceBakRepository.batchSave(taskInstances, batchSize, false);
            //还原操作
            DynamicClassUtils.changeTableAnnotationValue(TaskInstanceBak.class, mainTableName);
        }
    }

    public void batchSaveTaskNodeLogBak(List<TaskNodeLogBak> logBaks, String tableName, String mainTableName, Integer batchSize) {
        synchronized (TaskInstanceHost.class) {
            DynamicClassUtils.changeTableAnnotationValue(TaskNodeLogBak.class, tableName);
            taskNodeLogBakRepository.batchSave(logBaks, batchSize, false);
            //还原操作
            DynamicClassUtils.changeTableAnnotationValue(TaskNodeLogBak.class, mainTableName);
        }

    }

    //创建表
    public GeneralResult<Void> createTaskNodeBakTable(String tableName,String databaseDriver) {
        synchronized (TaskNode.class) {
            String templateTable = TaskDataBakScheduler.TASK_NODE_BAK;
            try {
                if (taskNodeBakRepository.countTable(tableName,databaseDriver) <= 0) {
                    taskNodeBakRepository.createTable(tableName, databaseDriver, new TaskNodeBak());
                    this.saveTableLog(tableName, templateTable, Status.Inst.SUCCESS.name());
                }
                return new GeneralResult<>(true, "创建成功");
            } catch (Exception e) {
                this.saveTableLog(tableName, templateTable, Status.Inst.FAILED.name());
                log.error("create task_node_bak table error:", e);
                return new GeneralResult<>(false, "创建失败");
            }
        }
    }

    //创建表
    public GeneralResult<Void> createNodeRelationBakTable(String tableName,String databaseDriver) {
        synchronized (NodeRelation.class) {
            String templateTable = TaskDataBakScheduler.TASK_NODE_RELATION_BAK;
            try {
                if (nodeRelationBakRepository.countTable(tableName,databaseDriver) <= 0) {
                    nodeRelationBakRepository.createTable(tableName, databaseDriver, new NodeRelationBak());
                    this.saveTableLog(tableName, templateTable, Status.Inst.SUCCESS.name());
                }
                return new GeneralResult<>(true, "创建成功");
            } catch (Exception e) {
                this.saveTableLog(tableName, templateTable, Status.Inst.FAILED.name());
                log.error("create task_node_relation_bak table error:", e);
                return new GeneralResult<>(false, "创建失败");
            }
        }
    }

    //创建表
    public GeneralResult<Void> createTaskInstanceHostBakTable(String tableName,String databaseDriver) {
        synchronized (TaskInstanceHost.class) {
            String templateTable = TaskDataBakScheduler.TASK_INSTANCE_HOST_BAK;
            try {
                if (taskInstanceHostBakRepository.countTable(tableName,databaseDriver) <= 0) {
                    taskInstanceHostBakRepository.createTable(tableName, databaseDriver, new TaskInstanceHostBak());
                    this.saveTableLog(tableName, templateTable, Status.Inst.SUCCESS.name());
                }
                return new GeneralResult<>(true, "创建成功");
            } catch (Exception e) {
                this.saveTableLog(tableName, templateTable, Status.Inst.FAILED.name());
                log.error("create task_instance_host_bak table error:", e);
                return new GeneralResult<>(false, "创建失败");
            }
        }
    }

    //创建表
    public GeneralResult<Void> createTaskInstanceBakTable(String tableName,String databaseDriver) {
        synchronized (TaskInstance.class) {
            String templateTable = TaskDataBakScheduler.TASK_INSTANCE_BAK;
            try {
                if (taskInstanceBakRepository.countTable(tableName,databaseDriver) <= 0) {
                    taskInstanceBakRepository.createTable(tableName, databaseDriver,new TaskInstanceBak());
                    this.saveTableLog(tableName, templateTable, Status.Inst.SUCCESS.name());
                }
                return new GeneralResult<>(true, "创建成功");
            } catch (Exception e) {
                this.saveTableLog(tableName, templateTable, Status.Inst.FAILED.name());
                log.error("create task_instance_bak table error:", e);
                return new GeneralResult<>(false, "创建失败");
            }
        }
    }

    public GeneralResult<Void> createTaskNodeLogBakTable(String tableName,String databaseDriver) {
        synchronized (TaskNodeLog.class) {
            String templateTable = TaskDataBakScheduler.TASK_NODE_LOG_BAK;
            try {
                if (taskNodeLogBakRepository.countTable(tableName,databaseDriver) <= 0) {
                    taskNodeLogBakRepository.createTable(tableName, databaseDriver, new TaskNodeLogBak());
                    this.saveTableLog(tableName, templateTable, Status.Inst.SUCCESS.name());
                }
                return new GeneralResult<>(true, "创建成功");
            } catch (Exception e) {
                this.saveTableLog(tableName, templateTable, Status.Inst.FAILED.name());
                log.error("create task_node_log_bak table error:", e);
                return new GeneralResult<>(false, "创建失败");
            }
        }
    }

    private void saveTableLog(String tableName, String templateTableName, String status) {
        TableLog tableLog = new TableLog();
        tableLog.setName(tableName);
        tableLog.setCode("BACKUP");
        tableLog.setStatus(status);
        tableLog.setTemplateTable(templateTableName);
        tableLog.setGmtCreate(new Date());
        tableLog.setDeleted(false);
        tableLogRepository.save(tableLog);
    }

    /**
     * 获取动态表的后缀
     *
     * @param localDate
     * @return
     */
    private String getTableSuffix(LocalDateTime localDate) {
        //2023
        int year = localDate.getYear();
        //4
        int month = localDate.getMonth().getValue();
        String months = String.valueOf(month).length() > 1 ? String.valueOf(month) : "0" + month;
        //202304
        String suffix = year + months;
        return suffix;
    }

}
