package com.bocloud.cop.entity;

import java.util.Date;
import java.util.List;
import java.util.Map;

import lombok.AllArgsConstructor;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.utils.DateDeserializer;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.IgnoreUpdate;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;

import lombok.Data;

@EqualsAndHashCode(callSuper = true)
@AllArgsConstructor
@NoArgsConstructor
@Data
@Table("task_host")
public class TaskHost extends Generic {

    @PK(value = PKStrategy.AUTO)
    private Long id; // ID
    @Column("ip")
    private String ip;
    @Column("ips")
    private Object ips;
    @Column("catalog")
    private String catalog; // 类型（大类）
    @Column("category")
    private String category; // 类型（小类）
    @Column("os_category")
    private String osCategory; // 操作系统类型
    @Column("known")
    private Boolean known;
    @Column("segment_id")
    private Long segmentId;
    @IgnoreAll
    private String segmentName;
    @IgnoreAll
    private Boolean isTemplate;
    /**
     * 实例id：数据来源为:IMS时有值 pm:uuid,vm:instance_id
     */
    @Column("instance_id")
    private String instanceId;
    /**
     * 数据来源：IMS(IMS同步),IMPORT(批量导入),CREATE(手动创建),SALT(salt同步)
     */
    @Column("source_type")
    private String sourceType;
    /**
     * 平台id，数据来源为：IMS时有值
     */
    @Column("vendor_id")
    private Long vendorId;
    @Column("vendor_type")
    private String vendorType;

    @Column("vendor_name")
    private String vendorName;
    /**
     * AUTO 自动同步，MANUAL 手动配置
     */
    @Column("update_mode")
    private String updateMode;

    /**
     * 租户id
     */
    @Column("tenant_id")
    private Long tenantId;

    /**
     * 操作系统
     */
    @Column("os_name")
    private String osName;

    /**
     * 作系统版本号
     */
    @Column("os_version")
    private String osVersion;

    /**
     * 连接协议
     */
    @Column("conn_protocol")
    private String connProtocol;

    /**
     * saltId
     */
    @Column("salt_id")
    private Long saltId;

    /**
     * 关联认证账号id
     */
    @IgnoreAll
    private List<Long> identificationIds;

    /**
     * 关联用户id
     */
    @IgnoreAll
    private List<Long> userGroupIds;

    @IgnoreAll
    private List<String> userGroupNames;

    @IgnoreAll
    private List<String> connProtocols;

    /**
     * 认证账号
     */
    @IgnoreAll
    private List<Map<String, Object>> identifications;

    @IgnoreAll
    private String tenantName;

    public void setIps(Object ips) {
        if (null == ips || ips instanceof String) {
            this.ips = ips;
        } else {
            this.ips = JSONArray.toJSONString(ips);
        }
    }

    public String getIps() {
        if (null == ips) {
            return null;
        }
        return ips.toString();
    }

    @IgnoreUpdate
    @Column(value = "gmt_create", nullable = false)
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;
    @Column("gmt_modify")
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtModify;
    @Column("creator_id")
    private Long creatorId;
    @Column("owner_id")
    private Long ownerId;
    @Column("mender_id")
    private Long menderId;
    @Column("name")
    private String name;
    @Column("status")
    private String status;
    @Column("is_deleted")
    private Boolean deleted;
    @Column("props")
    private String props;
    @Column("remark")
    private String remark;
    /**
     * 类型（小类）
     */
    @Column("user_catalog")
    private String userCatalog;
    /**
     * 数据中心
     */
    @Column
    private String dataCenter;
    /**
     * 数据中心名称
     */
    @IgnoreAll
    private String dataCenterName;
    /**
     * 机房
     */
    @Column
    private String room;
    /**
     * 机房名称
     */
    @IgnoreAll
    private String roomName;
    /**
     * 机柜
     */
    @Column
    private String rack;
    /**
     * 机柜名称
     */
    @IgnoreAll
    private String rackName;
    /**
     * 槽位
     */
    @Column
    private String slot;
    /**
     * 槽位名称
     */
    @IgnoreAll
    private String slotName;

    /**
     * 用户名
     */
    @IgnoreAll
    private String username;

    /**
     * 密码
     */
    @Column("password")
    private String password;

    @Column("salt_minion_id")
    private String saltMinionId;

    @Override
    public String toString() {
        return JSONObject.toJSONString(this);
    }

    public Date getGmtCreate() {
        return this.gmtCreate;
    }

    public Date getGmtModify() {
        return this.gmtModify;
    }

    public Long getCreatorId() {
        return this.creatorId;
    }

    public Long getOwnerId() {
        return this.ownerId;
    }

    public Long getMenderId() {
        return this.menderId;
    }

    public String getName() {
        return this.name;
    }

    public String getStatus() {
        return this.status;
    }

    public Boolean getDeleted() {
        return this.deleted;
    }

    public String getProps() {
        return this.props;
    }

    public String getRemark() {
        return this.remark;
    }

    @JsonDeserialize(using = DateDeserializer.class)
    public void setGmtCreate(Date gmtCreate) {
        this.gmtCreate = gmtCreate;
    }

    @JsonDeserialize(using = DateDeserializer.class)
    public void setGmtModify(Date gmtModify) {
        this.gmtModify = gmtModify;
    }

    public void setCreatorId(Long creatorId) {
        this.creatorId = creatorId;
    }

    public void setOwnerId(Long ownerId) {
        this.ownerId = ownerId;
    }

    public void setMenderId(Long menderId) {
        this.menderId = menderId;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public void setDeleted(Boolean deleted) {
        this.deleted = deleted;
    }

    public void setProps(String props) {
        this.props = props;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }

    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }
}
