package com.bocloud.cop.entity;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.annotation.ColumnType;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.DateDeserializer;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.*;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;
import java.util.Date;
import java.util.List;

/**
 * @author huliwei
 * @date 2023/4/21
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@Table("task_instance_bak")
public class TaskInstanceBak extends Generic{

    @PK(value = PKStrategy.SELF)
    private Long id; // ID
    @Column(value = "uuid",length = 64)
    private String uuid; // uuid
    @Column(value = "type",length = 32)
    private String type; // 任务类型：temporary-临时，standard-标准，inspection-巡检，compliance-合规，patch-补丁，compare-配置对比，backup-备份
    @Column(value = "code",length = 64)
    private String code; // 任务种类：standard-标准，scheduled-定时
    @ColumnType(value = "longtext")
    @Column(value = "graph",encryptable = true)
    private String graph;
    @Column(value = "cron",length = 64)
    private String cron; // cron表达式
    @Column("gmt_start")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtStart; // 开始时间
    @Column("gmt_finish")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtFinish; // 结束时间
    @Column("task_id")
    private Long taskId; // 任务ID
    @Column(value = "execute_mode",length = 24)
    private String executeMode; // 启动方式
    @Column(value = "execute_host",length = 32)
    private String executeHost;// 执行主机
    @Column("tenant_id")
    private Long tenantId; // 租户ID
    @Column(value = "catalog",length = 64)
    private RequestContext.Catalog catalog;// 账号类别
    @Column(value = "rollbacks",length = 1)
    private Boolean rollbacks;// 超时回滚
    @Column("timeout")
    private Long timeout;// 超时时间
    @Column(value = "is_backup",length = 1)
    private Boolean isBackup; // 是否纯备份任务
    @Column(value = "email_enabled",length = 1)
    private Boolean emailEnabled;
    @Column("priority")
    private Integer priority;
    /**
     * 所属平台
     */
    @Column(value = "platform",length = 100)
    private String platform;
    /**
     * 是否限制执行
     */
    @Column(value = "is_limited",length = 1)
    private Boolean isLimited;
    /**
     * 限制释放时间
     */
    @Column("limited_release_time")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date limitedReleaseTime;
    @Column("platform_alert_id")
    private Long platformAlertId;

    @IgnoreUpdate
    @Column(value = "gmt_create", nullable = false
    )
    @JsonSerialize(
            using = DateSerializer.class
    )
    @JsonDeserialize(
            using = DateDeserializer.class
    )
    @DateTimeFormat(
            pattern = "yyyy-MM-dd HH:mm:ss"
    )
    private Date gmtCreate;
    @Column("gmt_modify")
    @JsonSerialize(
            using = DateSerializer.class
    )
    @JsonDeserialize(
            using = DateDeserializer.class
    )
    @DateTimeFormat(
            pattern = "yyyy-MM-dd HH:mm:ss"
    )
    private Date gmtModify;
    @IgnoreUpdate
    @Column(value = "creator_id",nullable = false)
    private Long creatorId;
    @Column("owner_id")
    private Long ownerId;
    @Column("mender_id")
    private Long menderId;
    @Column(value = "name",length = 256)
    private String name;
    @Column(value = "status",length = 16)
    private String status;
    @Column(value = "is_deleted",length = 1,nullable = false)
    private Boolean deleted;
    @Column(value = "props",length = 1024)
    private String props;
    @Column(value = "remark",length = 512)
    private String remark;
    @IgnoreAll
    private String tableName;
    @IgnoreAll
    private Double cost; // 总耗时
    @IgnoreAll
    private String operator; // 启动人
    @IgnoreAll
    private List<JSONObject> results;

}
