package com.bocloud.cop.enums;

import org.apache.commons.lang3.StringUtils;

import com.megatron.common.model.GeneralResult;

/**
 * 类型枚举类，用来提供各种资源的枚举类型<br/>
 * 目前支持 编排任务类型，任务节点类型和脚本类型
 * 
 * @author dmw
 *
 */
public class Category {

    public static enum Task {
        /**
         * 运维类型
         */
        MAINTAIN("maintain"),
        /**
         * 资源类型
         */
        RESOURCE("resource"),
        /**
         * 软件类型
         */
        SOFTWARE("software"),
        /**
         * 混合类型
         */
        MULTIFUL("multiful");

        private String value;

        Task(String value) {
            this.value = value;
        }

        public String getValue() {
            return this.value;
        }
    }

    /**
     * 节点类型
     * 
     * @author dmw
     *
     */
    public static enum Node {
        /**
         * 开始节点
         */
        START("start"),
        /**
         * 开始节点
         */
        STOP("stop"),
        /**
         * 开始节点
         */
        JUDGE("judge"),
        /**
         * 运维节点
         */
        MAINTAIN("maintain"),
        /**
         * 资源节点
         */
        RESOURCE("resource"),
        /**
         * 软件节点
         */
        SOFTWARE("software");

        public static GeneralResult<Node> hasValue(String value) {
            if (StringUtils.isEmpty(value)) {
                return new GeneralResult<>(false, "参数为空！");
            }
            Node[] nodes = Node.values();
            for (Node node : nodes) {
                if (node.name().equalsIgnoreCase(value)) {
                    return new GeneralResult<Node>(true, node, "success");
                }
            }
            return new GeneralResult<>(false, "数值不存在！");
        }

        private String value;

        Node(String value) {
            this.value = value;
        }

        public String getValue() {
            return this.value;
        }
    }

    /**
     * 脚本类型
     * 
     * @author dmw
     *
     */
    public static enum Script {
        /**
         * 
         */
        SHELL,
        /**
        * 
        */
        POWERSHELL,
        /**
        * 
        */
        PYTHON,
        /**
        * 
        */
        BAT,
        /**
        * 
        */
        PERL,
        /**
        * 
        */
        PLAYBOOK,
        /**
        * 
        */
        DATABASE;

        public static GeneralResult<Script> hasValue(String value) {
            if (StringUtils.isEmpty(value)) {
                return new GeneralResult<>(false, "参数为空！");
            }
            Script[] scripts = Script.values();
            for (Script script : scripts) {
                if (script.name().equalsIgnoreCase(value)) {
                    return new GeneralResult<Script>(true, script, "success");
                }
            }
            return new GeneralResult<>(false, "数值不存在！");
        }
    }
}
