package com.bocloud.cop.model;

import com.alibaba.fastjson.annotation.JSONField;
import com.bocloud.cop.constant.CommonConstant;

/**
 * 与Worker服务交互的主机用户对象
 * 
 * @author sxp
 * @since Dec 27, 2017
 */
public class UserArg {
	/**
	 * 用户名
	 */
	private String name;

	private String comment;

	private String password;

	private String group;

	/**
	 * 过期时间，日期转化成的相对于1970年1月1日的秒
	 */
	private Long expires;

	/**
	 * windows用户密码是否过期
	 */
	@JSONField(name = CommonConstant.PSWD_EXPIRED)
	private Boolean passwordExpired;

	private String shell;

	/**
	 * 密码的加密类型
	 */
	@JSONField(name = CommonConstant.CRYPTED_TYPE)
	private String cryptedType;

	/**
	 * 在删除用户时，表示是否强制删除；其它情况不需要该参数
	 */
	private boolean force = false;

	public UserArg() {
		super();
	}

	public UserArg(String name, String comment, String password, String group, Long expires, String shell,
			String cryptedType, boolean force) {
		super();
		this.name = name;
		this.comment = comment;
		this.password = password;
		this.group = group;
		this.expires = expires;
		this.shell = shell;
		this.cryptedType = cryptedType;
		this.force = force;
	}

	public UserArg(String name, String comment, String password, String group, Long expires, String shell,
			boolean force) {
		this(name, comment, password, group, expires, shell, null, force);
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getComment() {
		return comment;
	}

	public void setComment(String comment) {
		this.comment = comment;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getGroup() {
		return group;
	}

	public void setGroup(String group) {
		this.group = group;
	}

	public Long getExpires() {
		return expires;
	}

	public void setExpires(Long expires) {
		this.expires = expires;
	}

	public String getShell() {
		return shell;
	}

	public void setShell(String shell) {
		this.shell = shell;
	}

	public String getCryptedType() {
		return cryptedType;
	}

	public void setCryptedType(String cryptedType) {
		this.cryptedType = cryptedType;
	}

	public boolean isForce() {
		return force;
	}

	public void setForce(boolean force) {
		this.force = force;
	}

	public Boolean getPasswordExpired() {
		return passwordExpired;
	}

	public void setPasswordExpired(Boolean passwordExpired) {
		this.passwordExpired = passwordExpired;
	}

	@Override
	public String toString() {
		return "UserArg [name=" + name + ", comment=" + comment + ", password=" + password + ", group=" + group
				+ ", expires=" + expires + ", shell=" + shell + ", cryptedType=" + cryptedType + ", force=" + force
				+ "]";
	}

}
