package com.bocloud.cop.model.inspection;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeSet;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;

import lombok.Data;

@Data
public class ScriptLog {

    /**
     * script_run_status: 输出脚本执行状态0 success(查询巡检结果成功，脚本正常执行) 1 error(查询巡检结果失败) 2 warning(脚本执行异常，但是查询到了巡检结果)
     * script_errror_info: 如果script_run_status为1或2 输出脚本执行的错误信息 inspect_result: 输出巡检的结果，数值类型或字符串类型 ，数值类型支持
     * 整型，浮点型，百分比，如果script_run_status为0或2是，则该字段必填 ----
     * PERCENT====/root++++60%@@@@mnt++++61%||||IO_PERCENT====/root++++62%@@@@mnt++++63% 解析---- PERCENT //多值匹配的key /root
     * 60% //value列表，[/root]为某个value指定的对象 mnt 61% IO_PERCENT /root 62% mnt 63% inspect_info: 巡检结果的附加信息 可以为空,竖线|分割
     */

    private Integer scriptStatus;
    private String scriptErrorInfo;
    private String inspectResult;
    private List<String> inspectMessage; // 存储scriptMessage解析出来的数据
    private Map<String, ScriptLogResult> inspectResultMap; // 存储将inspectResult解析出来的数据

    /**
     * 将输出的日志解析并返回脚本日志的格式化数据
     *
     * @param logString
     * @param warningRuleEntity
     * @return
     * @throws RuntimeException
     */
    public static ScriptLog parasLog(String logString, WarningRuleEntity warningRuleEntity) throws RuntimeException {
        ScriptLog scriptLog = new ScriptLog();
        // 解析 stdout：
        JSONObject obj = JSONObject.parseObject(logString);
        logString = obj.getString("stdout");

        String[] logs = logString.split("[\n]|[\r]");
        for (String log : logs) {
            if (log.trim().length() == 0) {
                continue;
            }
            int colonIndex = log.indexOf(":");
            if (colonIndex < 0) {
                continue;
            }
            String key = log.substring(0, colonIndex);
            switch (key) {
                case "script_run_status":
                    String script_run_status = log.substring(colonIndex + 1).trim();
                    try {
                        Integer script_run_status_int = Integer.parseInt(script_run_status);
                        scriptLog.setScriptStatus(script_run_status_int);
                    } catch (NumberFormatException e) {
                        throw new RuntimeException("脚本执行结果解析失败：script_run_status不能为\"" + script_run_status + "\"");
                    }
                    break;
                case "script_status": // 合规脚本执行结果
                    script_run_status = log.substring(colonIndex + 1).trim();
                    try {
                        Integer script_run_status_int = Integer.parseInt(script_run_status);
                        scriptLog.setScriptStatus(script_run_status_int);
                    } catch (NumberFormatException e) {
                        throw new RuntimeException("脚本执行结果解析失败：script_status不能为\"" + script_run_status + "\"");
                    }
                    break;
                case "script_error_info":
                    String script_error_info = log.substring(colonIndex + 1).trim();
                    scriptLog.setScriptErrorInfo(script_error_info);
                    break;
                case "inspect_result":
                    String inspect_result = log.substring(colonIndex + 1).trim();
                    scriptLog.setInspectResult(inspect_result);
                    break;
                case "inspect_info":
                    String inspect_info = log.substring(colonIndex + 1).trim();
                    String[] inspectInfoArray = inspect_info.split("\\|\\|\\|\\|");
                    List<String> inspectInfoList = Arrays.asList(inspectInfoArray);
                    scriptLog.setInspectMessage(inspectInfoList);
                    break;
                default:
                    break;
            }
        }

        if (scriptLog.getScriptStatus() == null) {
            throw new RuntimeException("脚本执行结果解析失败：script_run_status不能为空");
        }
        // 将输出的日志 同告警比对并将数据加载进scriptLog
        if (null != warningRuleEntity
            && (scriptLog.getScriptStatus().equals(0) || scriptLog.getScriptStatus().equals(2))) {
            if (scriptLog.getInspectResult() == null) {
                throw new RuntimeException("脚本执行结果解析失败：script_run_status为0或2时，inspect_result不能为空");
            }
            try {
                scriptLog.setInspectResultMap(handleInspectResult2(scriptLog, warningRuleEntity));
            } catch (RuntimeException e) {
                scriptLog = errorLog(scriptLog, e.getMessage());
            }
        } else if (scriptLog.getScriptStatus().equals(1)) {

        } else if (scriptLog.getScriptStatus().equals(0)) {

        } else {
            throw new RuntimeException("脚本执行结果解析失败：script_run_status不能为" + scriptLog.getScriptStatus());
        }

        return scriptLog;
    }

    /**
     * 解析inspect_result 字段并返回结果
     *
     * @param scriptLog
     *            PERCENT====/root++++70%@@@@mnt++++60%||||IO_PERCENT====/root++++70%@@@@mnt++++60%
     * @param entity
     * @throws RuntimeException
     */
    private static Map<String, ScriptLogResult> handleInspectResult2(ScriptLog scriptLog, WarningRuleEntity entity)
        throws RuntimeException {
        String inspectResult = scriptLog.getInspectResult();
        if (inspectResult == null) {
            throw new RuntimeException("脚本执行结果解析失败：script_run_status不能为空");
        }
        Map<String, ScriptLogResult> warnings = new HashMap<>();
        // 单值匹配
        if (!entity.isMulitCompare()) {
            /// root++++70%@@@@mnt++++60%
            String key = null;
            if (inspectResult.contains("====")) {
                int index = inspectResult.indexOf("====");
                key = inspectResult.substring(0, index);
                inspectResult = inspectResult.substring(index + 4);
            }
            Map<String, String> valueMap = paramsValueMap(inspectResult);

            ScriptLogResult result =
                new ScriptLogResult(key, entity.getWarningRules(), handleValues(valueMap, entity.getWarningRules()));
            warnings.put(null, result);
        } else {// 多值匹配
            String[] keyValuesArray = inspectResult.split("\\|\\|\\|\\|");
            // 存储如下结构
            // PERCENT====/root++++70%@@@@mnt++++60%||||IO_PERCENT====/root++++70%@@@@mnt++++60%
            //
            // PERCENT /root 70%
            // mnt 60%
            // IO_PERCENT /root 70%
            // mnt 60%
            Map<String, Map<String, String>> keyValueMap = new HashMap<>();
            for (String keyValues : keyValuesArray) {
                // PERCENT====/root++++70%@@@@mnt++++60%
                if (StringUtils.isBlank(keyValues)) {
                    continue;
                }
                keyValues = keyValues.trim();
                if (!keyValues.contains("====")) {
                    throw new RuntimeException("脚本执行结果解析失败：请检查inspect_result格式[多值匹配解析key失败]");
                }
                String[] keyAndValues = keyValues.split("====");
                // PERCENT
                String key = keyAndValues[0];

                if (keyAndValues.length == 1) {
                    keyValueMap.put(key, null);
                } else {
                    String values = keyAndValues[1];
                    keyValueMap.put(key, paramsValueMap(values));
                }
                /// root++++70%@@@@mnt++++60%
            }
            Map<String, TreeSet<WarningRule>> warningRulesMap = entity.getWarningRulesMap();
            for (Map.Entry<String, TreeSet<WarningRule>> warningRulesEntry : warningRulesMap.entrySet()) {
                String warningKey = warningRulesEntry.getKey();
                TreeSet<WarningRule> warningRules = warningRulesEntry.getValue();

                Map<String, String> valueMap = keyValueMap.get(warningKey);
                if (valueMap == null || valueMap.size() == 0) {
                    List<ScriptLogResultEntity> entities = new ArrayList<>();
                    entities.add(new ScriptLogResultEntity(null, "[未解析出结果]",
                        WarningRuleResult.error("未解析出【" + warningKey + "】对应的值", getMaxLevel(warningRules))));
                    ScriptLogResult result = new ScriptLogResult(warningKey, warningRules, entities);
                    warnings.put(warningKey, result);
                    continue;
                }
                // 处理多个同key值
                // root 70%
                // mnt 60%
                ScriptLogResult result =
                    new ScriptLogResult(warningKey, warningRules, handleValues(valueMap, warningRules));
                warnings.put(warningKey, result);
            }
        }
        return warnings;
    }

    /**
     * 将inspect_result中的 ==== 之后的解析为Map
     *
     * @param values
     *            root++++70%@@@@mnt++++60%
     * @return root 70% mnt 60%
     * @throws RuntimeException
     */
    private static Map<String, String> paramsValueMap(String values) throws RuntimeException {
        Map<String, String> valueMap = new HashMap<>();
        if (!values.contains("@@@@")) {
            valueMap.put(null, values);
            return valueMap;
        }
        String[] valueArray = values.trim().split("@@@@");
        for (String value : valueArray) {
            String valueName;
            String valueValue;
            if (value.contains("++++")) {
                String[] valueNameAndValue = value.split("\\+\\+\\+\\+");
                valueName = valueNameAndValue[0];
                valueValue = valueNameAndValue[1];
                if (StringUtils.isAnyBlank(valueName, valueValue)) {
                    throw new RuntimeException("脚本执行结果解析失败：请检查inspect_result格式[解析某个值的多个结果失败]");
                }
            } else {
                throw new RuntimeException("脚本执行结果解析失败：请检查inspect_result格式[解析某个值的多个结果失败]");
            }
            valueMap.put(valueName.trim(), valueValue.trim());
        }
        return valueMap;
    }

    /**
     * 处理 paramsValueMap方法生产的map
     *
     * @param valueMap
     * @param warningRules
     * @return
     * @throws RuntimeException
     */
    private static List<ScriptLogResultEntity> handleValues(Map<String, String> valueMap,
        TreeSet<WarningRule> warningRules) {
        List<ScriptLogResultEntity> entities = new ArrayList<>();
        for (Map.Entry<String, String> valueEntity : valueMap.entrySet()) {
            String valueName = valueEntity.getKey();
            String value = valueEntity.getValue();
            entities.add(new ScriptLogResultEntity(valueName, value, handleOneResult(value, warningRules)));
        }
        return entities;
    }

    /**
     * 从WarningRule列表中选出level最高的一个
     *
     * @param set
     *            倒序的WarningRule 列表
     * @return level最高
     * @throws RuntimeException
     */
    private static WarningRule getMaxLevel(TreeSet<WarningRule> set) throws RuntimeException {
        if (set == null || set.size() == 0) {
            throw new RuntimeException("告警数据异常，请联系管理员");
        }
        WarningRule maxLevel = set.first();
        if (maxLevel == null) {
            throw new RuntimeException("告警数据异常，请联系管理员");
        }
        return maxLevel;
    }

    /**
     * 处理分析某个值是否触发告警并返回结果
     *
     * @param value
     *            value 70%
     * @param warningRules
     *            level自高到低排序的规则列表
     * @return 如果触发了告警 则返回
     */
    private static WarningRuleResult handleOneResult(String value, TreeSet<WarningRule> warningRules) {
        if (warningRules != null) {
            for (WarningRule rule : warningRules) {
                WarningRule.RULE enumRule = rule.getRule();
                WarningRuleResult result = enumRule.validate(value, rule);
                if (result.hasErrorOrWarning()) {
                    // 有一个错误就跳出
                    return result;
                }
            }
        }
        return null;
    }

    /**
     * 生成错误日志
     *
     * @param error
     * @return
     */
    public static ScriptLog errorLog(String error) {
        ScriptLog log = new ScriptLog();
        log.setScriptStatus(1);
        log.setScriptErrorInfo(error);
        return log;
    }

    /**
     * 将scriptLog 转为错误日志
     *
     * @param scriptLog
     * @param error
     * @return
     */
    public static ScriptLog errorLog(ScriptLog scriptLog, String error) {
        scriptLog.setScriptStatus(1);
        scriptLog.setScriptErrorInfo(error);
        return scriptLog;
    }

    public enum RunStatus {
        SUCCESS, ERROR
    }

}
