package com.bocloud.cop.model.inspection;

import com.alibaba.fastjson.JSONObject;

/**
 * 告警结果实体 用于封装单个告警结果
 */
public class WarningRuleResult {
    /**
     * 是否成功执行，表示没有出现数据异常
     */
    private boolean success = true;
    /**
     * true 匹配到了告警，该结果表示 产生了告警
     */
    private boolean match = false;
    /**
     * 输出数据异常
     */
    private String errorInfo = "";
    /**
     * 输出告警
     */
    private String warningInfo = "";

    private Integer warningLevel;

    private String targetRule;

    public static WarningRuleResult error(String error, WarningRule warningRule) {
        WarningRuleResult result = new WarningRuleResult();
        result.success = false;
        result.errorInfo = error;
        result.warningLevel = warningRule.getWarnLevel();
        result.targetRule = JSONObject.toJSONString(warningRule);;
        return result;
    }

    public static WarningRuleResult pass() {
        return new WarningRuleResult();
    }

    public static WarningRuleResult warning(String warningInfo, WarningRule warningRule) {
        WarningRuleResult result = new WarningRuleResult();
        result.match = true;
        result.warningInfo = warningInfo;
        result.warningLevel = warningRule.getWarnLevel();
        result.targetRule = JSONObject.toJSONString(warningRule);;
        return result;
    }

    public boolean hasErrorOrWarning() {
        return (!success) || (success && match);
    }

    public String getErrorOrWarning() {
        return warningInfo + errorInfo;
    }

    public boolean isSuccess() {
        return success;
    }

    public String getErrorInfo() {
        return errorInfo;
    }

    public boolean isMatch() {
        return match;
    }

    public String getWarningInfo() {
        return warningInfo;
    }

    public String getTargetRule() {
        return targetRule;
    }

    public Integer getWarningLevel() {
        return warningLevel;
    }

    public String toString() {
        JSONObject json = new JSONObject();
        json.put("level", warningLevel);
        json.put("message", errorInfo + warningInfo);
        return json.toJSONString();
    }
}