package com.bocloud.cop.repository;

import com.bocloud.cop.entity.Command;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 命令管理DAO接口实现
 * 
 * @author weiwei
 *
 */
@Repository("commandRepository")
public class CommandRepository extends BasicGenericDao<Command, Long> {

    public CommandRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<Command> list(Pager pager) {
        String sql = "select * from command where is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "");
        return this.list(Command.class, sql, this.getQueryBuilder().getParam(pager.getParams()));
    }

    public List<Command> list() {
        String sql = "select * from command where is_deleted = 0";
        return this.list(Command.class, sql);
    }

    public List<Command> listByRid(Long roleId) {
        String sql =
            "select c.* from command c,role_command r where c.is_deleted = 0 and r.role_id = :roleId and c.id = r.command_id";
        return this.list(Command.class, sql, MapTools.simpleMap("roleId", roleId));
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from command where is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public boolean delete(Long id, Long userId) {
        String sql = "update command set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public Command query(Long id) {
        String sql = "select * from command where is_deleted = 0 and id = :id";
        List<Command> list = this.list(Command.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public Command query(Param param) {
        String sql = "select * from command where is_deleted = 0 ";
        List<Param> params = new ArrayList<>();
        params.add(param);
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "");
        List<Command> list = this.list(Command.class, sql, this.getQueryBuilder().getParam(params));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean exists(String name) {
        String sql = "select id from command where name = :name and is_deleted = 0";
        List<Command> list = this.list(Command.class, sql, MapTools.simpleMap("name", name));
        return !list.isEmpty();
    }

}
