package com.bocloud.cop.repository;

import com.bocloud.cop.entity.NetworkGroup;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Repository("networkGroupRepository")
public class NetworkGroupRepository extends JdbcGenericDao<NetworkGroup, Long> {

    public NetworkGroupRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<NetworkGroup> list(Pager pager) throws Exception {
        String sql = "select * from network_group where is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "");
        return this.list(NetworkGroup.class, sql, this.getQueryBuilder().getParam(pager.getParams()));
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from network_group where is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public List<NetworkGroup> list() throws Exception {
        String sql = "select * from network_group where is_deleted = 0";
        return this.list(NetworkGroup.class, sql);
    }

    public NetworkGroup query(Long id) {
        String sql = "select a.* from network_group a where a.is_deleted = 0 and a.id = :id";
        List<NetworkGroup> list = this.list(NetworkGroup.class, sql, MapTools.simpleMap("id", id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql =
            "update network_group set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean exists(String name) {
        String sql = "select id from network_group a where a.is_deleted = 0 and a.name = :name";
        List<NetworkGroup> list = this.list(NetworkGroup.class, sql, MapTools.simpleMap("name", name));
        return !list.isEmpty();
    }

}
