package com.bocloud.cop.repository;

import java.util.List;
import java.util.Map;

import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import com.bocloud.cop.entity.NodeRelation;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;

/**
 * 任务主机DAO接口实现
 * 
 * @author weiwei
 * @version 1.0
 * @since 2017.07.07
 *
 */
@Repository("nodeRelationRepository")
public class NodeRelationRepository extends BasicGenericDao<NodeRelation, Long> {

    public NodeRelationRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public boolean batchSave(List<NodeRelation> relations) {
        return this.batchSave(relations, relations.size());
    }

    public boolean delete(Long id) {
        String sql = "delete from task_node_relation where id = :id";
        return this.execute(sql, MapTools.simpleMap(Common.ID, id)) > 0;
    }

    public NodeRelation query(Long id) {
        String sql = "select * from task_node_relation where id = :id";
        List<NodeRelation> list = this.list(NodeRelation.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean updateStatus(Long instanceId, Long source, String status) {
        String sql =
            "update task_node_relation set status = :status where source = :source and instance_id = :instanceId";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        params.put("source", source);
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

    public boolean deleteByInstanceId(Long instanceId) {
        String sql = "delete from task_node_relation where instance_id = :instanceId";
        return this.execute(sql, MapTools.simpleMap("instanceId", instanceId)) > 0;
    }

    public List<NodeRelation> listByInstances(List<Long> instanceIds) {
        String sql = "select * from task_node_relation a where a.instance_id in (:instanceIds)";
        Map<String, Object> params = MapTools.simpleMap("instanceIds", instanceIds);
        return this.list(NodeRelation.class, sql, params);
    }

    public boolean deleteByInstanceIds(List<Long> instanceIds) {
        String sql = "delete from task_node_relation where instance_id in (:instanceIds)";
        Map<String, Object> params = MapTools.simpleMap("instanceIds", instanceIds);
        return this.execute(sql, params) > 0;
    }

    public boolean updateFailedStatus(Long instanceId, Long source, String status) throws Exception {
        String sql =
            "update task_node_relation set status = :status where instance_id = :instanceId and source = :source";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        params.put("source", source);
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

    public List<NodeRelation> list(Long instanceId, String status) {
        String sql = "select * from task_node_relation a where a.instance_id = :instanceId and a.status = :status";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        params.put("status", status);
        return this.list(NodeRelation.class, sql, params);
    }

}
