package com.bocloud.cop.repository;

import com.bocloud.cop.entity.Script;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 脚本DAO接口实现类
 */
@Repository("scriptRepository")
public class ScriptRepository extends JdbcGenericDao<Script, Long> {

    public ScriptRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<Script> list(Pager pager, Long userId, Long groupId) {
        String sql =
            "select a.*,b.name group_name from script a left join script_group b on a.group_id = b.id where a.is_deleted = 0";
        if (null != userId) {
            sql += "  and (a.is_open = 1 or ( a.is_open = 0 and a.creator_id = :userId))";
        }
        if (null != groupId && 0L != groupId) {
            sql += " and (a.group_id = " + groupId + " or a.group_ids like '%[" + groupId
                + ",%' or a.group_ids like '%, " + groupId + ",%')";
        }
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(pager.getParams());
        param.put("userId", userId);
        return this.list(Script.class, sql, param);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter, Long userId, Long groupId) {
        String sql = "select a.id,a.name from script a where a.is_deleted = 0";
        if (null != userId) {
            sql += "  and (a.is_open = 1 or ( a.is_open = 0 and a.creator_id = :userId))";
        }
        if (null != groupId && 0L != groupId) {
            sql += " and (a.group_id = " + groupId + " or a.group_ids like '%[" + groupId
                + ",%' or a.group_ids like '%, " + groupId + ",%')";
        }
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        param.put("userId", userId);
        List<Script> scripts = this.list(Script.class, sql, param);
        List<SimpleBean> beans = new ArrayList<>();
        for (Script script : scripts) {
            beans.add(new SimpleBean(script.getId(), script.getName()));
        }
        return beans;
    }

    public int count(List<Param> params, Long userId, Long groupId) {
        String sql = "select count(1) from script a where a.is_deleted = 0";
        if (null != userId) {
            sql += "  and (a.is_open = 1 or ( a.is_open = 0 and a.creator_id = :userId))";
        }
        if (null != groupId && 0L != groupId) {
            sql += " and (a.group_id = " + groupId + " or a.group_ids like '%[" + groupId
                + ",%' or a.group_ids like '%, " + groupId + ",%')";
        }
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        param.put("userId", userId);
        return this.countQuery(sql, param).intValue();
    }

    public boolean delete(Long id, Long userId) {
        String sql = "update script set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public Script query(Long id) {
        String sql =
            "select a.*,b.name group_name from script a left join script_group b on a.group_id = b.id where a.id = :id and a.is_deleted = 0";
        List<Script> list = this.list(Script.class, sql, MapTools.simpleMap("id", id));
        return list.isEmpty() ? null : list.get(0);
    }

    public Script get(Long id) {
        String sql = "select * from script a where a.id = :id";
        List<Script> list = this.list(Script.class, sql, MapTools.simpleMap("id", id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean exists(String name, Long groupId) {
        String sql = "select id from script a where a.name = :name and a.group_id = :groupId and a.is_deleted = 0";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        paramMap.put("groupId", groupId);
        List<Script> list = this.list(Script.class, sql, paramMap);
        return !list.isEmpty();
    }

    public boolean updateStatus(Long id, String status, Long userId) {
        String sql =
            "update script set status = :status, gmt_modify = :gmtModify, mender_id = :menderId where is_deleted = 0 and id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("status", status);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from script a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) {
        String sql = "select a.id,a.name,a.status from script a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<Script> scripts = this.list(Script.class, sql, this.getQueryBuilder().getParam(params));
        List<SimpleBean> beans = new ArrayList<>();
        for (Script script : scripts) {
            beans.add(new SimpleBean(script.getId(), script.getName(), script.getStatus()));
        }
        return beans;
    }

}
