package com.bocloud.cop.repository;

import com.bocloud.cop.entity.Software;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 软件文件DAO接口实现类
 */
@Repository("softwareRepository")
public class SoftwareRepository extends JdbcGenericDao<Software, Long> {

    public SoftwareRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<Software> list(Pager pager, Long groupId) {
        String sql =
            "select a.*,b.name group_name from software a left join software_group b on a.group_id = b.id where a.is_deleted = 0";
        if (null != groupId && 0L != groupId) {
            sql += " and (a.group_id = " + groupId + " or a.group_ids like '%" + groupId + ",%')";
        }
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(Software.class, sql, this.getQueryBuilder().getParam(pager.getParams()));
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter, Long groupId) {
        String sql = "select a.id,a.name,a.path from software a where a.is_deleted = 0";
        if (null != groupId && 0L != groupId) {
            sql += " and (a.group_id = " + groupId + " or a.group_ids like '%" + groupId + ",%')";
        }
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<Software> softwares = this.list(Software.class, sql, this.getQueryBuilder().getParam(params));
        List<SimpleBean> beans = new ArrayList<>();
        for (Software software : softwares) {
            beans.add(new SimpleBean(software.getId(), software.getName()));
        }
        return beans;
    }

    public int count(List<Param> params, Long groupId) {
        String sql = "select count(1) from software a where a.is_deleted = 0";
        if (null != groupId && 0L != groupId) {
            sql += " and (a.group_id = " + groupId + " or a.group_ids like '%" + groupId + ",%')";
        }
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public boolean delete(Long id, Long userId) {
        String sql =
            "update software set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public Software query(Long id) {
        String sql =
            "select a.*,b.name group_name from software a left join software_group b on a.group_id = b.id where a.id = :id and a.is_deleted = 0";
        List<Software> list = this.list(Software.class, sql, MapTools.simpleMap("id", id));
        return list.isEmpty() ? null : list.get(0);
    }

    public Software exists(String name) {
        String sql = "select * from software a where a.name = :name and a.is_deleted = 0";
        List<Software> list = this.list(Software.class, sql, MapTools.simpleMap("name", name));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean updateStatus(Long id, String status, Long userId) {
        String sql =
            "update software set status = :status, gmt_modify = :gmtModify, mender_id = :menderId where is_deleted = 0 and id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("status", status);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

}
