package com.bocloud.cop.repository;

import java.util.List;
import java.util.Set;

import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import com.bocloud.cop.entity.TaskConfig;
import com.megatron.common.exception.DataBaseException;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;

@Repository("taskConfigRepository")
public class TaskConfigRepository extends BasicGenericDao<TaskConfig, Long> {

    public TaskConfigRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public boolean modify(TaskConfig config) throws DataBaseException {
        return this.baseUpdate(config);
    }

    public TaskConfig query(Long id) throws DataBaseException {
        String sql = "select * from task_config where id = :id and is_deleted = 0";
        List<TaskConfig> list = this.list(TaskConfig.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public List<TaskConfig> listAll() throws DataBaseException {
        String sql = "select a.* from task_config a where a.is_deleted = 0";
        return this.list(TaskConfig.class, sql);
    }

    public TaskConfig queryByCode(String code) throws DataBaseException {
        String sql = "select * from task_config where code = :code and is_deleted = 0";
        List<TaskConfig> list = this.list(TaskConfig.class, sql, MapTools.simpleMap(Common.CODE, code));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean batchUpdate(List<TaskConfig> configs, String[] fields) throws DataBaseException {
        return this.batchUpdate(configs, fields, configs.size());
    }

    public List<TaskConfig> listByCodes(Set<String> codes) throws DataBaseException {
        String sql = "select * from task_config where code in (:code) and is_deleted = 0 ";
        return this.list(TaskConfig.class, sql, MapTools.simpleMap(Common.CODE, codes));
    }

    public List<TaskConfig> listByCategory(String category) throws DataBaseException {
        String sql = "select * from task_config where category = (:category) and is_deleted = 0";
        return this.list(TaskConfig.class, sql, MapTools.simpleMap("category", category));
    }

}
