package com.bocloud.cop.repository;

import com.bocloud.cop.entity.TaskFailRecord;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @author lyy
 * @date 2023年04月04日 10:16
 */
@Repository("taskFailRecordRepository")
public class TaskFailRecordRepository extends BasicGenericDao<TaskFailRecord, Long> {
    public TaskFailRecordRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<TaskFailRecord> list(Pager pager) throws Exception {
        String sql = "select a.* from task_fail_record a where 1 != 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(pager.getParams());
        return this.list(TaskFailRecord.class, sql, param);
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from task_fail_record a where 1 != 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    /**
     * 查询指定记录
     *
     * @param instanceId 任务实例的id
     * @return
     */
    public List<TaskFailRecord> list(Long instanceId) {
        String sql = "select a.* from task_fail_record a where instance_id =:instanceId ";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        return this.list(TaskFailRecord.class, sql, params);
    }

    /**
     * 根据实例id删除错误记录
     *
     * @param id
     */
    public void delete(Long id) {
        String sql = "delete from task_fail_record where id =:id ";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        this.execute(sql, params);
    }

    /**
     * 根据实例id删除错误记录
     *
     * @param instanceId
     */
    public void deleteByInstanceId(Long instanceId) {
        String sql = "delete from task_fail_record where instance_id =:instanceId ";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        this.execute(sql, params);
    }
}
