package com.bocloud.cop.repository;

import com.bocloud.cop.entity.TaskHost;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 任务主机DAO接口实现
 *
 * @author weiwei
 * @version 1.0
 * @since 2017.01.17
 */
@Repository("taskHostRepository")
public class TaskHostRepository extends BasicGenericDao<TaskHost, Long> {

    public TaskHostRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<TaskHost> list(int page, int rows, List<Param> params, Map<String, String> sorter, Long tenantId) {
        String sql = "select * from task_host where is_deleted = 0";
        if (null != tenantId && 0 != tenantId) {
            sql += " and tenant_id = " + tenantId;
        }
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "");
        return this.list(TaskHost.class, sql, this.getQueryBuilder().getParam(params));
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter, Long tenantId) {
        String sql = "select a.id,a.name,a.ip from task_host a where a.is_deleted = 0";
        if (null != tenantId && 0 != tenantId) {
            sql += " and tenant_id = " + tenantId;
        }
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<TaskHost> list = this.list(TaskHost.class, sql, this.getQueryBuilder().getParam(params));
        List<SimpleBean> beans = new ArrayList<>();
        for (TaskHost host : list) {
            beans.add(new SimpleBean(host.getId(), host.getName(), host.getIp()));
        }
        return beans;
    }

    public int count(List<Param> params, Long userGroupId) {
        String sql = "select count(DISTINCT th.id) from task_host th left join "
            + " resource_permission_relation thpr on th.id = thpr.resource_id and thpr.resource_type = 'HOST' and thpr.permission_type ='USERGROUP' "
            + " where th.is_deleted = 0 ";
        Map<String, Object> paramMap = this.getQueryBuilder().getParam(params);
        if (null != userGroupId && 0 != userGroupId) {
            sql += " and thpr.permission_id = :userGroupId ";
            paramMap.put("userGroupId", userGroupId);
        }
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "th");
        return this.countQuery(sql, paramMap).intValue();
    }

    public int count(List<Param> params, List<Long> groupIds, RequestContext context) {
        String sql = "select count(DISTINCT th.id) from task_host th left join "
            + " resource_permission_relation thpr on th.id = thpr.resource_id and thpr.resource_type = 'HOST' and thpr.permission_type ='USERGROUP' "
            + " where th.is_deleted = 0 ";
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        if (RequestContext.Catalog.Tenant != context.getCatalog()) {
            if (groupIds.size() > 0) {
                sql += " and (thpr.permission_id in (:groupIds) or th.owner_id = :userId) ";
                param.put("groupIds", groupIds);
            } else {
                sql += " and th.owner_id = :userId ";
            }
        }
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "th");
        param.put("userId", context.getTarget());
        return this.countQuery(sql, param).intValue();
    }

    /**
     * 查询未知状态
     *
     * @param params
     * @param status
     * @return
     */
    public int countForUnknowStatus(List<Param> params, List<String> status, Long userGroupId) {
        String sql = "select count(DISTINCT th.id) from task_host th left join "
            + " resource_permission_relation thpr on th.id = thpr.resource_id and thpr.resource_type = 'HOST' and thpr.permission_type ='USERGROUP' "
            + " where th.is_deleted = 0 and th.status not in (:statusList) ";
        Map<String, Object> paramMap = this.getQueryBuilder().getParam(params);
        if (null != userGroupId && 0 != userGroupId) {
            sql += " and thpr.permission_id = :userGroupId ";
            paramMap.put("userGroupId", userGroupId);
        }
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "th");
        paramMap.put("statusList", status);
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 查询未知状态
     */
    public int countForUnknowStatus(List<Param> params, List<String> status, List<Long> groupIds,
        RequestContext context) {
        String sql = "select count(DISTINCT th.id) from task_host th left join "
            + " resource_permission_relation thpr on th.id = thpr.resource_id and thpr.resource_type = 'HOST' and thpr.permission_type ='USERGROUP' "
            + " where th.is_deleted = 0 and th.status not in (:statusList) ";
        Map<String, Object> paramMap = this.getQueryBuilder().getParam(params);
        if (RequestContext.Catalog.Tenant != context.getCatalog()) {
            if (groupIds.size() > 0) {
                sql += " and (thpr.permission_id in (:groupIds) or th.owner_id = :userId) ";
                paramMap.put("groupIds", groupIds);
            } else {
                sql += " and th.owner_id = :userId ";
            }
        }
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "th");
        paramMap.put("statusList", status);
        paramMap.put("userId", context.getTarget());
        return this.countQuery(sql, paramMap).intValue();
    }

    public boolean delete(Long id) {
        String sql = "delete from task_host where id = :id";
        return this.execute(sql, MapTools.simpleMap(Common.ID, id)) > 0;
    }

    public TaskHost query(Long id) {
        String sql = "select a.*,b.name segment_name from task_host a "
            + " left join network_segment b on b.is_deleted = 0 and a.segment_id = b.id "
            + " where a.is_deleted = 0 and a.id =:id ";
        List<TaskHost> list = this.list(TaskHost.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean exists(String name) {
        String sql = "select * from task_host where name = :name and is_deleted = 0 ";
        List<TaskHost> list = this.list(TaskHost.class, sql, MapTools.simpleMap("name", name));
        return !list.isEmpty();
    }

    public List<TaskHost> list(Pager pager, Long userGroupId) {
        String sql = "select DISTINCT th.*,ns.name segment_name from task_host th "
            + "  left join resource_permission_relation thpr on th.id = thpr.resource_id and thpr.resource_type='HOST' and thpr.permission_type ='USERGROUP' "
            + "  left join network_segment ns on ns.is_deleted = 0 and th.segment_id = ns.id "
            + "  where th.is_deleted = 0 ";
        Map<String, Object> param = this.getQueryBuilder().getParam(pager.getParams());
        if (null != userGroupId && 0 != userGroupId) {
            sql += " and thpr.permission_id = :userGroupId ";
            param.put("userGroupId", userGroupId);
        }
        sql = this.getQueryBuilder().buildRaw(sql, pager, "th");
        return this.list(TaskHost.class, sql, param);
    }

    public List<TaskHost> list(Pager pager, List<Long> groupIds, RequestContext context) {
        String sql = "select DISTINCT th.*,ns.name segment_name from task_host th "
            + "  left join resource_permission_relation thpr on th.id = thpr.resource_id and thpr.resource_type='HOST' and thpr.permission_type ='USERGROUP' "
            + "  left join network_segment ns on ns.is_deleted = 0 and th.segment_id = ns.id "
            + "  where th.is_deleted = 0 ";
        Map<String, Object> param = this.getQueryBuilder().getParam(pager.getParams());
        if (RequestContext.Catalog.Tenant != context.getCatalog()) {
            if (groupIds.size() > 0) {
                sql += " and (thpr.permission_id in (:groupIds) or th.owner_id = :userId) ";
                param.put("groupIds", groupIds);
            } else {
                sql += " and th.owner_id = :userId ";
            }
        }
        sql = this.getQueryBuilder().buildRaw(sql, pager, "th");
        param.put("userId", context.getTarget());
        return this.list(TaskHost.class, sql, param);
    }

    /**
     * 查询未知状态
     *
     * @param pager
     * @param status
     * @return
     */
    public List<TaskHost> listForUnkownStatus(Pager pager, List<String> status, Long userGroupId) {
        String sql = "select DISTINCT th.*,ns.name segment_name from task_host th "
            + " left join resource_permission_relation thpr on th.id = thpr.resource_id and thpr.resource_type='HOST' and thpr.permission_type ='USERGROUP' "
            + " left join network_segment ns on ns.is_deleted = 0 and th.segment_id = ns.id "
            + " where th.is_deleted = 0 and th.status not in (:statusList) ";
        Map<String, Object> param = this.getQueryBuilder().getParam(pager.getParams());
        if (null != userGroupId && 0 != userGroupId) {
            sql += " and thpr.permission_id = :userGroupId ";
            param.put("userGroupId", userGroupId);
        }
        sql = this.getQueryBuilder().buildRaw(sql, pager, "th");
        param.put("statusList", status);
        return this.list(TaskHost.class, sql, param);
    }

    /**
     * 查询未知状态
     */
    public List<TaskHost> listForUnkownStatus(Pager pager, List<String> status, List<Long> groupIds,
        RequestContext context) {
        String sql = "select DISTINCT th.*,ns.name segment_name from task_host th "
            + "  left join resource_permission_relation thpr on th.id = thpr.resource_id and thpr.resource_type='HOST' and thpr.permission_type ='USERGROUP' "
            + "  left join network_segment ns on ns.is_deleted = 0 and th.segment_id = ns.id "
            + "  where th.is_deleted = 0 and th.status not in (:statusList) ";

        Map<String, Object> param = this.getQueryBuilder().getParam(pager.getParams());
        if (RequestContext.Catalog.Tenant != context.getCatalog()) {
            if (groupIds.size() > 0) {
                sql += " and (thpr.permission_id in (:groupIds) or th.owner_id = :userId) ";
                param.put("groupIds", groupIds);
            } else {
                sql += " and th.owner_id = :userId ";
            }
        }
        sql = this.getQueryBuilder().buildRaw(sql, pager, "th");
        param.put("statusList", status);
        param.put("userId", context.getTarget());
        return this.list(TaskHost.class, sql, param);
    }

    public List<TaskHost> listUnknown() throws Exception {
        String sql = "select * from task_host a where a.is_deleted = 0 and a.known = 0 and a.segment_id is not null";
        return this.list(TaskHost.class, sql);
    }

    public List<TaskHost> list(Long segmentId, Long start, Long count) throws Exception {
        String sql = "select * from task_host where is_deleted = 0 and segment_id = :segmentId order by id limit "
            + start + "," + count;
        return this.list(TaskHost.class, sql, MapTools.simpleMap("segmentId", segmentId));
    }

    public boolean updateKnown(Long id, boolean known) throws Exception {
        String sql = "update task_host set known = :known where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("known", known);
        return this.execute(sql, params) > 0;
    }

    public int count(Long segmentId) throws Exception {
        String sql = "select count(1) from task_host a where is_deleted = 0 and segment_id = :segmentId";
        return this.countQuery(sql, MapTools.simpleMap("segmentId", segmentId)).intValue();
    }

    public TaskHost query(String ip) {
        String sql = "select * from task_host where ip = :ip and is_deleted = 0 ";
        List<TaskHost> list = this.list(TaskHost.class, sql, MapTools.simpleMap("ip", ip));
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 查询指定平台下主机信息
     *
     * @param category
     * @param vendorId
     * @param sourceType
     * @return
     * @throws Exception
     */
    public List<TaskHost> listByVendor(String category, Long vendorId, String sourceType) throws Exception {
        String sql = "select * from task_host " + " where is_deleted = 0 and source_type = :sourceType "
            + " and vendor_id = :vendorId and category =:category ";
        Map<String, Object> params = MapTools.simpleMap("category", category);
        params.put("vendorId", vendorId);
        params.put("sourceType", sourceType);
        return this.list(TaskHost.class, sql, params);
    }

    /**
     * 查询指定平台下主机信息
     *
     * @param vendorId
     * @param sourceType
     * @return
     * @throws Exception
     */
    public List<TaskHost> listByVendor(Long vendorId, String sourceType) throws Exception {
        String sql = "select * from task_host " + " where is_deleted = 0 and source_type = :sourceType "
            + " and vendor_id = :vendorId ";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        params.put("sourceType", sourceType);
        return this.list(TaskHost.class, sql, params);
    }

    /**
     * 逻辑删除
     *
     * @param vendorId
     * @param sourceType
     * @return
     * @throws Exception
     */
    public Boolean removeByVendorId(Long vendorId, String sourceType) throws Exception {
        String sql = "update task_host set is_deleted = 1, gmt_modify = :gmtModify "
            + " where is_deleted = 0 and vendor_id = :vendorId and source_type =:sourceType ";
        Map<String, Object> params = MapTools.simpleMap("gmtModify", new Date());
        params.put("vendorId", vendorId);
        params.put("sourceType", sourceType);
        return this.execute(sql, params) > 0;
    }

    /**
     * 逻辑删除
     *
     * @param vendorId
     * @param sourceType
     * @return
     * @throws Exception
     */
    public Boolean removeByVendorId(Long vendorId, String sourceType, String category) throws Exception {
        String sql = "update task_host set is_deleted = 1, gmt_modify = :gmtModify "
            + " where is_deleted = 0 and vendor_id = :vendorId and source_type =:sourceType"
            + " and category = :category ";
        Map<String, Object> params = MapTools.simpleMap("gmtModify", new Date());
        params.put("vendorId", vendorId);
        params.put("sourceType", sourceType);
        params.put("category", category);
        return this.execute(sql, params) > 0;
    }

    /**
     * 逻辑删除
     *
     * @param id
     * @return
     */
    public boolean remove(Long id) {
        String sql = " update task_host set is_deleted = 1,gmt_modify = :gmtModify where id = :id and is_deleted = 0 ";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    /**
     * 逻辑删除
     *
     * @param instanceId
     * @param vendorId
     * @return
     */
    public boolean remove(String instanceId, Long vendorId, String category) {
        String sql = " update task_host set is_deleted = 1,gmt_modify = :gmtModify where is_deleted = 0 "
            + " and instance_id = :instanceId and vendor_id = :vendorId"
            + " and category = :category and source_type = 'IMS' ";
        Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
        params.put("instanceId", instanceId);
        params.put("gmtModify", new Date());
        params.put("category", category);
        return this.execute(sql, params) > 0;
    }

    /**
     * 根据信息查询同步主机资源的id
     *
     * @param instanceId
     *            唯一标识
     * @param vendorId
     *            平台id
     * @param category
     *            类型
     * @return 主机资源id
     */
    public Long queryId(String instanceId, Long vendorId, String category) {
        String sql = "select id from task_host where is_deleted = 0 "
            + " and instance_id = :instanceId and vendor_id = :vendorId"
            + " and category = :category and source_type = 'IMS' ";
        Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
        params.put("instanceId", instanceId);
        params.put("gmtModify", new Date());
        params.put("category", category);
        List<TaskHost> taskHostList = this.list(TaskHost.class, sql, params);
        return taskHostList.size() > 0 ? taskHostList.get(0).getId() : null;
    }

    /**
     * 通过ip查询第三方输入（手动输入与批量导入的VendorId为null）的对象
     *
     * @param ip
     * @return
     */
    public TaskHost queryByIpAndNoVendor(String ip) {
        String sql = "  select * from task_host where is_deleted = 0 and ip =:ip and vendor_id is null";
        List<TaskHost> list = this.list(TaskHost.class, sql, MapTools.simpleMap("ip", ip));
        return list.size() > 0 ? list.get(0) : null;
    }

    /**
     * 通过ip判断第三方输入（手动输入与批量导入的VendorId为null）的对象是否存在
     *
     * @param ip
     * @return
     */
    public boolean existsNoVendor(String ip) {
        String sql = "select * from task_host where is_deleted = 0 and ip =:ip and vendor_id is null ";
        List<TaskHost> list = this.list(TaskHost.class, sql, MapTools.simpleMap("ip", ip));
        return !list.isEmpty();
    }

    /**
     * 获取所有手动创建或者批量导入的主机
     *
     * @return
     */
    public List<TaskHost> listAllNoVendor() {
        String sql = "select * from task_host where is_deleted = 0 and vendor_id is null ";
        return this.list(TaskHost.class, sql);
    }

    /**
     * 逻辑删除(批量)
     *
     * @param vendorId
     * @param sourceType
     * @return
     * @throws Exception
     */
    public Boolean batchRemove(List<String> instanceIds, Long vendorId, String sourceType, String category)
        throws Exception {
        String sql = "update task_host set is_deleted = 1, gmt_modify = :gmtModify "
            + " where is_deleted = 0 and vendor_id = :vendorId and source_type =:sourceType "
            + " and category =:category and instance_id in (:instanceIds) ";
        Map<String, Object> params = MapTools.simpleMap("instanceIds", instanceIds);
        params.put("gmtModify", new Date());
        params.put("vendorId", vendorId);
        params.put("sourceType", sourceType);
        params.put("category", category);
        return this.execute(sql, params) > 0;
    }

    /**
     * 逻辑删除(批量)
     *
     * @param vendorId
     * @param sourceType
     * @return
     * @throws Exception
     */
    public List<TaskHost> listByInstanceIds(List<String> instanceIds, Long vendorId, String sourceType, String category)
        throws Exception {
        String sql =
            "select * from task_host " + " where is_deleted = 0 and vendor_id = :vendorId and source_type =:sourceType "
                + " and category =:category and instance_id in (:instanceIds) ";
        Map<String, Object> params = MapTools.simpleMap("instanceIds", instanceIds);
        params.put("gmtModify", new Date());
        params.put("vendorId", vendorId);
        params.put("sourceType", sourceType);
        params.put("category", category);
        return this.list(TaskHost.class, sql, params);
    }

    /**
     * 通过ip查询第三方输入（手动输入与批量导入的VendorId为null）的对象
     *
     * @return
     */
    public TaskHost queryForIms(String instanceId, Long vendorId, String category) {
        String sql =
            "  select * from task_host where is_deleted = 0 and instance_id =:instanceId and vendor_id = :vendorId "
                + " and category = :category and source_type = 'IMS' ";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        params.put("vendorId", vendorId);
        params.put("category", category);
        List<TaskHost> list = this.list(TaskHost.class, sql, params);
        return list.size() > 0 ? list.get(0) : null;
    }

    /**
     * 通过ip查询第三方输入（手动输入与批量导入的VendorId为null）的对象
     *
     * @return
     */
    public List<TaskHost> listVendorId(String category, String sourceType) {
        String sql = "  select vendor_id from task_host where is_deleted = 0  "
            + " and category = :category and source_type = :sourceType group by vendor_id ";
        Map<String, Object> params = MapTools.simpleMap("category", category);
        params.put("sourceType", sourceType);
        return this.list(TaskHost.class, sql, params);
    }

    /**
     * 通过ip查询第三方输入（手动输入与批量导入的VendorId为null）的对象
     *
     * @return
     */
    public List<TaskHost> listVendorId(String sourceType) {
        String sql = "select vendor_id from task_host where is_deleted = 0  "
            + " and source_type = :sourceType group by vendor_id ";
        return this.list(TaskHost.class, sql, MapTools.simpleMap("sourceType", sourceType));
    }

    /**
     * 批量查询ids
     */
    public List<TaskHost> query(List<Long> ids) {
        String sql = "select a.* from task_host a where a.is_deleted = 0 and a.id in(:ids) ";
        return this.list(TaskHost.class, sql, MapTools.simpleMap("ids", ids));
    }

    public List<TaskHost> listFormView(Pager pager, List<Long> viewId) {
        String viewSql = "select a.* from task_host a where a.is_deleted = 0 and a.id in(:viewId) ";
        String sql = "select * from (" + viewSql + ") a where 1 =1 ";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(pager.getParams());
        param.put("viewId", viewId);
        return this.list(TaskHost.class, sql, param);
    }

    public int countFormView(List<Param> params, List<Long> viewId) {
        String viewSql = "select count(1) from task_host a where a.is_deleted = 0 and a.id in(:viewId) ";
        String sql = "select * from (" + viewSql + ") a where 1 =1 ";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        param.put("viewId", viewId);
        return this.countQuery(sql, param).intValue();
    }

    public List<TaskHost> queryByIds(List<Long> ids) {
        String sql = "select * from task_host where id in(:ids)";
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        List<TaskHost> list = this.list(TaskHost.class, sql, params);
        return list;
    }

    public List<TaskHost> queryByIps(List<String> ips) {
        String sql =
            "select a.*,b.name segment_name from task_host a left join network_segment b on b.is_deleted = 0 and a.segment_id = b.id where a.ip in (:ips)";
        Map<String, Object> params = MapTools.simpleMap("ips", ips);
        List<TaskHost> list = this.list(TaskHost.class, sql, params);
        return list;
    }

    public boolean clearSegment(Long segmentId) throws Exception {
        String sql =
            "update task_host set segment_id = null,ip = null,update_mode = 'MANUAL' where segment_id = :segmentId";
        Map<String, Object> params = MapTools.simpleMap("segmentId", segmentId);
        return this.execute(sql, params) > 0;
    }


    /**
     * 根据数据源类型进行删除
     * @param sourceType 数据源类型
     */
    public void removeBySourceType(String sourceType,Long saltId) {
        String sql = " update task_host set is_deleted = 1,gmt_modify = :gmtModify  where source_type =:sourceType and is_deleted = 0 ";
        if(null != saltId){
            sql += " and salt_id = " + saltId;
        }
        Map<String, Object> paramMap = MapTools.simpleMap("sourceType", sourceType);
        paramMap.put("gmtModify", new Date());
        this.execute(sql, paramMap);
    }

    /**
     * 根据saltId查询主机列表
     * @param saltId saltId，为空时查询所有saltId不为空的主机资源
     * @return 主机资源列表
     */
    public List<TaskHost> listBySaltId(Long saltId) {
        String sql = " select * from task_host where is_deleted = 0 ";
        if ( null != saltId ) {
            sql += " and salt_id = " + saltId;
        }else {
            sql += " and salt_id is not null ";
        }
        return this.list(TaskHost.class, sql);
    }


    /**
     * 查询出所有的主机资源
     *
     * @return 主机资源列表
     */
    public List<TaskHost> listAll() {
        String sql = "select * from task_host where is_deleted = 0  ";
        return this.list(TaskHost.class, sql);
    }
}
