package com.bocloud.cop.repository;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.TaskInstanceBak;
import com.bocloud.cop.utils.DynamicClassUtils;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * @author huliwei
 * @date 2023/4/21
 */
@Repository("taskInstanceBakRepository")
public class TaskInstanceBakRepository extends BasicGenericDao<TaskInstanceBak, Long> {
    public TaskInstanceBakRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, namedParameterJdbcTemplate, service);
    }
    public List<TaskInstanceBak> query(String taskInstanceBakName , String leftTime , String rightTime ) throws Exception {
        String sql = "select id from " + taskInstanceBakName + " a  where a.is_deleted = 0 and a.gmt_create >= :leftTime and a.gmt_create <= :rightTime ";
        Map<String, Object> params = new HashMap<>();
        params.put("leftTime",leftTime);
        params.put("rightTime",rightTime);
        List<TaskInstanceBak> list = this.list(TaskInstanceBak.class, sql, params);
        return list;
    }
    public int count(List<Param> params, String taskInstanceBakName, String taskInstanceHostBakName , String host) throws Exception {
        String sql = null;
        if (StringUtils.hasText(host)) {
            sql = "select count(1) from " + taskInstanceBakName + " a, " + taskInstanceHostBakName + " b where a.is_deleted = 0 and a.id = b.instance_id and b.host = '" + host + "'";
        } else {
            sql = "select count(1) from " + taskInstanceBakName + " a where a.is_deleted = 0";
        }
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = this.getQueryBuilder().getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }
    public List<TaskInstanceBak> list(Pager pager, List<String> finalInstanceTableNames, List<String> finalInstanceHostTableNames , String host) throws Exception {
        String sql = null;
        for (int i = 0; i < finalInstanceTableNames.size(); i++) {
            if( i > 0){
                sql += " union all ";
            }
            if(i == 0){
                if (StringUtils.hasText(host)) {
                    sql = "(select id,name,uuid,type,status,gmt_start,gmt_finish,cron,catalog,execute_mode,execute_host,platform,creator_id,gmt_create, '"+ finalInstanceTableNames.get(i)  +"' AS table_name  from " + finalInstanceTableNames.get(i) + " a, " + finalInstanceHostTableNames.get(i) + " b where a.is_deleted = 0 and a.id = b.instance_id and b.host = '" + host + "'";
                } else {
                    sql = "(select id,name,uuid,type,status,gmt_start,gmt_finish,cron,catalog,execute_mode,execute_host,platform,creator_id,gmt_create, '"+ finalInstanceTableNames.get(i)  +"' AS table_name  from " + finalInstanceTableNames.get(i) + " a where a.is_deleted = 0";
                }
            }else{
                if (StringUtils.hasText(host)) {
                    sql += "(select id,name,uuid,type,status,gmt_start,gmt_finish,cron,catalog,execute_mode,execute_host,platform,creator_id,gmt_create, '"+ finalInstanceTableNames.get(i)  +"' AS table_name  from " + finalInstanceTableNames.get(i) + " a, " + finalInstanceHostTableNames.get(i) + " b where a.is_deleted = 0 and a.id = b.instance_id and b.host = '" + host + "'";
                } else {
                    sql += "(select id,name,uuid,type,status,gmt_start,gmt_finish,cron,catalog,execute_mode,execute_host,platform,creator_id,gmt_create, '"+ finalInstanceTableNames.get(i)  +"' AS table_name  from " + finalInstanceTableNames.get(i) + " a where a.is_deleted = 0";
                }
            }
            sql = this.getQueryBuilder().buildRaw(sql, pager.getParams(),pager.getSorter(), "a");
            sql += " LIMIT 999999999 )";
        }
        if(!ListTool.isEmpty(finalInstanceTableNames)){
            Integer pageNum = pager.getPage();
            Integer pageSize = pager.getRows();
            pageNum = 0 >= pageNum ? 1 : pageNum;
            pageSize = 0 >= pageSize ? 10 : pageSize;
            sql +=  " LIMIT " + (pageNum - 1) * pageSize + "," + pageSize;
        }
        return this.list(TaskInstanceBak.class, sql);
    }

    public TaskInstanceBak query(Long id, String taskInstanceBakName ) throws Exception {
        String sql = "select a.* from " + taskInstanceBakName + " a where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<TaskInstanceBak> list = this.list(TaskInstanceBak.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public int countTable(String tableName,String databaseDriver) throws Exception {
        String sql;
        if("dm".equalsIgnoreCase(databaseDriver)){
            sql = "SELECT COUNT(*)  FROM all_tables WHERE OWNER = 'COP' AND TABLE_NAME = :tableName ";
        }else{
            sql = "select count(1) from information_schema.TABLES where table_schema = (select database()) AND TABLE_NAME = :tableName ";
        }
        Map<String, Object> params = MapTools.simpleMap("tableName", tableName);
        return this.countQuery(sql, params).intValue();
    }
    public void createTable(String tableName, String databaseDriver , TaskInstanceBak taskInstanceBak) throws Exception {
        // 拼接sql
        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("CREATE TABLE " + tableName + " (\n");
        Map<String, JSONObject> fieldMap = DynamicClassUtils.getDeclaredFieldsInfo(taskInstanceBak);
        // id默认为主键
        sqlBuilder.append("id bigint NOT NULL ,\n");
        for (Map.Entry<String, JSONObject> entry : fieldMap.entrySet()) {
            String fieldName = entry.getKey();
            JSONObject fieldJSON = entry.getValue();
            sqlBuilder.append("" + fieldName + "");
            String fieldType = DynamicClassUtils.judgeDataType(fieldJSON.getString("fieldType"));
            if (!fieldJSON.getBooleanValue("nullable")) {
                if ("varchar".equals(fieldType)) {
                    String fieldDBType = fieldJSON.getString("fieldDBType");
                    if(null != fieldDBType){
                        sqlBuilder.append(" " + fieldDBType + " NOT NULL,\n");
                    }else {
                        sqlBuilder.append(" " + fieldType + "(" + fieldJSON.getLong("length") + ") NOT NULL,\n");
                    }
                } else if ("tinyint".equals(fieldType)) {
                    if("dm".equalsIgnoreCase(databaseDriver)){
                        sqlBuilder.append(" " + "SMALLINT" + " NOT NULL,\n");
                    }else{
                        sqlBuilder.append(" " + fieldType + "(1) NOT NULL,\n");
                    }
                } else {
                    // 目前包括bigint、datetime
                    sqlBuilder.append(" " + fieldType + " NOT NULL,\n");
                }
            } else {
                if ("varchar".equals(fieldType)) {
                    String fieldDBType = fieldJSON.getString("fieldDBType");
                    if(null != fieldDBType){
                        if("dm".equalsIgnoreCase(databaseDriver)){
                            if("longtext".equals(fieldDBType)){
                                sqlBuilder.append(" " + "LONG" + " DEFAULT NULL,\n");
                            }else{
                                sqlBuilder.append(" " + fieldDBType + " DEFAULT NULL,\n");
                            }
                        }else{
                            sqlBuilder.append(" " + fieldDBType + " DEFAULT NULL,\n");
                        }
                    }else{
                        sqlBuilder.append(" " + fieldType + "(" + fieldJSON.getLong("length") + ") DEFAULT NULL,\n");
                    }
                } else if ("tinyint".equals(fieldType)) {
                    if("dm".equalsIgnoreCase(databaseDriver)){
                        sqlBuilder.append(" " + "SMALLINT" + " DEFAULT NULL,\n");
                    }else{
                        sqlBuilder.append(" " + fieldType + "(1) DEFAULT NULL,\n");
                    }
                } else {
                    // 目前包括bigint、datetime
                    sqlBuilder.append(" " + fieldType + " DEFAULT NULL,\n");
                }
            }
        }
        sqlBuilder.append("PRIMARY KEY (id) \n");
        if("dm".equalsIgnoreCase(databaseDriver)){
            sqlBuilder.append(") ;");
        }else{
            //创建表的同时创建索引
            sqlBuilder.append(", index "+ tableName +"_gmt_create (gmt_create)\n");
            sqlBuilder.append(") ENGINE=InnoDB DEFAULT CHARSET=utf8;");
        }
        this.execute(sqlBuilder.toString(), null);
    }
}
