package com.bocloud.cop.repository;

import com.bocloud.cop.entity.TaskInstanceIndex;
import com.megatron.common.model.Pager;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author lyy
 * @date 2023年04月04日 10:31
 */
@Repository("taskInstanceIndexRepository")
public class TaskInstanceIndexRepository extends JdbcGenericDao<TaskInstanceIndex, Long> {

    public TaskInstanceIndexRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<TaskInstanceIndex> list(Pager pager) {
        String sql = "select a.id , a.name , instance_num , target_host_num from task_instance_index a  where a.is_deleted = 0 and period = '1M' ";
        sql = this.getQueryBuilder().buildRaw(sql, pager.getParams(),null, "a");
        return this.list(TaskInstanceIndex.class, sql, this.getQueryBuilder().getParam(pager.getParams()));
    }

    public List<TaskInstanceIndex> listGroup(String name, String startTime, String endTime) {
        String sql = "select a.name ,sum(instance_num) instance_num_sum, sum(target_host_num)target_host_num_sum from task_instance_index a  where a.is_deleted = 0 ";
        Map<String, Object> params = new HashMap<>();
        if (StringUtils.isNotEmpty(name)) {
            sql += " and name like :name";
            params.put("name", "%" + name + "%");
        }
        if (StringUtils.isNotEmpty(startTime)) {
            sql += " and count_start_time >= :startTime ";
            params.put("startTime", startTime);
        }
        if (StringUtils.isNotEmpty(endTime)) {
            sql += " and count_end_time <= :endTime ";
            params.put("endTime", endTime);
        }
        sql += " group by name";
        return this.list(TaskInstanceIndex.class, sql, params);
    }

    public List<TaskInstanceIndex> listPaltform(String startTime, String endTime) {
        String sql = "select DISTINCT platform name from task_instance a where 1 = 1 and platform is not null ";
        Map<String, Object> params = new HashMap<>();
        if (org.apache.commons.lang3.StringUtils.isNotEmpty(startTime)) {
            sql += " and gmt_create >= :startTime ";
            params.put("startTime", startTime);
        }
        if (org.apache.commons.lang3.StringUtils.isNotEmpty(endTime)) {
            sql += " and gmt_create < :endTime ";
            params.put("endTime", endTime);
        }
        return this.list(TaskInstanceIndex.class, sql, params);
    }

    public List<TaskInstanceIndex> countInstanceNum(String startTime, String endTime) {
        String sql = "select platform name,count(1) instance_num from task_instance a where 1 = 1 and platform is not null ";
        Map<String, Object> params = new HashMap<>();
        if (null != startTime) {
            sql += " and gmt_create >= :startTime ";
            params.put("startTime", startTime);
        }
        if (null != startTime) {
            sql += " and gmt_create < :endTime ";
            params.put("endTime", endTime);
        }
        sql += " group by platform ";
        return this.list(TaskInstanceIndex.class, sql, params);
    }

    public Long countTargetHostNum(String name, String startTime, String endTime) {
        String sql = "select count(distinct a.host+a.instance_id) from task_instance_host a,task_instance b where 1 = 1 and a.instance_id = b.id ";

        Map<String, Object> params = new HashMap<>();
        if (StringUtils.isNotEmpty(name)) {
            sql += " and b.platform = :name";
            params.put("name", name);
        }
        if (null != startTime) {
            sql += " and b.gmt_create >= :startTime ";
            params.put("startTime", startTime);
        }
        if (null != startTime) {
            sql += " and b.gmt_create < :endTime ";
            params.put("endTime", endTime);
        }
        return this.countQuery(sql, params);

    }

    public TaskInstanceIndex getLast(String period) {
        String sql = "select count_end_time from task_instance_index a where 1 = 1 and period = :period order by  count_end_time desc limit 1";
        Map<String, Object> params = new HashMap<>();
        params.put("period", period);
        List<TaskInstanceIndex> list = this.list(TaskInstanceIndex.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public List<TaskInstanceIndex> listApplicationCount(String periodSymbol, List<String> names, String startTime, String endTime) {
        String sql = "select * from task_instance_index a  where a.is_deleted = 0 ";
        Map<String, Object> params = new HashMap<>();
        if (StringUtils.isNotEmpty(periodSymbol)) {
            sql += " and period = :period";
            params.put("period", periodSymbol);
        }
        if (CollectionUtils.isNotEmpty(names)) {
            sql += " and name in(:names) ";
            params.put("names", names);
        }
        if (StringUtils.isNotEmpty(startTime)) {
            sql += " and count_start_time >= :startTime ";
            params.put("startTime", startTime);
        }
        if (StringUtils.isNotEmpty(endTime)) {
            sql += " and count_end_time <= :endTime ";
            params.put("endTime", endTime);
        }
        return this.list(TaskInstanceIndex.class, sql, params);
    }

}