package com.bocloud.cop.repository;

import com.bocloud.cop.entity.TaskInstanceJob;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * 任务实例job DAO接口实现
 */
@Repository("taskInstanceJobRepository")
public class TaskInstanceJobRepository extends BasicGenericDao<TaskInstanceJob, Long> {

    public TaskInstanceJobRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<TaskInstanceJob> list() throws Exception {
        String sql = "select * from task_instance_job order by id";
        return this.list(TaskInstanceJob.class, sql);
    }

    public boolean delete(Long id) throws Exception {
        String sql = "delete from task_instance_job where id = :id";
        return this.execute(sql, MapTools.simpleMap(Common.ID, id)) > 0;
    }

    public boolean delete(String taskId) throws Exception {
        String sql = "delete from task_instance_job where task_id = :taskId";
        return this.execute(sql, MapTools.simpleMap("taskId", taskId)) > 0;
    }

    public int count() throws Exception {
        String sql = "select count(1) from task_instance_job a";
        sql = this.getQueryBuilder().buildRaw(sql, new ArrayList<>(), null, "a");
        return this.countQuery(sql, new HashMap<>()).intValue();
    }

    public List<TaskInstanceJob> list(int rows) throws Exception {
        String sql = "select * from task_instance_job";
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(1, rows, null, null), null);
        return this.list(TaskInstanceJob.class, sql, new HashMap<>());
    }

    public boolean delete(List<Long> ids) throws Exception {
        String sql = "delete from task_instance_job where";
        int size = ids.size();
        int index = 0;
        for (Long id : ids) {
            index++;
            if (index == size) {
                sql += " id = " + id;
            } else {
                sql += " id = " + id + " or";
            }
        }
        return this.execute(sql, new HashMap<>()) > 0;
    }

}
