package com.bocloud.cop.repository;

import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.enums.Status;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 任务实例接口实现
 * 
 * @author weiwei
 * @version 1.0
 * @since 2017.07.07
 */
@Repository("taskInstanceRepository")
public class TaskInstanceRepository extends JdbcGenericDao<TaskInstance, Long> {

    public TaskInstanceRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<TaskInstance> list(Pager pager, String taskInstanceBakName, String taskInstanceHostBakName , String host) throws Exception {
        String sql = null;
        if (StringUtils.hasText(host)) {
            sql = "select a.* , '"+ taskInstanceBakName +"' AS table_name  from " + taskInstanceBakName + " a, " + taskInstanceHostBakName + " b where a.is_deleted = 0 and a.id = b.instance_id and b.host = '" + host + "'";
        } else {
            sql = "select a.* , '"+ taskInstanceBakName +"' AS table_name  from " + taskInstanceBakName + " a where a.is_deleted = 0";
        }
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(TaskInstance.class, sql);
    }

    public TaskInstance queryByStartTime(String leftTime) throws Exception {
        String sql = "select id from task_instance a where a.is_deleted = 0 and a.gmt_create < :leftTime ";
        Map<String, Object> params = MapTools.simpleMap("leftTime", leftTime);
        List<TaskInstance> list = this.list(TaskInstance.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public TaskInstance queryByEndTime(String rightTime) throws Exception {
        String sql = "select id from task_instance a where a.is_deleted = 0 and a.gmt_create <= :rightTime ";
        Map<String, Object> params = MapTools.simpleMap("rightTime", rightTime);
        List<TaskInstance> list = this.list(TaskInstance.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }
    public int linkCount(List<Param> params, String executeWorker) throws Exception {
        String sql = null;
        if (StringUtils.hasText(executeWorker)) {
            sql =
                    "select count(1) from task_instance a where a.is_deleted = 0 and a.id in (select distinct instance_id from task_instance_host where execute_worker like '%"
                            + executeWorker + "%') ";
        } else {
            sql = "select count(1) from task_instance a where a.is_deleted = 0";
        }
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = this.getQueryBuilder().getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<TaskInstance> queryByIds(List<Long> ids) throws Exception {
        String sql =
                "select id,name,uuid,status,execute_host,gmt_create,platform,gmt_start,gmt_finish from task_instance a where a.is_deleted = 0 and a.id in (:ids) ORDER BY id DESC ";
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        List<TaskInstance> list = this.list(TaskInstance.class, sql, params);
        return list;
    }

    public List<TaskInstance> linkQuery(Pager pager, String executeWorker) throws Exception {
        String sql = null;
        if (StringUtils.hasText(executeWorker)) {
            sql =
                    "select  id,name,uuid,status,execute_host,gmt_create,platform,gmt_start,gmt_finish from task_instance a where a.is_deleted = 0 and a.id in (select distinct instance_id from task_instance_host where execute_worker like '%"
                            + executeWorker + "%') ";
        } else {
            sql = "select  id,name,uuid,status,execute_host,gmt_create,platform,gmt_start,gmt_finish from task_instance a where a.is_deleted = 0";
        }
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(TaskInstance.class, sql);
    }

    public List<TaskInstance> list(Pager pager, boolean await) {
        String sql = null;
        if (await) {
            sql =
                "select a.* from task_instance a where a.is_deleted = 0 and a.status in ('WAITTING','APPROVING','APPROVED','REFUSED')";
        } else {
            sql =
                "select a.* from task_instance a where a.is_deleted = 0 and a.status not in ('APPROVING','APPROVED','REFUSED')";
        }
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(TaskInstance.class, sql);
    }

    public TaskInstance query(Long id) {
        String sql = "select a.* from task_instance a where a.is_deleted = 0 and a.id = :id";
        List<TaskInstance> list = this.list(TaskInstance.class, sql, MapTools.simpleMap("id", id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean remove(Long id, Long userId) {
        String sql =
            "update task_instance set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public int count(List<Param> params) {
        String sql =
            "select count(1) from task_instance a where a.is_deleted = 0 and a.status not in ('APPROVING','APPROVED','REFUSED')";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public boolean updateStatus(Long id, String status, Long userId) {
        String sql =
            "update task_instance set status = :status, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("status", status);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<TaskInstance> listByHost(Status.Inst status, String executeHost, String excludeHost) {
        String sql = "select * from task_instance where status = :status ";
        Map<String, Object> params = MapTools.simpleMap("status", status.name());
        if (StringUtils.hasText(executeHost)) {
            sql += " and execute_host = :executeHost ";
            params.put("executeHost", executeHost);
        }
        if (StringUtils.hasText(excludeHost)) {
            sql += " and execute_host != :excludeHost";
            params.put("excludeHost", excludeHost);
        }
        return list(TaskInstance.class, sql, params);
    }

    public List<TaskInstance> listExpInstance(String executeHost, String excludeHost) {
        StringBuffer sqlBuffer = new StringBuffer(
            "SELECT * FROM task_instance WHERE is_deleted=0 AND status IN ('RUNNING','SUSPENDED','CANCELING','WAITTING') ");
        Map<String, Object> params = new HashMap<>();
        if (StringUtils.hasText(executeHost)) {
            sqlBuffer.append("AND execute_host = :executeHost ");
            params.put("executeHost", executeHost);
        }
        if (StringUtils.hasText(excludeHost)) {
            sqlBuffer.append("AND execute_host != :excludeHost ");
            params.put("excludeHost", excludeHost);
        }
        // sqlBuffer.append("OR execute_host IS NULL");
        return list(TaskInstance.class, sqlBuffer.toString(), params);
    }

    public List<TaskInstance> listAwaits(Pager pager) {
        String sql =
            "select a.* from task_instance a where a.is_deleted = 0 and a.status in ('WAITTING','APPROVING','APPROVED','REFUSED')";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(TaskInstance.class, sql);
    }

    public int countAwaits(List<Param> params) {
        String sql =
            "select count(1) from task_instance a where a.is_deleted = 0 and a.status in ('WAITTING','APPROVING','APPROVED','REFUSED')";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) {
        String sql = "select a.id,a.name,a.status from task_instance a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<TaskInstance> instances = this.list(TaskInstance.class, sql, this.getQueryBuilder().getParam(params));
        List<SimpleBean> beans = new ArrayList<>();
        for (TaskInstance instance : instances) {
            beans.add(new SimpleBean(instance.getId(), instance.getName(), instance.getStatus()));
        }
        return beans;
    }

    public List<TaskInstance> listByTask(Long taskId) {
        String sql = "select a.* from task_instance a where a.is_deleted = 0 and a.task_id = :taskId";
        return this.list(TaskInstance.class, sql, MapTools.simpleMap("taskId", taskId));
    }

    public List<TaskInstance> getInstanceNums(String start, String end) {
        String sql =
            "select distinct task_id,count(task_id) count from task_instance where is_deleted = 0 and gmt_create >= '"
                + start + "' and gmt_create <= '" + end + "' group by task_id order by count desc";
        return this.list(TaskInstance.class, sql);
    }

    public TaskInstance get(String uuid) throws Exception {
        String sql = "select * from task_instance a where a.is_deleted = 0 and a.uuid = :uuid";
        List<TaskInstance> list = this.list(TaskInstance.class, sql, MapTools.simpleMap("uuid", uuid));
        return list.isEmpty() ? null : list.get(0);
    }

    public List<TaskInstance> list(Pager pager, Long commandId) {
        String sql =
            "select a.* from task_instance a,command_instance b where a.is_deleted = 0 and a.id = b.instance_id and b.command_id = "
                + commandId;
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(TaskInstance.class, sql);
    }

    public int count(List<Param> params, Long commandId) {
        String sql =
            "select count(1) from task_instance a,command_instance b where a.is_deleted = 0 and a.id = b.instance_id and b.command_id = "
                + +commandId;
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public boolean updateTime(Long id) throws Exception {
        String sql = "update task_instance set gmt_create = :gmtCreate, gmt_modify = :gmtModify where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtCreate", new Date());
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public boolean updateLimitedDiscard(Long id, Long userId, Long platformAlertId) throws Exception {
        String sql =
                "update task_instance set is_limited = 1, status = 'CANCELED', platform_alert_id = :platformAlertId, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("platformAlertId", platformAlertId);
        return this.execute(sql, params) > 0;
    }

    public boolean updateLimitedQueue(Long id, Long userId, Long platformAlertId) throws Exception {
        String sql =
                "update task_instance set is_limited = 1, platform_alert_id = :platformAlertId, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("platformAlertId", platformAlertId);
        return this.execute(sql, params) > 0;
    }

    public List<TaskInstance> listLimited(String name, Integer instanceNum) throws Exception {
        String sql = "select * from task_instance a where 1 = 1 and a.status='CREATED' and  a.is_limited = 1 ";
        Map<String, Object> params = new HashMap<>();
        if (StringUtils.hasText(name)) {
            sql += " and a.platform = :name";
            params.put("name", name);
        }
        sql += " order by a.gmt_create ";
        if (null != instanceNum) {
            sql += " limit " + instanceNum;
        }
        return list(TaskInstance.class, sql, params);
    }

    public boolean updateLimitedRelease(Long id, Long userId) throws Exception {
        String sql =
                "update task_instance set is_limited = 0,limited_release_time = :limitedReleaseTime, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("limitedReleaseTime", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public Integer countInstanceNum(String name, String startTime, String endTime) {
        String sql = "select count(1) from task_instance a where 1 = 1 ";
        Map<String, Object> params = new HashMap<>();
        if (StringUtils.hasText(name)) {
            sql += " and a.platform = :name";
            params.put("name", name);
        }
        if (null != startTime) {
            sql += " and gmt_create >= :startTime ";
            params.put("startTime", startTime);
        }
        if (null != endTime) {
            sql += " and gmt_create < :endTime ";
            params.put("endTime", endTime);
        }
        return this.countQuery(sql, params).intValue();

    }

    public Integer countTargetHostNum(String name, String startTime, String endTime) {

        String sql = "select count(1) from task_instance_host a,task_instance b where 1 = 1 and a.instance_id = b.id ";
        Map<String, Object> params = new HashMap<>();
        if (StringUtils.hasText(name)) {
            sql += " and b.platform = :name";
            params.put("name", name);
        }
        if (null != startTime) {
            sql += " and b.gmt_create >= :startTime ";
            params.put("startTime", startTime);
        }
        if (null != startTime) {
            sql += " and b.gmt_create < :endTime ";
            params.put("endTime", endTime);
        }
        return this.countQuery(sql, params).intValue();
    }

    public Integer countInstanceNumLast(String name, String startTime, String endTime) {
        String sql = "select count(1) from task_instance a where 1 = 1 ";
        Map<String, Object> params = new HashMap<>();
        if (StringUtils.hasText(name)) {
            sql += " and a.platform = :name";
            params.put("name", name);
        }
        if (null != startTime && null != endTime) {
            // 如果在当前周期中 告警配置被禁用了，会放行对应的数据。这部分数据也需要汇总
            sql += " and ((a.limited_release_time >= :startTime and a.limited_release_time < :endTime) or (a.is_limited=1 and a.status='CREATED' ))";
            sql += " and a.gmt_create < :startTime  ";
            params.put("startTime", startTime);
            params.put("endTime", endTime);
        }

        return this.countQuery(sql, params).intValue();

    }

    public Integer countTargetHostNumLast(String name, String startTime, String endTime) {
        String sql = "select count(1) from task_instance_host a,task_instance b where 1 = 1 and a.instance_id = b.id ";

        Map<String, Object> params = new HashMap<>();
        if (StringUtils.hasText(name)) {
            sql += " and b.platform = :name";
            params.put("name", name);
        }
        if (null != startTime && null != endTime) {
            sql += " and ((b.limited_release_time >= :startTime and b.limited_release_time < :endTime) or (b.is_limited=1 and b.status='CREATED' ))";
            sql += "  and b.gmt_create < :startTime  ";
            params.put("startTime", startTime);
            params.put("endTime", endTime);
        }
        return this.countQuery(sql, params).intValue();

    }
    public Integer countTargetHostNum(Long instanceId) {
        String sql = "select count(distinct a.host) from task_instance_host a where a.instance_id =:instanceId";
        Map<String, Object> params = new HashMap<>();
        params.put("instanceId", instanceId);
        return this.countQuery(sql, params).intValue();

    }

    public List<TaskInstance> list(List<Param> params) throws Exception {
        String sql = "select * from task_instance a where 1 = 1";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = this.getQueryBuilder().getParam(params);
        return this.list(TaskInstance.class, sql, paramMap);
    }

    public boolean delete(List<Long> ids)  {
        String sql = "delete from task_instance where id in (:ids)";
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        return this.execute(sql, params) > 0;
    }

    public boolean initStatus(Long id, String status, Long userId) throws Exception {
        String sql =
            "update task_instance set status = :status, gmt_modify = :gmtModify, mender_id = :menderId, gmt_start = null, gmt_finish = null where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("status", status);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<TaskInstance> list(Pager pager, String host) {
        String sql = null;
        if (StringUtils.hasText(host)) {
            sql =
                    "select distinct a.* from task_instance a,task_instance_host b where a.is_deleted = 0  and a.status not in ('APPROVING','APPROVED','REFUSED') and a.id = b.instance_id and b.host = '"
                            + host + "'";
        } else {
            sql =
                    "select a.* from task_instance a where a.is_deleted = 0 and a.status not in ('APPROVING','APPROVED','REFUSED')";
        }
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(TaskInstance.class, sql);
    }

    public int count(List<Param> params, String host) {
        String sql = null;
        if (StringUtils.hasText(host)) {
            sql =
                    "select count(distinct a.id) from task_instance a,task_instance_host b where a.is_deleted = 0 and a.status not in ('APPROVING','APPROVED','REFUSED') and a.id = b.instance_id and b.host = '"
                            + host + "'";
        } else {
            sql =
                    "select count(1) from task_instance a where a.is_deleted = 0 and a.status not in ('APPROVING','APPROVED','REFUSED')";
        }
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    /**
     * 计算当前周期已经执行及之前周期下发当前周期执行的任务数
     * @param name
     * @param startTime
     * @param endTime
     * @return
     */
    public Integer countDealedInstanceNum(String name, String startTime, String endTime) {
        String sql = "select count(1) from task_instance a where 1 = 1 ";
        Map<String, Object> params = new HashMap<>();
        if (org.apache.commons.lang3.StringUtils.isNotEmpty(name)) {
            sql += " and a.platform = :name";
            params.put("name", name);
        }
        if (null != startTime && null != endTime) {
            sql +=
                    " and ( (a.limited_release_time >= :startTime and a.limited_release_time < :endTime and a.gmt_create < :startTime ) ";
            sql += " or (a.gmt_create >= :startTime and a.gmt_create < :endTime and a.is_limited!=1))";

            params.put("startTime", startTime);
            params.put("endTime", endTime);
        }
        return this.countQuery(sql, params).intValue();
    }


    /**
     * 计算当前周期已经执行及之前周期下发当前周期执行的目标主机数
     * @param name
     * @param startTime
     * @param endTime
     * @return
     */
    public Integer countDealedTargetHostNum(String name, String startTime, String endTime) {
        String sql = "select count(distinct a.host+a.instance_id) from task_instance_host a,task_instance b where a.instance_id = b.id ";
        Map<String, Object> params = new HashMap<>();
        if (org.apache.commons.lang3.StringUtils.isNotEmpty(name)) {
            sql += " and b.platform = :name";
            params.put("name", name);
        }
        if (null != startTime && null != endTime) {
            sql += " and ((b.limited_release_time >= :startTime and b.limited_release_time < :endTime  and b.gmt_create < :startTime ) ";
            sql += " or (b.gmt_create >= :startTime and b.gmt_create < :endTime and b.is_limited!=1))";
            params.put("startTime", startTime);
            params.put("endTime", endTime);
        }
        return this.countQuery(sql, params).intValue();

    }

    public List<TaskInstance> list(List<Param> params, int rows) throws Exception {
        String sql = "select * from task_instance a where 1 = 1";
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(1, rows, params, new HashMap<>()), "a");
        Map<String, Object> paramMap = this.getQueryBuilder().getParam(params);
        return this.list(TaskInstance.class, sql, paramMap);
    }

}
