package com.bocloud.cop.repository;

import com.bocloud.cop.entity.TaskNodeLog;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 任务节点日志DAO接口实现
 * 
 * @author weiwei
 */
@Repository("taskNodeLogRepository")
public class TaskNodeLogRepository extends BasicGenericDao<TaskNodeLog, Long> {

    public TaskNodeLogRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public boolean deleteByInstanceId(Long instanceId) {
        String sql = "delete from task_node_log where instance_id = :instanceId";
        return this.execute(sql, MapTools.simpleMap("instanceId", instanceId)) > 0;
    }

    public List<TaskNodeLog> list(Long nodeId) {
        String sql = "select * from task_node_log where node_id = :nodeId order by id asc";
        return this.list(TaskNodeLog.class, sql, MapTools.simpleMap("nodeId", nodeId));
    }

    public TaskNodeLog get(Long nodeId, Integer priority) {
        String sql = "select * from task_node_log where node_id = :nodeId and priority = :priority";
        Map<String, Object> params = MapTools.simpleMap("nodeId", nodeId);
        params.put("priority", priority);
        List<TaskNodeLog> list = this.list(TaskNodeLog.class, sql, params);
        return ListTool.isEmpty(list) ? null : list.get(0);
    }

    public TaskNodeLog getLast(Long nodeId) {
        String sql = "select * from task_node_log where node_id = :nodeId order by id desc";
        List<TaskNodeLog> list = this.list(TaskNodeLog.class, sql, MapTools.simpleMap("nodeId", nodeId));
        return ListTool.isEmpty(list) ? null : list.get(0);
    }

    public List<TaskNodeLog> listByInstances(List<Long> instanceIds) {
        String sql = "select * from task_node_log a where a.instance_id in (:instanceIds)";
        Map<String, Object> params = MapTools.simpleMap("instanceIds", instanceIds);
        return this.list(TaskNodeLog.class, sql, params);
    }

    public boolean deleteByInstanceIds(List<Long> instanceIds) {
        String sql = "delete from task_node_log where instance_id in (:instanceIds)";
        Map<String, Object> params = MapTools.simpleMap("instanceIds", instanceIds);
        return this.execute(sql, params) > 0;
    }

    public boolean deleteByNodeId(Long nodeId) {
        String sql = "delete from task_node_log where node_id = :nodeId";
        return this.execute(sql, MapTools.simpleMap("nodeId", nodeId)) > 0;
    }

}
