package com.bocloud.cop.repository;

import com.bocloud.cop.entity.TaskNode;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 任务节点DAO接口实现类
 * 
 * @author wei wei
 * @version 1.0
 * @since 2017.07.07
 */
@Repository("taskNodeRepository")
public class TaskNodeRepository extends JdbcGenericDao<TaskNode, Long> {

    public TaskNodeRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<TaskNode> list(int page, int rows, List<Param> params, Map<String, String> sorter) {
        String sql = "select * from task_node a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(TaskNode.class, sql, this.getQueryBuilder().getParam(params));
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from task_node a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public boolean remove(Long id, Long userId) {
        String sql =
            "update task_node set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<TaskNode> listByInstance(Long instanceId) {
        String sql =
            "select * from task_node a where a.is_deleted = 0 and a.instance_id = :instanceId order by a.gmt_start asc";
        return this.list(TaskNode.class, sql, MapTools.simpleMap("instanceId", instanceId));
    }

    public TaskNode query(Long id) {
        String sql = "select * from task_node a where a.id = :id";
        List<TaskNode> list = this.list(TaskNode.class, sql, MapTools.simpleMap("id", id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean updateStatus(Long id, String status, Long userId) {
        String sql =
            "update task_node set status = :status, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

    public boolean removeByInstance(Long instanceId, Long userId) {
        String sql =
            "update task_node set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where instance_id = :instanceId";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public TaskNode getByNodeId(String nodeId, Long instanceId) {
        String sql = "select a.* from task_node a where a.node_id = :nodeId and a.instance_id = :instanceId";
        Map<String, Object> paramMap = MapTools.simpleMap("nodeId", nodeId);
        paramMap.put("instanceId", instanceId);
        List<TaskNode> list = this.list(TaskNode.class, sql, paramMap);
        return list.isEmpty() ? null : list.get(0);
    }

    public List<TaskNode> listByTarget(Long target, Long instanceId) {
        String sql = "select a.* from task_node a where a.is_deleted = 0 and a.instance_id = :instanceId"
            + " and a.id in (select b.source from task_node_relation b where b.instance_id = a.instance_id and target = :target)";
        Map<String, Object> paramMap = MapTools.simpleMap("target", target);
        paramMap.put("instanceId", instanceId);
        return this.list(TaskNode.class, sql, paramMap);
    }

    public List<TaskNode> listByStatus(Long instanceId, String status) {
        String sql =
            "select a.* from task_node a where a.is_deleted = 0 and a.status = :status and a.instance_id = :instanceId";
        Map<String, Object> paramMap = MapTools.simpleMap("status", status);
        paramMap.put("instanceId", instanceId);
        return this.list(TaskNode.class, sql, paramMap);
    }

    public TaskNode getByUuid(String nodeId) {
        String sql = "select a.* from task_node a where a.uuid = :nodeId";
        List<TaskNode> list = this.list(TaskNode.class, sql, MapTools.simpleMap("nodeId", nodeId));
        return list.isEmpty() ? null : list.get(0);
    }

    public List<TaskNode> listByInstances(List<Long> instanceIds) {
        String sql = "select * from task_node a where a.instance_id in (:instanceIds)";
        Map<String, Object> params = MapTools.simpleMap("instanceIds", instanceIds);
        return this.list(TaskNode.class, sql, params);
    }

    public boolean deleteByInstanceIds(List<Long> instanceIds) {
        String sql = "delete from task_node where instance_id in (:instanceIds)";
        Map<String, Object> params = MapTools.simpleMap("instanceIds", instanceIds);
        return this.execute(sql, params) > 0;
    }

    public boolean updateFailedStatus(Long id, String status, Long userId) throws Exception {
        String sql =
            "update task_node set output = null, status = :status, gmt_modify = :gmtModify, mender_id = :menderId where id = :id and status='FAILED'";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }
}
