package com.bocloud.cop.repository;

import com.bocloud.cop.entity.TaskPlatformAlert;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 任务实例平台告警dao
 *
 * @author majiali
 * @version 1.0
 * @since 2022.08.24
 */
@Repository("taskPlatformAlertRepository")
public class TaskPlatformAlertRepository extends JdbcGenericDao<TaskPlatformAlert, Long> {

    public TaskPlatformAlertRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public TaskPlatformAlert query(Long id) throws Exception {
        String sql =
                "select a.* from task_platform_alert a where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<TaskPlatformAlert> list = this.list(TaskPlatformAlert.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean remove(Long id, Long userId) throws Exception {
        String sql =
                "update task_platform_alert set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<TaskPlatformAlert> list(Pager pager) throws Exception {
        String sql =
                "select a.* from task_platform_alert a  where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(pager.getParams());
        return this.list(TaskPlatformAlert.class, sql, param);
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from task_platform_alert a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public List<TaskPlatformAlert> getByPlatformConfigId(Long platformConfigId) throws Exception {
        String sql =
                "select a.* from task_platform_alert a where a.is_deleted = 0 and platform_config_id = :platformConfigId";
        Map<String, Object> params = MapTools.simpleMap("platformConfigId", platformConfigId);
        return this.list(TaskPlatformAlert.class, sql, params);
    }

    public List<TaskPlatformAlert> getAll() throws Exception {
        String sql =
                "select a.* from task_platform_alert a where a.is_deleted = 0 and a.status != 'INVALID' order by a.gmt_modify desc";
        return this.list(TaskPlatformAlert.class, sql);
    }

    /**
     * @param platformConfigId
     * @param userId
     * @Description:将告警设置未失效状态，并删除
     * @Author majiali
     * @CreateTime 2022/9/23 9:41
     */
    public boolean updateInvalidAndDel(Long platformConfigId, Long userId) throws Exception {
        String sql =
                "update task_platform_alert set is_deleted = 1, status = 'INVALID', gmt_modify = :gmtModify, mender_id = :menderId where platform_config_id = :platformConfigId";
        Map<String, Object> params = MapTools.simpleMap("platformConfigId", platformConfigId);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<TaskPlatformAlert> getByTime(String currTime) throws Exception {
        String sql =
                "select b.* from task_platform_alert b,task_platform_config a where b.platform_config_id=a.id  and b.status in ('CREATED','PROCESSED') and b.is_deleted=0 and a.is_deleted = 0 and a.is_opened =1 ";
        Map<String, Object> params = new HashMap<>();
        if (null != currTime) {
            sql += " and a.start_time <= :currTime ";
            sql += " and a.end_time >= :currTime ";
            params.put("currTime", currTime);
        }
        sql += " order by b.gmt_create ";
        return this.list(TaskPlatformAlert.class, sql, params);
    }

}
