package com.bocloud.cop.repository;

import com.bocloud.cop.entity.TaskPlatformConfig;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 任务实例平台dao
 *
 * @author majiali
 * @version 1.0
 * @since 2022.08.24
 */
@Repository("taskPlatformConfigRepository")
public class TaskPlatformConfigRepository extends JdbcGenericDao<TaskPlatformConfig, Long> {

    public TaskPlatformConfigRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public TaskPlatformConfig query(Long id) throws Exception {
        String sql =
                "select a.* from task_platform_config a where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<TaskPlatformConfig> list = this.list(TaskPlatformConfig.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean remove(Long id, Long userId) throws Exception {
        String sql =
                "update task_platform_config set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<TaskPlatformConfig> getByPlatform(String platform) throws Exception {
        String sql = "select * from task_platform_config a where a.is_deleted = 0 and a.platform = :platform";
        Map<String, Object> params = MapTools.simpleMap("platform", platform);
        return this.list(TaskPlatformConfig.class, sql, params);
    }

    public List<TaskPlatformConfig> list(Pager pager) throws Exception {
        String sql =
                "select a.* from task_platform_config a  where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(pager.getParams());
        return this.list(TaskPlatformConfig.class, sql, param);
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from task_platform_config a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public boolean updateOpened(Long id, Boolean opened, Long userId) throws Exception {
        String sql =
                "update task_platform_config set is_opened = :opened, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("opened", opened);
        return this.execute(sql, params) > 0;
    }

    public List<TaskPlatformConfig> getAll() throws Exception {
        String sql =
                "select a.* from task_platform_config a where a.is_deleted = 0  and is_opened =1 ";
        return this.list(TaskPlatformConfig.class, sql);
    }


    public List<TaskPlatformConfig> getByTime(String currTime) throws Exception {
        String sql =
                "select a.* from task_platform_config a where a.is_deleted = 0 and is_opened =1 ";
        Map<String, Object> params = new HashMap<>();
        if (null != currTime) {
            sql += " and start_time <= :currTime ";
            sql += " and end_time >= :currTime ";
            params.put("currTime", currTime);
        }

        sql += " order by start_time ";
        return this.list(TaskPlatformConfig.class, sql,params);
    }

    public TaskPlatformConfig exists(String name) throws Exception {
        String sql = "select * from task_platform_config a where a.name = :name and a.is_deleted = 0";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        List<TaskPlatformConfig> list = this.list(TaskPlatformConfig.class, sql, paramMap);
        return list.isEmpty() ? null : list.get(0);
    }

}
