package com.bocloud.cop.repository;

import com.bocloud.cop.entity.TaskPlatform;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 任务实例平台告警dao
 *
 * @author majiali
 * @version 1.0
 * @since 2022.08.24
 */
@Repository("taskPlatformRepository")
public class TaskPlatformRepository extends JdbcGenericDao<TaskPlatform, Long> {

    public TaskPlatformRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<TaskPlatform> listAll() throws Exception {
        String sql =
                "select a.* from task_platform a  where a.is_deleted = 0";
        return this.list(TaskPlatform.class, sql);
    }

    public List<TaskPlatform> list(Pager pager) throws Exception {
        String sql =
                "select a.* from task_platform a  where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(pager.getParams());
        return this.list(TaskPlatform.class, sql, param);
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from task_platform a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public boolean remove(Long id, Long userId) throws Exception {
        String sql =
                "update task_platform set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public TaskPlatform exists(String name) throws Exception {
        String sql = "select * from task_platform a where a.name = :name and a.is_deleted = 0";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        List<TaskPlatform> list = this.list(TaskPlatform.class, sql, paramMap);
        return list.isEmpty() ? null : list.get(0);
    }
    public TaskPlatform query(Long id) throws Exception {
        String sql = "select a.* from task_platform a where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<TaskPlatform> list = this.list(TaskPlatform.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

}
