package com.bocloud.cop.repository;


import com.bocloud.cop.entity.TaskTreatment;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author huliwei
 * @date 2023/4/23
 */
@Repository("taskTreatmentRepository")
public class TaskTreatmentRepository extends JdbcGenericDao<TaskTreatment, Long> {

    public TaskTreatmentRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<TaskTreatment> list(Pager pager) throws Exception {
        String sql = "select a.* from task_treatment a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(TaskTreatment.class, sql);
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from task_treatment a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public boolean exists(String name) throws Exception {
        String sql = "select * from task_treatment a where a.is_deleted = 0 and a.name = :name";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        List<TaskTreatment> list = this.list(TaskTreatment.class, sql, params);
        return !list.isEmpty();
    }

    public boolean remove(Long id, Long userId) throws Exception {
        String sql = "update task_treatment set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public TaskTreatment query(Long id) throws Exception {
        String sql = "select a.* from task_treatment a where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<TaskTreatment> list = this.list(TaskTreatment.class, sql, paramMap);
        return list.isEmpty() ? null : list.get(0);
    }

    public List<TaskTreatment> listAll() throws Exception {
        String sql = "select a.* from task_treatment a where a.is_deleted = 0";
        List<TaskTreatment> list = this.list(TaskTreatment.class, sql);
        return list;
    }

}
