package com.bocloud.cop.repository;

import com.bocloud.cop.entity.TaskWorker;
import com.bocloud.cop.entity.TaskWorker.Category;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Repository("taskWorkerRepository")
public class TaskWorkerRepository extends JdbcGenericDao<TaskWorker, Long> {

    public TaskWorkerRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<TaskWorker> listByWorkers(List<Long> ids) throws Exception {
        String sql = "select * from task_worker a where a.is_deleted = 0 and a.id in(:ids)";
        Map<String, Object> paramMap = MapTools.simpleMap("ids", ids);
        return this.list(TaskWorker.class, sql, paramMap);
    }

    public List<TaskWorker> list(Pager pager) throws Exception {
        String sql =
            "select a.*,b.name group_name from task_worker a left join network_group b on a.group_id = b.id where a.is_deleted is not null";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(TaskWorker.class, sql, this.getQueryBuilder().getParam(pager.getParams()));
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from task_worker where 1 = 1";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "");
        if (sql.toLowerCase().contains("category = 'SaltMaster'".toLowerCase())) {
            sql += " and is_deleted = 0";
        }
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public List<TaskWorker> list() throws Exception {
        String sql =
            "select a.*,b.name group_name from task_worker a left join network_group b on a.group_id = b.id where a.is_deleted is not null";
        return this.list(TaskWorker.class, sql);
    }

    public TaskWorker query(Long id) {
        String sql =
            "select a.*,b.name group_name from task_worker a left join network_group b on a.group_id = b.id  where a.is_deleted = 0 and a.id = :id";
        List<TaskWorker> list = this.list(TaskWorker.class, sql, MapTools.simpleMap("id", id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql =
            "update task_worker set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<TaskWorker> listWorkers(String category) throws Exception {
        String sql = "select * from task_worker a where a.is_deleted = 0 and a.category = :category";
        return this.list(TaskWorker.class, sql, MapTools.simpleMap("category", category));
    }

    public boolean delete(String instance, String category) throws Exception {
        String sql =
            "update task_worker set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where instance = :instance and category = :category";
        Map<String, Object> params = MapTools.simpleMap("instance", instance);
        params.put("category", category);
        params.put("gmtModify", new Date());
        params.put("menderId", 1L);
        return this.execute(sql, params) > 0;
    }

    /**
     * 查询worker实例是否存在与数据库中
     *
     * @param instance
     *            实例地址 包括IP:PORT
     * @return 是否存在 true：存在，false : 不存在
     */
    public TaskWorker exist(String instance, String category) throws Exception {
        String sql = "select * from task_worker a where a.instance = :instance and a.category = :category";
        if (Category.SaltMaster.name().equalsIgnoreCase(category)) {
            sql += " and a.is_deleted = 0";
        }
        Map<String, Object> paramMap = MapTools.simpleMap("instance", instance);
        paramMap.put("category", category);
        List<TaskWorker> list = this.list(TaskWorker.class, sql, paramMap);
        return list.isEmpty() ? null : list.get(0);
    }

    public TaskWorker getWorker(String instance, String category) throws Exception {
        String sql =
            "select * from task_worker a where a.is_deleted = 0 and a.instance = :instance and a.category = :category";
        Map<String, Object> paramMap = MapTools.simpleMap("instance", instance);
        paramMap.put("category", category);
        List<TaskWorker> list = this.list(TaskWorker.class, sql, paramMap);
        return list.isEmpty() ? null : list.get(0);
    }

    public List<TaskWorker> listGenerals(Long groupId, String category, Boolean isExclude) throws Exception {
        String sql =
            "select * from task_worker a where a.is_deleted = 0 and a.is_super = false and a.group_id = :groupId and a.category = :category and a.is_exclude = :isExclude ";
        Map<String, Object> paramMap = MapTools.simpleMap("groupId", groupId);
        paramMap.put("category", category);
        paramMap.put("isExclude", isExclude);
        return this.list(TaskWorker.class, sql, paramMap);
    }

    public List<TaskWorker> listSuper(String category, Boolean isExclude) throws Exception {
        String sql =
            "select * from task_worker a where a.is_deleted = 0 and a.is_super = true and a.category = :category and a.is_exclude = :isExclude ";
        Map<String, Object> paramMap = MapTools.simpleMap("category", category);
        paramMap.put("isExclude", isExclude);
        return this.list(TaskWorker.class, sql, paramMap);
    }

    public List<TaskWorker> listGenerals(Long groupId, String category) throws Exception {
        String sql =
            "select * from task_worker a where a.is_deleted = 0 and a.is_super = false and a.group_id = :groupId and a.category = :category";
        Map<String, Object> paramMap = MapTools.simpleMap("groupId", groupId);
        paramMap.put("category", category);
        return this.list(TaskWorker.class, sql, paramMap);
    }

    public List<TaskWorker> listSupers() throws Exception {
        String sql = "select * from task_worker where is_deleted = 0 and is_super = true";
        return this.list(TaskWorker.class, sql);
    }

    public List<TaskWorker> listSupers(String category) throws Exception {
        String sql =
            "select * from task_worker a where a.is_deleted = 0 and a.is_super = true and a.category = :category";
        return this.list(TaskWorker.class, sql, MapTools.simpleMap("category", category));
    }

    public List<TaskWorker> listByGroupId(Long groupId) {
        String sql =
            "select * from task_worker a where a.is_deleted = 0 and a.is_super = false and a.group_id = :groupId";
        return this.list(TaskWorker.class, sql, MapTools.simpleMap("groupId", groupId));
    }

    public boolean update(Long id,String remark,Long userId) throws Exception {
        String sql =
                "update task_worker set remark=:remark, gmt_modify = :gmtModify, mender_id = :menderId where id=:id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("remark", remark);
        return this.execute(sql, params) > 0;
    }

}
