package com.bocloud.cop.repository.backup;

import com.bocloud.cop.entity.backup.TaskBackupScene;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author huliwei
 * @date 2022/10/19
 */
@Repository("taskBackupSceneRepository")
public class TaskBackupSceneRepository extends BasicGenericDao<TaskBackupScene, Long> {
    public TaskBackupSceneRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql = "update task_backup_scene set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id ";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public TaskBackupScene query(Long id) throws Exception {
        String sql = " select a.* from task_backup_scene a where a.id = :id and a.is_deleted=0 ";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<TaskBackupScene> taskBackupScenes = this.list(TaskBackupScene.class, sql, params);
        if (CollectionUtils.isEmpty(taskBackupScenes)) {
            return null;
        }
        return taskBackupScenes.get(0);
    }

    public int countSameNameOrCode(String name, String code) throws Exception {
        String sql = "select count(1) from task_backup_scene where is_deleted=0 ";
        if (null != code) {
            sql += " and code = :code ";
        } else if (null != name) {
            sql += " and name = :name ";
        }
        Map<String, Object> params = MapTools.simpleMap("code", code);
        params.put("name", name);
        return this.countQuery(sql, params).intValue();
    }

    public List<TaskBackupScene> list(Pager pager) throws Exception {
        String sql = "select a.* from task_backup_scene a where 1=1 and a.is_deleted=0 ";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        Map<String, Object> paramMap = this.getQueryBuilder().getParam(pager.getParams());
        List<TaskBackupScene> list = this.list(TaskBackupScene.class, sql, paramMap);

        return list;
    }

    public List<TaskBackupScene> list(List<Param> params, Map<String, String> sorter) throws Exception {

        String sql = "select a.* from task_backup_scene a where 1=1 and is_deleted=0 ";
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        List<TaskBackupScene> list = this.list(TaskBackupScene.class, sql, param);
        return list;
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from task_backup_scene a where 1=1 and a.is_deleted=0 ";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }
}
