package com.bocloud.cop.repository.compare;


import com.bocloud.cop.entity.compare.StandardDocument;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Author liuyuanyuan
 * @Date 2022-10-292 13:46:31
 * @Description:
 */
@Repository
public class StandardDocumentRepository extends BasicGenericDao<StandardDocument, Long> {

    public StandardDocumentRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<StandardDocument> queryByVersionIds(List<Long> versionIds) throws Exception {
        String sql = "select a.*,b.name as version_name, b.code as version_code from standard_document a left join standard_document_version b on a.version_id = b.id where 1=1 and a.is_deleted=0 and a.version_id in (:versionId) " ;
        Map<String, Object> paramMap = MapTools.simpleMap("versionId", versionIds);
        return this.list(StandardDocument.class, sql, paramMap);
    }

    public List<StandardDocument> queryByVersionId(Long versionId) throws Exception {
        String sql = "select a.*,b.name as version_name from standard_document a left join standard_document_version b on a.version_id = b.id where 1=1 and a.is_deleted=0 and a.version_id = :versionId " ;
        Map<String, Object> paramMap = MapTools.simpleMap("versionId", versionId);
        return this.list(StandardDocument.class, sql, paramMap);
    }

    public List<StandardDocument> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.*,b.name as version_name from standard_document a left join standard_document_version b on a.version_id = b.id where 1=1 and a.is_deleted=0 " ;
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = this.getQueryBuilder().getParam(params);
        return this.list(StandardDocument.class, sql, paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.id,a.name from standard_document a where 1=1 and a.is_deleted=0 " ;
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = this.getQueryBuilder().getParam(params);
        List<StandardDocument> list = this.list(StandardDocument.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (StandardDocument standardDocument : list) {
            beans.add(new SimpleBean(standardDocument.getId(), standardDocument.getName()));
        }
        return beans;
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from standard_document a where 1=1 and a.is_deleted=0 " ;
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public StandardDocument query(Long id) throws Exception {
        String sql = "select a.*, b.name as version_name, b.code as version_code from standard_document a left join standard_document_version b on a.version_id = b.id where a.id = :id and a.is_deleted = 0 " ;
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<StandardDocument> list = this.list(StandardDocument.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public List<StandardDocument> queryByIds(List<Long> ids) throws Exception {
        String sql = "select * from standard_document where id in (:ids) and is_deleted = 0" ;
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        return this.list(StandardDocument.class, sql, params);
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql = "update standard_document set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id" ;
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean exists(String name, Long versionId) throws Exception {
        String sql = "select * from standard_document where is_deleted = 0 and name = :name and version_id = :versionId " ;
        Map<String, Object> params = MapTools.simpleMap("name", name);
        params.put("versionId", versionId);
        List<StandardDocument> list = this.list(StandardDocument.class, sql, params);
        return !list.isEmpty();
    }

}