package com.bocloud.cop.repository.inspection;

import com.bocloud.cop.entity.inspection.InspectRecordWarning;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

@Repository("inspectRecordWarningRepository")
public class InspectRecordWarningRepository extends JdbcGenericDao<InspectRecordWarning, Long> {

    public InspectRecordWarningRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<InspectRecordWarning> findAllByRecordId(Long recordId) {
        String sql = "select a.* from inspect_record_warning a where a.record_id =:recordId   and a.is_deleted = 0";
        return this.list(InspectRecordWarning.class, sql, MapTools.simpleMap("recordId", recordId));
    }

    public int count(Long recordId, List<Param> params) {
        String sql = "select count(1) from inspect_record_warning a where a.is_deleted = 0 and record_id =:recordId";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        param.put("recordId", recordId);
        return this.countQuery(sql, param).intValue();
    }

    public List<InspectRecordWarning> list(Long recordId, Pager pager) {
        String sql = "select a.* from inspect_record_warning a  where a.is_deleted = 0 and record_id =:recordId";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(pager.getParams());
        param.put("recordId", recordId);
        return this.list(InspectRecordWarning.class, sql, param);
    }

}
