package com.bocloud.cop.repository.inspection;

import com.bocloud.cop.entity.inspection.InspectTiming;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

@Repository("inspectTimingRepository")
public class InspectTimingRepository extends JdbcGenericDao<InspectTiming, Long> {

    public InspectTimingRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public Integer delete(Long id, Long menderId) {
        String sql =
            "update inspect_timing set is_deleted = 1 , mender_id = :menderId ,gmt_modify = now() where id =:id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("menderId", menderId);
        return this.execute(sql, params);
    }

    public List<InspectTiming> findAllByRunning(Boolean running) {
        String sql = "select a.* from inspect_timing a where a.running =:running  and a.is_deleted = 0";
        return this.list(InspectTiming.class, sql, MapTools.simpleMap("running", running));
    }

    public Integer changeRunning(Long id, Long menderId, Boolean running) {
        String sql =
            "update inspect_timing set running = :running , mender_id = :menderId ,gmt_modify = now() where id =:id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("menderId", menderId);
        params.put("running", running);
        return this.execute(sql, params);
    }

    public Long countAllByRoutineId(Long inspectId) {
        String sql = "select count(a.id) from inspect_timing a where a.inspect_id= :inspectId and a.is_deleted = 0";
        return this.countQuery(sql, MapTools.simpleMap("inspectId", inspectId));
    }

    public InspectTiming findOne(Long id) {
        String sql = "select a.* from inspect_timing a where a.id = :id and  a.is_deleted = 0";
        return this.list(InspectTiming.class, sql, MapTools.simpleMap("id", id)).stream().findFirst().orElse(null);
    }

    public List<InspectTiming> list(Pager pager) {
        String sql = "select a.* from inspect_timing a  where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(InspectTiming.class, sql);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from inspect_timing a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public boolean exists(String name, String type) {
        String sql = "select id from inspect_timing a where a.is_deleted = 0 and a.name = :name and a.type = :type";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        params.put("type", type);
        List<InspectTiming> list = this.list(InspectTiming.class, sql, params);
        return !list.isEmpty();
    }

}
