package com.bocloud.cop.repository.module;

import java.util.Date;
import java.util.List;
import java.util.Map;

import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import com.bocloud.cop.entity.module.NodeModuleGroup;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;

/**
 * 编排节点类型分组DAO接口实现类
 */
@Repository("nodeModuleGroupRepository")
public class NodeModuleGroupRepository extends JdbcGenericDao<NodeModuleGroup, Long> {

    public NodeModuleGroupRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<NodeModuleGroup> list(Long parentId) {
        if (null == parentId) {
            parentId = 0L;
        }
        String sql = "select * from task_node_module_group a where a.is_deleted = 0 and a.parent_id = :parentId";
        Map<String, Object> params = MapTools.simpleMap("parentId", parentId);
        return this.list(NodeModuleGroup.class, sql, params);
    }

    public boolean delete(Long id, Long userId) {
        String sql =
            "update task_node_module_group set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public NodeModuleGroup query(Long id) {
        String sql = "select * from task_node_module_group a where is_deleted = 0 and id = :id";
        List<NodeModuleGroup> list = this.list(NodeModuleGroup.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean exists(String name, Long parentId) {
        String sql =
            "select id from task_node_module_group where is_deleted = 0 and name = :name and parent_id = :parentId";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        params.put("parentId", parentId);
        List<NodeModuleGroup> list = this.list(NodeModuleGroup.class, sql, params);
        return !list.isEmpty();
    }

    public NodeModuleGroup getGroup(String code) {
        String sql = "select * from task_node_module_group a where is_deleted = 0 and code = :code";
        List<NodeModuleGroup> list = this.list(NodeModuleGroup.class, sql, MapTools.simpleMap("code", code));
        return list.isEmpty() ? null : list.get(0);
    }

}
