package com.bocloud.cop.repository.patch;

import com.bocloud.cop.entity.patch.PatchDeploy;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Repository("patchDeployRepository")
public class PatchDeployRepository extends JdbcGenericDao<PatchDeploy, Long> {

    public PatchDeployRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from (select p.*,c.name patch_name from patch_deploy p "
            + "left join patch  c on p.patch_id = c.id where p.is_deleted = 0 ) a where 1=1 ";
        sql = this.getQueryBuilder().build(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public List<PatchDeploy> list(Pager pager) {
        String sql = "select * from (select p.*,c.name patch_name from patch_deploy p "
            + "left join patch  c on p.patch_id = c.id where p.is_deleted = 0 ) a where 1=1 ";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(PatchDeploy.class, sql);
    }

    public boolean remove(Long id, Long userId) {
        String sql = "update patch_deploy set is_deleted = 1,mender_id = :userId,gmt_modify = :time where id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap(Common.ID, id);
        paramMap.put("time", new Date());
        paramMap.put("userId", userId);
        return this.execute(sql, paramMap) > 0;
    }

    public PatchDeploy detail(Long id) {
        String sql = "select a.* from patch_deploy a where a.is_deleted = 0 and a.id = :id ";
        List<PatchDeploy> list = this.list(PatchDeploy.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean run(Long id, Long userId) {
        String sql =
            "UPDATE patch_deploy set status = 'RUNNING',mender_id = :userId,gmt_modify = :gmtModify where id = :id ";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("userId", userId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public boolean timing(Date date, Long userId, Long id) {
        String sql =
            "UPDATE patch_deploy set timing = :time,mender_id = :userId,gmt_modify = :gmtModify,is_timing = 0 where id = :id ";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("time", date);
        params.put("userId", userId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public boolean cancel(Long id, Long userId) {
        String sql =
            "update patch_deploy set is_timing = 1,mender_id = :userId,gmt_modify = :gmtModify,timing = :timing where id = :id ";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("userId", userId);
        params.put("gmtModify", new Date());
        params.put("timing", null);
        return this.execute(sql, params) > 0;
    }

    public PatchDeploy selectDeployById(Long id) {
        String sql = "select * from patch_deploy a where a.is_deleted = 0 and a.id = :id ";
        return this.list(PatchDeploy.class, sql, MapTools.simpleMap(Common.ID, id)).stream().findFirst().orElse(null);
    }

    public boolean updateInstanceId(Long instanceId, Long id) {
        String sql = "update patch_deploy set instance_id = :instanceId where id = :id ";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("instanceId", instanceId);
        return this.execute(sql, params) > 0;
    }

    public PatchDeploy selectInstanceId(Long instanceId) {
        String sql = "select * from patch_deploy where instance_id = :instanceId";
        List<PatchDeploy> list = this.list(PatchDeploy.class, sql, MapTools.simpleMap("instanceId", instanceId));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean updateStatus(Long instanceId) {
        String sql = "update patch_deploy set status = 'COMPLETE' where instance_id = :instanceId";
        return this.execute(sql, MapTools.simpleMap("instanceId", instanceId)) > 0;
    }

    public boolean exist(String name) {
        String sql = "select count(1) from patch_deploy where name = :name and is_deleted = 0";
        return this.countQuery(sql, MapTools.simpleMap("name", name)).intValue() > 0;
    }

    public int count(Long templateId) {
        String sql = "select count(1) from patch_deploy a where a.is_deleted = 0 and execute_template = :templateId";
        return this.countQuery(sql, MapTools.simpleMap("templateId", templateId)).intValue();
    }

    public boolean updateStatus(Long id, String status, Long userId) {
        String sql =
            "update patch_deploy set status = :status, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

    public List<PatchDeploy> listByPatchId(Long patchId) {
        String sql = "select a.* from patch_deploy a where a.patch_id =:patchId and a.is_deleted = 0 ";
        return this.list(PatchDeploy.class, sql, MapTools.simpleMap("patchId", patchId));
    }

}
