package com.bocloud.cop.core;

import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
public class WorkerStrategyCache {

    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private LockFactory lockFactory;

    /**
     * 添加worker 策略信息缓存
     *
     * @param key
     *            key
     * @param value
     *            任务执行数
     */
    public void addCache(String key, String value) {
        try (AutoCloseLock lock = lockFactory.getACLock(key)) {
            Assert.isTrue(lock.acquire(60, TimeUnit.SECONDS), "请求超时");
            redisTemplate.opsForValue().set(key, value);
        } catch (Exception e) {
            log.error("配置worker策略相关缓存失败:", e);
        }
    }

    /**
     * 删除worker 策略相关信息缓存
     *
     * @param key
     *            key
     */
    public void delCache(String key) {
        try (AutoCloseLock lock = lockFactory.getACLock(key)) {
            Assert.isTrue(lock.acquire(60, TimeUnit.SECONDS), "请求超时");
            redisTemplate.delete(key);
        } catch (Exception e) {
            log.error("删除worker策略相关缓存失败:", e);
        }
    }

}
