package com.bocloud.cop.intf;

import com.bocloud.cop.entity.Software;
import com.bocloud.cop.model.SoftwareBean;
import com.megatron.common.model.*;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;

import java.util.List;

/**
 * 软件抽象Service接口
 * 
 * @author wei wei
 * @version 1.0
 * @since 2016.08.16
 */
public interface SoftwareService {

    /**
     * 查询软件列表
     * 
     * @param pager
     * @return
     */
    public GeneralResult<GridBean<Software>> list(Pager pager);

    /**
     * （自服务）查询分配的文件
     * 
     * @param pager
     * @param context
     * @return
     */
    public GeneralResult<GridBean<Software>> listAllot(Pager pager, RequestContext context);

    public GeneralResult<GridBean<SimpleBean>> simple(Pager pager);

    /**
     * 创建软件
     * 
     * @param software
     * @return
     */
    public GeneralResult<Software> create(Software software);

    /**
     * 更新软件
     * 
     * @param bean
     * @param userId
     * @return
     */
    public GeneralResult<SoftwareBean> modify(SoftwareBean bean, Long userId);

    /**
     * 删除软件
     * 
     * @param id
     * @param userId
     * @return
     */
    public GeneralResult<String> remove(Long id, Long userId);

    /**
     * 查看软件详情
     * 
     * @param id
     * @return
     */
    public GeneralResult<Software> detail(Long id);

    /**
     * 判断是否存在
     * 
     * @param name
     * @return
     */
    public GeneralResult<Software> exists(String name);

    /**
     * 判断文件是否存在
     * 
     * @param name
     * @return
     */
    public GeneralResult<Software> fileNotExists(String name);

    public GeneralResult<Void> updateStatus(Long id, String status, Long userId);

    public void compressDownload(List<String> paths, HttpServletResponse response);

    public void singleDownload(String path, HttpServletRequest request, HttpServletResponse response);

    /**
     * 分配
     *
     * @param tenantId
     * @param softwareIds
     * @return
     */
    public GeneralResult<Void> allot(Long tenantId, List<Long> softwareIds);

    /**
     * 分配租户列表
     *
     * @param tenantId
     * @return
     */
    public GeneralResult<List<Software>> listAllotSoftwares(Long tenantId);
}
