package com.bocloud.cop.intf;

import java.io.IOException;
import java.util.List;
import java.util.Map;

import org.springframework.web.multipart.MultipartFile;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.NetworkSegment;
import com.bocloud.cop.entity.Task;
import com.bocloud.cop.entity.TaskHost;
import com.bocloud.cop.model.TaskHostBean;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;

/**
 * 任务主机service接口
 * 
 * @author weiwei
 * @version 1.0
 * @since 2017.01.17
 *
 */
public interface TaskHostService {

    /**
     * 创建
     * 
     * @param host
     * @return
     */
    public GeneralResult<Void> create(TaskHost host, RequestContext context);

    /**
     * 修改
     * 
     * @param bean
     * @return
     */
    public GeneralResult<Void> modify(TaskHostBean bean, RequestContext context);

    /**
     * 删除
     * 
     * @param id
     * @return
     */
    public GeneralResult<Void> remove(Long id);

    /**
     * 详情
     * 
     * @param id
     * @return
     */
    public GeneralResult<TaskHost> detail(Long id);

    GeneralResult<GridBean<TaskHost>> list(Pager pager, RequestContext context);

    GeneralResult<Void> configure(Long hostId, Long segmentId);

    GeneralResult<Task> configHostAllow(Long hostId);

    /**
     * 同步IMS服务主机数据
     */
    void syncHostFromIMS(RequestContext context, String category);

    /**
     * 批量导入
     * 
     * @param excel
     * @param context
     * @return
     * @throws IOException
     */
    GeneralResult<Void> upload(MultipartFile excel, RequestContext context) throws IOException;

    /**
     * 导入模板下载
     * 
     * @param paramsMap
     * @return
     */
    GeneralResult<?> download(Map<String, Object> paramsMap);

    /**
     * 根据通知消息更新taskhost
     * 
     * @param action
     * @param data
     */
    void modifyByMessage(String action, JSONObject data, String category);

    /**
     * 同步taskHost网段
     */
    void syncSegment();

    GeneralResult<NetworkSegment> hasWorker(String host);

    /**
     * 权限配置
     * 
     * @param ids
     *            主机资源id
     * @param userGroupIds
     *            用户id
     * @return 配置结果
     */
    GeneralResult<Void> configPermissionRelation(List<Long> ids, List<Long> userGroupIds);

    /**
     * 关联账号
     * 
     * @param ids
     *            主机资源id
     * @param identificationIds
     *            认证账号id
     * @return 配置结果
     */
    GeneralResult<Void> configIdentificationRelation(List<Long> ids, List<Long> identificationIds);

    /**
     * 查询主机资源列表
     * 
     * @param pager
     * @return
     */
    GeneralResult<GridBean<TaskHost>> listForManage(Pager pager);

    /**
     * 查询主机信息以及认证账号信息
     * 
     * @param pager
     *            分页信息
     * @param context
     * @return
     */
    GeneralResult<GridBean<TaskHost>> listTaskHostAndIdentification(Pager pager, RequestContext context);

    /**
     * 同步saltMinion信息
     * @param context 操作人
     * @param saltId saltMaster id
     * @return 操作结果
     */
    GeneralResult<Void> syncSaltMinions(RequestContext context, Long saltId);

}
