package com.bocloud.cop.intf;

import java.util.Date;
import java.util.List;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.Task;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.model.TaskBean;
import com.bocloud.cop.model.params.TaskTransferParams;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;

public interface TaskService {

    /**
     * 查询任务列表
     * 
     * @param pager
     * @return
     */
    public GeneralResult<GridBean<Task>> list(Pager pager, RequestContext context);

    /**
     * 创建任务
     * 
     * @param task
     * @param context
     * @return
     */
    public GeneralResult<Task> create(Task task, RequestContext context);

    public GeneralResult<Task> onekeyRelease(JSONObject task, RequestContext context);

    /**
     * 更新任务
     * 
     * @param bean
     * @param context
     * @return
     */
    public GeneralResult<Task> modify(TaskBean bean, RequestContext context);

    /**
     * 删除任务
     * 
     * @param id
     * @param userId
     * @return
     */
    public GeneralResult<Void> remove(Long id, Long userId);

    /**
     * 查看任务详情
     * 
     * @param id
     * @return
     */
    public GeneralResult<Task> detail(Long id);

    /**
     * 克隆任务
     * 
     * @param id
     * @return
     */
    public GeneralResult<Task> clone(Long id);

    /**
     * 任务基本信息
     * 
     * @param id
     * @return
     */
    public Task basic(Long id);

    /**
     * 任务实例化
     * 
     * @param taskId
     * @param mode
     * @param type
     * @param context
     * @return
     */
    public TaskInstance initInstance(Long taskId, String mode, String type, RequestContext context);

    /**
     * 启动定时任务
     * 
     * @param id
     * @param userId
     * @return
     */
    public GeneralResult<Void> active(Long id, Long userId);

    /**
     * 暂停定时任务
     * 
     * @param id
     * @param userId
     * @return
     */
    public GeneralResult<Void> suspend(Long id, Long userId);

    /**
     * 转换或生成定时任务
     * 
     * @param id
     * @param params
     * @param context
     * @return
     */
    public GeneralResult<Void> transferCron(Long id, TaskTransferParams params, RequestContext context);

    /**
     * 改变状态
     * 
     * @param id
     * @param status
     * @param userId
     * @return
     */
    public GeneralResult<Void> updateStatus(Long id, String status, Long userId);

    public GeneralResult<Void> isValid(String expression);

    public GeneralResult<Task> get(String uuid);

    public GeneralResult<Void> updateExecute(Long id, Date date);

    public GeneralResult<List<Task>> listSchedule();

    public GeneralResult<Void> execute(JSONObject object);

}
