package com.bocloud.cop.schedule.inspection.report;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.inspection.InspectReportDetail;

/**
 * 巡检报表Excel工具
 *
 * @author jiaohongtao
 * @version 1.0
 * @since 2020年11月10日
 */
public class InspectReportExcelUtil {

    private static final short BOLD_FONT = 20;
    private static final short NORMAL_FONT = 12;

    public static void excel(InspectReportDetail reportDetail, HSSFWorkbook workbook) {

        HSSFSheet sheet = workbook.createSheet();
        sheet.setColumnWidth(0, 16 * 256);
        sheet.setColumnWidth(1, 20 * 256);
        sheet.setColumnWidth(2, 30 * 256);
        sheet.setColumnWidth(3, 40 * 256);
        sheet.setColumnWidth(4, 40 * 256);
        sheet.addMergedRegion(new CellRangeAddress(0, 1, 0, 5));
        HSSFRow row = sheet.createRow(0);
        HSSFCell cell = row.createCell(0);
        cell.setCellStyle(getStyle(workbook, BOLD_FONT, true));
        cell.setCellValue(reportDetail.getTitle());

        // 添加基本信息
        String baseMsg = reportDetail.getBaseMsg();
        int baseSize = 0;
        if (StringUtils.isNotBlank(baseMsg)) {
            List<JSONObject> jsonArray = JSONArray.parseArray(baseMsg, JSONObject.class);
            baseSize = jsonArray.size();

            // 合并基本信息的第二列
            for (int i = 2; i < baseSize + 2; i++) {
                sheet.addMergedRegion(new CellRangeAddress(i, i, 1, 5));
            }

            for (int i = 0; i < baseSize; i++) {
                JSONObject object = jsonArray.get(i);
                // 创建行
                HSSFRow row2 = sheet.createRow(i + 2);
                // 设置第一个
                HSSFCell cell2 = row2.createCell(0);
                // String name = object.getString("name");
                String value = object.get("value") + "";
                cell2.setCellValue(object.getString("remark"));
                cell2.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
                // 设置第二个
                cell2 = row2.createCell(1);
                cell2.setCellValue(value);
                cell2.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
            }
        }

        // 添加列表头部信息
        HSSFRow row13 = sheet.createRow(baseSize + 5);

        String listMsg = reportDetail.getListMsg();
        if (StringUtils.isNotBlank(listMsg)) {
            List<JSONObject> listArray = JSONArray.parseArray(listMsg, JSONObject.class);
            List<String> codes = listArray.stream().map(l -> l.getString("code")).collect(Collectors.toList());

            int j = 0;
            if (codes.contains("ip")) {
                for (JSONObject list : listArray) {
                    if ("ip".equals(list.getString("code"))) {
                        HSSFCell cell14 = row13.createCell(j);
                        cell14.setCellValue(list.getString("name"));
                        cell14.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
                    }
                }
                j++;
            }
            if (codes.contains("itemName")) {
                for (JSONObject list : listArray) {
                    if ("itemName".equals(list.getString("code"))) {
                        HSSFCell cell14 = row13.createCell(j);
                        cell14.setCellValue(list.getString("name"));
                        cell14.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
                    }
                }
                j++;
            }
            if (codes.contains("inspectResult")) {
                for (JSONObject list : listArray) {
                    if ("inspectResult".equals(list.getString("code"))) {
                        HSSFCell cell14 = row13.createCell(j);
                        cell14.setCellValue(list.getString("name"));
                        cell14.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
                    }
                }
                j++;
            }
            if (codes.contains("rule")) {
                for (JSONObject list : listArray) {
                    if ("rule".equals(list.getString("code"))) {
                        HSSFCell cell14 = row13.createCell(j);
                        cell14.setCellValue(list.getString("name"));
                        cell14.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
                    }
                }
                j++;
            }
            if (codes.contains("threshold")) {
                for (JSONObject list : listArray) {
                    if ("threshold".equals(list.getString("code"))) {
                        HSSFCell cell14 = row13.createCell(j);
                        cell14.setCellValue(list.getString("name"));
                        cell14.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
                    }
                }
                j++;
            }
            if (codes.contains("warning")) {
                for (JSONObject list : listArray) {
                    if ("warning".equals(list.getString("code"))) {
                        HSSFCell cell14 = row13.createCell(j);
                        cell14.setCellValue(list.getString("name"));
                        cell14.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
                    }
                }
            }
        }

        // 添加列表内容信息
        List<JSONObject> listObjects = reportDetail.getListObjects();
        for (int i = 0, listObjectsSize = listObjects.size(); i < listObjectsSize; i++) {
            JSONObject listObject = listObjects.get(i);
            int x = 0;
            HSSFRow row14 = sheet.createRow(baseSize + 6 + i);

            Set<String> listKeys = listObject.keySet();
            if (listKeys.contains("ip")) {
                HSSFCell cell14 = row14.createCell(x);
                cell14.setCellValue(listObject.get("ip") + "");
                cell14.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
                x++;
            }
            if (listKeys.contains("itemName")) {
                HSSFCell cell14 = row14.createCell(x);
                cell14.setCellValue(listObject.get("itemName") + "");
                cell14.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
                x++;
            }
            if (listKeys.contains("inspectResult")) {
                HSSFCell cell14 = row14.createCell(x);
                String inspectResult = listObject.getString("inspectResult");
                cell14.setCellValue(StringUtils.isNotBlank(inspectResult) ? inspectResult : " ");
                cell14.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
                x++;
            }
            if (listKeys.contains("rule")) {
                HSSFCell cell14 = row14.createCell(x);
                cell14.setCellValue(listObject.getString("rule"));
                cell14.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
                x++;
            }
            if (listKeys.contains("threshold")) {
                HSSFCell cell14 = row14.createCell(x);
                cell14.setCellValue(listObject.get("threshold") + "");
                cell14.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
                x++;
            }
            if (listKeys.contains("warning")) {
                HSSFCell cell14 = row14.createCell(x);
                String warning = listObject.getString("warning");
                cell14.setCellValue(StringUtils.isNotBlank(warning) ? warning : " ");
                cell14.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
            }
        }
    }

    /**
     * 标题或者第一列数据信息单元格样式
     *
     * @param fontHeight
     *            标题20， 第一列12，第二列12
     */
    private static HSSFCellStyle getStyle(HSSFWorkbook workbook, short fontHeight, boolean isBold) {

        HSSFFont font = workbook.createFont();
        font.setFontHeightInPoints((fontHeight));
        font.setBold(isBold);
        font.setFontName("宋体");

        HSSFCellStyle style = workbook.createCellStyle();
        style.setFont(font);
        style.setWrapText(true);
        style.setAlignment(HorizontalAlignment.CENTER);
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        return style;
    }

}
