package com.bocloud.cop.schedule.inspection.report;

import java.awt.Color;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFPalette;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.inspection.InspectRecordWarning;
import com.bocloud.cop.model.inspection.WarningRule;

public class InspectReportExeclTool {
    public static void exportExcel(HSSFWorkbook workbook, int sheetNum, String sheetTitle, List<List<String>> result,
        String title) throws Exception {
        // 生成一个表格
        HSSFSheet sheet = workbook.createSheet();
        workbook.setSheetName(sheetNum, sheetTitle);
        // 设置表格默认列宽度为20个字节
        sheet.setDefaultColumnWidth((short)20);
        // 生成一个样式
        HSSFCellStyle style = workbook.createCellStyle();
        // 设置这些样式
        style.setFillForegroundColor(HSSFColor.PALE_BLUE.index);

        style.setFillBackgroundColor(HSSFColor.WHITE.index);
        style.setFillForegroundColor(HSSFColor.LIGHT_ORANGE.index);

        // 生成一个字体
        HSSFFont font = workbook.createFont();
        font.setColor(HSSFColor.BLACK.index);
        font.setFontHeightInPoints((short)12);
        // 把字体应用到当前的样式
        style.setFont(font);

        // 指定当单元格内容显示不下时自动换行
        style.setWrapText(true);
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        style.setAlignment(HorizontalAlignment.CENTER);

        // 产生表格标题行
        HSSFRow row = sheet.createRow(0);
        row.createCell(0).setCellValue("巡检报表");
        CellRangeAddress region1 = new CellRangeAddress(0, 0, 0, 2);
        sheet.addMergedRegion(region1);
        if (result != null) {
            int index = 1;
            for (List<String> m : result) {
                row = sheet.createRow(index);
                HSSFCell cell = row.createCell(0);
                cell.setCellValue(m.get(0));
                cell = row.createCell(1);
                cell.setCellValue(m.get(1));
                cell = row.createCell(2);
                if ("部分异常".equals(m.get(1))) {
                    String link = "HYPERLINK(\"[" + title + "-" + getSecondTimestamp(new Date()) + ".xls" + "]"
                        + m.get(0) + "!A1\",\"详情\")";
                    cell.setCellFormula(link);
                } else {
                    cell.setCellValue("详情");
                }
                index++;
            }
        }
    }

    public static int getSecondTimestamp(Date date) {
        if (null == date) {
            return 0;
        }
        String timestamp = String.valueOf(date.getTime());
        int length = timestamp.length();
        if (length > 3) {
            return Integer.valueOf(timestamp.substring(0, length - 3));
        } else {
            return 0;
        }
    }

    private static final short NORMAL_FONT = 12;

    public static void exportExcel1(HSSFWorkbook workbook, int sheetNum, String sheetTitle,
        List<InspectRecordWarning> result) {
        // 生成一个表格
        HSSFSheet sheet = workbook.createSheet();
        workbook.setSheetName(sheetNum, sheetTitle);
        // 设置表格默认列宽度为20个字节
        sheet.setDefaultColumnWidth((short)20);
        // 生成一个样式
        HSSFCellStyle style = workbook.createCellStyle();
        // 设置这些样式
        style.setFillForegroundColor(HSSFColor.PALE_BLUE.index);

        style.setFillBackgroundColor(HSSFColor.WHITE.index);
        style.setFillForegroundColor(HSSFColor.LIGHT_ORANGE.index);

        int rowNum = 0;
        // 生成一个字体
        HSSFFont font = workbook.createFont();
        font.setColor(HSSFColor.BLACK.index);
        font.setFontHeightInPoints((short)12);
        // 把字体应用到当前的样式
        style.setFont(font);
        HSSFRow row = sheet.createRow(rowNum);
        HSSFCell cell = null;
        cell = row.createCell(0);
        cell.setCellValue("资源组名称");
        cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell = row.createCell(1);
        cell.setCellValue("主机名称");
        cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell = row.createCell(2);
        cell.setCellValue("主机ip");
        cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell = row.createCell(3);
        cell.setCellValue("指标项");
        cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell = row.createCell(4);
        cell.setCellValue("指标值");
        cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell = row.createCell(5);
        cell.setCellValue("告警");
        cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell = row.createCell(6);
        cell.setCellValue("规则");
        cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell = row.createCell(7);
        cell.setCellValue("阈值");
        cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        for (InspectRecordWarning inspectRecordWarning : result) {
            rowNum++;
            row = sheet.createRow(rowNum);
            HSSFCell cell1 = row.createCell(0);
            cell1.setCellValue(inspectRecordWarning.getGroupName());
            cell1.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell1 = row.createCell(1);
            cell1.setCellValue(inspectRecordWarning.getResourceName());
            cell1.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell1 = row.createCell(2);
            cell1.setCellValue(inspectRecordWarning.getIp());
            cell1.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell1 = row.createCell(3);
            cell1.setCellValue(
                StringUtils.isEmpty(inspectRecordWarning.getInspectKey()) ? inspectRecordWarning.getItemName()
                    : inspectRecordWarning.getItemName() + "_" + inspectRecordWarning.getInspectKey());
            cell1.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell1 = row.createCell(4);
            cell1
                .setCellValue(StringUtils.isEmpty(inspectRecordWarning.getValueName()) ? inspectRecordWarning.getValue()
                    : inspectRecordWarning.getValueName() + ":" + inspectRecordWarning.getValue());
            cell1.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            String rule = inspectRecordWarning.getTargetRule();
            String description = "-";
            String ruleName = "-";
            String threshold = "-";
            String fontColor = null;
            Integer level = null;
            if (rule != null) {
                JSONObject targetRule = JSONObject.parseObject(rule);
                // color
                fontColor = targetRule.getString("color").replace("#", "");
                // warning
                description = targetRule.getString("description");
                // rule
                String ruleSign = targetRule.getString("rule");
                ruleName = WarningRule.RULE.valueOf(ruleSign).getName();
                threshold = targetRule.getString("threshold");
                level = targetRule.getInteger("warnLevel");
            }

            cell1 = row.createCell(5);
            cell1.setCellValue(description);
            cell1.setCellStyle(getWarningStyle(workbook, NORMAL_FONT, false, fontColor, level));

            cell1 = row.createCell(6);
            cell1.setCellValue(ruleName);
            cell1.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell1 = row.createCell(7);
            cell1.setCellValue(threshold);
            cell1.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
        }
    }

    /**
     * 标题或者第一列数据信息单元格样式
     *
     * @param fontHeight
     *            标题20， 第一列12，第二列12
     */
    private static HSSFCellStyle getStyle(HSSFWorkbook workbook, short fontHeight, boolean isBold) {

        HSSFFont font = workbook.createFont();
        font.setFontHeightInPoints((fontHeight));
        font.setBold(isBold);
        font.setFontName("宋体");
        HSSFCellStyle style = workbook.createCellStyle();
        style.setFont(font);
        style.setWrapText(true);
        style.setAlignment(HorizontalAlignment.CENTER);
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        return style;
    }

    private static HSSFCellStyle getWarningStyle(HSSFWorkbook workbook, short fontHeight, boolean isBold,
        String colorString, Integer level) {
        HSSFFont font = workbook.createFont();
        font.setFontHeightInPoints((fontHeight));
        font.setBold(isBold);
        font.setFontName("宋体");

        if ((!StringUtils.isEmpty(colorString)) && level != null && level > 0 && level <= 10) {
            String color16 = colorString.replace("#", "");
            Integer colorInt = Integer.parseInt(color16, 16);
            Color color = new Color(colorInt);
            // 获取工作簿的调色面板

            HSSFPalette palette = workbook.getCustomPalette();
            // 将调色面板中蓝色索引替换为红色
            short index = warningColors[level];
            palette.setColorAtIndex(index, (byte)color.getRed(), (byte)color.getGreen(), (byte)color.getBlue());
            // 会出现红色字体
            font.setColor(index);
        }
        HSSFCellStyle style = workbook.createCellStyle();
        style.setFont(font);
        style.setWrapText(true);
        style.setAlignment(HorizontalAlignment.CENTER);
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        return style;
    }

    /**
     * 从库里选几个颜色进行覆盖
     */
    private static short[] warningColors =
        {HSSFColor.GREY_25_PERCENT.index, HSSFColor.GREY_40_PERCENT.index, HSSFColor.GREY_50_PERCENT.index,
            HSSFColor.GREY_80_PERCENT.index, HSSFColor.LIGHT_CORNFLOWER_BLUE.index, HSSFColor.LEMON_CHIFFON.index,
            HSSFColor.ROYAL_BLUE.index, HSSFColor.AQUA.index, HSSFColor.TAN.index, HSSFColor.PALE_BLUE.index};
}
