package com.bocloud.cop.schedule.inspection.report;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.inspection.InspectRecordWarning;
import com.bocloud.cop.entity.inspection.InspectReportDetail;
import com.bocloud.cop.model.inspection.WarningRule;
import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Chunk;
import com.itextpdf.text.Document;
import com.itextpdf.text.Element;
import com.itextpdf.text.Font;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;

public class InspectReportPdfTool {

    public static byte[] pdf(InspectReportDetail reportDetail, List<List<InspectRecordWarning>> dataList,
        List<List<String>> data) throws Exception {
        // 第一步，实例化一个document对象
        Document document = new Document(PageSize.A4, 50, 50, 50, 50);
        // 清空response，否则可能会对程序有影响，因为在此之前有可能也用到了response，若不reset，后面的setCntentoType()有可能失效
        // 第二步，设置要到出的路径
        // 如果是浏览器通过request请求需要在浏览器中输出则使用下面方式
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        // 第三步,设置字符,宋体
        BaseFont bfChinese = BaseFont.createFont("STSong-Light", "UniGB-UCS2-H", BaseFont.NOT_EMBEDDED);
        Font titleFont = new Font(bfChinese, 16.0F, Font.NORMAL);
        Font subChinese = new Font(bfChinese, 14.0F, Font.NORMAL);
        Font fontChinese = new Font(bfChinese, 12.0F, Font.NORMAL);
        // 第四步，将pdf文件输出到磁盘
        PdfWriter.getInstance(document, baos);
        // 第五步，打开生成的pdf文件
        document.open();

        Font greenFont = new Font(bfChinese, 10.0F, Font.NORMAL);
        greenFont.setColor(BaseColor.GREEN);
        Font redFont = new Font(bfChinese, 10.0F, Font.NORMAL);
        redFont.setColor(BaseColor.RED);
        Font orangeFont = new Font(bfChinese, 10.0F, Font.NORMAL);
        orangeFont.setColor(BaseColor.ORANGE);

        // 设置总标题
        String title = reportDetail.getTitle();
        Paragraph paragraph = new Paragraph();
        paragraph.setAlignment(Element.ALIGN_CENTER);
        paragraph.add(new Chunk(title, titleFont).setLocalDestination(title));
        document.add(paragraph);
        document.add(new Paragraph("\n"));

        PdfPTable table = new PdfPTable(3);
        table.setHorizontalAlignment(Element.ALIGN_CENTER);
        float[] widths = {40f, 40f, 40f};
        table.setWidths(widths);

        PdfPCell cell;

        for (List<String> datum : data) {
            cell = new PdfPCell(new Paragraph(datum.get(0), fontChinese));
            setCellCenter(cell);
            table.addCell(cell);

            cell = new PdfPCell(new Paragraph(datum.get(1), fontChinese));
            setCellCenter(cell);
            table.addCell(cell);

            cell = new PdfPCell(new Paragraph(datum.get(2), fontChinese));
            setCellCenter(cell);
            table.addCell(cell);
        }

        List<String> sub = new ArrayList<>();
        for (List<String> datum : data) {
            if ("部分异常".equals(datum.get(1))) {
                sub.add(datum.get(0));
            }
        }

        int i = 0;
        for (List<InspectRecordWarning> recordWarnings : dataList) {
            if (recordWarnings.size() > 0) {
                cell = new PdfPCell(new Paragraph(sub.get(i), subChinese));
                cell.setColspan(3);
                setAndAddCellStyle(table, cell);
                PdfPTable table1 = new PdfPTable(8);
                table1.setHorizontalAlignment(Element.ALIGN_CENTER);
                float[] floats = {16f, 16f, 16f, 16f, 16f, 16f, 16f, 16f};
                table1.setWidths(floats);
                cell = new PdfPCell(getDetailInfo(recordWarnings, table1, fontChinese));
                cell.setColspan(3);
                setAndAddCellStyle(table, cell);
                i++;
            }
        }
        document.add(table);
        // 第七步，关闭document
        document.close();
        return baos.toByteArray();
    }

    public static byte[] pdfOfReportId(String title, List<List<InspectRecordWarning>> dataList) throws Exception {
        // 第一步，实例化一个document对象
        Document document = new Document();
        // 清空response，否则可能会对程序有影响，因为在此之前有可能也用到了response，若不reset，后面的setCntentoType()有可能失效
        // 第二步，设置要到出的路径
        // 如果是浏览器通过request请求需要在浏览器中输出则使用下面方式
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        // 第三步,设置字符,宋体
        BaseFont bfChinese = BaseFont.createFont("STSong-Light", "UniGB-UCS2-H", BaseFont.NOT_EMBEDDED);
        Font fontChinese = new Font(bfChinese, 12.0F, Font.NORMAL);
        Font titleChinese = new Font(bfChinese, 16.0F, Font.NORMAL);
        // 第四步，将pdf文件输出到磁盘
        PdfWriter.getInstance(document, baos);
        // 第五步，打开生成的pdf文件
        document.open();

        Font greenFont = new Font(bfChinese, 10.0F, Font.NORMAL);
        greenFont.setColor(BaseColor.GREEN);
        Font redFont = new Font(bfChinese, 10.0F, Font.NORMAL);
        redFont.setColor(BaseColor.RED);
        Font orangeFont = new Font(bfChinese, 10.0F, Font.NORMAL);
        orangeFont.setColor(BaseColor.ORANGE);

        // 设置总标题
        Paragraph paragraph = new Paragraph();
        paragraph.setAlignment(Element.ALIGN_CENTER);
        paragraph.add(new Chunk(title, titleChinese).setLocalDestination(title));
        document.add(paragraph);
        document.add(new Paragraph("\n"));
        PdfPCell cell;

        PdfPTable table = new PdfPTable(8);
        float[] floats = {40f, 40f, 40f, 40f, 40f, 40f, 40f, 40f};
        table.setWidths(floats);

        cell = new PdfPCell(new Paragraph("资源组名称", fontChinese));
        setCellCenter(cell);
        table.addCell(cell);
        cell = new PdfPCell(new Paragraph("主机名称", fontChinese));
        setCellCenter(cell);
        table.addCell(cell);

        cell = new PdfPCell(new Paragraph("主机ip", fontChinese));
        setCellCenter(cell);
        table.addCell(cell);

        cell = new PdfPCell(new Paragraph("指标项", fontChinese));
        setCellCenter(cell);
        table.addCell(cell);

        cell = new PdfPCell(new Paragraph("指标值", fontChinese));
        setCellCenter(cell);
        table.addCell(cell);

        cell = new PdfPCell(new Paragraph("告警", fontChinese));
        setCellCenter(cell);
        table.addCell(cell);

        cell = new PdfPCell(new Paragraph("规则", fontChinese));
        setCellCenter(cell);
        table.addCell(cell);

        cell = new PdfPCell(new Paragraph("阈值", fontChinese));
        setCellCenter(cell);
        table.addCell(cell);

        for (List<InspectRecordWarning> list : dataList) {
            for (InspectRecordWarning item : list) {
                cell = new PdfPCell(new Paragraph(item.getGroupName(), fontChinese));
                setCellCenter(cell);
                table.addCell(cell);

                cell = new PdfPCell(new Paragraph(
                    StringUtils.isEmpty(item.getResourceName()) ? item.getIp() : item.getResourceName(), fontChinese));
                setCellCenter(cell);
                table.addCell(cell);

                cell = new PdfPCell(new Paragraph(item.getIp(), fontChinese));
                setCellCenter(cell);
                table.addCell(cell);

                String itemName = StringUtils.isEmpty(item.getInspectKey()) ? item.getItemName()
                    : item.getItemName() + "_" + item.getInspectKey();
                cell = new PdfPCell(new Paragraph(itemName, fontChinese));
                setCellCenter(cell);
                table.addCell(cell);

                String inspectValue = StringUtils.isEmpty(item.getValueName()) ? item.getValue()
                    : item.getValueName() + ":" + item.getValue();
                cell = new PdfPCell(new Paragraph(inspectValue, fontChinese));
                setCellCenter(cell);
                table.addCell(cell);

                String rule = item.getTargetRule();
                if (StringUtils.isEmpty(rule)) {
                    cell = new PdfPCell(new Paragraph("-", fontChinese));
                    setCellCenter(cell);
                    table.addCell(cell);

                    cell = new PdfPCell(new Paragraph("-", fontChinese));
                    setCellCenter(cell);
                    table.addCell(cell);

                    cell = new PdfPCell(new Paragraph("-", fontChinese));
                    setCellCenter(cell);
                    table.addCell(cell);
                } else {
                    BaseFont bfChinese1 = BaseFont.createFont("STSong-Light", "UniGB-UCS2-H", BaseFont.NOT_EMBEDDED);
                    Font colorfulFont = new Font(bfChinese1, 8.0F, Font.NORMAL);

                    // {"color":"#E50F5D","description":"二级告警","rule":"GT","threshold":"55%","warnLevel":2}

                    // 告警标记
                    JSONObject ruleJson = JSONObject.parseObject(rule);
                    String color16 = ruleJson.getString("color").replace("#", "");
                    Integer color = Integer.parseInt(color16, 16) + (255 << 24);
                    colorfulFont.setColor(new BaseColor(color));
                    String description = ruleJson.getString("description");
                    cell = new PdfPCell(new Paragraph(description, colorfulFont));
                    setCellCenter(cell);
                    table.addCell(cell);

                    // 规则符号转文字
                    String ruleSign = ruleJson.getString("rule");
                    cell = new PdfPCell(new Paragraph(WarningRule.RULE.valueOf(ruleSign).getName(), fontChinese));
                    setCellCenter(cell);
                    table.addCell(cell);

                    // 阈值
                    cell = new PdfPCell(new Paragraph(ruleJson.getString("threshold"), fontChinese));
                    setCellCenter(cell);
                    table.addCell(cell);

                }
            }
        }

        table.addCell(cell);
        document.add(table);
        // 第七步，关闭document
        document.close();
        return baos.toByteArray();
    }

    private static PdfPTable getDetailInfo(List<InspectRecordWarning> lists, PdfPTable warningTable, Font fontZH)
        throws Exception {
        PdfPCell cell;

        // 2.0版本添加
        cell = new PdfPCell(new Paragraph("资源组名称", fontZH));
        setCellCenter(cell);
        warningTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("主机名称", fontZH));
        setCellCenter(cell);
        warningTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("主机ip", fontZH));
        setCellCenter(cell);
        warningTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("指标项", fontZH));
        setCellCenter(cell);
        warningTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("指标值", fontZH));
        setCellCenter(cell);
        warningTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("告警", fontZH));
        setCellCenter(cell);
        warningTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("规则", fontZH));
        setCellCenter(cell);
        warningTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("阈值", fontZH));
        setCellCenter(cell);
        warningTable.addCell(cell);

        // for (List<InspectRecordWarning> list : lists) {
        for (InspectRecordWarning item : lists) {
            cell = new PdfPCell(new Paragraph(item.getGroupName(), fontZH));
            setCellCenter(cell);
            warningTable.addCell(cell);
            cell = new PdfPCell(new Paragraph(
                StringUtils.isEmpty(item.getResourceName()) ? item.getIp() : item.getResourceName(), fontZH));
            setCellCenter(cell);
            warningTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(item.getIp(), fontZH));
            setCellCenter(cell);
            warningTable.addCell(cell);

            String itemName = StringUtils.isEmpty(item.getInspectKey()) ? item.getItemName()
                : item.getItemName() + "_" + item.getInspectKey();
            cell = new PdfPCell(new Paragraph(itemName, fontZH));
            setCellCenter(cell);
            warningTable.addCell(cell);

            String inspectValue = StringUtils.isEmpty(item.getValueName()) ? item.getValue()
                : item.getValueName() + ":" + item.getValue();
            cell = new PdfPCell(new Paragraph(inspectValue, fontZH));
            setCellCenter(cell);
            warningTable.addCell(cell);

            String rule = item.getTargetRule();
            if (StringUtils.isEmpty(rule)) {
                cell = new PdfPCell(new Paragraph("-", fontZH));
                setCellCenter(cell);
                warningTable.addCell(cell);

                cell = new PdfPCell(new Paragraph("-", fontZH));
                setCellCenter(cell);
                warningTable.addCell(cell);

                cell = new PdfPCell(new Paragraph("-", fontZH));
                setCellCenter(cell);
                warningTable.addCell(cell);
            } else {
                BaseFont bfChinese = BaseFont.createFont("STSong-Light", "UniGB-UCS2-H", BaseFont.NOT_EMBEDDED);
                Font colorfulFont = new Font(bfChinese, 8.0F, Font.NORMAL);

                // {"color":"#E50F5D","description":"二级告警","rule":"GT","threshold":"55%","warnLevel":2}

                // 告警标记
                JSONObject ruleJson = JSONObject.parseObject(rule);
                String color16 = ruleJson.getString("color").replace("#", "");
                Integer color = Integer.parseInt(color16, 16) + (255 << 24);
                colorfulFont.setColor(new BaseColor(color));
                String description = ruleJson.getString("description");
                cell = new PdfPCell(new Paragraph(description, colorfulFont));
                setCellCenter(cell);
                warningTable.addCell(cell);

                // 规则符号转文字
                String ruleSign = ruleJson.getString("rule");
                cell = new PdfPCell(new Paragraph(WarningRule.RULE.valueOf(ruleSign).getName(), fontZH));
                setCellCenter(cell);
                warningTable.addCell(cell);

                // 阈值
                cell = new PdfPCell(new Paragraph(ruleJson.getString("threshold"), fontZH));
                setCellCenter(cell);
                warningTable.addCell(cell);

            }
            // }
        }
        return warningTable;
    }

    private static void setAndAddCellStyle(PdfPTable table, PdfPCell cell) {
        cell.setMinimumHeight(30.0F);
        cell.setPaddingTop(10f);
        cell.setPaddingBottom(10f);
        setCellCenter(cell);
        cell.setBorder(Rectangle.NO_BORDER);
        table.addCell(cell);
    }

    private static void setCellCenter(PdfPCell cell) {
        cell.setMinimumHeight(40);
        cell.setHorizontalAlignment(Element.ALIGN_CENTER);
        cell.setVerticalAlignment(Element.ALIGN_MIDDLE);
    }
}
