package com.bocloud.cop.service;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.cop.entity.Software;
import com.bocloud.cop.entity.SoftwareTenant;
import com.bocloud.cop.enums.ApplyStatus;
import com.bocloud.cop.intf.SoftwareService;
import com.bocloud.cop.model.SoftwareBean;
import com.bocloud.cop.repository.SoftwareRepository;
import com.bocloud.cop.repository.SoftwareTenantRepository;
import com.bocloud.cop.repository.TaskTemplateRepository;
import com.bocloud.cop.service.utils.CompressDownloadUtil;
import com.bocloud.cop.service.utils.StorageConfig;
import com.google.common.collect.Lists;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.io.*;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 文件抽象Service接口实现类
 *
 * @author wei wei
 * @version 1.0
 * @since 2016.08.16
 */
@Slf4j
@Service("softwareService")
public class SoftwareServiceImpl implements SoftwareService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private SoftwareRepository softwareRepository;
    @Autowired
    private TaskTemplateRepository taskTemplateRepository;
    @Autowired
    private StorageConfig storageConfig;
    @Autowired
    private SoftwareTenantRepository softwareTenantRepository;

    @Override
    public GeneralResult<GridBean<Software>> list(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            if (pager.getSimple()) {
                pager.setRows(Integer.MAX_VALUE);
            }
            Long groupId = null;
            for (Param pm : pager.getParams()) {
                Map<String, Object> param = pm.getParam();
                Object obj = param.get("groupId");
                if (null != obj) {
                    groupId = Long.valueOf(obj.toString());
                    param.remove("groupId");
                    break;
                }
            }
            int total = softwareRepository.count(pager.getParams(), groupId);
            List<Software> list = softwareRepository.list(pager, groupId);
            GridBean<Software> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询文件成功");
        } catch (Exception e) {
            log.error("List software fail:", e);
            return new GeneralResult<>(false, "查询文件失败");
        }
    }

    @Override
    public GeneralResult<GridBean<Software>> listAllot(Pager pager, RequestContext context) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            if (pager.getSimple()) {
                pager.setRows(Integer.MAX_VALUE);
            }
            List<SoftwareTenant> allotList = softwareTenantRepository.listByTenantId(context.getTenant());
            Set<Long> softwareIds = allotList.stream().map(SoftwareTenant::getSoftwareId).collect(Collectors.toSet());
            int total = 0;
            List<Software> list = new ArrayList<>();
            if (null != softwareIds && softwareIds.size() > 0) {
                pager.getParams().add(new Param(MapTools.simpleMap("id", StringUtils.join(softwareIds, ",")), Sign.IN));
                total = softwareRepository.count(pager.getParams(), null);
                list = softwareRepository.list(pager, null);
            }
            GridBean<Software> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询文件成功");
        } catch (Exception e) {
            log.error("listAllot software fail:", e);
            return new GeneralResult<>(false, "查询文件失败");
        }
    }

    @Override
    public GeneralResult<GridBean<SimpleBean>> simple(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            if (pager.getSimple()) {
                pager.setRows(Integer.MAX_VALUE);
            }
            Long groupId = null;
            for (Param pm : pager.getParams()) {
                Map<String, Object> param = pm.getParam();
                Object obj = param.get("groupId");
                if (null != obj) {
                    groupId = Long.valueOf(obj.toString());
                    param.remove("groupId");
                    break;
                }
            }
            int total = softwareRepository.count(pager.getParams(), groupId);
            List<SimpleBean> beans = softwareRepository.list(pager.getParams(), pager.getSorter(), groupId);
            GridBean<SimpleBean> bean = new GridBean<>(1, 1, total, beans);
            return new GeneralResult<>(true, bean, "查询文件成功");
        } catch (Exception e) {
            log.error("List software fail:", e);
            return new GeneralResult<>(false, "查询文件失败");
        }

    }

    @Override
    public GeneralResult<Software> create(Software software) {
        try {
            GeneralResult<?> result = exists(software.getName());
            if (result.isSuccess()) {
                return new GeneralResult<>(false, "文件名称已存在!");
            }
            software.setStatus(ApplyStatus.DEVELOPING.name());
            software.setGroupIds(JSONArray.toJSONString(software.getGroupIds()));
            softwareRepository.save(software);
            return new GeneralResult<>(true, software, "添加文件成功");
        } catch (Exception e) {
            log.error("Create software fail:", e);
            return new GeneralResult<>(false, "添加文件失败");
        }

    }

    @Override
    public GeneralResult<SoftwareBean> modify(SoftwareBean bean, Long userId) {
        String path = Software.class.getSimpleName() + "_" + bean.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Software software = softwareRepository.query(bean.getId());
            Assert.notNull(software, "文件不存在！");
            if (ApplyStatus.APPROVED.name().equalsIgnoreCase(software.getStatus())) {
                return new GeneralResult<>(false, "选择的文件已发布，请先下架后再操作！");
            }
            if (!bean.getName().equals(software.getName())) {
                GeneralResult<?> result = exists(bean.getName());
                if (result.isSuccess()) {
                    return new GeneralResult<>(false, "此名称已存在!");
                }
            }
            BeanUtils.copyProperties(bean, software);
            software.setMenderId(userId);
            software.setGroupIds(JSONArray.toJSONString(bean.getGroupIds()));
            Boolean result = softwareRepository.update(software);
            return result ? new GeneralResult<>(true, bean, "修改文件成功") : new GeneralResult<>(false, "修改文件失败");
        } catch (Exception e) {
            log.error("Modify software fail:", e);
            return new GeneralResult<>(false, "修改文件失败");
        }
    }

    @Override
    public GeneralResult<String> remove(Long id, Long userId) {
        String path = Software.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Software software = softwareRepository.query(id);
            if (null == software) {
                return new GeneralResult<>(false, "文件不存在");
            }
            if (ApplyStatus.APPROVED.name().equalsIgnoreCase(software.getStatus())) {
                return new GeneralResult<>(false, "选择的文件已发布，请先下架后再操作！");
            }
            List<Param> params = Lists.newArrayList();
            params.add(new Param(MapTools.simpleMap("graph", software.getPath()), Sign.LK));
            int count = taskTemplateRepository.count(params);
            if (count > 0) {
                return new GeneralResult<>(false, "该文件已被编排模板引用无法删除");
            }
            softwareRepository.delete(id, userId);
            return new GeneralResult<>(true, software.getPath(), "删除文件成功");
        } catch (Exception e) {
            log.error("Remove software fail:", e);
            return new GeneralResult<>(false, "删除文件失败");
        }
    }

    @Override
    public GeneralResult<Software> detail(Long id) {
        try {
            Software software = softwareRepository.query(id);
            return new GeneralResult<>(true, software, "查询文件详情成功");
        } catch (Exception e) {
            log.error("Query software detail fail:", e);
            return new GeneralResult<>(false, "查询文件详情失败");
        }
    }

    @Override
    public GeneralResult<Software> exists(String name) {
        try {
            Software software = softwareRepository.exists(name);
            if (null == software) {
                return new GeneralResult<>(false, "文件名称不存在");
            }
            return new GeneralResult<>(true, software, "文件名称已存在");
        } catch (Exception e) {
            log.error("get software error:", e);
            return new GeneralResult<>(false, "根据名称查询文件是否存在时出错，" + e.getMessage());
        }

    }

    /**
     * 根据名称查询该文件是否不存在
     */
    @Override
    public GeneralResult<Software> fileNotExists(String name) {
        return this.exists(name);
    }

    @Override
    public void compressDownload(List<String> paths, HttpServletResponse response) {
        try {
            List<String> fpathList = new ArrayList<>(paths.size());
            String prePath = storageConfig.getPath() + "/";

            for (String fileName : paths) {
                fpathList.add(prePath + fileName);
            }
            List<File> files = new ArrayList<>();
            fpathList.forEach(file -> {
                if (Objects.nonNull(file)) {
                    files.add(new File(file));
                }
            });
            files.forEach(file -> {
                if (!file.exists()) {
                    return;
                }
            });
            // 设置响应头
            DateFormat format = new SimpleDateFormat("yyyyMMddhhmmss");
            String downloadName = "补丁文件-" + format.format(new Date()) + ".zip";
            CompressDownloadUtil.setDownloadResponse(response, downloadName);
            String zipFilePath = prePath + downloadName;
            // 将多个文件压缩到指定路径下
            CompressDownloadUtil.compressZip(files, new FileOutputStream(zipFilePath));
            // -- 下载压缩包
            CompressDownloadUtil.downloadFile(response.getOutputStream(), zipFilePath);
            // -- 删除临时生成的ZIP文件
            CompressDownloadUtil.deleteFile(zipFilePath);
        } catch (Exception e) {
            log.error("下载补丁文件失败：", e);
        }

    }

    @Override
    public void singleDownload(String path, HttpServletRequest request, HttpServletResponse response) {
        response.setCharacterEncoding("utf-8");
        response.setContentType("multipart/form-data");
        try {
            String fileName = new String(path.substring(path.lastIndexOf("/") + 1).getBytes());
            final String userAgent = request.getHeader("user-agent").toLowerCase();
            if (userAgent.contains("trident") || userAgent.contains("edge")) {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } else {
                fileName = new String(fileName.getBytes("utf-8"), "iso-8859-1");
            }
            response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
        } catch (UnsupportedEncodingException e1) {
            log.error("set code failure:", e1);
        }
        OutputStream os = null;
        InputStream inputStream = null;
        try {
            String filepath = this.storageConfig.getPath() + path;
            File file = new File(filepath);
            os = response.getOutputStream();
            inputStream = new FileInputStream(file);
            byte[] b = new byte[2048];
            int length;
            while ((length = inputStream.read(b)) != -1) {
                os.write(b, 0, length);
            }
            inputStream.close();
        } catch (IOException e) {
            log.error("Download fail:", e);
        } finally {
            try {
                if (null != inputStream) {
                    inputStream.close();
                }
                if (null != os) {
                    os.close();
                    os.flush();
                }
            } catch (IOException e) {
                log.error("Download fail:", e);
            }
        }
    }

    @Override
    public GeneralResult<Void> updateStatus(Long id, String status, Long userId) {
        String path = Software.class.getSimpleName() + "_" + id;
        boolean result = false;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Software software = this.softwareRepository.query(id);
            if (null == software) {
                return new GeneralResult<>(false, "文件不存在");
            }
            result = this.softwareRepository.updateStatus(id, status, userId);
        } catch (Exception e) {
            log.error("Update software status fail:", e);
            return new GeneralResult<>(false, "状态更新失败");
        }
        return result ? new GeneralResult<>(true, "状态更新成功") : new GeneralResult<>(false, "状态更新失败");
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public GeneralResult<Void> allot(Long tenantId, List<Long> softwareIds) {
        try {
            List<SoftwareTenant> softwareTenants = softwareTenantRepository.listByTenantId(tenantId);
            List<Long> allotIds = softwareTenants.stream().map(SoftwareTenant::getId).collect(Collectors.toList());
            if (!ListTool.isEmpty(allotIds)) {
                softwareTenantRepository.delete(allotIds);
            }
            List<SoftwareTenant> allotList = new ArrayList<>();
            for (Long softwareId : softwareIds) {
                allotList.add(new SoftwareTenant(softwareId, tenantId));
            }
            if (!ListTool.isEmpty(allotList)) {
                softwareTenantRepository.batchSave(allotList, 1000);
            }
            return new GeneralResult<>(true, "操作成功");
        } catch (Exception e) {
            log.error("software allot error: ", e);
            return new GeneralResult<>(false, "操作失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<List<Software>> listAllotSoftwares(Long tenantId) {
        try {
            List<SoftwareTenant> softwareTenants = softwareTenantRepository.listByTenantId(tenantId);
            List<Long> softwareIds =
                softwareTenants.stream().map(SoftwareTenant::getSoftwareId).collect(Collectors.toList());
            List<Software> list = new ArrayList<>();
            if (!ListTool.isEmpty(softwareIds)) {
                Pager pager = new Pager();
                pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
                pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
                pager.getSorter().put(Common.ID, Common.ONE);
                pager.setPage(1);
                pager.setRows(Integer.MAX_VALUE);
                pager.getParams().add(new Param(MapTools.simpleMap("id", StringUtils.join(softwareIds, ",")), Sign.IN));
                list = softwareRepository.list(pager, null);
            }
            return new GeneralResult<>(true, list, "查询成功");
        } catch (Exception e) {
            log.error("list allot tenants error: ", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }
}
