package com.bocloud.cop.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.TableLog;
import com.bocloud.cop.entity.TaskInstanceBak;
import com.bocloud.cop.entity.TaskNodeBak;
import com.bocloud.cop.entity.TaskNodeLogBak;
import com.bocloud.cop.intf.TaskInstanceBakService;
import com.bocloud.cop.model.InstanceGraph;
import com.bocloud.cop.model.InstanceNode;
import com.bocloud.cop.model.NodeConfig;
import com.bocloud.cop.model.result.HostResult;
import com.bocloud.cop.repository.TableLogRepository;
import com.bocloud.cop.repository.TaskInstanceBakRepository;
import com.bocloud.cop.repository.TaskNodeBakRepository;
import com.bocloud.cop.repository.TaskNodeLogBakRepository;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.JSONTools;
import com.megatron.common.utils.ListTool;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 备份任务Service接口实现类
 *
 * @author majiali
 * @version 1.0
 * @since 2022.11.14
 */
@Slf4j
@Service("taskInstanceBakService")
public class TaskInstanceBakServiceImpl implements TaskInstanceBakService {

    @Autowired
    private TaskInstanceBakRepository taskInstanceBakRepository;
    @Autowired
    private TaskNodeBakRepository taskNodeBakRepository;
    @Autowired
    private TableLogRepository tableLogRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private TaskNodeLogBakRepository taskNodeLogBakRepository;
    @Value("${spring.datasource.druid.dialect:mysql}")
    private String databaseDriver;

    @Override
    public GeneralResult<GridBean<TaskInstanceBak>> list(Pager pager, RequestContext context) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            String host = null;
            String startTime = null;
            String endTime = null;
            for (Param param : pager.getParams()) {
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("host")) {
                    host = paramMap.get("host").toString();
                    paramMap.remove("host");
                }
                if (paramMap.containsKey("gmtCreate")) {
                    String time = paramMap.get("gmtCreate").toString();
                    if(Sign.GET.name().equals(param.getSign().name())){
                        startTime = time;
                    }else if(Sign.LET.name().equals(param.getSign().name())){
                        endTime = time;
                    }
                }
            }
            if(StringUtils.isEmpty(startTime)||StringUtils.isEmpty(endTime)){
                return new GeneralResult<>(false, "日期必传！");
            }
            List<TaskInstanceBak> list = new ArrayList<>();
            int total = 0;
            //根据模板表查询出所有日志表 然后 使用并行流查询包含参数日期的库表
            List<TableLog> tableLogs = tableLogRepository.query("task_instance_bak");
            String finalTimeGet = startTime;
            String finalTimeLet = endTime;
            //获取到数据中包含所查询的日期的历史备份表
            List<String> tableList = tableLogs.parallelStream().map(item -> item.getName()).filter(item -> {
                List<TaskInstanceBak> query = new ArrayList<>();
                try {
                    query = taskInstanceBakRepository.query(item, finalTimeGet, finalTimeLet);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                return !ListTool.isEmpty(query);
            }).collect(Collectors.toList());
            List<String> finalInstanceTableNames = new ArrayList<>();
            List<String> finalInstanceHostTableNames = new ArrayList<>();
            //按照表明排序实际就是按照数据的创建时间排序
            if(!ListTool.isEmpty(tableList)){
                List<String> tableNames = tableList.stream().sorted((o1,o2) -> o2.compareTo(o1)).collect(Collectors.toList());
                finalInstanceTableNames.addAll(tableNames);
                for (String tableName : tableNames) {
                    int i = tableName.lastIndexOf("_");
                    if(i < 0){
                        continue;
                    }
                    String substring = tableName.substring(i+1);
                    finalInstanceHostTableNames.add("task_instance_host_bak_" + substring);
                }
            }
            if(!ListTool.isEmpty(finalInstanceTableNames)){
                for (int i = 0; i < finalInstanceTableNames.size(); i++) {
                    total += taskInstanceBakRepository.count(pager.getParams(),finalInstanceTableNames.get(i),finalInstanceHostTableNames.get(i), host);
                }
                list = taskInstanceBakRepository.list(pager, finalInstanceTableNames,finalInstanceHostTableNames,host);
            }
            for (TaskInstanceBak instance : list) {
                if (null == instance.getGmtFinish()) {
                    instance.setCost(0.00);
                } else {
                    BigDecimal start = new BigDecimal(instance.getGmtStart().getTime());
                    BigDecimal end = new BigDecimal(instance.getGmtFinish().getTime());
                    instance.setCost(end.subtract(start).divide(new BigDecimal(1000), 3, RoundingMode.HALF_UP).doubleValue());
                }
                instance.setOperator(redisTemplate.opsForValue().get("User_Name_" + instance.getCreatorId()));
            }
            GridBean<TaskInstanceBak> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, bean, "查询备份任务记录成功");
        } catch (Exception e) {
            log.error("List task record fail:", e);
            return new GeneralResult<>(false, "查询备份任务记录失败");
        }
    }

    @Override
    public GeneralResult<TaskInstanceBak> detail(Long id,String taskInstanceBakName) {
        try {
            TaskInstanceBak instanceBak = this.taskInstanceBakRepository.query(id,taskInstanceBakName);
            Assert.notNull(instanceBak, "数据不存在！");
            InstanceGraph graph = JSONObject.parseObject(instanceBak.getGraph(), InstanceGraph.class);
            List<InstanceNode> graphNodes = graph.getNodes();
            Map<String, InstanceNode> nodeMap = Maps.newHashMap();
            for (InstanceNode taskNode : graphNodes) {
                NodeConfig config = JSONObject.parseObject(taskNode.getConfig(), NodeConfig.class);
                nodeMap.put(config.getId(), taskNode);
            }
            String taskNodeBakName = "task_node";
            int i = taskInstanceBakName.lastIndexOf("_");
            if(i >= 0){
                String substring = taskInstanceBakName.substring(i+1);
                taskNodeBakName = "task_node_bak_" + substring;
            }
            List<TaskNodeBak> nodes = this.taskNodeBakRepository.listByInstance(instanceBak.getId(),taskNodeBakName);
            List<JSONObject> results = Lists.newArrayList();
            for (TaskNodeBak node : nodes) {
                if (nodeMap.containsKey(node.getNodeId())) {
                    InstanceNode taskNode = nodeMap.get(node.getNodeId());
                    taskNode.setId(node.getId());
                    taskNode.setStatus(node.getStatus());
                }
                if (null == node.getGmtFinish() || null == node.getGmtStart()) {
                    node.setCost(0.00);
                } else {
                    BigDecimal start = new BigDecimal(node.getGmtStart().getTime());
                    BigDecimal end = new BigDecimal(node.getGmtFinish().getTime());
                    node.setCost(
                            end.subtract(start).divide(new BigDecimal(1000), 3, RoundingMode.HALF_UP).doubleValue());
                }
                if (node.getCode().contains("maintain.backup") && null != node.getOutput()) {
                    GeneralResult<?> result = JSONObject.parseObject(node.getOutput(), GeneralResult.class);
                    List<HostResult> hostResults =
                            JSONArray.parseArray(JSONObject.toJSONString(result.getData()), HostResult.class);
                    if (!ListTool.isEmpty(hostResults)) {
                        for (HostResult hostResult : hostResults) {
                            if (hostResult.isFailed()) {
                                continue;
                            }
                            JSONObject object = new JSONObject();
                            object.put("host", hostResult.getHost());
                            JSONObject message = JSONTools.isJSONObj(hostResult.getMessage());
                            if (null == message) {
                                object.put("path", hostResult.getMessage());
                            } else {
                                object.put("path", message.getString("dest"));
                            }
                            results.add(object);
                        }
                    }
                }
            }
            if (null == instanceBak.getGmtFinish()) {
                instanceBak.setCost(0.00);
            } else {
                BigDecimal start = new BigDecimal(instanceBak.getGmtStart().getTime());
                BigDecimal end = new BigDecimal(instanceBak.getGmtFinish().getTime());
                instanceBak.setCost(end.subtract(start).divide(new BigDecimal(1000), 3, RoundingMode.HALF_UP).doubleValue());
            }
            JSONObject object = JSONObject.parseObject(instanceBak.getGraph());
            JSONArray array = new JSONArray();
            for (InstanceNode instanceNode : graphNodes) {
                JSONObject node = JSONObject.parseObject(JSONObject.toJSONString(instanceNode));
                node.put("input", JSONObject.parseObject(instanceNode.getInput()));
                node.put("config", JSONObject.parseObject(instanceNode.getConfig()));
                node.put("control", JSONObject.parseObject(instanceNode.getControl()));
                array.add(node);
            }
            object.put("nodes", array);
            instanceBak.setGraph(JSONObject.toJSONString(object));
            instanceBak.setResults(results);
            RequestContext user = JSONObject.parseObject(instanceBak.getProps(), RequestContext.class);
            if (null != user) {
                if (RequestContext.Catalog.Manager == user.getCatalog()) {
                    instanceBak.setOperator(redisTemplate.opsForValue().get("User_Name_" + instanceBak.getCreatorId()));
                } else {
                    instanceBak.setOperator(redisTemplate.opsForValue().get(user.getCatalog() + "_Name_" + instanceBak.getCreatorId()));
                }
            }
            return new GeneralResult<>(true, instanceBak, "查询备份任务记录详细成功");
        } catch (Exception e) {
            log.error("Query task record detail fail:", e);
            return new GeneralResult<>(false, "查询备份任务记录详细失败");
        }
    }

    @Override
    public GeneralResult<TaskNodeBak> detailNode(Long id,String taskInstanceBakName) {
        try {
            String taskNodeBakName = "task_node";
            int i = taskInstanceBakName.lastIndexOf("_");
            if(i >= 0){
                String substring = taskInstanceBakName.substring(i+1);
                taskNodeBakName = "task_node_bak_" + substring;
            }
            int countTable = taskNodeBakRepository.countTable(taskNodeBakName, databaseDriver);
            if(countTable<=0){
                TaskNodeBak taskNodeBak = new TaskNodeBak();
                return new GeneralResult<>(true, taskNodeBak, "查询备份任务节点详细成功");
            }
            TaskNodeBak node = this.taskNodeBakRepository.query(id,taskNodeBakName);
            Assert.notNull(node, "数据不存在！");
            if (null == node.getGmtFinish() || null == node.getGmtStart()) {
                node.setCost(0.00);
            } else {
                BigDecimal start = new BigDecimal(node.getGmtStart().getTime());
                BigDecimal end = new BigDecimal(node.getGmtFinish().getTime());
                node.setCost(end.subtract(start).divide(new BigDecimal(1000), 3, RoundingMode.HALF_UP).doubleValue());
            }
            List<Object> successes = Lists.newArrayList();
            List<Object> failures = Lists.newArrayList();
            if (null != node.getOutput()) {
                GeneralResult<?> result = JSONObject.parseObject(node.getOutput(), GeneralResult.class);
                if (!node.getCode().contains("method") && !node.getCode().contains("database")
                        && !node.getCode().contains("resource")) {
                    if (null != result.getData()) {
                        List<HostResult> results =
                                JSONArray.parseArray(JSONObject.toJSONString(result.getData()), HostResult.class);
                        for (HostResult hostResult : results) {
                            if (hostResult.isSuccess()) {
                                successes.add(hostResult);
                            } else {
                                failures.add(hostResult);
                            }
                        }
                    }
                }
            }
            node.setSuccesses(successes);
            node.setFailures(failures);
            return new GeneralResult<>(true, node, "查询备份任务节点详细成功");
        } catch (Exception e) {
            log.error("Query record detail fail:", e);
            return new GeneralResult<>(false, "查询备份任务节点详细失败");
        }
    }

    @Override
    public GeneralResult<TaskNodeLogBak> getLog(Long nodeId,String taskInstanceBakName, Integer priority, Boolean end) {
        try {
            String taskNodeLogBakName = "task_node_log";
            int i = taskInstanceBakName.lastIndexOf("_");
            if(i >= 0){
                String substring = taskInstanceBakName.substring(i+1);
                taskNodeLogBakName = "task_node_log_bak_" + substring;
            }
            int countTable = taskNodeLogBakRepository.countTable(taskNodeLogBakName, databaseDriver);
            if(countTable<=0){
                TaskNodeLogBak taskNodeLogBak = new TaskNodeLogBak();
                return new GeneralResult<>(true, taskNodeLogBak, "查询任务节点日志成功");
            }
            if (null != end && end) {
                TaskNodeLogBak log = taskNodeLogBakRepository.get(nodeId, priority,taskNodeLogBakName);
                if (null == log) {
                    log = new TaskNodeLogBak();
                    log.setLast(true);
                } else {
                    TaskNodeLogBak nextLog = taskNodeLogBakRepository.get(nodeId, priority,taskNodeLogBakName);
                    log.setLast(null == nextLog);
                }
                if (!StringUtils.isEmpty(log.getContent())) {
                    StringBuilder builder = new StringBuilder();
                    List<String> contents = JSONObject.parseArray(log.getContent(), String.class);
                    for (String content : contents) {
                        builder.append(content + "\n");
                    }
                    log.setContent(builder.toString());
                }
                return new GeneralResult<>(true, log, "查询任务节点日志成功");
            }
            TaskNodeLogBak log = null;
            if (null == priority) {
                log = taskNodeLogBakRepository.getLast(nodeId,taskNodeLogBakName);
                if (null == log) {
                    log = new TaskNodeLogBak();
                    log.setPriority(1);
                }
            } else {
                log = taskNodeLogBakRepository.get(nodeId, priority,taskNodeLogBakName);
                if (null == log) {
                    log = new TaskNodeLogBak();
                    log.setPriority(priority);
                }
            }
            if (!StringUtils.isEmpty(log.getContent())) {
                StringBuilder builder = new StringBuilder();
                List<String> contents = JSONObject.parseArray(log.getContent(), String.class);
                for (String content : contents) {
                    builder.append(content + "\n");
                }
                log.setContent(builder.toString());
            }
            return new GeneralResult<>(true, log, "查询任务节点日志成功");
        } catch (Exception e) {
            log.error("Query record fail:", e);
            return new GeneralResult<>(false, "查询任务节点日志失败");
        }
    }

}
