package com.bocloud.cop.service;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.support.CronExpression;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.constant.CacheConstant;
import com.bocloud.cop.entity.CommandInstance;
import com.bocloud.cop.entity.Identification;
import com.bocloud.cop.entity.NetworkSegment;
import com.bocloud.cop.entity.NodeRelation;
import com.bocloud.cop.entity.Task;
import com.bocloud.cop.entity.TaskHost;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.TaskInstanceHost;
import com.bocloud.cop.entity.TaskNode;
import com.bocloud.cop.enums.ApplyStatus;
import com.bocloud.cop.enums.AuthenMode;
import com.bocloud.cop.enums.Catalog;
import com.bocloud.cop.enums.Category;
import com.bocloud.cop.enums.ExecuteMode;
import com.bocloud.cop.enums.NodeCode;
import com.bocloud.cop.enums.Status;
import com.bocloud.cop.enums.TaskHostType;
import com.bocloud.cop.intf.CommandService;
import com.bocloud.cop.intf.TaskHostService;
import com.bocloud.cop.intf.TaskService;
import com.bocloud.cop.model.NodeConfig;
import com.bocloud.cop.model.NodeControl;
import com.bocloud.cop.model.Target;
import com.bocloud.cop.model.TaskBean;
import com.bocloud.cop.model.TaskEdge;
import com.bocloud.cop.model.TaskGraph;
import com.bocloud.cop.model.WorkerLoadInfo;
import com.bocloud.cop.model.params.TaskTransferParams;
import com.bocloud.cop.repository.CommandInstanceRepository;
import com.bocloud.cop.repository.IdentificationRepository;
import com.bocloud.cop.repository.NodeRelationRepository;
import com.bocloud.cop.repository.TaskHostRepository;
import com.bocloud.cop.repository.TaskInstanceHostRepository;
import com.bocloud.cop.repository.TaskInstanceRepository;
import com.bocloud.cop.repository.TaskNodeRepository;
import com.bocloud.cop.repository.TaskRepository;
import com.bocloud.cop.service.utils.NodeHelper;
import com.bocloud.cop.service.utils.worker.TaskWorkerHelper;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Sign;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.IDFactory;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 任务抽象Service接口实现类
 *
 * @author wei wei
 * @version 1.0
 * @since 2016.08.17
 */
@Slf4j
@Service("taskService")
public class TaskServiceImpl implements TaskService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private TaskRepository taskRepository;
    @Autowired
    private TaskInstanceRepository taskInstanceRepository;
    @Autowired
    private TaskNodeRepository taskNodeRepository;
    @Autowired
    private NodeRelationRepository nodeRelationRepository;
    @Autowired
    private IdentificationRepository identificationRepository;
    @Autowired
    private TaskHostService taskHostService;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private CommandService commandService;
    @Autowired
    private CommandInstanceRepository commandInstanceRepository;
    @Autowired
    private TaskHostRepository taskHostRepository;
    @Autowired
    private TaskInstanceHostRepository taskInstanceHostRepository;

    @Override
    public GeneralResult<GridBean<Task>> list(Pager pager, RequestContext context) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            if (com.megatron.common.model.RequestContext.Catalog.Tenant == context.getCatalog()) {
                pager.getParams().add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
            } else if (com.megatron.common.model.RequestContext.Catalog.User == context.getCatalog()) {
                pager.getParams().add(new Param(MapTools.simpleMap("creatorId", context.getTarget()), Sign.EQ));
                pager.getParams().add(new Param(MapTools.simpleMap("catalog", context.getCatalog()), Sign.EQ));
            }
            GridBean<Task> bean = null;
            if (null != pager.getSimple() && pager.getSimple()) {
                if (com.megatron.common.model.RequestContext.Catalog.Tenant == context.getCatalog()) {
                    pager.getParams().add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
                }
                int total = taskRepository.count(pager.getParams());
                List<SimpleBean> list = taskRepository.list(pager.getParams(), pager.getSorter());
                List<Task> tasks = new ArrayList<>();
                for (SimpleBean simpleBean : list) {
                    Task task = new Task();
                    task.setId(simpleBean.getId());
                    task.setName(simpleBean.getName());
                    task.setStatus(simpleBean.getValue());
                    tasks.add(task);
                }
                bean = new GridBean<>(pager.getPage(), pager.getRows(), total, tasks);
            } else {
                int total = taskRepository.count(pager.getParams());
                List<Task> list = taskRepository.list(pager);
                for (Task task : list) {
                    if (com.megatron.common.model.RequestContext.Catalog.Tenant == task.getCatalog()) {
                        task.setCreator(redisTemplate.opsForValue().get("Tenant_Name_" + task.getCreatorId()));
                        if (null != task.getMenderId()) {
                            task.setMender(redisTemplate.opsForValue().get("Tenant_Name_" + task.getMenderId()));
                        }
                    } else {
                        task.setCreator(redisTemplate.opsForValue().get("User_Name_" + task.getCreatorId()));
                        if (null != task.getMenderId()) {
                            task.setMender(redisTemplate.opsForValue().get("User_Name_" + task.getMenderId()));
                        }
                    }
                    if (null != task.getTenantId() && task.getTenantId() != 0) {
                        task.setTenantName(redisTemplate.opsForValue().get("Tenant_Name_" + task.getTenantId()));
                    }
                    // 执行方式
                    if (StringUtils.hasText(task.getTarget())) {
                        Target target = JSONObject.parseObject(task.getTarget(), Target.class);
                        task.setExecuteType(target.getExecuteType());
                    }
                }
                bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<>(true, bean, "查询任务成功");
        } catch (

        Exception e) {
            log.error("List task fail:", e);
            return new GeneralResult<>(false, "查询任务失败");
        }
    }

    @Override
    public GeneralResult<Task> create(Task task, RequestContext context) {
        try {
            // 判断某种类的任务名称是否存在
            boolean flag = taskRepository.exists(task.getName(), task.getCode(), context.getTenant());
            if (flag) {
                return new GeneralResult<>(false, "该名称已存在，请重新输入!");
            }
            GeneralResult<Task> result = saveTask(task, context, true, false);
            if (result.isSuccess()) {
                return new GeneralResult<>(true, result.getData(), "添加任务成功");
            }
            return result;
        } catch (Exception e) {
            log.error("query task error:", e);
            return new GeneralResult<>(false, "添加任务失败");
        }
    }

    @Override
    public GeneralResult<Task> onekeyRelease(JSONObject params, RequestContext context) {
        GeneralResult<Task> result;
        if (null != params.getLong("id")) {
            TaskBean bean = JSON.parseObject(JSONObject.toJSONString(params), TaskBean.class);
            result = modify(bean, context);
        } else {
            Task task = JSON.parseObject(JSONObject.toJSONString(params), Task.class);
            result = create(task, context);
        }
        if (result.isFailed()) {
            return new GeneralResult<>(false, result.getMessage());
        }
        Task task = result.getData();
        GeneralResult<Void> rt = updateStatus(task.getId(), ApplyStatus.APPROVED.name(), context.getTarget());
        if (rt.isFailed()) {
            return new GeneralResult<>(false, "发布失败");
        }
        return new GeneralResult<>(true, "操作成功");
    }

    private GeneralResult<Task> saveTask(Task task, RequestContext context, boolean isAdd, boolean temp) {
        TaskGraph graph = JSONObject.parseObject(JSONObject.toJSONString(task.getGraph()), TaskGraph.class);
        List<TaskNode> nodes = graph.getNodes();
        List<TaskEdge> edges = graph.getEdges();
        Map<String, String> ingoreNodes = NodeHelper.getIngoreNodes(nodes);
        Map<String, List<String>> preLinks = NodeHelper.getPreLinks(edges, ingoreNodes);
        Map<String, List<TaskEdge>> nextEdges = NodeHelper.getNextEdges(edges, ingoreNodes);
        Map<String, TaskNode> nodeMaps = new HashMap<>();
        for (TaskNode node : nodes) {
            NodeConfig config = JSONObject.parseObject(node.getConfig(), NodeConfig.class);
            nodeMaps.put(config.getId(), node);
        }
        // 默认场景
        if (null == task.getSceneId()) {
            task.setSceneId(1L);
        }
        try {
            if (!StringUtils.hasText(task.getCode())) {
                task.setCode("standard");
            }
            // 校验cron表达式
            boolean repeat = false;
            if (task.getCode().contains(Catalog.Task.SCHEDULED.getValue())) {
                GeneralResult<Void> result = isValid(task.getCron());
                if (result.isFailed()) {
                    return new GeneralResult<>(false, result.getMessage());
                }
                CronExpression expression = CronExpression.parse(task.getCron());
                LocalDateTime time = expression.next(LocalDateTime.now());
                if (null != time && null != expression.next(time)) {
                    repeat = true;
                }
                task.setStatus(ApplyStatus.CREATED.name());
            }
            // 判断是否存在闭环
            boolean suspend = false;
            for (TaskNode node : nodes) {
                String code = node.getModuleType();
                NodeConfig config = JSONObject.parseObject(node.getConfig(), NodeConfig.class);
                if (null == config || config.isIgnore() || code.contains(NodeCode.Ctrl.Start.getValue())
                    || code.contains(NodeCode.Ctrl.Stop.getValue())) {
                    continue;
                }
                node.setNodeId(config.getId());
                if (code.contains(NodeCode.Ctrl.Judge.getValue())) {
                    List<String> links = preLinks.get(config.getId());
                    if (ListTool.isEmpty(links)) {
                        return new GeneralResult<>(false, "判断节点的入度数量为0");
                    }
                    for (String nodeId : links) {
                        TaskNode preNode = nodeMaps.get(nodeId);
                        NodeControl nodeControl = JSONObject.parseObject(preNode.getControl(), NodeControl.class);
                        if (nodeControl.isSkippable()) {
                            return new GeneralResult<>(false, "判断节点前置节点不允许失败跳过，请检查");
                        }
                    }
                    List<TaskEdge> list = nextEdges.get(config.getId());
                    if (ListTool.isEmpty(list) || list.size() != 2) {
                        return new GeneralResult<>(false, "判断节点的出度有误，请检查");
                    }
                    if (list.get(0).isDicision() == list.get(1).isDicision()) {
                        return new GeneralResult<>(false, "判断节点的出度有误，请检查");
                    }
                    for (TaskEdge edge : list) {
                        String target = edge.getTarget();
                        target = ingoreNodes.containsKey(target) ? ingoreNodes.get(target) : target;
                        links = preLinks.get(target);
                        if (null != links && links.size() > 1) {
                            return new GeneralResult<>(false, "判断节点所连接的后置节点存在其他前置节点连接，请检查");
                        }
                    }
                }
                if (NodeHelper.isCycle(node.getNodeId(), preLinks, nodes.size() - ingoreNodes.size())) {
                    return new GeneralResult<>(false, "作业存在闭环无法创建");
                }
                if (code.contains("software")
                    || (code.contains("maintain") && !code.contains("method") && !code.contains("database"))) {
                    JSONObject input = JSONObject.parseObject(node.getInput());
                    JSONObject target = input.getJSONObject("target");
                    JSONObject depend = input.getJSONObject("depend");
                    boolean exists = false;
                    if (null != target) {
                        JSONArray array = target.getJSONArray("data");
                        if (null != array && array.size() > 0) {
                            exists = true;
                        }
                    }
                    if (null != depend && "host".equals(depend.getString("type"))) {
                        JSONArray array = depend.getJSONArray("data");
                        if (null != array && array.size() > 0) {
                            exists = true;
                        }
                    }
                    if (!exists) {
                        return new GeneralResult<>(false, "作业节点【" + node.getName() + "】没有关联目标主机");
                    }
                }
                NodeControl control = JSONObject.parseObject(node.getControl(), NodeControl.class);
                if (null != control && control.isSuspensible()) {
                    suspend = control.isSuspensible();
                }
                if (repeat) {
                    if (code.contains("software")) {
                        return new GeneralResult<>(false, "该定时任务会重复执行多次，故不可存在软件安装节点，当前任务无法创建成功");
                    }
                }
            }
            task.setHasSuspend(suspend);
            if (suspend && StringUtils.hasText(task.getCron())) {
                return new GeneralResult<>(false, "作业存在完成后暂停节点，定时任务无法创建成功");
            }
            // isAdd为true新增任务
            if (isAdd && !temp) {
                task.setId(null);
                task.preSave(context.getTarget(), context.getTarget(), context.getTenant(), context.getCatalog());
                task.setUuid(IDFactory.instance().uuid());
                task.setGmtCreate(new Date());
                task.setGmtModify(new Date());
                task.setStatus(com.megatron.common.model.RequestContext.Catalog.Manager == task.getCatalog()
                    ? ApplyStatus.DEVELOPING.name() : ApplyStatus.APPROVED.name());
                task.setGraph(JSONObject.toJSONString(task.getGraph()));
                task.setEmailEnabled(null == task.getEmailEnabled() ? false : task.getEmailEnabled());
                this.taskRepository.save(task);
            } else {
                // 更新任务
                task.setGmtModify(new Date());
                task.setMenderId(context.getTarget());
                task.setGraph(JSONObject.toJSONString(task.getGraph()));
                this.taskRepository.update(task);
            }
        } catch (Exception e) {
            log.error("Save task fail:", e);
            return new GeneralResult<>(false, "保存任务失败");
        }
        return new GeneralResult<>(true, task, "保存任务成功");
    }

    /**
     * 校验cron表达式是否合法
     *
     * @param expression
     * @return
     */
    @Override
    public GeneralResult<Void> isValid(String expression) {
        String message = "保存任务失败：Cron表达式有误或者执行时间是过去的时间";
        try {
            String[] fields = StringUtils.tokenizeToStringArray(expression, " ");
            List<Integer> years = Lists.newArrayList();
            if (null == fields || fields.length < 6 || fields.length > 7) {
                return new GeneralResult<>(false, message);
            } else if (fields.length == 7) {
                if (fields[6].contains("/")) {
                    return new GeneralResult<>(false, message);
                }
                String[] values = StringUtils.delimitedListToStringArray(fields[6], ",");
                for (String value : values) {
                    int[] range = getRange(value, 0, 10000);
                    for (int i : range) {
                        years.add(i);
                    }
                }
                int currentYear = Calendar.getInstance().get(Calendar.YEAR);
                boolean flag = false;
                for (Integer year : years) {
                    if (year >= currentYear) {
                        flag = true;
                    }
                }
                if (!flag) {
                    return new GeneralResult<>(false, message);
                }
                expression =
                    fields[0] + " " + fields[1] + " " + fields[2] + " " + fields[3] + " " + fields[4] + " " + fields[5];
            }
            CronExpression generator = CronExpression.parse(expression);
            LocalDateTime time = generator.next(LocalDateTime.now());
            if (null == time) {
                return new GeneralResult<>(false, message);
            }
            if (fields.length == 7) {
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(Date.from(time.atZone(ZoneId.systemDefault()).toInstant()));
                int execYear = calendar.get(Calendar.YEAR);
                for (Integer year : years) {
                    if (execYear > year) {
                        return new GeneralResult<>(false, message);
                    }
                }
            }
            return new GeneralResult<>(true, "success");
        } catch (Exception e) {
            log.warn("Cron expression is invalid");
            return new GeneralResult<>(false, message);
        }
    }

    private int[] getRange(String field, int min, int max) {
        int[] result = new int[2];
        if (!field.contains("-")) {
            result[0] = result[1] = Integer.valueOf(field);
        } else {
            String[] split = StringUtils.delimitedListToStringArray(field, "-");
            if (split.length > 2) {
                throw new IllegalArgumentException("Range has more than two fields: '" + field + "' in expression");
            }
            result[0] = Integer.valueOf(split[0]);
            result[1] = Integer.valueOf(split[1]);
        }
        if (result[0] >= max || result[1] >= max) {
            throw new IllegalArgumentException("Range exceeds maximum (" + max + "): '" + field + "' in expression");
        }
        if (result[0] < min || result[1] < min) {
            throw new IllegalArgumentException("Range less than minimum (" + min + "): '" + field + "' in expression");
        }
        if (result[0] > result[1]) {
            throw new IllegalArgumentException("Invalid inverted range: '" + field + "' in expression");
        }
        return result;
    }

    @Override
    public GeneralResult<Task> modify(TaskBean bean, RequestContext context) {
        String path = Task.class.getSimpleName() + "_" + bean.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Task task = this.taskRepository.query(bean.getId());
            Assert.notNull(task, "数据不存在！");
            if (!task.getName().equals(bean.getName())) {
                // 判断某种类的任务名称是否存在
                boolean flag = taskRepository.exists(bean.getName(), bean.getCode(), context.getTenant());
                if (flag) {
                    return new GeneralResult<>(false, "该名称已存在，请重新输入!");
                }
            }
            if (task.getStatus().equals(Status.Inst.WAITTING.name())) {
                return new GeneralResult<>(false, "修改失败：任务正在执行");
            }
            if (task.getStatus().equals(Status.Inst.END.name())) {
                return new GeneralResult<>(false, "修改失败：定时任务已结束");
            }
            if ("scheduled".equalsIgnoreCase(task.getCode())) {
                task.setRunTime(bean.getRunTime());
                task.setIsLoop(bean.getIsLoop());
                task.setCron(bean.getCron());
                task.setName(bean.getName());
                task.setEmailEnabled(bean.getEmailEnabled());
                task = (Task)settingCron(task);
                GeneralResult<Void> result = isValid(task.getCron());
                if (result.isFailed()) {
                    return new GeneralResult<>(result.isSuccess(), result.getMessage());
                }
                task.setGmtModify(new Date());
                task.setMenderId(context.getTarget());
                this.taskRepository.update(task);
                return new GeneralResult<>(true, "修改任务成功");
            }
            BeanUtils.copyProperties(bean, task);
            GeneralResult<Task> result = saveTask(task, context, false, false);
            if (result.isFailed()) {
                return result;
            }
            return new GeneralResult<>(true, result.getData(), "修改任务成功");
        } catch (Exception e) {
            log.error("Update task fail:", e);
            return new GeneralResult<>(false, "修改任务失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = Task.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Task task = this.taskRepository.query(id);
            Assert.notNull(task, "数据不存在！");
            Assert.isTrue(!task.getStatus().equals(Status.Inst.WAITTING.name()), "任务正在执行");
            boolean result = this.taskRepository.delete(id, userId);
            return result ? new GeneralResult<>(true, "删除任务成功") : new GeneralResult<>(false, "删除任务失败");
        } catch (Exception e) {
            log.error("Remove task fail:", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    @Override
    public GeneralResult<Task> detail(Long id) {
        try {
            Task task = this.taskRepository.query(id);
            Assert.notNull(task, "数据不存在！");
            if (com.megatron.common.model.RequestContext.Catalog.Tenant == task.getCatalog()) {
                task.setCreator(redisTemplate.opsForValue().get("Tenant_Name_" + task.getCreatorId()));
                if (null != task.getMenderId()) {
                    task.setMender(redisTemplate.opsForValue().get("Tenant_Name_" + task.getMenderId()));
                }
            } else {
                task.setCreator(redisTemplate.opsForValue().get("User_Name_" + task.getCreatorId()));
                if (null != task.getMenderId()) {
                    task.setMender(redisTemplate.opsForValue().get("User_Name_" + task.getMenderId()));
                }
            }
            if (null != task.getTenantId() && task.getTenantId() != 0) {
                task.setTenantName(redisTemplate.opsForValue().get("Tenant_Name_" + task.getTenantId()));
            }
            // 执行方式
            if (StringUtils.hasText(task.getTarget())) {
                Target target = JSONObject.parseObject(task.getTarget(), Target.class);
                task.setExecuteType(target.getExecuteType());
            }
            return new GeneralResult<>(true, task, "查询任务详细成功");
        } catch (Exception e) {
            log.error("Query task detail fail:", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    @Override
    public GeneralResult<Task> clone(Long id) {
        try {
            Task task = this.taskRepository.query(id);
            Assert.notNull(task, "数据不存在！");
            task.setId(null);
            return new GeneralResult<>(true, task, "克隆成功");
        } catch (Exception e) {
            log.error("Clone task failed!", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    @Override
    public Task basic(Long id) {
        try {
            return this.taskRepository.query(id);
        } catch (Exception e) {
            log.error("Query task fail:", e);
            return null;
        }
    }

    @Override
    public TaskInstance initInstance(Long taskId, String mode, String type, RequestContext context) {
        Task task = null;
        try {
            task = this.taskRepository.query(taskId);
            Assert.notNull(task, "数据不存在！");
            // 创建任务实例
            Long userId = context.getTarget();
            TaskInstance instance = new TaskInstance(userId, userId, userId, task.getName(), Status.Inst.CREATED.name(),
                task.getRemark(), task.getCode(), task.getGraph().toString(), task.getCron(), context.getTenant(),
                context.getCatalog());
            if (!StringUtils.hasText(type)) {
                type = Catalog.Task.STANDARD.getValue();
            }
            instance.setType(type);
            instance.setUuid(IDFactory.instance().uuid());
            instance.setName(instance.getName() + "_" + instance.getUuid().substring(0, 8));
            instance.setTaskId(taskId);
            if (!StringUtils.hasText(mode)) {
                mode = ExecuteMode.WEB.name();
            }
            instance.setExecuteMode(mode);
            instance.setTenantId(context.getTenant());
            instance.setIsBackup(false);
            instance.setProps(JSONObject.toJSONString(context));
            instance.setEmailEnabled(task.getEmailEnabled());
            this.taskInstanceRepository.save(instance);
            List<Long> commandIds = new ArrayList<>();
            // 解析任务节点
            TaskGraph graph = JSONObject.parseObject(instance.getGraph(), TaskGraph.class);
            List<TaskNode> nodes = graph.getNodes();
            Map<String, Long> nodeMap = Maps.newHashMap();
            boolean isBackup = true;
            List<TaskInstanceHost> instanceHosts = new ArrayList<>();
            for (TaskNode node : nodes) {
                NodeConfig config = JSONObject.parseObject(node.getConfig(), NodeConfig.class);
                if (null == config || config.isIgnore()) {
                    continue;
                }
                String code = node.getModuleType();
                if (!code.contains("start") && !code.contains("stop") && !code.contains("backup")) {
                    isBackup = false;
                }
                if (code.startsWith("task")) {
                    node.setCatalog(Catalog.Node.TASK.name());
                    if (code.contains("maintain")) {
                        node.setCategory(Category.Node.MAINTAIN.name());
                    } else if (code.contains("software")) {
                        node.setCategory(Category.Node.SOFTWARE.name());
                    }
                } else {
                    node.setCatalog(Catalog.Node.CTRL.name());
                    if (code.contains("start")) {
                        node.setCategory(Category.Node.START.name());
                    } else if (code.contains("stop")) {
                        node.setCategory(Category.Node.STOP.name());
                    } else if (code.contains("judge")) {
                        node.setCategory(Category.Node.JUDGE.name());
                    }
                }
                List<TaskInstanceHost> taskInstanceHosts = new ArrayList<>();
                if (!code.contains("ctrl")) {
                    JSONObject input = JSONObject.parseObject(node.getInput());
                    JSONObject target = input.getJSONObject("target");
                    if (null != target) {
                        List<JSONObject> hosts = JSONObject.parseArray(target.getString("data"), JSONObject.class);
                        if (ListTool.isEmpty(hosts)) {
                            hosts = new ArrayList<>();
                        }
                        for (JSONObject host : hosts) {
                            // 记录任务实例和主机关系
                            TaskInstanceHost taskInstanceHost = new TaskInstanceHost();
                            taskInstanceHost.setHost(host.getString("host"));
                            taskInstanceHost.setInstanceId(instance.getId());
                            taskInstanceHosts.add(taskInstanceHost);
                            if (null == host.getLong("id")) {
                                continue;
                            }
                            TaskHost taskHost = taskHostRepository.query(host.getLong("id"));
                            if (null == taskHost) {
                                continue;
                            }
                            host.put("segmentId", taskHost.getSegmentId());
                            if (TaskHostType.ConnProtocol.SaltStack.name()
                                .equalsIgnoreCase(target.getString("executeType"))) {
                                if (null == taskHost.getSaltId()) {
                                    log.warn("服务器(" + taskHost.getName() + ")缺少saltId");
                                    return null;
                                }
                                host.put("saltId", taskHost.getSaltId());
                                // 必须拥有saltMinionId,salt返回的执行结果key是minionId
                                if (null == taskHost.getSaltMinionId()) {
                                    log.warn("服务器(" + taskHost.getName() + ")缺少saltMinionId");
                                    return null;
                                }
                                host.put("saltMinionId", taskHost.getSaltMinionId());
                            }
                        }
                        if (AuthenMode.BATCH.name().equalsIgnoreCase(target.getString("authenMode"))) {
                            Identification ident = identificationRepository.query(target.getLong("identId"));
                            if (null == ident) {
                                this.taskInstanceRepository.remove(instance.getId(), userId);
                                log.warn("认证账户为空，任务实例化失败");
                                return null;
                            }
                            for (JSONObject host : hosts) {
                                host.put("user", ident.getUsername());
                                host.put("pasd", ident.getPassword());
                            }
                        } else if (AuthenMode.ACCOUNT.name().equalsIgnoreCase(target.getString("authenMode"))) {
                            for (JSONObject host : hosts) {
                                Long identId = host.getLong("identId");
                                if (null == identId) {
                                    log.warn("认证账户为空，任务实例化失败");
                                    this.taskInstanceRepository.remove(instance.getId(), userId);
                                    return null;
                                }
                                Identification ident = identificationRepository.query(identId);
                                if (null == ident) {
                                    log.warn("认证账户为空，任务实例化失败");
                                    this.taskInstanceRepository.remove(instance.getId(), userId);
                                    return null;
                                }
                                host.put("user", ident.getUsername());
                                host.put("pasd", ident.getPassword());
                            }
                        }
                        target.put("data", hosts);
                        input.put("target", target);
                        node.setInput(input.toString());
                    }
                }
                node.setUuid(IDFactory.instance().uuid());
                node.setName(node.getName() + "_" + node.getUuid().substring(0, 8));
                node.preSave(userId, userId, instance.getId(), Status.Node.READY.name());
                node.setNodeId(config.getId());
                this.taskNodeRepository.save(node);
                if (!taskInstanceHosts.isEmpty()) {
                    taskInstanceHosts.forEach(taskInstanceHost -> {
                        taskInstanceHost.setNodeId(node.getId());
                        taskInstanceHost.setNodeName(node.getName());
                    });
                    instanceHosts.addAll(taskInstanceHosts);
                }
                nodeMap.put(node.getNodeId(), node.getId());

                if (node.getModuleType().contains("maintain.script")) {
                    List<Long> commands = commandService.getCommands(node.getInput());
                    for (Long command : commands) {
                        if (!commandIds.contains(command)) {
                            commandIds.add(command);
                        }
                    }
                }
            }
            if (isBackup) {
                instance.setIsBackup(isBackup);
                taskInstanceRepository.update(instance);
            }
            // 解析任务节点关系
            List<TaskEdge> edges = graph.getEdges();
            Map<String, String> ingoreNodes = NodeHelper.getIngoreNodes(nodes);
            if (!ListTool.isEmpty(edges)) {
                List<NodeRelation> relations = new ArrayList<>();
                for (TaskEdge edge : edges) {
                    if (edge.isIgnore()) {
                        continue;
                    }
                    String source = edge.getSource();
                    source = ingoreNodes.containsKey(source) ? ingoreNodes.get(source) : source;
                    String target = edge.getTarget();
                    target = ingoreNodes.containsKey(target) ? ingoreNodes.get(target) : target;
                    NodeRelation relation = new NodeRelation(nodeMap.get(source), nodeMap.get(target), instance.getId(),
                        Status.Node.READY.name(), edge.isDicision());
                    relations.add(relation);
                }
                if (!relations.isEmpty()) {
                    this.nodeRelationRepository.batchSave(relations);
                }
            }
            // 处理命令与任务实例关系
            List<CommandInstance> commandInstances = new ArrayList<>();
            for (Long commandId : commandIds) {
                commandInstances.add(new CommandInstance(commandId, instance.getId()));
            }
            if (!commandInstances.isEmpty()) {
                commandInstanceRepository.batchSave(commandInstances, 100);
            }
            // 处理任务实例链路数据
            if (!instanceHosts.isEmpty()) {
                taskInstanceHostRepository.batchSave(instanceHosts, 100);
            }
            taskInstanceRepository.updateTime(instance.getId());
            instance.setContext(context);
            instance.setNodes(nodes);
            return instance;
        } catch (Exception e) {
            log.error("initInstance [{}] exception !", taskId, e);
            return null;
        }
    }

    @Override
    public GeneralResult<Void> suspend(Long id, Long userId) {
        String path = Task.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            boolean result = taskRepository.updateStatus(id, Status.Inst.SUSPENDED.name(), userId);
            return result ? new GeneralResult<>(true, "暂停定时任务成功!") : new GeneralResult<>(false, "暂停定时任务失败!");
        } catch (Exception e) {
            log.error("stop crontask fail:", e);
            return new GeneralResult<>(false, "暂停定时任务失败!");
        }
    }

    @Override
    public GeneralResult<Void> active(Long id, Long userId) {
        String path = Task.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            boolean result = taskRepository.updateStatus(id, Status.Inst.WAITTING.name(), userId);
            return result ? new GeneralResult<>(true, "启动定时任务成功!") : new GeneralResult<>(false, "启动定时任务失败!");
        } catch (Exception e) {
            log.error("start crontask fail:", e);
            return new GeneralResult<>(false, "启动定时任务失败!");
        }
    }

    @Override
    public GeneralResult<Void> transferCron(Long id, TaskTransferParams params, RequestContext context) {
        // 设置一次性执行Cron
        params = (TaskTransferParams)settingCron(params);
        String crons = params.getCron();
        String path = Task.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Task task = this.taskRepository.query(id);
            Assert.notNull(task, "数据不存在！");
            if (task.getHasSuspend()) {
                return new GeneralResult<>(false, "任务存在完成后暂停节点，无法转换定时任务");
            }
            GeneralResult<Void> result = isValid(crons);
            if (result.isFailed()) {
                return result;
            }
            // 判断是否存在多次
            boolean repeat = hasNext(crons);
            if (repeat) {
                TaskGraph graph = JSONObject.parseObject(task.getGraph().toString(), TaskGraph.class);
                List<TaskNode> nodes = graph.getNodes();
                for (TaskNode node : nodes) {
                    String code = node.getModuleType();
                    if (!StringUtils.hasText(code)) {
                        continue;
                    }
                    if (code.contains("software")) {
                        return new GeneralResult<>(false, "该定时任务会重复执行多次，故不可存在软件安装节点，当前任务无法创建成功");
                    }
                }
            }
            task.setName(task.getName());
            task.setCron(crons);
            task.setCode(Catalog.Task.SCHEDULED.getValue());
            task.setStatus(ApplyStatus.CREATED.name());
            String content = "常规任务【" + task.getName() + "】生成定时任务成功";
            task.setGmtCreate(new Date());
            task.setGmtModify(task.getGmtCreate());
            task.setIsLoop(params.getIsLoop());
            task.setRunTime(params.getRunTime());
            if (params.getIsNew()) {
                task.setId(null);
                task.setUuid(IDFactory.instance().uuid());
                task.preSave(context.getTarget(), context.getTarget(), context.getTenant(), context.getCatalog());
                taskRepository.save(task);
            } else {
                task.setMenderId(context.getTarget());
                taskRepository.update(task);
                content = "常规任务【" + task.getName() + "】转换定时任务成功";
            }
            return new GeneralResult<>(true, content);
        } catch (Exception e) {
            log.error("start crontask fail:", e);
            return new GeneralResult<>(false, "常规任务生成定时任务失败");
        }
    }

    @Override
    public GeneralResult<Void> updateStatus(Long id, String status, Long userId) {
        String path = Task.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            taskRepository.updateStatus(id, status, userId);
            return new GeneralResult<>(true, "修改任务状态成功");
        } catch (Exception e) {
            log.error("update task status fail:", e);
            return new GeneralResult<>(false, "修改任务状态失败");
        }
    }

    @Override
    public GeneralResult<Task> get(String uuid) {
        try {
            Task task = this.taskRepository.get(uuid);
            Assert.notNull(task, "数据不存在！");
            return new GeneralResult<>(true, task, "查询任务详细成功");
        } catch (Exception e) {
            log.error("Query task detail fail:", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    @Override
    public GeneralResult<Void> updateExecute(Long id, Date date) {
        String path = Task.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            taskRepository.updateExecute(id, date);
            return new GeneralResult<>(true, "修改任务执行时间成功");
        } catch (Exception e) {
            log.error("Update task execute time fail:", e);
            return new GeneralResult<>(false, "修改任务执行时间失败");
        }
    }

    @Override
    public GeneralResult<List<Task>> listSchedule() {
        try {
            List<Task> tasks = taskRepository.listSchedule();
            return new GeneralResult<>(true, tasks, "查询定时作业成功");
        } catch (Exception e) {
            log.error("List task fail:", e);
            return new GeneralResult<>(false, "查询定时作业失败");
        }
    }

    @Override
    public GeneralResult<Void> execute(JSONObject object) {
        if (null == object || object.isEmpty() || !object.containsKey("module") || !object.containsKey("targets")) {
            return new GeneralResult<>(false, "任务参数有误");
        }
        List<JSONObject> targets = JSONObject.parseArray(JSON.toJSONString(object.get("targets")), JSONObject.class);
        if (ListTool.isEmpty(targets)) {
            return new GeneralResult<>(false, "目标机器列表为空");
        }
        try {
            GeneralResult<NetworkSegment> hasWorker = taskHostService.hasWorker(targets.get(0).getString("host"));
            if (hasWorker.isFailed()) {
                return new GeneralResult<>(false, hasWorker.getMessage());
            }
            GeneralResult<?> result = handleTask(object, hasWorker.getData().getGroupId());
            if (result.isFailed()) {
                return new GeneralResult<Void>(false, result.getMessage());
            }
            return new GeneralResult<>(true, "任务下发执行成功");
        } catch (Exception e) {
            log.error("execute task fail:", e);
            return new GeneralResult<>(false, "任务下发执行失败");
        }
    }

    private GeneralResult<?> handleTask(JSONObject params, Long groupId) {
        String key = "";
        if (null == groupId) {
            key = CacheConstant.SUPER_WORKERS;
        } else {
            key = CacheConstant.WORKERGROUP_ + groupId;
        }
        String value = redisTemplate.opsForValue().get(key);
        WorkerLoadInfo loadInfo = JSONObject.parseObject(value, WorkerLoadInfo.class);
        if (null == loadInfo || ListTool.isEmpty(loadInfo.getWorkers())) {
            return new GeneralResult<>(false, "网段对应分组未配置执行worker");
        }
        String instance = getInstance(key, loadInfo);
        if (!StringUtils.hasText(instance)) {
            return new GeneralResult<>(false, "网段对应分组未配置执行worker");
        }
        boolean sync = false;
        if (params.containsKey("sync")) {
            sync = params.getBooleanValue("sync");
            params.remove("sync");
        }
        return TaskWorkerHelper.invoke(instance, params, sync);
    }

    private String getInstance(String key, WorkerLoadInfo loadInfo) {
        try (AutoCloseLock lock = lockFactory.getACLock(key)) {
            Assert.isTrue(lock.acquire(300, TimeUnit.SECONDS), "请求超时");
            List<String> workers = loadInfo.getWorkers();
            int position = loadInfo.getPosition();
            if (position >= workers.size()) {
                position = 0;
            }
            String instance = workers.get(position);
            position++;
            loadInfo.setPosition(position);
            redisTemplate.opsForValue().set(key, JSONObject.toJSONString(loadInfo));
            return instance;
        } catch (Exception e) {
            log.error("配置worker负载缓存失败:", e);
            return null;
        }
    }

    private Object settingCron(Object params) {
        DateFormat format = new SimpleDateFormat("ss mm HH dd MM ? yyyy");
        if (params instanceof TaskTransferParams) {
            TaskTransferParams transferParams = (TaskTransferParams)params;
            if (null != transferParams.getIsLoop() && !transferParams.getIsLoop()) {
                ((TaskTransferParams)params).setCron(format.format(((TaskTransferParams)params).getRunTime()));
            }
        } else if (params instanceof Task) {
            Task task = (Task)params;
            if (null != task.getIsLoop() && !task.getIsLoop()) {
                ((Task)params).setCron(format.format(((Task)params).getRunTime()));
            } else {
                ((Task)params).setRunTime(null);
            }
        }
        return params;
    }

    private boolean hasNext(String crons) {
        String[] fields = StringUtils.tokenizeToStringArray(crons, " ");
        if (fields.length == 7) {
            return false;
        }
        CronExpression expression = CronExpression.parse(crons);
        LocalDateTime time = expression.next(LocalDateTime.now());
        if (null != time && null != expression.next(time)) {
            return true;
        }
        return false;
    }

}
