package com.bocloud.cop.service;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.Script;
import com.bocloud.cop.entity.TaskNode;
import com.bocloud.cop.entity.TaskTemplate;
import com.bocloud.cop.entity.TaskTemplateTenant;
import com.bocloud.cop.enums.ApplyStatus;
import com.bocloud.cop.enums.NodeCode;
import com.bocloud.cop.enums.TaskNodeSort;
import com.bocloud.cop.intf.TaskTemplateService;
import com.bocloud.cop.model.NodeConfig;
import com.bocloud.cop.model.NodeControl;
import com.bocloud.cop.model.TaskEdge;
import com.bocloud.cop.model.TaskGraph;
import com.bocloud.cop.model.TaskGraphBean;
import com.bocloud.cop.model.TaskNodeBean;
import com.bocloud.cop.repository.ScriptRepository;
import com.bocloud.cop.repository.TaskRepository;
import com.bocloud.cop.repository.TaskTemplateRepository;
import com.bocloud.cop.repository.TaskTemplateTenantRepository;
import com.bocloud.cop.repository.patch.PatchDeployRepository;
import com.bocloud.cop.service.utils.NodeHelper;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.model.Sign;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import io.jsonwebtoken.lang.Collections;
import lombok.extern.slf4j.Slf4j;

/**
 * 作业模板抽象Service接口实现类
 *
 * @author wei wei
 * @version 1.0
 * @since 2019.01.11
 */
@Slf4j
@Service("taskTemplateService")
public class TaskTemplateServiceImpl implements TaskTemplateService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private TaskTemplateRepository taskTemplateRepository;
    @Autowired
    private TaskRepository taskRepository;
    @Autowired
    private ScriptRepository scriptRepository;
    @Autowired
    private PatchDeployRepository patchDeployRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private TaskTemplateTenantRepository taskTemplateTenantRepository;

    @Override
    public GeneralResult<GridBean<TaskTemplate>> list(Pager pager, RequestContext context) {
        GridBean<TaskTemplate> gridBean = null;
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            if (com.megatron.common.model.RequestContext.Catalog.Tenant == context.getCatalog()) {
                pager.getParams().add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
            } else if (com.megatron.common.model.RequestContext.Catalog.User == context.getCatalog()) {
                pager.getParams().add(new Param(MapTools.simpleMap("creatorId", context.getTarget()), Sign.EQ));
                pager.getParams().add(new Param(MapTools.simpleMap("catalog", context.getCatalog()), Sign.EQ));
            }
            int total = taskTemplateRepository.count(pager.getParams());
            List<TaskTemplate> list = taskTemplateRepository.list(pager);
            for (TaskTemplate template : list) {
                pager.getParams().clear();
                pager.getParams().add(new Param(MapTools.simpleMap("templateId", template.getId()), Sign.EQ));
                template.setTaskCount(taskRepository.count(pager.getParams()));
                if (com.megatron.common.model.RequestContext.Catalog.Tenant == template.getCatalog()) {
                    template.setCreator(redisTemplate.opsForValue().get("Tenant_Name_" + template.getCreatorId()));
                    if (null != template.getMenderId()) {
                        template.setMender(redisTemplate.opsForValue().get("Tenant_Name_" + template.getMenderId()));
                    }
                } else {
                    template.setCreator(redisTemplate.opsForValue().get("User_Name_" + template.getCreatorId()));
                    if (null != template.getMenderId()) {
                        template.setMender(redisTemplate.opsForValue().get("User_Name_" + template.getMenderId()));
                    }
                }
                if (null != template.getTenantId() && template.getTenantId() != 0) {
                    template.setTenantName(redisTemplate.opsForValue().get("Tenant_Name_" + template.getTenantId()));
                }
            }
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
        } catch (Exception e) {
            log.error("List task template fail:", e);
            return new GeneralResult<>(false, "查询作业模板失败");
        }
        return new GeneralResult<>(true, gridBean, "查询作业模板成功");
    }

    @Override
    public GeneralResult<GridBean<TaskTemplate>> listAllot(Pager pager, RequestContext context) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = taskTemplateRepository.countByTenant(pager.getParams(), context.getTenant());
            List<TaskTemplate> list = taskTemplateRepository.listByTenant(pager, context.getTenant());
            GridBean<TaskTemplate> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, bean, "查询公共作业模板成功");
        } catch (Exception e) {
            log.error("List task template fail:", e);
            return new GeneralResult<>(false, "查询公共作业模板失败");
        }
    }

    @Override
    public GeneralResult<GridBean<SimpleBean>> simple(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = taskTemplateRepository.count(pager.getParams());
            List<SimpleBean> list = taskTemplateRepository.list(pager.getParams(), pager.getSorter());
            GridBean<SimpleBean> bean = new GridBean<>(1, 1, total, list);
            return new GeneralResult<>(true, bean, "查询作业模板成功");
        } catch (Exception e) {
            log.error("List task template fail:", e);
            return new GeneralResult<>(false, "查询作业模板失败");
        }
    }

    @Override
    public GeneralResult<TaskTemplate> create(TaskTemplate template, RequestContext context) {
        try {

            template.setCreatorId(context.getTarget());
            template.setMenderId(context.getTarget());
            template.setTenantId(context.getTenant());
            template.setCatalog(context.getCatalog());
            if (null == template.getSceneId()) {
                template.setSceneId(1L);
            }
            // 判断某种类的作业模板名称是否存在
            boolean flag = taskTemplateRepository.exists(template.getName(), context.getTenant());
            if (flag) {
                return new GeneralResult<>(false, "该模板名称已存在，无法添加");
            }
            boolean checkScript = true;
            if (!(Catalog.Manager == context.getCatalog())) {
                checkScript = false;
            }
            GeneralResult<TaskTemplate> result = saveTask(template, true, checkScript);
            if (result.isSuccess()) {
                return new GeneralResult<>(true, result.getData(), "添加作业模板成功");
            }
            return result;
        } catch (Exception e) {
            log.error("query task template error:", e);
            return new GeneralResult<>(false, "添加作业模板失败");
        }
    }

    private GeneralResult<TaskTemplate> saveTask(TaskTemplate template, boolean isAdd, boolean checkScript) {
        TaskGraph graph = JSONObject.parseObject(JSONObject.toJSONString(template.getGraph()), TaskGraph.class);
        List<TaskNode> nodes = graph.getNodes();
        if (ListTool.isEmpty(nodes)) {
            return new GeneralResult<>(false, "保存作业模板失败：不能没有任务节点");
        }
        List<TaskEdge> edges = graph.getEdges();
        Map<String, String> ingoreNodes = NodeHelper.getIngoreNodes(nodes);
        Map<String, List<String>> preLinks = NodeHelper.getPreLinks(edges, ingoreNodes);
        Map<String, List<TaskEdge>> nextEdges = NodeHelper.getNextEdges(edges, ingoreNodes);
        Map<String, TaskNode> nodeMaps = new HashMap<>();
        for (TaskNode node: nodes) {
            NodeConfig config = JSONObject.parseObject(node.getConfig(), NodeConfig.class);
            nodeMaps.put(config.getId(), node);
        }
        try {
            // 判断是否存在闭环
            boolean suspend = false;
            boolean reuse = false;
            boolean maintain = true;
            for (TaskNode node : nodes) {
                String code = node.getModuleType();
                // 是否内置节点
                boolean internal = node.getCode().contains("task.maintain.custom");
                NodeConfig config = JSONObject.parseObject(node.getConfig(), NodeConfig.class);
                if (null == config || config.isIgnore() || code.contains(NodeCode.Ctrl.Start.getValue())
                    || code.contains(NodeCode.Ctrl.Stop.getValue())) {
                    continue;
                }
                node.setNodeId(config.getId());
                if (code.contains(NodeCode.Ctrl.Judge.getValue())) {
                    List<String> links = preLinks.get(config.getId());
                    if (ListTool.isEmpty(links)) {
                        return new GeneralResult<>(false, "判断节点的入度数量为0");
                    }
                    for (String nodeId: links) {
                        TaskNode preNode = nodeMaps.get(nodeId);
                        NodeControl nodeControl = JSONObject.parseObject(preNode.getControl(), NodeControl.class);
                        if (nodeControl.isSkippable()) {
                            return new GeneralResult<>(false, "判断节点前置节点不允许失败跳过，请检查");
                        }
                    }
                    List<TaskEdge> list = nextEdges.get(config.getId());
                    if (ListTool.isEmpty(list) || list.size() != 2) {
                        return new GeneralResult<>(false, "判断节点的出度有误，请检查");
                    }
                    if (list.get(0).isDicision() == list.get(1).isDicision()) {
                        return new GeneralResult<>(false, "判断节点的出度有误，请检查");
                    }
                    for (TaskEdge edge : list) {
                        String target = edge.getTarget();
                        target = ingoreNodes.containsKey(target) ? ingoreNodes.get(target) : target;
                        links = preLinks.get(target);
                        if (null != links && links.size() > 1) {
                            return new GeneralResult<>(false, "判断节点所连接的后置节点存在其他前置节点连接，请检查");
                        }
                    }
                } else if ("task.maintain.scripts".equalsIgnoreCase(code) && !internal && checkScript) {
                    JSONObject input = JSONObject.parseObject(node.getInput());
                    JSONObject fronts = input.getJSONObject("fronts");
                    List<JSONObject> scripts = JSONObject.parseArray(fronts.getString("scripts"), JSONObject.class);
                    for (JSONObject object : scripts) {
                        Script script = scriptRepository.query(object.getLong("scriptId"));
                        if (null == script) {
                            return new GeneralResult<>(false, "脚本执行节点【" + node.getName() + "】所选脚本不存在");
                        }
                        if (!ApplyStatus.APPROVED.name().equalsIgnoreCase(script.getStatus())) {
                            return new GeneralResult<>(false, "脚本执行节点【" + node.getName() + "】所选脚本没有进行审批或者没有审批通过");
                        }
                        if (!script.getContent().equals(object.getString("content"))) {
                            return new GeneralResult<>(false, "脚本执行节点【" + node.getName() + "】执行脚本和所选的脚本内容不一致");
                        }
                    }
                } else if ("task.maintain.script".equalsIgnoreCase(code) && !internal && checkScript) {
                    JSONObject input = JSONObject.parseObject(node.getInput());
                    JSONObject fronts = input.getJSONObject("fronts");
                    Script script = scriptRepository.query(fronts.getLong("scriptId"));
                    if (null == script) {
                        return new GeneralResult<>(false, "脚本执行节点【" + node.getName() + "】所选脚本不存在");
                    }
                    if (!ApplyStatus.APPROVED.name().equalsIgnoreCase(script.getStatus())) {
                        return new GeneralResult<>(false, "脚本执行节点【" + node.getName() + "】所选脚本没有进行审批或者没有审批通过");
                    }
                    if (!script.getContent().equals(fronts.getString("content"))) {
                        return new GeneralResult<>(false, "脚本执行节点【" + node.getName() + "】执行脚本和所选的脚本内容不一致");
                    }
                }
                if (NodeHelper.isCycle(node.getNodeId(), preLinks, nodes.size() - ingoreNodes.size())) {
                    return new GeneralResult<>(false, "作业模板存在闭环无法创建");
                }
                NodeControl control = JSONObject.parseObject(node.getControl(), NodeControl.class);
                if (null != control && control.isSuspensible()) {
                    suspend = control.isSuspensible();
                }
            }
            template.setHasSuspend(suspend);
            template.setHasReuse(reuse);
            template.setInternal(false);
            template.setMaintain(maintain);
            template.setGraph(JSONObject.toJSONString(template.getGraph()));
            // isAdd为true新增作业模板
            if (isAdd) {
                template.setId(null);
                template.setGmtCreate(new Date());
                template.setGmtModify(new Date());
                template.setStatus(Catalog.Manager == template.getCatalog() ? ApplyStatus.DEVELOPING.name()
                    : ApplyStatus.APPROVED.name());
                this.taskTemplateRepository.save(template);
            } else {
                // 更新作业模板
                template.setGmtModify(new Date());
                this.taskTemplateRepository.update(template);
            }
        } catch (Exception e) {
            log.error("Save task template fail:", e);
            return new GeneralResult<>(false, "保存作业模板失败");
        }
        return new GeneralResult<>(true, template, "保存作业模板成功");
    }

    @Override
    public GeneralResult<TaskTemplate> modify(TaskTemplate template, RequestContext context) {
        String path = TaskTemplate.class.getSimpleName() + "_" + template.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            TaskTemplate taskTemplate = this.taskTemplateRepository.query(template.getId());
            Assert.notNull(taskTemplate, "数据不存在！");
            if (!taskTemplate.getName().equals(template.getName())) {
                // 判断某种类的作业模板名称是否存在
                boolean flag = taskTemplateRepository.exists(template.getName(), context.getTenant());
                if (flag) {
                    return new GeneralResult<>(false, "该名称已存在，请重新输入!");
                }
            }
            taskTemplate.setGraph(template.getGraph());
            taskTemplate.setSceneId(template.getSceneId());
            taskTemplate.setName(template.getName());
            taskTemplate.setRemark(template.getRemark());
            taskTemplate.setMenderId(context.getTarget());
            boolean checkScript = true;
            if (!Catalog.Manager.name().equalsIgnoreCase(context.getCatalog().name())) {
                checkScript = false;
            }
            GeneralResult<TaskTemplate> result = saveTask(taskTemplate, false, checkScript);
            if (result.isSuccess()) {
                return new GeneralResult<>(true, result.getData(), "修改作业模板成功");
            }
            return result;
        } catch (Exception e) {
            log.error("Update task fail:", e);
            return new GeneralResult<>(false, "修改作业模板失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = TaskTemplate.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            int count = patchDeployRepository.count(id);
            if (count > 0) {
                return new GeneralResult<>(false, "作业模板被补丁部署任务引用无法删除");
            }
            this.taskTemplateRepository.remove(id, userId);
            return new GeneralResult<>(true, "删除作业模板成功");
        } catch (Exception e) {
            log.error("Remove task template fail:", e);
            return new GeneralResult<>(false, "删除作业模板失败");
        }
    }

    @Override
    public GeneralResult<TaskTemplate> detail(Long id) {
        try {
            TaskTemplate template = this.taskTemplateRepository.query(id);
            Assert.notNull(template, "数据不存在！");
            // 对数据进行处理？
            TaskGraphBean graph = JSONObject.parseObject(template.getGraph().toString(), TaskGraphBean.class);
            List<TaskNodeBean> nodes = graph.getNodes();
            List<TaskEdge> edges = graph.getEdges();
            Map<String, String> ingoreNodes = NodeHelper.getIngoreNodesForBean(nodes);
            Map<String, List<String>> nextLinks = NodeHelper.getNextLinks(edges, ingoreNodes);
            Map<String, List<String>> preLinks = NodeHelper.getPreLinks(edges, ingoreNodes);
            // 转换节点列表为Map
            Map<String, TaskNodeBean> nodeMap = new HashMap<>();
            nodes.stream().forEach(node -> {
                NodeConfig config = node.getConfig();
                nodeMap.put(config.getId(), node);
            });
            // 获取开始节点
            List<TaskNodeBean> starts = nodes.stream().filter(node -> {
                NodeConfig config = node.getConfig();
                String nodeId = config.getId();
                return !preLinks.keySet().contains(nodeId);
            }).collect(Collectors.toList());
            List<TaskNodeBean> nodeBeans = new ArrayList<>();
            // 获取节点的层级
            starts.stream().forEach(node -> {
                int levelInd = 1;
                NodeConfig config = node.getConfig();
                String nodeId = config.getId();
                if (config.isIgnore()) {
                    return;
                }
                node.setLevel(levelInd);
                nodeBeans.add(node);
                handleNodesLevel(levelInd, nodeId, nextLinks, nodeBeans, nodeMap);
            });
            // 根据层级排序
            nodes =
                nodeBeans.stream().sorted(Comparator.comparingInt(TaskNodeBean::getLevel)).collect(Collectors.toList());
            List<TaskNodeBean> list = new ArrayList<>();
            // 如果存在主从节点，处理主从节点排序
            if (ingoreNodes.keySet().size() > 0) {
                Map<String, List<TaskNodeBean>> salveNodeMap = getSalveNodes(ingoreNodes, nodeMap);
                nodes.stream().forEach(node -> {
                    list.add(node);
                    NodeConfig config = node.getConfig();
                    String nodeId = config.getId();
                    List<TaskNodeBean> slaves = salveNodeMap.get(nodeId);
                    if (Collections.isEmpty(slaves)) {
                        return;
                    }
                    slaves = slaves.stream().sorted(Comparator.comparingInt(TaskNodeBean::getSort))
                        .collect(Collectors.toList());
                    list.addAll(slaves);
                });
            }

            graph.setNodes(0 == list.size() ? nodes : list);
            template.setGraph(JSONObject.toJSONString(graph));
            return new GeneralResult<>(true, template, "查询作业模板详细成功");
        } catch (Exception e) {
            log.error("Query task detail template fail:", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    /**
     * 处理节点层级（包含层级关系）
     * 
     * @param levelInd
     *            层级
     * @param nodeId
     *            节点唯一编码
     * @param nextLinks
     *            子节点关系map
     * @param nodeBeans
     *            节点列表
     * @param nodeMap
     *            节点map
     */
    private void handleNodesLevel(int levelInd, String nodeId, Map<String, List<String>> nextLinks,
        List<TaskNodeBean> nodeBeans, Map<String, TaskNodeBean> nodeMap) {
        List<String> nextNodes = nextLinks.get(nodeId);
        if (!Collections.isEmpty(nextNodes)) {
            for (String nextNodeId : nextNodes) {
                int nextLevelInd = levelInd + 1;
                TaskNodeBean nextNode = nodeMap.get(nextNodeId);
                if (null == nextNode.getLevel()) {
                    nextNode.setLevel(nextLevelInd);
                    nodeBeans.add(nextNode);
                    // log.info("node层级:{} {}", nextNodeId, nextLevelInd);
                } else if (nextLevelInd > nextNode.getLevel()) {
                    // log.info("node层级变更:{} {}->{}", nextNodeId, nextNode.getLevel(), nextLevelInd);
                    nextNode.setLevel(nextLevelInd);
                }
                handleNodesLevel(nextLevelInd, nextNodeId, nextLinks, nodeBeans, nodeMap);
            }
        }
    }

    /**
     * 获取主从关系，主节点为key
     * 
     * @param ingoreMap
     *            主从关系Map，从节点为key
     * @return
     */
    private Map<String, List<TaskNodeBean>> getSalveNodes(Map<String, String> ingoreMap,
        Map<String, TaskNodeBean> taskNodeMap) {
        Map<String, List<TaskNodeBean>> map = new HashMap<>();
        ingoreMap.keySet().stream().forEach(key -> {
            TaskNodeBean salveNode = taskNodeMap.get(key);
            NodeConfig config = salveNode.getConfig();
            TaskNodeSort sort = TaskNodeSort.getByName(config.getRole());
            salveNode.setSort(null == sort ? 0 : sort.getValue());
            String value = ingoreMap.get(key);
            List<TaskNodeBean> salveNodes = map.get(value);
            if (null == salveNodes) {
                salveNodes = new ArrayList<>();
                salveNodes.add(salveNode);
                map.put(value, salveNodes);
            } else {
                salveNodes.add(salveNode);
                map.put(value, salveNodes);
            }
        });
        return map;
    }

    @Override
    public GeneralResult<TaskTemplate> clone(Long id) {
        try {
            TaskTemplate template = this.taskTemplateRepository.query(id);
            Assert.notNull(template, "数据不存在！");
            template.setId(null);
            return new GeneralResult<>(true, template, "克隆成功");
        } catch (Exception e) {
            log.error("Clone task template failed!", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    @Override
    public TaskTemplate basic(Long id) {
        try {
            return this.taskTemplateRepository.query(id);
        } catch (Exception e) {
            log.error("Query task detail template fail:", e);
            return null;
        }
    }

    @Override
    public GeneralResult<Void> updateStatus(Long id, String status, Long userId) {
        String path = TaskTemplate.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            taskTemplateRepository.updateStatus(id, status, userId);
            return new GeneralResult<>(true, "修改作业模板状态成功");
        } catch (Exception e) {
            log.error("update task template status fail:", e);
            return new GeneralResult<>(false, "修改作业模板状态失败");
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public GeneralResult<Void> allot(Long tenantId, List<Long> templateIds) {
        try {
            if (!ListTool.isEmpty(templateIds)) {
                Boolean hasSaltStack = taskTemplateRepository.hasSaltStack(templateIds);
                if (hasSaltStack) {
                    throw new IllegalArgumentException("不支持分配非Remote模式的模板");
                }
            }
            List<TaskTemplateTenant> templateTenants = taskTemplateTenantRepository.listByTenantId(tenantId);
            List<Long> allotIds = templateTenants.stream().map(TaskTemplateTenant::getId).collect(Collectors.toList());
            if (!ListTool.isEmpty(allotIds)) {
                taskTemplateTenantRepository.delete(allotIds);
            }
            List<TaskTemplateTenant> list = new ArrayList<>();
            for (Long templateId : templateIds) {
                list.add(new TaskTemplateTenant(templateId, tenantId));
            }
            if (!ListTool.isEmpty(list)) {
                taskTemplateTenantRepository.batchSave(list, 1000);
            }
            return new GeneralResult<>(true, "操作成功");
        } catch (Exception e) {
            log.error("task template allot error: ", e);
            return new GeneralResult<>(false, "操作失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<List<TaskTemplate>> listAllotTemplates(Long tenantId) {
        try {
            List<TaskTemplateTenant> templateTenants = taskTemplateTenantRepository.listByTenantId(tenantId);
            List<Long> templateIds =
                templateTenants.stream().map(TaskTemplateTenant::getTemplateId).collect(Collectors.toList());
            List<TaskTemplate> list = new ArrayList<>();
            if (!ListTool.isEmpty(templateIds)) {
                Pager pager = new Pager(1, Integer.MAX_VALUE, new ArrayList<>(), new HashMap<>());
                pager.getSorter().put(Common.ID, Common.ONE);
                pager.getParams().add(new Param(MapTools.simpleMap("id", StringUtils.join(templateIds, ",")), Sign.IN));
                list = taskTemplateRepository.list(pager);
            }
            return new GeneralResult<>(true, list, "查询成功");
        } catch (Exception e) {
            log.error("list allot tenants error: ", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

}
