package com.bocloud.cop.service.inspection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.inspection.InspectItem;
import com.bocloud.cop.entity.inspection.InspectRecord;
import com.bocloud.cop.entity.inspection.InspectRecordItem;
import com.bocloud.cop.entity.inspection.InspectRecordWarning;
import com.bocloud.cop.entity.inspection.InspectWarnLevel;
import com.bocloud.cop.entity.inspection.RecordItem;
import com.bocloud.cop.intf.inspection.DataAdaptorService;
import com.bocloud.cop.intf.inspection.InspectItemService;
import com.bocloud.cop.model.inspection.ScriptLog;
import com.bocloud.cop.model.inspection.ScriptLogResult;
import com.bocloud.cop.model.inspection.ScriptLogResultEntity;
import com.bocloud.cop.model.inspection.WarningRuleEntity;
import com.bocloud.cop.model.inspection.WarningRuleResult;
import com.bocloud.cop.repository.inspection.InspectRecordItemRepository;
import com.bocloud.cop.repository.inspection.InspectRecordRepository;
import com.bocloud.cop.repository.inspection.InspectRecordWarningRepository;
import com.bocloud.cop.repository.inspection.InspectWarnRepository;
import com.bocloud.cop.service.utils.GenericEntityUtil;
import com.megatron.common.model.GeneralResult;

import lombok.extern.slf4j.Slf4j;

@Service("inspect")
@Slf4j
public class InspectDataAdaptorServiceImpl implements DataAdaptorService {

    @Autowired
    private InspectItemService inspectItemService;
    @Autowired
    private InspectRecordWarningRepository inspectRecordWarningRepository;
    @Autowired
    private InspectWarnRepository inspectWarnRepository;
    @Autowired
    private InspectRecordRepository inspectRecordRepository;
    @Autowired
    private InspectRecordItemRepository inspectRecordItemRepository;

    @Override
    public void handleContent(RecordItem item, String content) {
        log.info("handleContent----content::::" + JSON.toJSONString(content));
        InspectRecordItem recordItem = (InspectRecordItem)item;
        // Map<String,Map<Long,RecordItem>>
        Map<String, InspectWarnLevel> warningLevelMap = loadWarningLevel();
        ScriptLog scriptLog;// 存储日志解析结果
        // 解析日志
        try {
            if (StringUtils.isEmpty(content)) {
                throw new RuntimeException("解析结果失败：未查询到输出日志");
            }
            GeneralResult<InspectItem> result = inspectItemService.get(recordItem.getItemId());
            if (result.isFailed()) {
                throw new RuntimeException("未查询到巡检项：" + result.getMessage());
            }
            InspectItem inspectItem = result.getData();
            recordItem.setTargetRule(inspectItem.getTargetRule());
            Boolean mulitiCompare = inspectItem.getMultiCompare();
            if (mulitiCompare == null) {
                mulitiCompare = false;
            }
            // 实际解析日志
            scriptLog = ScriptLog.parasLog(content,
                WarningRuleEntity.paras(inspectItem.getTargetRule(), mulitiCompare, warningLevelMap));
        } catch (RuntimeException e) {
            scriptLog = ScriptLog.errorLog(e.getMessage());
        } catch (Exception e) {
            log.error("日志解析失败", e);
            scriptLog = ScriptLog.errorLog("服务器错误,请联系管理员查询日志");
        }

        // 将巡检结果转存数据库
        try {
            if (scriptLog.getScriptStatus() != null
                && scriptLog.getScriptStatus() == ScriptLog.RunStatus.SUCCESS.ordinal()) {
                recordItem.setStatus(InspectRecordItem.Status.SUCCESS.name());
            } else {
                recordItem.setStatus(InspectRecordItem.Status.ERROR.name());
                recordItem.setScriptErrorInfo(scriptLog.getScriptErrorInfo());
            }
            recordItem.setInspectResult(scriptLog.getInspectResult());
            recordItem.setLog(JSONObject.toJSONString(scriptLog.getInspectMessage()));
            if (scriptLog.getInspectResult() != null && null != scriptLog.getInspectResultMap()
                && scriptLog.getInspectResultMap().size() > 0) {
                List<InspectRecordWarning> warnings = new ArrayList<>();
                WarningRuleResult max = null;// 加载最严重的告警
                Collection<ScriptLogResult> logResults = scriptLog.getInspectResultMap().values();
                List<String> warningInfos = new ArrayList<>();
                for (ScriptLogResult logResult : logResults) {
                    // 多值匹配时的值的key 如果是单值匹配 则为null
                    String key = logResult.getKey();
                    // 上述key对应的告警，可能为空
                    List<ScriptLogResultEntity> entities = logResult.getEntities();
                    for (ScriptLogResultEntity entity : entities) {
                        WarningRuleResult warn = entity.getWarningRuleResult();
                        if (warn == null) {
                            continue;
                        }
                        if (warn.hasErrorOrWarning()) {
                            InspectRecordWarning warning = new InspectRecordWarning();
                            warning.setGroupName(recordItem.getGroupName());
                            warning.setItemName(recordItem.getItemName());
                            warning.setItemId(recordItem.getItemId());
                            warning.setIp(item.getIp());
                            warning.setRecordId(recordItem.getRecordId());
                            warning.setContent(warn.getErrorOrWarning());
                            warningInfos.add(warn.getErrorOrWarning());
                            warning.setCreatorId(recordItem.getCreatorId());
                            warning.setMenderId(recordItem.getMenderId());
                            warning.setLevel(warn.getWarningLevel());
                            warning.setInspectKey(key);
                            warning.setValue(entity.getValue());
                            warning.setValueName(entity.getValueName());
                            warning.setResourceName(recordItem.getHostName());
                            warning.setTargetRule(warn.getTargetRule());
                            warnings.add(warning);
                            if (max == null || max.getWarningLevel() < warn.getWarningLevel()) {
                                max = warn;
                            }
                        }
                    }
                }
                if (max != null) {
                    InspectWarnLevel level = warningLevelMap.get(max.getWarningLevel() + "");
                    JSONObject levelJson = new JSONObject();
                    levelJson.put("level", level.getWarnLevel());
                    levelJson.put("color", level.getColor());
                    levelJson.put("id", level.getId());
                    levelJson.put("remark", level.getRemark());
                    levelJson.put("info", max.getErrorOrWarning());
                    recordItem.setMaxWarning(levelJson.toJSONString());
                }
                recordItem.setWarningInfo(JSONObject.toJSONString(warningInfos));
                recordItem.setResult(JSONObject.toJSONString(logResults));
                if (warnings.size() > 0) {
                    recordItem.setWarning(true);
                    log.info("handleContent.warningList::::" + JSON.toJSONString(warnings));
                    inspectRecordWarningRepository.batchSave(warnings, warnings.size());
                }
            }
        } catch (Exception e) {
            log.error("处理【巡检告警或错误】失败", e);
        }

        // 更新数据
        try {
            GenericEntityUtil.fillModifyData(recordItem);
            inspectRecordItemRepository.update(recordItem);
        } catch (Exception e) {
            log.error("更新【巡检记录项目】失败", e);
        }
    }

    @Override
    public void synRecord(Long recordId) {
        log.info("synRecord::::" + recordId);
        InspectRecord record = inspectRecordRepository.findOne(recordId);
        Long runnings = inspectRecordItemRepository.countByStatusAndRecordId(InspectRecordItem.Status.RUNNING.name(),
            record.getId());
        if (runnings == 0) {
            Double totalTime = (System.currentTimeMillis() - record.getGmtCreate().getTime()) / 1000.0;
            Long errors = inspectRecordItemRepository.countByStatusAndRecordId(InspectRecordItem.Status.ERROR.name(),
                record.getId());
            String status = errors > 0 ? InspectRecord.Status.ERROR.name() : InspectRecord.Status.SUCCESS.name();
            Boolean warning = null;
            if (InspectRecord.Status.SUCCESS.name().equals(status)) {
                Long warnings = inspectRecordItemRepository.countByWarningAndRecordId(true, record.getId());
                warning = warnings > 0;
            }
            log.info("synRecord.status::::" + status);
            log.info("synRecord.warning::::" + warning);
            inspectRecordRepository.updateStatus(record.getId(), status, warning, totalTime);
        }
    }

    /**
     * 加载所有告警 避免多次查询
     *
     * @return
     */
    private Map<String, InspectWarnLevel> loadWarningLevel() {
        List<InspectWarnLevel> list = inspectWarnRepository.findAll();
        return list.stream().collect(Collectors.toMap(InspectWarnLevel::getWarnLevel, item -> item));
    }
}
