package com.bocloud.cop.service.inspection;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.TaskNode;
import com.bocloud.cop.entity.inspection.*;
import com.bocloud.cop.enums.ComplianceCategory;
import com.bocloud.cop.enums.InspectCategory;
import com.bocloud.cop.enums.InspectTypeEnums;
import com.bocloud.cop.intf.inspection.DataAdaptorService;
import com.bocloud.cop.intf.inspection.InspectRecordService;
import com.bocloud.cop.model.ResultTrend;
import com.bocloud.cop.model.StatResult;
import com.bocloud.cop.model.TrendData;
import com.bocloud.cop.model.inspection.ManualInspectRecord;
import com.bocloud.cop.model.inspection.ManualInspectRecordItem;
import com.bocloud.cop.repository.TaskNodeRepository;
import com.bocloud.cop.repository.inspection.InspectItemRepository;
import com.bocloud.cop.repository.inspection.InspectRecordItemRepository;
import com.bocloud.cop.repository.inspection.InspectRecordRepository;
import com.bocloud.cop.repository.inspection.InspectRecordWarningRepository;
import com.bocloud.cop.schedule.inspection.report.ComplianceRecordPdfUtil;
import com.bocloud.cop.schedule.inspection.report.InspectRecordPdfUtil;
import com.bocloud.cop.service.utils.GenericEntityUtil;
import com.google.common.collect.Lists;
import com.itextpdf.text.DocumentException;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.ContentDisposition;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class InspectRecordServiceImpl implements InspectRecordService {

    @Autowired
    private InspectRecordRepository inspectRecordRepository;
    @Autowired
    private InspectRecordItemRepository inspectRecordItemRepository;
    @Autowired
    private InspectRecordWarningRepository inspectRecordWarningRepository;
    @Autowired
    private InspectItemRepository inspectItemRepository;
    @Autowired
    private TaskNodeRepository taskNodeRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    @Qualifier("inspect")
    private DataAdaptorService inspectDataAdaptor;
    @Autowired
    @Qualifier("compliance")
    private DataAdaptorService complianceDataAdaptor;

    @Override
    public GeneralResult<InspectRecord> get(Long id) {
        try {
            InspectRecord record = inspectRecordRepository.findOne(id);
            if (record == null) {
                return new GeneralResult<>(false, "查询失败，没有数据");
            }

            List<InspectRecordItem> items = inspectRecordItemRepository.findAllByRecordId(record.getId());
            record.setRecordItems(items);
            record.setCreator(redisTemplate.opsForValue().get("User_Name_" + record.getCreatorId()));
            if (InspectTypeEnums.INSPECTION.name().equalsIgnoreCase(record.getType())) { // 巡检详情
                List<InspectRecordWarning> warnings = inspectRecordWarningRepository.findAllByRecordId(record.getId());
                record.setRecordWarnings(warnings);
            } else if (InspectTypeEnums.COMPLIANCE.name().equalsIgnoreCase(record.getType())) { // 合规详情
                record.setItemDetail(itemDetail(items));
                record.setHostDetail(hostDetail(items));
                record.setComplianceDetail(complianceDetail(items));
                record.setErrorDetail(errorDetail(items));
            }
            return new GeneralResult<>(true, record, "查询成功");
        } catch (Exception e) {
            log.error("Query inspectRecord detail fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public void pdf(Long id, HttpServletResponse response) {
        GeneralResult<InspectRecord> GeneralResult = this.get(id);
        Assert.isTrue(GeneralResult.isSuccess(), GeneralResult.getMessage());
        InspectRecord inspectRecord = GeneralResult.getData();
        Assert.notNull(inspectRecord, "未找到id为" + id + "的巡检记录");

        ContentDisposition disposition =
            ContentDisposition.attachment().filename(inspectRecord.getName() + ".pdf", StandardCharsets.UTF_8).build();
        response.addHeader(HttpHeaders.CONTENT_DISPOSITION, disposition.toString());
        response.setStatus(HttpStatus.OK.value());

        try {
            if (InspectTypeEnums.INSPECTION.name().equalsIgnoreCase(inspectRecord.getType())) {
                InspectRecordPdfUtil.covertPdf(inspectRecord, response.getOutputStream());
            } else if (InspectTypeEnums.COMPLIANCE.name().equalsIgnoreCase(inspectRecord.getType())) {
                ComplianceRecordPdfUtil.covertPdf(inspectRecord, response.getOutputStream());
            }
            response.getOutputStream().flush();
            response.getOutputStream().close();
        } catch (IOException | DocumentException e) {
            log.error("将巡检记录导出为PDF失败：{}", e.getMessage(), e);
            throw new RuntimeException(e);
        }
    }

    @Override
    public void excel(Long id, HttpServletResponse response) {
        GeneralResult<InspectRecord> GeneralResult = this.get(id);
        Assert.isTrue(GeneralResult.isSuccess(), GeneralResult.getMessage());
        InspectRecord inspectRecord = GeneralResult.getData();
        Assert.notNull(inspectRecord, "未找到id为" + id + "的巡检记录");

        ContentDisposition disposition =
            ContentDisposition.attachment().filename(inspectRecord.getName() + ".xls", StandardCharsets.UTF_8).build();
        response.addHeader(HttpHeaders.CONTENT_DISPOSITION, disposition.toString());
        response.setStatus(HttpStatus.OK.value());

        try {
            if (InspectTypeEnums.INSPECTION.name().equalsIgnoreCase(inspectRecord.getType())) {
                InspectRecordPdfUtil.covertExcel(inspectRecord, response.getOutputStream());
            } else if (InspectTypeEnums.COMPLIANCE.name().equalsIgnoreCase(inspectRecord.getType())) {
                ComplianceRecordPdfUtil.covertExcel(inspectRecord, response.getOutputStream());
            }
            response.getOutputStream().flush();
            response.getOutputStream().close();
        } catch (IOException e) {
            log.error("将巡检记录导出为EXCEL失败：{}", e.getMessage(), e);
            throw new RuntimeException(e);
        }
    }

    /**
     * @param limitDay
     *            0 当天数据 n n天内的数据 null 所有
     * @param executeType
     *            0 快速脚本 1 常规巡检 2 定时巡检 null 所有
     * @param status
     *            0 启动中 1 执行中 2 成功 3 错误 null 所有
     * @param warning
     *            0 正常 1 告警
     * @param userId
     *            用户id null 所有
     * @param type
     *            INSPECTION / COMPLIANCE 巡检/合规
     * @return
     */
    @Override
    public Long count(Integer limitDay, Integer executeType, String status, Boolean warning, Long userId,
        Long accessUserId, String type) {
        return inspectRecordRepository.count(limitDay, executeType, status, warning, userId, type);
    }

    @Override
    public Map<String, Object> countGroupByCategory(Integer limitDay, String type) {
        return inspectRecordRepository.countGroupByCategory(limitDay, type);
    }

    @Override
    public Map<String, Object> getChartData(Integer limitDay, String category, Long userId, String type) {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        calendar.add(Calendar.DATE, -limitDay);
        Date fromDate = calendar.getTime();
        Map<String, Map<String, Long>> map = initMap(fromDate);
        List total = inspectRecordRepository.listCountByDateBase(fromDate, category, null, null, type);
        List success = inspectRecordRepository.listCountByDateBase(fromDate, category,
            InspectRecord.Status.SUCCESS.name(), null, type);
        List warning = inspectRecordRepository.listCountByDateBase(fromDate, category, null, true, type);
        map = fillData(map, total, "total");
        map = fillData(map, success, "success");
        map = fillData(map, warning, "warning");
        return outPutMap(map);
    }

    @Override
    public Map<String, Object> getPeiChartData(Integer limitDay, String category, Long userId, String type) {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        calendar.add(Calendar.DATE, -limitDay);
        Date fromDate = calendar.getTime();
        Long success =
            inspectRecordRepository.countByStatus(fromDate, InspectRecord.Status.SUCCESS.name(), category, type);
        Long fail = inspectRecordRepository.countByStatus(fromDate, InspectRecord.Status.ERROR.name(), category, type);
        Map<String, Object> map = new HashMap<>();
        map.put("success", success);
        map.put("fail", fail);
        return map;
    }

    @Override
    public GeneralResult<InspectRecord> save(ManualInspectRecord manualRecord, Long userId) {
        try {
            if (StringUtils.isEmpty(manualRecord.getName())) {
                return new GeneralResult<>(false, "请输入巡检名称");
            }
            if (manualRecord.getWarning() == null) {
                return new GeneralResult<>(false, "巡检告警状态不能为空");
            }
            if (manualRecord.getRecordItems() == null || manualRecord.getRecordItems().size() <= 0) {
                return new GeneralResult<>(false, "巡检详情不能为空");
            }
            for (ManualInspectRecordItem item : manualRecord.getRecordItems()) {
                if (item == null) {
                    return new GeneralResult<>(false, "巡检详情条目不能为空");
                }
                if (StringUtils.isEmpty(item.getIp())) {
                    return new GeneralResult<>(false, "巡检详情条目-主机ip不能为空");
                }
                if (StringUtils.isEmpty(item.getName())) {
                    return new GeneralResult<>(false, "巡检详情条目-巡检项目名称不能为空");
                }
                if (StringUtils.isEmpty(item.getValue())) {
                    return new GeneralResult<>(false, "巡检详情条目-巡检结果不能为空");
                }
            }
            InspectRecord record = new InspectRecord();
            record.setKeyStamp("R-" + UUID.randomUUID());
            record.setWarning(manualRecord.getWarning());
            record.setExecuteType(InspectRecord.Type.MANUAL.ordinal());
            record.setStatus(InspectRecord.Status.SUCCESS.name());
            record.setName(manualRecord.getName());
            GenericEntityUtil.fillCreateData(record, userId);
            inspectRecordRepository.save(record);
            List<ManualInspectRecordItem> manualRecordItems = manualRecord.getRecordItems();
            if (ListTool.isEmpty(manualRecordItems)) {
                throw new RuntimeException("请输入至少一条执行详情");
            }
            List<InspectRecordItem> recordItems = new ArrayList<>();
            List<InspectRecordWarning> warnings = new ArrayList<>();
            for (ManualInspectRecordItem manualRecordItem : manualRecordItems) {
                InspectRecordItem recordItem = new InspectRecordItem();
                if (!StringUtils.isEmpty(manualRecordItem.getWarning())) {
                    recordItem.setWarning(true);
                    recordItem.setWarningInfo("[\"" + manualRecordItem.getWarning() + "\"]");
                    InspectRecordWarning warning = new InspectRecordWarning();
                    warning.setRecordId(record.getId());
                    warning.setIp(manualRecordItem.getIp());
                    warning.setItemName("人工填写");
                    warning.setGroupName("-");
                    warning.setContent(manualRecordItem.getName() + "-" + manualRecordItem.getValue() + ":"
                        + manualRecordItem.getWarning());
                    GenericEntityUtil.fillCreateData(warning, userId);
                    warnings.add(warning);
                } else {
                    recordItem.setWarning(false);
                }
                recordItem.setStatus(InspectRecordItem.Status.SUCCESS.name());
                recordItem.setLog("{\"" + manualRecordItem.getName() + "\":\"" + manualRecordItem.getValue() + "\"}");
                recordItem.setRecordId(record.getId());
                recordItem.setIp(manualRecordItem.getIp());
                recordItem.setGroupName("");
                recordItem.setItemName("人工填写");
                recordItems.add(recordItem);
            }
            List<InspectRecordItem> items = batchSave(recordItems, userId);
            if (warnings.size() > 0) {
                inspectRecordWarningRepository.batchSave(warnings, warnings.size());
                record.setRecordWarnings(warnings);
            }
            record.setRecordItems(items);
            return new GeneralResult<>(true, record, "保存成功");
        } catch (Exception e) {
            log.error("Save inspectRecord fail:", e);
            return new GeneralResult<>(false, "保存失败");
        }
    }

    @Override
    public GeneralResult<GridBean<InspectRecord>> list(Pager pager, Long userId) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = inspectRecordRepository.count(pager.getParams());
            List<InspectRecord> records = inspectRecordRepository.list(pager);
            for (InspectRecord record : records) {
                record.setCreator(redisTemplate.opsForValue().get("User_Name_" + record.getCreatorId()));
                if (StringUtils.isEmpty(record.getCategory())) {
                    continue;
                }
                if (InspectTypeEnums.INSPECTION.name().equalsIgnoreCase(record.getType())) {
                    record.setCategory(InspectCategory.valueOf(record.getCategory()).getName());
                } else {
                    record.setCategory(ComplianceCategory.valueOf(record.getCategory()).getName());
                }
            }
            GridBean<InspectRecord> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, records);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("List inspectRecord fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<GridBean<InspectRecordItem>> listItems(Long recordId, Pager pager) {
        return listItems(recordId, pager, false);
    }

    @Override
    public GeneralResult<GridBean<InspectRecordWarning>> listWarning(Long recordId, Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = inspectRecordWarningRepository.count(recordId, pager.getParams());
            List<InspectRecordWarning> warnings = inspectRecordWarningRepository.list(recordId, pager);
            GridBean<InspectRecordWarning> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, warnings);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("List inpect record warning fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<GridBean<InspectRecordItem>> listError(Long recordId, Pager pager) {
        return listItems(recordId, pager, true);
    }

    // 合规详情
    @Override
    public GeneralResult<GridBean<JSONObject>> getDetail(Pager pager, Long id, String type) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<InspectRecordItem> items = inspectRecordItemRepository.listByRecordIdAndParams(id, pager);
            List<JSONObject> array;
            switch (type) {
                case "itemDetail": // 基线详情
                    array = itemDetail(items);
                    break;
                case "hostDetail": // 设备详情
                    array = hostDetail(items);
                    break;
                case "complianceDetail": // 合规详情
                    array = complianceDetail(items);
                    break;
                case "errorDetail": // 错误详情
                    array = errorDetail(items);
                    break;
                default:
                    return new GeneralResult<>(false, "查询失败，无法判断查询类型");
            }
            List<JSONObject> list = new ArrayList<JSONObject>();
            int total = array.size();
            int page = pager.getPage() == null ? 1 : pager.getPage();
            int rows = pager.getRows() == null ? 10 : pager.getRows();
            if (total != 0) {
                int value = total / rows + (total % rows == 0 ? 0 : 1);
                if (page >= value) {
                    page = value;
                }
                if (page < value) {
                    list.addAll(array.subList((page - 1) * rows, page * rows));
                } else {
                    list.addAll(array.subList((page - 1) * rows, total));
                }
            }
            GridBean<JSONObject> bean = new GridBean<>(page, rows, total, list);
            return new GeneralResult<>(true, bean, "查询成功");
        } catch (Exception e) {
            log.error("Query compliance record detail fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    /**
     * 基线详情
     *
     * @param items
     * @return
     */
    private List<JSONObject> itemDetail(List<InspectRecordItem> items) {
        Map<String, List<InspectRecordItem>> groupBy =
            items.stream().collect(Collectors.groupingBy(InspectRecordItem::getItemName));

        List<JSONObject> array = new ArrayList<>();
        groupBy.forEach((k, v) -> {
            int num = v.size(); // 设备数总和
            int passNum =
                    (int) v.stream().filter(a -> null != a.getWarning() && a.getWarning()).count(); // 合规设备数
            int error = (int) v.stream().filter(a -> "ERROR".equals(a.getStatus())).count();
            JSONObject param = new JSONObject();
            param.put("itemName", k); // 基线名称
            param.put("passHost", passNum); // 合规设备数
            param.put("noPassHost", num - passNum - error); // 不合规设备数
            param.put("error", error); // 错误设备数
            array.add(param);
        });
        return array;
    }

    /**
     * 设备详情
     *
     * @param items
     * @return
     */
    private List<JSONObject> hostDetail(List<InspectRecordItem> items) {
        Map<String, List<InspectRecordItem>> groupBy =
            items.stream().collect(Collectors.groupingBy(InspectRecordItem::getIp));
        List<JSONObject> array = new ArrayList<>();
        groupBy.forEach((k, v) -> {
            int num = v.size(); // 设备数总和
            int passNum = (int) v.stream().filter(InspectRecordItem::getWarning).count(); // 合规设备数
            int error = (int) v.stream().filter(a -> "ERROR".equals(a.getStatus())).count();
            StringBuilder groupNames = new StringBuilder();
            Map<String, String> groupRecord = new HashMap<>();
            for (int i = 0; i < v.size(); i++) {
                String groupName = v.get(i).getGroupName();
                if (groupRecord.containsKey(groupName)) {
                    continue;
                }
                groupNames.append(groupName).append("\n");
                groupRecord.put(groupName, groupName);
            }
            JSONObject param = new JSONObject();
            param.put("groupName", groupNames.toString()); // 资源组名称
            param.put("hostName", v.get(0).getHostName()); // 主机名称
            param.put("ip", k); // ip
            param.put("passHost", passNum); // 合规设备数
            param.put("noPassHost", num - passNum - error); // 不合规设备数
            param.put("error", error); // 错误设备数
            array.add(param);
        });
        return array;
    }

    /**
     * 合规详情
     *
     * @param items
     * @return
     */
    private List<JSONObject> complianceDetail(List<InspectRecordItem> items) {
        List<JSONObject> array = new ArrayList<>();
        items.stream().filter(i -> !"ERROR".equals(i.getStatus())) // 过滤错误的记录
            .forEach(a -> {
                JSONObject param = new JSONObject();
                param.put("groupName", a.getGroupName()); // 资源组名称
                param.put("hostName", a.getHostName());// 主机名称
                param.put("itemName", a.getItemName());// 基线名称
                param.put("ip", a.getIp()); // 主机ip
                param.put("result", getResultValue(a.getInspectResult())); // 当前值
                param.put("rule", getRule(a.getItemId()));// 基线标准
                param.put("pass", getStatus(a)); // 合规状态 三种状态 合规：不合规：执行中
                array.add(param);
            });
        return array;
    }

    private String getStatus(InspectRecordItem item) {
        if ("RUNNING".equals(item.getStatus())) {
            return "执行中";
        }
        return item.getWarning() ? "合规" : "不合规";
    }

    /**
     * @param scriptResult
     * @return
     */
    private String getResultValue(String scriptResult) {
        if (null == scriptResult) {
            return "";
        }
        List<String> messageGroups = new ArrayList<>();
        if (scriptResult.contains("@@@@")) {
            String[] messages = scriptResult.split("@@@@");
            for (String s : messages) {
                String[] split = s.split("\\+\\+\\+\\+");
                messageGroups.add(split[1]);
            }
        } else {
            String[] messages = scriptResult.split("[|][|][|][|]");
            messageGroups.addAll(Arrays.asList(messages));
        }
        StringBuffer buffer = new StringBuffer();
        for (String result : messageGroups) { // 获取合规结果，将结果放进map集合中
            String[] param = result.split("====");
            if (param.length == 0) {
                continue;
            }
            buffer.append(param[0]);
            buffer.append("=");
            buffer.append(param.length == 2 ? param[1] : " ");
            buffer.append("；");
        }
        return buffer.toString();
    }

    /**
     * 获取基线规则
     *
     * @param id
     * @return
     */
    private String getRule(Long id) {
        StringBuilder buffer = new StringBuilder();
        InspectItem item = inspectItemRepository.query(id);
        if (null == item) {
            return "对应的合规基线数据不存在";
        }
        JSONArray array = JSONArray.parseArray(item.getTargetRule());
        if (null == array) {
            return "";
        }
        for (int i = 0; i < array.size(); i++) {
            JSONObject j = array.getJSONObject(i);
            buffer.append(j.getString("name"));
            buffer.append(transformRule(j.getString("rule")));
            buffer.append(j.getString("value"));
            buffer.append("为");
            buffer.append(j.getBoolean("pass") ? "合规" : "不合规");
            buffer.append("；");
        }
        return buffer.toString();
    }

    /**
     * 规则转换成中文
     *
     * @param rule
     * @return
     */
    private String transformRule(String rule) {
        switch (rule) {
            case "GT": // 大于
                return "大于";
            case "EQ": // 等于
                return "等于";
            case "NEQ": // 不等于
                return "不等于";
            case "LT": // 小于
                return "小于";
            case "GE": // 大于等于
                return "大于等于";
            case "LE": // 小于等于
                return "小于等于";
            case "IN": // 包含
                return "包含";
            case "NIN": // 不包含
                return "不包含";
            default:
                return "规则不匹配！";
        }
    }

    /**
     * 错误详情
     *
     * @param items
     * @return
     */
    private List<JSONObject> errorDetail(List<InspectRecordItem> items) {
        List<JSONObject> array = new ArrayList<>();
        List<InspectRecordItem> recordItems =
            items.stream().filter(item -> "ERROR".equals(item.getStatus())).collect(Collectors.toList());
        recordItems.forEach(item -> {
            JSONObject param = new JSONObject();
            param.put("groupName", item.getGroupName()); // 资源组名称
            param.put("hostName", item.getHostName());// 主机名称
            param.put("itemName", item.getItemName());
            param.put("ip", item.getIp()); // 主机ip
            param.put("errorInfo", item.getScriptErrorInfo()); // 错误信息
            array.add(param);
        });
        return array;
    }

    private Map<String, Map<String, Long>> initMap(Date fromDate) {
        Map<String, Map<String, Long>> map = new TreeMap<>((o1, o2) -> o1.compareTo(o2));
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(fromDate);
        Date now = new Date();
        while (calendar.getTime().before(now)) {
            String dateString = sdf.format(calendar.getTime());
            Map<String, Long> innerMap = new HashMap<>();
            innerMap.put("total", 0L);
            innerMap.put("success", 0L);
            innerMap.put("warning", 0L);
            map.put(dateString, innerMap);
            calendar.add(Calendar.DATE, 1);
        }
        return map;
    }

    /**
     * 填充map
     *
     * @param map
     *            被填充的map
     * @param list
     *            数据
     * @param key
     *            类型 total success
     * @return
     */
    private Map<String, Map<String, Long>> fillData(Map<String, Map<String, Long>> map, List list, String key) {
        for (Object item : list) {
            Map itemMap = (Map)item;
            Long count = (Long)itemMap.get("count");
            String date = (String)itemMap.get("create_date");
            Map<String, Long> innerMap = map.get(date);
            if (innerMap != null) {
                innerMap.put(key, count);
            }
        }
        return map;
    }

    /**
     * 填充map
     *
     * @param map
     *            被填充的map
     * @return
     */
    private Map<String, Object> outPutMap(Map<String, Map<String, Long>> map) {
        Map<String, Object> outPutMap = new HashMap<>();
        List<String> date = new ArrayList<>();
        List<Long> success = new ArrayList<>();
        List<Long> total = new ArrayList<>();
        List<Long> warning = new ArrayList<>();
        for (Map.Entry<String, Map<String, Long>> entry : map.entrySet()) {
            date.add(entry.getKey());
            total.add(entry.getValue().get("total"));
            success.add(entry.getValue().get("success"));
            warning.add(entry.getValue().get("warning"));
        }
        outPutMap.put("date", date);
        outPutMap.put("total", total);
        outPutMap.put("success", success);
        outPutMap.put("warning", warning);
        return outPutMap;
    }

    @Override
    public GeneralResult<List<StatResult>> statsStatus(RequestContext context, String type) {
        Pager pager = new Pager();
        pager.setRows(Integer.MAX_VALUE);
        List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
        Param param = new Param(MapTools.simpleMap("type", type), Sign.EQ);
        params.add(param);
        pager.setParams(params);
        List<InspectRecord> records = inspectRecordRepository.list(pager);
        int starting = 0;
        int running = 0;
        int success = 0;
        int failed = 0;
        for (InspectRecord record : records) {
            if (InspectRecord.Status.STARTING.name().equalsIgnoreCase(record.getStatus())) {
                starting++;
            } else if (InspectRecord.Status.RUNNING.name().equalsIgnoreCase(record.getStatus())) {
                running++;
            } else if (InspectRecord.Status.SUCCESS.name().equalsIgnoreCase(record.getStatus())) {
                success++;
            } else if (InspectRecord.Status.ERROR.name().equalsIgnoreCase(record.getStatus())) {
                failed++;
            }
        }
        List<StatResult> results = new ArrayList<>();
        results.add(new StatResult("启动中", starting));
        results.add(new StatResult("执行中", running));
        results.add(new StatResult("成功", success));
        results.add(new StatResult("失败", failed));
        return new GeneralResult<>(true, results, "获取执行状态统计成功");
    }

    @Override
    public GeneralResult<ResultTrend> statsTrend(RequestContext context, String type) {
        List<String> keys = Lists.newArrayList();
        List<String> dateKeys = Lists.newArrayList();
        List<TrendData> values = Lists.newArrayList();
        List<Integer> runs = Lists.newArrayList();
        List<Integer> success = Lists.newArrayList();
        List<Integer> fails = Lists.newArrayList();
        List<Integer> starts = Lists.newArrayList();
        List<Param> params = Lists.newArrayList();

        Calendar calendar = null;
        for (int i = 29; i >= 0; i--) {
            calendar = Calendar.getInstance();
            calendar.add(Calendar.DATE, -i);
            int year = calendar.get(Calendar.YEAR);
            int month = calendar.get(Calendar.MONTH) + 1;
            int day = calendar.get(Calendar.DAY_OF_MONTH);
            String months = month < 10 ? "0" + month : "" + month;
            String days = day < 10 ? "0" + day : "" + day;
            dateKeys.add(year + "-" + months + "-" + days);
            keys.add(months + "-" + days);
        }
        params.add(new Param(MapTools.simpleMap("gmtCreate", dateKeys.get(0) + " 00:00:00"), Sign.GET));
        params.add(new Param(MapTools.simpleMap("gmtCreate", dateKeys.get(keys.size() - 1) + " 23:59:59"), Sign.LET));
        params.add(new Param(MapTools.simpleMap("type", type), Sign.EQ));

        List<InspectRecord> records = inspectRecordRepository.list(new Pager(1, Integer.MAX_VALUE, params, null));
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
        Map<String, List<InspectRecord>> map = new HashMap<>();
        for (InspectRecord record : records) {
            String key = format.format(record.getGmtCreate()).substring(0, 10);
            if (map.containsKey(key)) {
                List<InspectRecord> list = map.get(key);
                list.add(record);
                map.put(key, list);
            } else {
                List<InspectRecord> list = new ArrayList<>();
                list.add(record);
                map.put(key, list);
            }
        }
        for (String key : dateKeys) {
            int successes = 0;
            int faileds = 0;
            int runnings = 0;
            int starting = 0;
            List<InspectRecord> list = map.get(key);
            if (!ListTool.isEmpty(list)) {
                for (InspectRecord record : list) {
                    if (InspectRecord.Status.STARTING.name().equalsIgnoreCase(record.getStatus())) {
                        starting++;
                        continue;
                    } else if (InspectRecord.Status.RUNNING.name().equalsIgnoreCase(record.getStatus())) {
                        runnings++;
                        continue;
                    } else if (InspectRecord.Status.SUCCESS.name().equalsIgnoreCase(record.getStatus())) {
                        successes++;
                        continue;
                    } else if (InspectRecord.Status.ERROR.name().equalsIgnoreCase(record.getStatus())) {
                        faileds++;
                        continue;
                    }
                }
            }
            success.add(successes);
            fails.add(faileds);
            runs.add(runnings);
            starts.add(starting);
        }
        values.add(new TrendData("成功", success));
        values.add(new TrendData("失败", fails));
        values.add(new TrendData("执行中", runs));
        values.add(new TrendData("启动中", starts));
        return new GeneralResult<>(true, new ResultTrend(keys, values), "获取执行趋势数据成功");
    }

    @Override
    public GeneralResult<List<InspectRecord>> limit(Pager pager, Long userId) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<InspectRecord> records = inspectRecordRepository.list(pager);
            for (InspectRecord record : records) {
                record.setCreator(redisTemplate.opsForValue().get("User_Name_" + record.getCreatorId()));
                if (StringUtils.isEmpty(record.getCategory())) {
                    continue;
                }
                if (InspectTypeEnums.INSPECTION.name().equalsIgnoreCase(record.getType())) {
                    record.setCategory(InspectCategory.valueOf(record.getCategory()).getName());
                } else {
                    record.setCategory(ComplianceCategory.valueOf(record.getCategory()).getName());
                }
            }
            return new GeneralResult<>(true, records, "查询成功");
        } catch (Exception e) {
            log.error("List inspectRecord fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public void synRecord(String source, Long recordId) {
        if ("compliance".equalsIgnoreCase(source)) {// 同步合规记录
            synInspectRecord(recordId, complianceDataAdaptor);
        } else if ("inspection".equalsIgnoreCase(source)) {// 同步巡检记录
            synInspectRecord(recordId, inspectDataAdaptor);
        } else {
            throw new RuntimeException("未知的source取值：" + source);
        }
    }

    @Transactional
    public void synInspectRecord(Long recordId, DataAdaptorService adaptor) {
        InspectRecord record = inspectRecordRepository.findOne(recordId);
        if (record == null || record.getExecuteType() == InspectRecord.Type.MANUAL.ordinal()
            || !record.getStatus().equals(InspectRecord.Status.RUNNING.name())) {
            return;
        }
        List<? extends RecordItem> items = inspectRecordItemRepository.findAllByRecordId(recordId);
        for (RecordItem item : items) {
            synInspectRecord(item, adaptor);
        }
        adaptor.synRecord(recordId);
    }

    /**
     * 同步某个任务实例对应的数据
     */
    @Override
    public void synInspectRecord(RecordItem item, DataAdaptorService adaptor) {
        // 查询数据
        TaskNode taskNode = taskNodeRepository.query(item.getJobNodeId());
        Assert.notNull(taskNode, "未找到id为" + item.getJobNodeId() + "的任务节点");
        if (taskNode.getStatus().equalsIgnoreCase("SUCCESS") || taskNode.getStatus().equalsIgnoreCase("FAILED")) {
            JSONObject outJson = JSONObject.parseObject(taskNode.getOutput());
            JSONArray outDatas = outJson.getJSONArray("data");
            if (ListTool.isEmpty(outDatas)) {
                String errorInfo = outJson.getString("message");
                if (StringUtils.isEmpty(errorInfo)) {
                    errorInfo = "作业执行失败：未知的错误";
                }
                InspectRecordItem recordItem = (InspectRecordItem)item;
                recordItem.setStatus(InspectRecordItem.Status.ERROR.name());
                recordItem.setScriptErrorInfo(errorInfo);
                GenericEntityUtil.fillModifyData(recordItem);
                inspectRecordItemRepository.update(recordItem);
                return;
            }
            for (int i = 0; i < outDatas.size(); i++) {
                JSONObject outData = outDatas.getJSONObject(i);
                String message = outData.getString("message");
                Boolean success = outData.getBoolean("success");
                log.info("success:::::" + success);
                if (!Boolean.TRUE.equals(success)) {
                    InspectRecordItem recordItem = (InspectRecordItem)item;
                    recordItem.setStatus(InspectRecordItem.Status.ERROR.name());
                    recordItem.setScriptErrorInfo(message);
                    GenericEntityUtil.fillModifyData(recordItem);
                    inspectRecordItemRepository.update(recordItem);
                    continue;
                }
                adaptor.handleContent(item, message);
            }
        }
    }

    private List<InspectRecordItem> batchSave(List<InspectRecordItem> items, Long userId) {
        for (InspectRecordItem item : items) {
            GenericEntityUtil.fillCreateData(item, userId);
        }
        inspectRecordItemRepository.batchSave(items, items.size());
        return items;
    }

    private GeneralResult<GridBean<InspectRecordItem>> listItems(Long recordId, Pager pager, boolean onlyError) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = inspectRecordItemRepository.count(recordId, pager.getParams(), onlyError);
            List<InspectRecordItem> items = inspectRecordItemRepository.list(recordId, pager, onlyError);
            GridBean<InspectRecordItem> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, items);
            return new GeneralResult<>(true, bean, "查询成功");
        } catch (Exception e) {
            log.error("List inspectRecordItem fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

}
