package com.bocloud.cop.service.inspection;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.inspection.*;
import com.bocloud.cop.enums.InspectCategory;
import com.bocloud.cop.intf.inspection.InspectReportDetailService;
import com.bocloud.cop.repository.inspection.*;
import com.bocloud.cop.schedule.inspection.report.InspectReportExcelUtil;
import com.bocloud.cop.schedule.inspection.report.InspectReportExeclTool;
import com.bocloud.cop.schedule.inspection.report.InspectReportPdfUtil;
import com.bocloud.cop.service.utils.DateTransfer;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.ContentDisposition;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 巡检报表详情Service实现
 */
@Slf4j
@Service
public class InspectReportDetailServiceImpl implements InspectReportDetailService {

    @Autowired
    private InspectReportDetailRepository inspectReportDetailRepository;
    @Autowired
    private InspectReportRepository inspectReportRepository;
    @Autowired
    private InspectRecordRepository inspectRecordRepository;
    @Autowired
    private InspectRecordItemRepository inspectRecordItemRepository;
    @Autowired
    private InspectItemRepository inspectItemRepository;
    @Autowired
    InspectRecordWarningRepository inspectRecordWarningRepository;
    @Autowired
    StringRedisTemplate redisTemplate;

    @Override
    public GeneralResult<GridBean<InspectReportDetail>> list(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = inspectReportDetailRepository.count(pager.getParams());
            List<InspectReportDetail> list = inspectReportDetailRepository.list(pager);
            GridBean<InspectReportDetail> gridBean;
            if (pager.getSimple()) {
                gridBean = new GridBean<>(1, 1, total, list);
            } else {
                gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("List inspectReportDetail fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    private String fieldToColumn(String filed) {
        if (StringUtils.isEmpty(filed)) {
            return "";
        }
        StringBuilder builder = new StringBuilder(filed);
        builder.replace(0, 1, (filed.charAt(0) + "").toLowerCase());
        int count = 0;
        for (int i = 1; i < filed.length(); ++i) {
            char c = filed.charAt(i);
            if (c >= 'A' && c <= 'Z') {
                builder.replace(i + count, i + count + 1, (c + "").toLowerCase());
                builder.insert(i + count, "_");
                ++count;
            }
        }
        return builder.toString();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public GeneralResult<InspectReportDetail> create(Long reportId, Long userId) {
        InspectReportDetail entity = new InspectReportDetail();
        InspectReport report = inspectReportRepository.query(reportId);
        // 获取最新的定时任务的执行记录
        InspectRecord record = inspectRecordRepository.queryTimingLast(report.getInspectId());
        if (null == record) {
            return new GeneralResult<>(false, "未查询到id为[{" + report.getInspectId() + "}]执行的定时记录");
        }
        JSONObject recordJson = JSONObject.parseObject(JSONObject.toJSONString(record));
        // 处理基础信息
        List<JSONObject> msgs = JSONObject.parseArray(report.getBaseMsg(), JSONObject.class);
        for (JSONObject msg : msgs) {
            String key = msg.getString(Common.NAME);
            if ("userName".equals(key)) {
                key = "creatorId";
            }
            msg.put(Common.VALUE, recordJson.get(key));
        }
        entity.setBaseMsg(JSONObject.toJSONString(msgs));
        // 处理列表信息 比较规则rule
        String listMsg = report.getListMsg();
        List<JSONObject> listArray = JSONArray.parseArray(listMsg, JSONObject.class);

        Set<String> titleKey = new HashSet<>();
        // select * from table where 1 = 1
        StringBuilder appendSql = new StringBuilder();
        Map<String, Object> paramMap = new HashMap<>();
        listArray.forEach(list -> {
            String code = list.getString(Common.CODE);
            titleKey.add(code);
            // String ruleName = list.getString("rule");
            JSONArray valueArray = list.getJSONArray("value");
            if (InspectReport.ListMsg.ip.name().equals(code) || InspectReport.ListMsg.itemName.name().equals(code)) {
                // || inspectResult.name().equals(code)
                if (InspectReport.ListMsg.itemName.name().equals(code)) {
                    // 前端传的是ID。。 code = "inspect_item_name";
                    code = "item_id";
                }
                // transform(appendSql, MetaTool.fieldToColumn(code), ruleName, value, paramMap);
                transform(appendSql, fieldToColumn(code), valueArray, paramMap);
            }
        });
        entity.setListMsg(JSONArray.toJSONString(listArray));

        // 盛装列表内容信息
        List<JSONObject> listObjects = new ArrayList<>();
        List<InspectRecordItem> recordItems =
            inspectRecordItemRepository.listByRule(record.getId(), appendSql, paramMap);
        // 过滤ip，指标项，无关告警
        filterItemInfo(listArray, recordItems);
        recordItems.forEach(recordItem -> {
            // 每个列表内容信息的行，主机IP，指标项，指标值，规则，阈值，告警等级
            JSONObject object = new JSONObject();
            if (titleKey.contains("ip")) {
                object.put("ip", recordItem.getIp());
            }
            if (titleKey.contains("itemName")) {
                object.put("itemName", recordItem.getItemName());
            }
            if (titleKey.contains("inspectResult")) {
                object.put("inspectResult", recordItem.getInspectResult());
            }
            if (titleKey.contains("rule")) {
                // 在查询详情前做插入
                object.put("rule", listArray.get(0).getString("rule"));
            }
            if (titleKey.contains(InspectReport.ListMsg.threshold.name())) {
                // 在查询详情前做插入
                object.put(InspectReport.ListMsg.threshold.name(), "-");
            }
            if (titleKey.contains(InspectReport.ListMsg.warning.name())) {
                // TODO 告警
                String maxWarning = recordItem.getMaxWarning();
                if (StringUtils.isNotBlank(maxWarning)) {
                    JSONObject maxWarn = JSONObject.parseObject(maxWarning);
                    object.put(InspectReport.ListMsg.warning.name(), maxWarn.getString("remark"));
                } else {
                    object.put(InspectReport.ListMsg.warning.name(), "告警-");
                }
            }
            listObjects.add(object);
        });
        entity.setListObjects(listObjects);

        String period = report.getPeriod();
        String choose = report.getChoose();
        String generateTime = report.getGenerateTime();
        Integer week = report.getWeek();
        Integer day = report.getDay();
        Date currentTime = new Date();
        Date cronDate = DateTransfer.getDate(currentTime, period, choose, generateTime, week, day);
        entity.setGenerateTime(cronDate);
        entity.setTitle(report.getTitle());
        entity.setReportId(reportId);
        entity.setCreatorMender(userId);
        inspectReportDetailRepository.save(entity);
        GeneralResult<InspectReportDetail> result = detail(entity.getId());
        if (result.isSuccess()) {
            return new GeneralResult<>(true, result.getData(), "添加成功");
        }
        return new GeneralResult<>(true, entity, "添加成功");
    }

    /**
     * 过滤ip，指标，无关告警等信息
     *
     * @param objects
     *            列表信息
     * @param recordItems
     *            巡检记录指标信息
     */
    private void filterItemInfo(List<JSONObject> objects, List<InspectRecordItem> recordItems) {
        List<String> ipValues = new ArrayList<>();
        List<String> ruleValue = new ArrayList<>();
        List<Long> itemNames = new ArrayList<>();
        for (JSONObject object : objects) {
            if (InspectReport.ListMsg.ip.name().equals(object.getString("code"))) {
                ipValues = object.getJSONArray("value").toJavaList(String.class);
            }
            if (InspectReport.ListMsg.warning.name().equals(object.getString("code"))) {
                ruleValue = object.getJSONArray("value").toJavaList(String.class);
            }
            if (InspectReport.ListMsg.itemName.name().equals(object.getString("code"))) {
                itemNames = object.getJSONArray("value").toJavaList(Long.class);
            }
        }
        Iterator<InspectRecordItem> iterator = recordItems.iterator();
        while (iterator.hasNext()) {
            InspectRecordItem next = iterator.next();

            boolean flag = false;
            // ip
            if (!ListTool.isEmpty(ipValues) && !ipValues.contains(next.getIp())) {
                // 过滤出不包含的item
                flag = true;
            }
            // item
            if (!ListTool.isEmpty(itemNames) && !itemNames.contains(next.getItemId())) {
                // 过滤出不包含的item
                flag = true;
            }

            // warn
            if (!ListTool.isEmpty(ruleValue)) {
                String maxWarning = next.getMaxWarning();
                if (StringUtils.isNotBlank(maxWarning)) {
                    // {"color":"#362C78","level":"1","remark":"一级告警","id":10,"info":"/ - 巡检值【53%】大于阈值 【10%】"}
                    JSONObject maxObject = JSONObject.parseObject(maxWarning);
                    String warnLevel = maxObject.getString("level");
                    // 放过无告警item 过滤出不包含的告警等级
                    if (StringUtils.isNotBlank(warnLevel) && !ruleValue.contains(warnLevel)) {
                        flag = true;
                    }
                }
            }
            if (flag) {
                iterator.remove();
            }
        }
    }

    private void transform(StringBuilder appendSql, String name, JSONArray valueArray, Map<String, Object> paramMap) {
        if (valueArray != null) {
            List<Object> objects = valueArray.toJavaList(Object.class);
            if (!ListTool.isEmpty(objects)) {
                appendSql.append(" and ").append(name).append(" in(:").append(name).append(")");
                paramMap.put(name, objects);
            }
        }
    }

    @Override
    public GeneralResult<InspectReportDetail> detail(Long id) {
        try {
            InspectReportDetail entity = inspectReportDetailRepository.query(id);
            if (entity == null) {
                return new GeneralResult<>(false, "数据不存在");
            }
            InspectReport report = inspectReportRepository.query(entity.getReportId());
            // 获取最新的定时任务的执行记录
            InspectRecord timingLastRecord = inspectRecordRepository.queryTimingLastCompleted(report.getInspectId());
            entity.setReportName(timingLastRecord.getName());
            List<InspectRecordItem> allByRecordId =
                inspectRecordItemRepository.findAllByRecordId(timingLastRecord.getId());
            // 添加基本信息
            JSONArray baseArray = new JSONArray();
            String baseMsg = entity.getBaseMsg();
            if (StringUtils.isNotBlank(baseMsg)) {
                List<JSONObject> jsonArray = JSONArray.parseArray(baseMsg, JSONObject.class);
                for (JSONObject object : jsonArray) {
                    String name = object.getString("name");
                    String value = object.get("value") + "";
                    if ("category".equals(name)) {
                        value = InspectCategory.valueOf(value).getName();
                    } else if ("status".equals(name)) {
                        switch (value) {
                            case "SUCCESS":
                                value = "成功";
                                break;
                            case "ERROR":
                                value = "失败";
                                break;
                            default:
                                break;
                        }
                    } else if ("warning".equals(name)) {
                        switch (value) {
                            case "true":
                                value = "告警";
                                break;
                            case "false":
                                value = "正常";
                                break;
                            default:
                                break;
                        }
                    } else if ("userName".equals(name)) {
                        value = redisTemplate.opsForValue().get("User_Name_" + value);
                    }
                    object.put("value", value);
                    baseArray.add(object);
                }
            }
            entity.setBaseMsg(baseArray.toJSONString());

            // 盛装列表内容信息
            List<JSONObject> listObjects = new ArrayList<>();
            List<JSONObject> listArray = JSONArray.parseArray(entity.getListMsg(), JSONObject.class);

            // 过滤ip，指标项，无关告警
            filterItemInfo(listArray, allByRecordId);

            List<String> titleKey = new ArrayList<>();
            listArray.forEach(list -> titleKey.add(list.getString(Common.CODE)));

            Set<Long> itemId = allByRecordId.stream().map(InspectRecordItem::getItemId).collect(Collectors.toSet());
            List<InspectItem> inspectItems = inspectItemRepository.findAllByIdIn(itemId);
            for (InspectRecordItem recordItem : allByRecordId) {
                // 每个列表内容信息的行，主机IP，指标项，指标值，规则，阈值，告警等级
                JSONObject object = new JSONObject();
                if (titleKey.contains("ip")) {
                    object.put("ip", recordItem.getIp());
                }
                if (titleKey.contains("itemName")) {
                    object.put("itemName", recordItem.getItemName());
                }
                if (titleKey.contains("inspectResult")) {
                    String inspectResult = recordItem.getInspectResult();
                    if (StringUtils.isNotBlank(inspectResult)) {
                        inspectResult = inspectResult.replace("++++", ":").replace("====", ":").replace("@@@@", "\r\n")
                            .replace("||||", "\r\n");
                    }
                    object.put("inspectResult", inspectResult);
                }

                if (titleKey.contains("rule")) {
                    for (InspectItem item : inspectItems) {
                        String targetRule = item.getTargetRule();
                        // 如果没有规则，不填充该字段
                        if (StringUtils.isNotBlank(targetRule) && item.getId().equals(recordItem.getItemId())) {
                            List<JSONObject> warningRules = JSONArray.parseArray(targetRule, JSONObject.class);
                            String ruleValue =
                                warningRules.stream().map(warningRule -> transferRule(warningRule.getString("rule")))
                                    .collect(Collectors.joining("\n"));
                            object.put("rule", ruleValue);
                        }
                    }
                }
                if (titleKey.contains(InspectReport.ListMsg.threshold.name())) {
                    for (InspectItem item : inspectItems) {
                        String targetRule = item.getTargetRule();
                        // 如果没有规则，不填充该字段
                        if (StringUtils.isNotBlank(targetRule) && item.getId().equals(recordItem.getItemId())) {
                            List<JSONObject> warningRules = JSONArray.parseArray(targetRule, JSONObject.class);
                            String thresholdValue =
                                warningRules.stream().map(warningRule -> warningRule.getString("threshold"))
                                    .collect(Collectors.joining("\n"));
                            object.put(InspectReport.ListMsg.threshold.name(), thresholdValue);
                        }
                    }
                }
                if (titleKey.contains(InspectReport.ListMsg.warning.name())) {
                    String maxWarning = recordItem.getMaxWarning();
                    if (StringUtils.isNotBlank(maxWarning)) {
                        JSONObject maxWarn = JSONObject.parseObject(maxWarning);
                        object.put(InspectReport.ListMsg.warning.name(), maxWarn.getString("remark"));
                    } /* else {
                      object.put(warning.name(), "-");
                      }*/
                }
                listObjects.add(object);
            }
            List<JSONObject> jsonObjects = listObjects.stream().distinct().collect(Collectors.toList());
            entity.setListObjects(jsonObjects);
            return new GeneralResult<>(true, entity, "获取详情成功");
        } catch (Exception e) {
            log.error("Query inspectReportDetail detail fail:", e);
            return new GeneralResult<>(false, "获取详情失败");
        }
    }

    @Override
    public GeneralResult<List<InspectRecordWarning>> detail(Long id, Long key) {
        try {
            List<InspectRecordWarning> warnings = new ArrayList<>();
            InspectReportDetail entity = inspectReportDetailRepository.query(id);
            if (entity == null) {
                return new GeneralResult<>(false, "数据不存在");
            }
            InspectReport report = inspectReportRepository.query(entity.getReportId());
            if ("template".equals(report.getType())) {
                List<JSONObject> objects = JSONObject.parseArray(report.getListMsg(), JSONObject.class);
                for (JSONObject object : objects) {
                    if (!key.equals(object.getLong("key"))) {
                        continue;
                    }
                    JSONArray inspectIds = object.getJSONArray("inspectId");
                    for (Object inspectId : inspectIds) {
                        InspectRecord timingLastRecord =
                            inspectRecordRepository.queryTimingLast(Long.valueOf(String.valueOf(inspectId)));
                        List<InspectRecordWarning> recordWarnings = inspectRecordWarningRepository
                            .list(timingLastRecord.getId(), new Pager(1, 10000, null, null));
                        warnings.addAll(recordWarnings);
                    }
                }
            }
            return new GeneralResult<>(true, warnings, "获取详情成功");
        } catch (Exception e) {
            log.error("Query inspectReportDetail detail fail:", e);
            return new GeneralResult<>(false, "获取详情失败");
        }
    }

    /**
     * 转换规则为中文
     *
     * @param value
     *            规则
     */
    private static String transferRule(String value) {
        switch (value) {
            case "EQ":
                return "等于";
            case "NEQ":
                return "不等于";
            case "IN":
                return "包含";
            case "NIN":
                return "不包含";
            case "LE":
                return "小于";
            case "LT":
                return "大于";
            case "GE":
                return "大于等于";
            case "GT":
                return "大于";
            // case "ALL":
            default:
                // ALL，"空"
                return "未知";
        }
    }

    public void excelAll(Long id, HSSFWorkbook workbook) {
        int sheetNum = 0;
        try {
            InspectReportDetail detail = inspectReportDetailRepository.query(id);
            List<List<String>> data = new ArrayList<>();
            List<List<InspectRecordWarning>> dataList = new ArrayList<>();
            lists(detail, dataList, data);
            InspectReportExeclTool.exportExcel(workbook, sheetNum, "巡检报表概述", data, detail.getTitle());
            int i = 0;
            for (List<String> datum : data) {
                if ("部分异常".equals(datum.get(1))) {
                    sheetNum++;
                    InspectReportExeclTool.exportExcel1(workbook, sheetNum, datum.get(0), dataList.get(i));
                }
                i++;
            }
        } catch (Exception e) {
            log.error("excelAll异常信息：", e);
        }
    }

    private void lists(InspectReportDetail detail, List<List<InspectRecordWarning>> warnLists,
        List<List<String>> dataLists) {
        List<String> datas;
        JSONArray array = JSONArray.parseArray(detail.getListMsg());
        if (array != null) {
            for (Object arr : array) {
                datas = new ArrayList<>();
                JSONObject object = (JSONObject)JSON.toJSON(arr);
                JSONArray inspectIds = object.getJSONArray("inspectId");
                datas.add(object.getString("name"));
                String status = "总体正常";
                List<InspectRecordWarning> warnings = new ArrayList<>();
                if (inspectIds != null && inspectIds.size() > 0) {
                    for (Object inspectId : inspectIds) {
                        InspectRecord record =
                            inspectRecordRepository.queryTimingLast(Long.valueOf(String.valueOf(inspectId)));
                        if (record != null && record.getWarning()) {
                            status = "部分异常";
                            List<InspectRecordWarning> list =
                                inspectRecordWarningRepository.list(record.getId(), new Pager(1, 10000, null, null));
                            warnings.addAll(list);
                        }
                    }
                }
                datas.add(status);
                datas.add("详情");
                warnLists.add(warnings);
                dataLists.add(datas);
            }
        }
    }

    @Override
    public void pdf(Long id, HttpServletResponse response) {
        try {
            GeneralResult<InspectReportDetail> GeneralResult = this.detail(id);
            Assert.isTrue(GeneralResult.isSuccess(), GeneralResult.getMessage());
            InspectReportDetail detail = GeneralResult.getData();
            Assert.notNull(detail, "未找到id为" + id + "的巡检报表");
            ContentDisposition disposition =
                ContentDisposition.attachment().filename(detail.getTitle() + ".pdf", StandardCharsets.UTF_8).build();
            response.addHeader(HttpHeaders.CONTENT_DISPOSITION, disposition.toString());
            response.setStatus(HttpStatus.OK.value());
            response.getOutputStream().write(InspectReportPdfUtil.covertPdf(detail));
            response.getOutputStream().flush();
            response.getOutputStream().close();
        } catch (Exception e) {
            log.error("将巡检报表导出为PDF失败：{}", e.getMessage(), e);
            throw new RuntimeException(e);
        }
    }

    @Override
    public void excel(Long id, HttpServletResponse response) {
        try {
            GeneralResult<InspectReportDetail> GeneralResult = this.detail(id);
            Assert.isTrue(GeneralResult.isSuccess(), GeneralResult.getMessage());
            InspectReportDetail detail = GeneralResult.getData();
            Assert.notNull(detail, "未找到id为" + id + "的巡检报表");
            ContentDisposition disposition =
                ContentDisposition.attachment().filename(detail.getTitle() + ".xls", StandardCharsets.UTF_8).build();
            response.addHeader(HttpHeaders.CONTENT_DISPOSITION, disposition.toString());
            response.setStatus(HttpStatus.OK.value());
            HSSFWorkbook workbook = new HSSFWorkbook();
            InspectReportExcelUtil.excel(detail, workbook);
            workbook.write(response.getOutputStream());
            response.getOutputStream().flush();
            response.getOutputStream().close();
        } catch (Exception e) {
            log.error("将巡检报表导出为EXCEL失败：{}", e.getMessage(), e);
            throw new RuntimeException(e);
        }
    }

}
