package com.bocloud.cop.service.internal;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;

/**
 * cmp服务内部访问
 */
@Component("cmpInternalService")
public class CmpInternalService implements InitializingBean {

    private static Service service = Service.create(BoCloudService.CMP.getName());
    @Autowired
    private ServiceFactory serviceFactory;
    @Autowired
    private CurrentService currentService;
    private Map<String, Object> headers = new HashMap<>(4);

    /**
     * 根据平台类型查询所有的计算平台id
     *
     * @param vendorCategory
     *            平台类型
     * @return
     */
    public GeneralResult<?> listVendorIdByVendorCategory(String vendorCategory) {
        String url = "/v1/vendors";
        Map<String, Object> param = MapTools.simpleMap("page", 1);
        param.put("rows", Integer.MAX_VALUE);
        List<Param> params = new ArrayList<>();
        params.add(new Param(Collections.singletonMap("vendorCategory", vendorCategory), Sign.EQ));
        param.put("params", params);
        RemoteService remoteService = serviceFactory.build(service, url, HttpMethod.GET, headers, param);
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke();
    }

    /**
     * 根据平台查询所有的宿主机
     *
     * @param vendorId
     *            平台类型
     * @return
     */
    public GeneralResult<?> listHostByVender(Long vendorId) {
        String url = "/v1/hosts";
        Map<String, Object> param = MapTools.simpleMap("page", 1);
        param.put("rows", Integer.MAX_VALUE);
        List<Param> params = new ArrayList<>();
        params.add(new Param(Collections.singletonMap("vendorId", vendorId), Sign.EQ));
        param.put("params", params);
        RemoteService remoteService = serviceFactory.build(service, url, HttpMethod.GET, headers, param);
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke();
    }

    /**
     * 根据平台查询所有的云主机
     *
     * @param vendorId
     *            平台id
     * @return
     */
    public GeneralResult<?> listVmByVendor(Long vendorId) {
        String url = "/v1/vms";
        Map<String, Object> param = MapTools.simpleMap("page", 1);
        param.put("rows", Integer.MAX_VALUE);
        List<Param> params = new ArrayList<>();
        params.add(new Param(Collections.singletonMap("vendorId", vendorId), Sign.EQ));
        param.put("params", params);
        RemoteService remoteService = serviceFactory.build(service, url, HttpMethod.GET, headers, param);
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke();
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
