package com.bocloud.cop.service.internal;

import com.alibaba.fastjson.JSON;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.http.HttpClient;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.io.IOException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.Collections;
import java.util.List;

@Slf4j
@Component("cmpInternetService")
public class CmpInternetService {

    @Value("${cmp.host:https://127.0.0.1:60006}")
    private String cmpHost;

     private String getToken() {
         return new AESEncryptor().encrypt(
                 "Bsm+Cmp@v587:" + System.currentTimeMillis(), null
         ) ;
     }

    public GeneralResult<?> listVendorIdByVendorCategory(String vendorCategory) {
        try {
            List<Param> httpParams = Collections.singletonList(
                    new Param(Collections.singletonMap("vendorCategory", vendorCategory), Sign.EQ)
            );
            String httpUrl = cmpHost + "/api/cmp/v1/bsm/vendors" +
                    "?page=1" +
                    "&rows="+Integer.MAX_VALUE +
                    "&params="+ URLEncoder.encode(JSON.toJSONString(httpParams), StandardCharsets.UTF_8);
            HttpClient httpClient = new HttpClient();
            HttpGet httpGet = new HttpGet(httpUrl);
            httpGet.addHeader("bsm-token", this.getToken());
            CloseableHttpResponse httpResponse = httpClient.getHttpClient().execute(httpGet);
            int httpCode = httpResponse.getStatusLine().getStatusCode();
            String httpBody = EntityUtils.toString(httpResponse.getEntity());
            Assert.isTrue(httpCode == 200, httpBody);
            return JSON.parseObject(httpBody, GeneralResult.class);
        } catch (IOException e) {
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    public GeneralResult<?> listHostByVender(Long vendorId) {
        try {
            List<Param> httpParams = Collections.singletonList(
                    new Param(Collections.singletonMap("vendorId", vendorId), Sign.EQ)
            );
            String httpUrl = cmpHost + "/api/cmp/v1/bsm/hosts" +
                    "?page=1" +
                    "&rows="+Integer.MAX_VALUE +
                    "&params="+ URLEncoder.encode(JSON.toJSONString(httpParams), StandardCharsets.UTF_8);
            HttpClient httpClient = new HttpClient();
            HttpGet httpGet = new HttpGet(httpUrl);
            httpGet.addHeader("bsm-token", this.getToken());
            CloseableHttpResponse httpResponse = httpClient.getHttpClient().execute(httpGet);
            int httpCode = httpResponse.getStatusLine().getStatusCode();
            String httpBody = EntityUtils.toString(httpResponse.getEntity());
            Assert.isTrue(httpCode == 200, httpBody);
            return JSON.parseObject(httpBody, GeneralResult.class);
        } catch (IOException e) {
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    public GeneralResult<?> listVmByVendor(Long vendorId) {
        try {
            List<Param> httpParams = Collections.singletonList(
                    new Param(Collections.singletonMap("vendorId", vendorId), Sign.EQ)
            );
            String httpUrl = cmpHost + "/api/cmp/v1/bsm/vms" +
                    "?page=1" +
                    "&rows="+Integer.MAX_VALUE +
                    "&params="+ URLEncoder.encode(JSON.toJSONString(httpParams), StandardCharsets.UTF_8);
            HttpClient httpClient = new HttpClient();
            HttpGet httpGet = new HttpGet(httpUrl);
            httpGet.addHeader("bsm-token", this.getToken());
            CloseableHttpResponse httpResponse = httpClient.getHttpClient().execute(httpGet);
            int httpCode = httpResponse.getStatusLine().getStatusCode();
            String httpBody = EntityUtils.toString(httpResponse.getEntity());
            Assert.isTrue(httpCode == 200, httpBody);
            return JSON.parseObject(httpBody, GeneralResult.class);
        } catch (IOException e) {
            return new GeneralResult<>(false, e.getMessage());
        }
    }
}
