package com.bocloud.cop.service.internal;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.cop.model.FastTask;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.model.GeneralResult;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;

/**
 * 这里调用的是cop接口，所以标记为Deprecated，后续需要干掉
 */
@Component
public class CopInternalService implements InitializingBean {

    private static final Service SERVICE = Service.create(BoCloudService.COP.getName());
    private Map<String, Object> headers = new HashMap<>(4);

    @Autowired
    private ServiceFactory serviceFactory;
    @Autowired
    private CurrentService currentService;

    public GeneralResult<?> patchScriptsFast(FastTask task) {
        String url = "/v1/scripts/fast";
        RemoteService remoteService = serviceFactory.build(SERVICE, url, HttpMethod.POST, headers, task);
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke();
    }

    public GeneralResult<?> patchDeployPatchs(Long id) {
        String url = "/v1/deploy/patchs/" + id;
        Map<String, String> param = Collections.singletonMap("action", "run");
        RemoteService remoteService = serviceFactory.build(SERVICE, url, HttpMethod.PATCH, headers, param);
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke();
    }

    public boolean scheduleReportBroadcast(Long id, HttpMethod method) {
        boolean success = false;
        String taskUrl = "/v1/inspect/reports/" + id + "/schedule";
        List<RemoteService> services = serviceFactory.buildAll(SERVICE, taskUrl, method, headers, null);
        for (RemoteService service : services) {
            GeneralResult<?> result = service.invoke();
            if (result.isFailed()) {
                continue;
            }
            if ("fixed".equals(result.getData().toString())) {
                success = true;
                break;
            }
        }
        return success;
    }

    public boolean scheduleInspectBroadcast(Long id, HttpMethod method) {
        boolean success = false;
        String taskUrl = "/v1/inspect/timings/" + id + "/schedule";
        List<RemoteService> services = serviceFactory.buildAll(SERVICE, taskUrl, method, headers, null);
        for (RemoteService service : services) {
            GeneralResult<?> result = service.invoke();
            if (result.isFailed()) {
                continue;
            }
            if ("fixed".equals(result.getData().toString())) {
                success = true;
                break;
            }
        }
        return success;
    }

    public boolean scheduleInspectCompare(Long id, HttpMethod method) {
        boolean success = false;
        String taskUrl = "/v1/tasks/comparisons/" + id + "/schedule";
        List<RemoteService> services = serviceFactory.buildAll(SERVICE, taskUrl, method, headers, null);
        for (RemoteService service : services) {
            GeneralResult<?> result = service.invoke();
            if (result.isFailed()) {
                continue;
            }
            if ("fixed".equals(result.getData().toString())) {
                success = true;
                break;
            }
        }
        return success;
    }

    public boolean scheduleBackup(Long id, HttpMethod method) {
        boolean success = false;
        String taskUrl = "/v1/backup/" + id + "/schedule";
        List<RemoteService> services = serviceFactory.buildAll(SERVICE, taskUrl, method, headers, null);
        for (RemoteService service : services) {
            GeneralResult<?> result = service.invoke();
            if (result.isFailed()) {
                continue;
            }
            if ("fixed".equals(result.getData().toString())) {
                success = true;
                break;
            }
        }
        return success;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
