package com.bocloud.cop.service.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.cop.entity.TaskWorker;
import com.bocloud.cop.model.SaltMinionModel;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.model.GeneralResult;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class TseInternalService implements InitializingBean {

    private static final Service SERVICE = Service.create(BoCloudService.TSE.getName());
    private Map<String, Object> headers = new HashMap<>(4);

    @Autowired
    private ServiceFactory serviceFactory;
    @Autowired
    private CurrentService currentService;

    /**
     * 根据SaltMaster查询Master下的minions
     * @param worker saltMaster
     * @return minions列表
     */
    public GeneralResult<List<SaltMinionModel>> minions(TaskWorker worker) {
        String url = "/v1/slat/minions";
        RemoteService remoteService = serviceFactory.build(SERVICE, url, HttpMethod.POST, headers, worker);
        remoteService.setDataFormat(PostDataFormat.RAW);
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed()) {
            return new GeneralResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new GeneralResult<>(true, JSONObject.parseArray(data, SaltMinionModel.class), "success");
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
