package com.bocloud.cop.service.network;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.constant.CacheConstant;
import com.bocloud.cop.constant.OperationConstant;
import com.bocloud.cop.core.WorkerCache;
import com.bocloud.cop.entity.NetworkGroup;
import com.bocloud.cop.entity.TaskHost;
import com.bocloud.cop.entity.TaskWorker;
import com.bocloud.cop.enums.TaskHostType;
import com.bocloud.cop.intf.network.TaskWorkerService;
import com.bocloud.cop.model.SaltMinionModel;
import com.bocloud.cop.model.WorkerLoadInfo;
import com.bocloud.cop.model.salt.SaltCacheModel;
import com.bocloud.cop.repository.NetworkGroupRepository;
import com.bocloud.cop.repository.TaskHostRepository;
import com.bocloud.cop.repository.TaskWorkerRepository;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import com.suse.salt.netapi.AuthModule;
import com.suse.salt.netapi.client.AsyncHttpClient;
import com.suse.salt.netapi.client.SaltClient;
import com.suse.salt.netapi.client.impl.HttpAsyncClientImpl;
import com.suse.salt.netapi.datatypes.Token;
import com.suse.salt.netapi.parser.JsonParser;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.config.CookieSpecs;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.TrustAllStrategy;
import org.apache.http.impl.nio.client.CloseableHttpAsyncClient;
import org.apache.http.impl.nio.client.HttpAsyncClients;
import org.apache.http.ssl.SSLContexts;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import javax.net.ssl.SSLContext;
import java.io.IOException;
import java.net.URI;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Service("taskWorkerService")
public class TaskWorkerServiceImpl implements TaskWorkerService {

    @Autowired
    private TaskWorkerRepository taskWorkerRepository;
    @Autowired
    private NetworkGroupRepository networkGroupRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private WorkerCache workerCache;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private TaskHostRepository taskHostRepository;

    @Override
    public GeneralResult<GridBean<TaskWorker>> list(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = taskWorkerRepository.count(pager.getParams());
            List<TaskWorker> list = null;
            if (pager.getSimple()) {
                list = taskWorkerRepository.list();
            } else {
                list = taskWorkerRepository.list(pager);
                List<TaskWorker> workers = new ArrayList<TaskWorker>();
                for (TaskWorker worker : list) {
                    if (TaskWorker.Category.SaltMaster.equals(worker.getCategory()) && null != worker.getDeleted()
                        && worker.getDeleted()) {
                        continue;
                    }
                    workers.add(worker);
                    if (null != worker.getDeleted() && worker.getDeleted()) {
                        continue;
                    }
                    String value = redisTemplate.opsForValue().get("workmetric_" + worker.getInstance());
                    worker.setTasks(StringUtils.isEmpty(value) ? 0 : Integer.valueOf(value));
                }
                list = workers;
            }
            GridBean<TaskWorker> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, bean, "查询worker列表成功");
        } catch (Exception e) {
            log.error("List task worker fail:", e);
            return new GeneralResult<>(false, "查询worker列表失败");
        }
    }

    @Override
    public GeneralResult<Void> create(TaskWorker worker) {
        try {
            worker.setIsSuper(worker.getIsSuper());
            TaskWorker bean = taskWorkerRepository.exist(worker.getInstance(), worker.getCategory().name());
            if (null != bean) {
                return new GeneralResult<>(false, "添加worker失败，已存在相同实例");
            }
            if (TaskWorker.Category.SaltMaster.equals(worker.getCategory())) {
                GeneralResult<Void> result = checkSalt(worker);
                if (result.isFailed()) {
                    return result;
                }
            }
            taskWorkerRepository.save(worker);
            // 更新缓存
            refreshSaltCache(OperationConstant.CREATE, worker, null);
            return new GeneralResult<>(true, "添加worker成功");
        } catch (Exception e) {
            log.error("Create task worker fail:", e);
            return new GeneralResult<>(false, "添加worker失败");
        }
    }

    private GeneralResult<Void> checkSalt(TaskWorker worker) {
        SSLContext sslContext;
        try {
            sslContext = SSLContexts.custom().loadTrustMaterial(null, TrustAllStrategy.INSTANCE).build();
        } catch (NoSuchAlgorithmException | KeyManagementException | KeyStoreException e) {
            log.warn("Custom SSLContext Error: {}. Use Default SSLContext", e.getMessage(), e);
            sslContext = SSLContexts.createDefault();
        }
        RequestConfig requestConfig = RequestConfig.custom().setConnectionRequestTimeout(10000).setConnectTimeout(10000)
            .setSocketTimeout(20 * 60 * 1000).setCookieSpec(CookieSpecs.STANDARD).build();
        CloseableHttpAsyncClient httpAsyncClient = HttpAsyncClients.custom().setDefaultRequestConfig(requestConfig)
            .setSSLContext(sslContext).setSSLHostnameVerifier(NoopHostnameVerifier.INSTANCE).setMaxConnTotal(100)
            .setMaxConnPerRoute(100).build();
        try {
            httpAsyncClient.start();
            SaltClient client = new SaltClient(URI.create(worker.getProtocol() + "://" + worker.getInstance()),
                new HttpAsyncClientImpl(httpAsyncClient));
            Token token = client
                .login(worker.getUsername(), new AESEncryptor().decrypt(worker.getPassword(), null), AuthModule.PAM)
                .toCompletableFuture().join();
            if (null == token || StringUtils.isEmpty(token.getToken())) {
                return new GeneralResult<Void>(false, "校验SaltMaster连接失败，认证信息可能有误");
            }
            return new GeneralResult<Void>(true, "success");
        } catch (Exception e) {
            log.error("check salt master error:", e);
            return new GeneralResult<Void>(false, "校验SaltMaster连接失败：" + e.getMessage());
        } finally {
            if (null != httpAsyncClient) {
                try {
                    httpAsyncClient.close();
                } catch (IOException e) {
                }
            }
        }
    }

    @Override
    public GeneralResult<TaskWorker> detail(Long id) {
        try {
            TaskWorker worker = this.taskWorkerRepository.query(id);
            Assert.notNull(worker, "worker数据不存在！");
            return new GeneralResult<>(true, worker, "查询worker详细成功");
        } catch (Exception e) {
            log.error("Query network segment fail:", e);
            return new GeneralResult<>(false, "查询worker详细失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = TaskWorker.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            TaskWorker taskWorker = this.taskWorkerRepository.query(id);
            Assert.notNull(taskWorker, "worker数据不存在！");
            boolean result = this.taskWorkerRepository.delete(id, userId);
            // 更新缓存
            refreshSaltCache(OperationConstant.DELETE, taskWorker, null);
            // 清楚taskHost表中salt相关信息
            if (TaskWorker.Category.SaltMaster.equals(taskWorker.getCategory())) {
                removeTaskHostSaltInfo(taskWorker.getId());
            }
            return result ? new GeneralResult<>(true, "删除worker成功") : new GeneralResult<>(false, "删除worker失败");
        } catch (Exception e) {
            log.error("Remove task worker fail:", e);
            return new GeneralResult<>(false, "删除worker失败");
        }
    }

    @Override
    public GeneralResult<Void> modify(TaskWorker worker) {
        String path = TaskWorker.class.getSimpleName() + "_" + worker.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            TaskWorker taskWorker = taskWorkerRepository.query(worker.getId());
            Assert.notNull(taskWorker, "worker数据不存在");
            // 检查数据是否重复
            TaskWorker bean = taskWorkerRepository.exist(worker.getInstance(), worker.getCategory().name());
            if (null != bean && bean.getId().compareTo(worker.getId()) != 0) {
                return new GeneralResult<>(false, "修改worker失败，已存在相同的实例");
            }
            if (TaskWorker.Category.SaltMaster.equals(worker.getCategory())) {
                GeneralResult<Void> result = checkSalt(worker);
                if (result.isFailed()) {
                    return result;
                }
            }
            Long oldGroupId = taskWorker.getGroupId();
            boolean result = taskWorkerRepository.update(worker);
            // 更新缓存
            refreshSaltCache(OperationConstant.UPDATE, worker, oldGroupId);
            return result ? new GeneralResult<>(true, "修改worker成功") : new GeneralResult<>(false, "修改worker失败");
        } catch (Exception e) {
            log.error("Modify task worker fail:", e);
            return new GeneralResult<>(false, "修改worker失败");
        }
    }

    @Override
    public GeneralResult<Void> configure(Long workerId, Long groupId, Boolean isSuper) {
        String path = TaskWorker.class.getSimpleName() + "_" + workerId;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            TaskWorker worker = taskWorkerRepository.query(workerId);
            if (null == worker) {
                return new GeneralResult<>(false, "worker数据不存在");
            }
            if (worker.getDeleted()) {
                return new GeneralResult<>(false, "当前worker处于异常状态,不支持配置!");
            }
            Long oldGroupId = worker.getGroupId();
            NetworkGroup group = networkGroupRepository.query(groupId);
            if (null == group) {
                return new GeneralResult<>(false, "分组数据不存在");
            }
            worker.setGroupId(groupId);
            isSuper = null == isSuper ? false : isSuper;
            worker.setIsSuper(isSuper);
            if (TaskWorker.Category.SaltMaster.equals(worker.getCategory())) {
                boolean update = taskWorkerRepository.update(worker);
                if (update) {
                    refreshSaltCache(OperationConstant.UPDATE, worker, oldGroupId);
                }
                return update ? new GeneralResult<>(true, "配置worker成功") : new GeneralResult<>(false, "配置worker失败");
            }
            boolean update = taskWorkerRepository.update(worker);
            if (update) {
                refreshCache(oldGroupId, groupId, worker, isSuper);
            }
            return update ? new GeneralResult<>(true, "配置worker成功") : new GeneralResult<>(false, "配置worker失败");
        } catch (Exception e) {
            log.error("Configure task worker fail:", e);
            return new GeneralResult<>(false, "配置worker失败");
        }
    }

    private void refreshCache(Long oldGroupId, Long newGroupId, TaskWorker worker, Boolean isSuper) {
        if (newGroupId.equals(oldGroupId)) {
            if (isSuper) {
                // worker isSuper属性变为true，则在分组worker里面移除worker
                workerCache.delCache(CacheConstant.WORKERGROUP_ + newGroupId, worker.getInstance());
            } else {
                // worker isSuper属性变为false，则在分组worker里面添加worker
                workerCache.addCache(CacheConstant.WORKERGROUP_ + newGroupId, worker.getInstance());
            }
        } else {
            if (null != oldGroupId) {
                // worker都需从原分组中移除掉
                workerCache.delCache(CacheConstant.WORKERGROUP_ + oldGroupId, worker.getInstance());
            }
            if (!isSuper) {
                // worker isSuper属性变为false，则在新的分组worker里面添加worker
                workerCache.addCache(CacheConstant.WORKERGROUP_ + newGroupId, worker.getInstance());
            }
        }
        if (isSuper) {
            workerCache.addCache(CacheConstant.SUPER_WORKERS, worker.getInstance());
        } else {
            workerCache.delCache(CacheConstant.SUPER_WORKERS, worker.getInstance());
        }
    }

    /**
     * @param type
     *            操作类型
     * @param worker
     *            操作对象
     */
    private void refreshSaltCache(String type, TaskWorker worker, Long oldGroupId) {
        String key = CacheConstant.SALTMASTER_ + worker.getId();
        if (OperationConstant.DELETE.equals(type)) {
            // 移除salt缓存
            removeSaltCache(key, worker.getId());
        } else {
            // 添加、修改缓存
            addSaltChche(key, worker);
        }
    }

    // 向缓存中添加一个值
    private void addSaltChche(String key, TaskWorker worker) {
        String value = redisTemplate.opsForValue().get(key);
        SaltCacheModel model = JSONObject.parseObject(value, SaltCacheModel.class);
        if (null == model) {
            model = new SaltCacheModel();
        }
        model.add(worker);
        redisTemplate.opsForValue().set(key, JSONObject.toJSONString(model));
    }

    /**
     * 移除缓存中的一个值
     *
     * @param key
     */
    private void removeSaltCache(String key, Long id) {
        String value = redisTemplate.opsForValue().get(key);
        SaltCacheModel model = JSONObject.parseObject(value, SaltCacheModel.class);
        if (null != model) {
            model.remove(id);
            if (ListTool.isEmpty(model.getSaltModels())) {
                redisTemplate.delete(key);
                return;
            }
            redisTemplate.opsForValue().set(key, JSONObject.toJSONString(model));
        }
    }

    @Override
    public GeneralResult<Void> exclude(List<Long> workerIds, RequestContext context) {
        try {
            List<TaskWorker> workers = taskWorkerRepository.listByWorkers(workerIds);
            // 处理普通worker
            List<TaskWorker> generalWorkers =
                workers.stream().filter(worker -> !worker.getIsSuper()).collect(Collectors.toList());
            Map<Long, List<TaskWorker>> workerMap = generalWorkers.stream().filter(worker ->
                null != worker.getGroupId()
            ).collect(Collectors.groupingBy(TaskWorker::getGroupId));
            // worker排除检验
            List<String> errors = new ArrayList<>();
            for (Long groupId : workerMap.keySet()) {
                GeneralResult<Void> result = excludeGroupCheck(groupId, workerMap.get(groupId));
                if (result.isFailed()) {
                    errors.add(result.getMessage());
                }
            }
            if (!ListTool.isEmpty(errors)) {
                return new GeneralResult<>(false, "worker排除失败:" + errors);
            }
            for (Long groupId : workerMap.keySet()) {
                GeneralResult<Void> result =
                    excludeAndRecoveryWorkers(groupId, workerMap.get(groupId), context, "exclude", false);
                if (result.isFailed()) {
                    return result;
                }
            }
            // 处理超级worker
            List<TaskWorker> superWorkers =
                workers.stream().filter(TaskWorker::getIsSuper).collect(Collectors.toList());
            if (!ListTool.isEmpty(superWorkers)) {
                GeneralResult<Void> result = excludeAndRecoveryWorkers(null, superWorkers, context, "exclude", true);
                if (result.isFailed()) {
                    return result;
                }
            }
            return new GeneralResult<>(true, "worker排除成功");
        } catch (Exception e) {
            log.error("network group exclude workers is fail:", e);
            return new GeneralResult<>(false, "worker排除失败");
        }
    }

    private GeneralResult<Void> excludeAndRecoveryWorkers(Long id, List<TaskWorker> currentWorkers,
        RequestContext context, String action, boolean isSuper) {
        String path = TaskWorker.class.getSimpleName() + "_excludeWorker_" + (isSuper ? "super" : id);
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(60, TimeUnit.SECONDS), "请求超时");
            List<TaskWorker> workers = null;
            if (isSuper) {
                workers = taskWorkerRepository.listSupers();
            } else {
                workers = taskWorkerRepository.listGenerals(id, TaskWorker.Category.Ansible.name());
            }
            List<TaskWorker> taskWorkers;
            Map<Long, TaskWorker> workerMap =
                currentWorkers.stream().collect(Collectors.toMap(TaskWorker::getId, Function.identity()));
            if ("exclude".equals(action)) {
                taskWorkers =
                    workers.stream().filter(worker -> workerMap.containsKey(worker.getId()) || (null != worker.getIsExclude() && worker.getIsExclude()))
                        .collect(Collectors.toList());
                if (workers.size() - taskWorkers.size() < 1) {
                    return new GeneralResult<>(false, "排除worker实例后少于1个节点，请检查！");
                }
            } else if ("recovery".equals(action)) {
                taskWorkers = workers.stream().filter(TaskWorker::getIsExclude)
                    .filter(worker -> !workerMap.containsKey(worker.getId())).collect(Collectors.toList());
            } else {
                return new GeneralResult<>(false, "未知的操作");
            }
            // 先获取redis上存在的worker负载信息，然后更新负载信息
            List<String> excludeWorkers =
                taskWorkers.stream().map(TaskWorker::getInstance).collect(Collectors.toList());
            String key = isSuper ? CacheConstant.SUPER_WORKERS : CacheConstant.WORKERGROUP_ + id;
            GeneralResult<Void> result = addCache(key, excludeWorkers);
            if (result.isFailed()) {
                return result;
            }
            // 更新数据库
            List<TaskWorker> updateWorkers = currentWorkers.stream().peek(worker -> {
                worker.setIsExclude("exclude".equals(action));
                worker.setMenderId(context.getTarget());
                worker.setGmtModify(new Date());
            }).collect(Collectors.toList());
            taskWorkerRepository.batchUpdate(updateWorkers, 100);
            return new GeneralResult<>(true, "网段分组内worker排除恢复成功");
        } catch (Exception e) {
            log.error("network group exclude recovery workers is fail:", e);
            return new GeneralResult<>(false, "网段分组【" + (isSuper ? "super" : id) + "】内worker排除恢复失败");
        }
    }

    public GeneralResult<Void> addCache(String key, List<String> excludeWorkers) {
        try (AutoCloseLock lock = lockFactory.getACLock(key)) {
            Assert.isTrue(lock.acquire(60, TimeUnit.SECONDS), "请求超时");
            String value = redisTemplate.opsForValue().get(key);
            WorkerLoadInfo info = JSONObject.parseObject(value, WorkerLoadInfo.class);
            log.info("网段分组【" + key + "】的排除worker缓存");
            if (null == info) {
                return new GeneralResult<>(false, "当前网段分组【" + key + "】缓存数据不存在");
            }
            info.setExcludeWorkers(excludeWorkers);
            redisTemplate.opsForValue().set(key, JSONObject.toJSONString(info));
            return new GeneralResult<>(true, "配置排除worker负载缓存成功");
        } catch (Exception e) {
            log.error("配置排除worker负载缓存失败:", e);
            return new GeneralResult<>(false, "网络分组【" + key + "】配置排除worker负载缓存失败");
        }
    }

    private GeneralResult<Void> excludeGroupCheck(Long id, List<TaskWorker> currentWorkers) {
        try {
            NetworkGroup group = this.networkGroupRepository.query(id);
            if (null == group) {
                return new GeneralResult<>(false, "网段分组【" + id + "】数据不存在！");
            }
            // 网段分组下的所有worker
            List<TaskWorker> groupWorkers = taskWorkerRepository.listGenerals(id, TaskWorker.Category.Ansible.name());
            Map<Long, TaskWorker> workerMap =
                currentWorkers.stream().collect(Collectors.toMap(TaskWorker::getId, Function.identity()));
            // 排除的worker
            List<TaskWorker> excludeWorkers =
                groupWorkers.stream().filter(worker -> workerMap.containsKey(worker.getId()) || (null != worker.getIsExclude() && worker.getIsExclude()))
                    .collect(Collectors.toList());
            if (groupWorkers.size() - excludeWorkers.size() < 1) {
                return new GeneralResult<>(false, "网段分组【" + group.getName() + "】排除worker实例后少于1个节点！");
            }
            return new GeneralResult<>(true, "网段分组内worker排除检查成功");
        } catch (Exception e) {
            log.error("network group exclude  workers check is fail:", e);
            return new GeneralResult<>(false, "网段分组【" + id + "】内worker排除检查失败");
        }
    }

    @Override
    public GeneralResult<Void> recovery(List<Long> workerIds, RequestContext context) {
        try {
            List<TaskWorker> workers = taskWorkerRepository.listByWorkers(workerIds);
            // 处理普通worker
            List<TaskWorker> generalWorkers =
                workers.stream().filter(worker -> !worker.getIsSuper()).collect(Collectors.toList());
            Map<Long, List<TaskWorker>> workerMap = generalWorkers.stream().filter(worker -> {
                return null != worker.getGroupId();
            }).collect(Collectors.groupingBy(TaskWorker::getGroupId));
            // 遍历处理分组内worker
            for (Long groupId : workerMap.keySet()) {
                GeneralResult<Void> result =
                    excludeAndRecoveryWorkers(groupId, workerMap.get(groupId), context, "recovery", false);
                if (result.isFailed()) {
                    return result;
                }
            }
            // 处理超级worker
            List<TaskWorker> superWorkers =
                workers.stream().filter(TaskWorker::getIsSuper).collect(Collectors.toList());
            if (!ListTool.isEmpty(superWorkers)) {
                GeneralResult<Void> result = excludeAndRecoveryWorkers(null, superWorkers, context, "recovery", true);
                if (result.isFailed()) {
                    return result;
                }
            }
            return new GeneralResult<>(true, "worker排除恢复成功");
        } catch (Exception e) {
            log.error("network group exclude recovery workers is fail:", e);
            return new GeneralResult<>(false, "worker排除恢复失败");
        }
    }

    @Override
    public GeneralResult<List<SaltMinionModel>> minions(TaskWorker worker) {
        SSLContext sslContext;
        try {
            sslContext = SSLContexts.custom().loadTrustMaterial(null, TrustAllStrategy.INSTANCE).build();
        } catch (NoSuchAlgorithmException | KeyManagementException | KeyStoreException e) {
            log.warn("Custom SSLContext Error: {}. Use Default SSLContext", e.getMessage(), e);
            sslContext = SSLContexts.createDefault();
        }
        RequestConfig requestConfig = RequestConfig.custom().setConnectionRequestTimeout(10000).setConnectTimeout(10000)
            .setSocketTimeout(20 * 60 * 1000).setCookieSpec(CookieSpecs.STANDARD).build();
        CloseableHttpAsyncClient httpAsyncClient = HttpAsyncClients.custom().setDefaultRequestConfig(requestConfig)
            .setSSLContext(sslContext).setSSLHostnameVerifier(NoopHostnameVerifier.INSTANCE).setMaxConnTotal(100)
            .setMaxConnPerRoute(100).build();
        try {
            // 获取minions
            httpAsyncClient.start();
            AsyncHttpClient httpClient = new HttpAsyncClientImpl(httpAsyncClient);
            String server = worker.getProtocol() + "://" + worker.getInstance();
            SaltClient client = new SaltClient(URI.create(server), httpClient);
            Token token = client
                .login(worker.getUsername(), new AESEncryptor().decrypt(worker.getPassword(), null), AuthModule.PAM)
                .toCompletableFuture().join();
            URI uri = URI.create(server).resolve("/minions");
            Map<String, Object> response =
                httpClient.get(uri, Collections.singletonMap("X-Auth-Token", token.getToken()), JsonParser.MAP)
                    .toCompletableFuture().join();
            JSONArray returns = JSON.parseObject(JSON.toJSONString(response)).getJSONArray("return");

            List<SaltMinionModel> minions = new ArrayList<>();
            for (int i = 0; i < returns.size(); i++) {
                JSONObject minionList = returns.getJSONObject(i);
                for (Map.Entry<String, Object> entry : minionList.entrySet()) {
                    if (entry.getValue() instanceof Boolean) {
                        continue;
                    }
                    SaltMinionModel host = new SaltMinionModel();
                    JSONObject value = JSON.parseObject(entry.getValue().toString());
                    host.setOs(value.getString("os"));
                    host.setKernel(value.getString("kernel"));
                    host.setOsMajorRelease(value.getString("osmajorrelease"));
                    host.setHost(value.getString("host"));
                    host.setSaltMinionId(entry.getKey());
                    List<String> ipv4List = value.getJSONArray("ipv4").toJavaList(String.class);
                    StringBuilder ipv4s = new StringBuilder();
                    for (String ipv4 : ipv4List) {
                        if ("127.0.0.1".equals(ipv4)) {
                            continue;
                        }
                        ipv4s.append(ipv4).append(",");
                    }
                    host.setIpv4s(ipv4s.length() > 0 ? ipv4s.substring(0, ipv4s.length() - 1) : ipv4s.toString());
                    List<String> ipv6List = value.getJSONArray("ipv6").toJavaList(String.class);
                    StringBuilder ipv6s = new StringBuilder();
                    for (String ipv6 : ipv6List) {
                        if ("::1".equals(ipv6)) {
                            continue;
                        }
                        ipv6s.append(ipv6).append(",");
                    }
                    host.setIpv6s(ipv6s.length() > 0 ? ipv6s.substring(0, ipv6s.length() - 1) : ipv6s.toString());
                    minions.add(host);
                }
            }
            return new GeneralResult<>(true, minions, "查询minion成功");
        } catch (Exception e) {
            log.error("查询minons失败:", e);
            return new GeneralResult<>(false, "查询minion失败:" + e.getMessage());
        }
    }

    /**
     * 删除主机资源中salt相关信息
     * 
     * @param saltId
     *            saltMasterId
     */
    private void removeTaskHostSaltInfo(Long saltId) {
        // SaltMaster不存在时，删除数据源是Salt的主机资源
        taskHostRepository.removeBySourceType(TaskHostType.SourceType.SALT.name(), saltId);
        // 其他数据的主机资源，删除saltId并在连接协议中删除salt协议
        List<TaskHost> hosts = taskHostRepository.listBySaltId(saltId);
        hosts.forEach(host -> {
            host.setSaltId(null);
            if (StringUtils.isNotBlank(host.getConnProtocol())) {
                List<String> protocols =
                    JSONArray.parseArray(host.getConnProtocol().replaceAll("\\\\", ""), String.class);
                protocols.remove(TaskHostType.ConnProtocol.SaltStack.name());
                host.setConnProtocol(protocols.size() > 0 ? JSONObject.toJSONString(protocols) : null);
            }
            taskHostRepository.update(host);
        });
    }


    @Override
    public GeneralResult<?> updateRemark(Long id,String remark,Long userId) {
        try {
            TaskWorker worker = this.taskWorkerRepository.query(id);
            Assert.notNull(worker, "worker数据不存在！");
            taskWorkerRepository.update(id, remark, userId);
            return new GeneralResult<>(true, "更新成功");
        } catch (Exception e) {
            log.error("update remark fail:", e);
            return new GeneralResult<>(false, "更新失败");
        }
    }

}
