package com.bocloud.cop.service.utils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import com.bocloud.cop.entity.inspection.InspectReport;

import lombok.extern.slf4j.Slf4j;

/**
 * 巡检报表工具日期转换
 */
@Slf4j
public class DateTransfer {

    /**
     * 获取计算后的时间
     *
     * @param period
     *            周期
     * @param choose
     *            本次/下次（天/周/月）
     * @param generateTime
     *            指定小时 00:00
     * @param day
     *            号
     * @return 计算后的时间
     */
    public static Date getDate(Date date, String period, String choose, String generateTime, Integer week,
        Integer day) {
        // 使用Calendar 获取当前日期和时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        // Date currentTime = calendar.getTime();
        Date currentTime = date;
        // 转换格式 使用format将这个日期转换成我们需要的格式
        SimpleDateFormat sdfDay = new SimpleDateFormat("yyyy-MM-dd ");
        /*period = "Month"; choose = "current"; generateTime = "18:00";*/

        String transferTime = null;
        if ("Day".equals(period)) {
            // 日报
            if ("current".equals(choose)) {
                transferTime = sdfDay.format(currentTime) + generateTime;
            } else if ("next".equals(choose)) {
                currentTime = getNextPeriod(currentTime, "day");
                transferTime = sdfDay.format(currentTime) + generateTime;
            }
        } else if ("Week".equals(period)) {
            // 周报
            Date weekDate = getWeekDate(currentTime, choose, week);
            transferTime = sdfDay.format(weekDate) + generateTime;
        } else if ("Month".equals(period)) {
            // 月报
            Date monthDate = getMonthDate(date, choose, day);
            transferTime = sdfDay.format(monthDate) + generateTime;
        }
        transferTime = transferTime + ":00";
        try {
            return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse(transferTime);
        } catch (ParseException e) {
            throw new RuntimeException("格式转换错误");
        }
    }

    /**
     * 获取当前日期的月份有多少天，即最后一天为多少号
     *
     * @param date
     *            当前日期
     */
    public static int getCurrentMonthLastDay(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        // 把日期设置为当月第一天
        calendar.set(Calendar.DATE, 1);
        // 日期回滚一天，也就是最后一天
        calendar.roll(Calendar.DATE, -1);
        return calendar.get(Calendar.DATE);
    }

    /**
     * 计算差几号的日期
     *
     * @param choose
     *            本月/下月
     * @param day
     *            号
     */
    public static Date getMonthDate(Date date, String choose, int day) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);

        int currentMonthDay = calendar.get(Calendar.DAY_OF_MONTH);
        int createDay;
        if ("current".equals(choose)) {
            if (day >= currentMonthDay) {
                createDay = day - currentMonthDay;
            } else {
                createDay = getCurrentMonthLastDay(new Date()) - currentMonthDay + day;
                log.info("已推迟下个月的" + day + "号执行");
            }
        } else if ("next".equals(choose)) {
            createDay = getCurrentMonthLastDay(date) - currentMonthDay + day;
        } else {
            throw new RuntimeException("choose不合法");
        }

        calendar.add(Calendar.DAY_OF_MONTH, createDay);
        return calendar.getTime();
    }

    /**
     * 计算差几周的日期
     *
     * @param choose
     *            本周/下周
     * @param week
     *            星期
     */
    public static Date getWeekDate(Date date, String choose, int week) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        int currentWeek = getDateWeek(date);
        int createDay;
        // 多一天,i wanna why
        // week += 1;
        if ("current".equals(choose)) {
            if (week >= currentWeek) {
                createDay = week - currentWeek;
            } else {
                createDay = 7 - currentWeek + week;
                log.info("已推迟下周的" + InspectReport.Week.values()[week].getName() + "执行");
            }
        } else if ("next".equals(choose)) {
            createDay = 7 - currentWeek + week;
        } else {
            throw new RuntimeException("不合法");
        }

        calendar.add(Calendar.DAY_OF_MONTH, createDay);
        return calendar.getTime();
    }

    /**
     * 获取当前星期几
     */
    public static int getDateWeek(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        return c.get(Calendar.DAY_OF_WEEK) - 1;
    }

    /**
     * 获取指定日期的下一天/下一周/下一月日期
     *
     * @param date
     *            给定指定日期
     * @param next
     *            日/周/月 1/7/31
     * @return 计算后的date
     */
    public static Date getNextPeriod(Date date, String next) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        switch (next) {
            case "day":
                calendar.add(Calendar.DAY_OF_MONTH, 1);
                break;
            case "week":
                calendar.add(Calendar.WEEK_OF_YEAR, 1);
                break;
            case "month":
                calendar.add(Calendar.MONTH, 1);
                break;
            default:
                throw new RuntimeException("日期格式不对，转换失败");
        }
        return calendar.getTime();
    }
}
