package com.bocloud.cop.service.utils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.Task;
import com.bocloud.cop.model.*;
import com.bocloud.cop.model.params.DeployGaiaParams;
import com.bocloud.cop.service.utils.worker.WorkerDataUtils;
import com.megatron.common.model.GeneralResult;
import org.apache.commons.lang3.StringUtils;

import java.util.*;

public class DeployGaiaTaskBuilder {
    public static Task getTemplateTask(DeployGaiaParams params) {
        JSONObject task = new JSONObject();
        task.put("code", "standard");
        String name = "gaia部署任务";
        String type = params.getArgs().getType();
        if ("add".equalsIgnoreCase(type)) {
            name = "gaia部署任务";
        } else if ("delete".equalsIgnoreCase(type)) {
            name = "gaia卸载任务";
        }
        task.put("name", name);
        task.put("sceneId", 8);
        JSONArray nodes = new JSONArray();
        List<TaskEdge> edges = new ArrayList<>();
        // 配置任务节点信息
        getGaiaNodes(params, nodes, edges);
        JSONObject graph = new JSONObject();
        graph.put("nodes", nodes);
        graph.put("edges", edges);
        task.put("graph", graph);
        return JSONObject.parseObject(task.toJSONString(), Task.class);
    }

    private static void getGaiaNodes(DeployGaiaParams params, JSONArray nodes, List<TaskEdge> edges) {
        Map<String, List<WorkerHost>> userMap = new HashMap<>();
        Target target = params.getTarget();
        List<WorkerHost> data = target.getData();
        for (WorkerHost host : data) {
            String key = host.getBecomeUser();
            key = StringUtils.isEmpty(key) ? "nouser" : key;
            if (userMap.containsKey(key)) {
                List<WorkerHost> list = userMap.get(key);
                list.add(host);
                userMap.put(key, list);
            } else {
                List<WorkerHost> list = new ArrayList<>();
                list.add(host);
                userMap.put(key, list);
            }
        }
        String name = "gaia部署任务节点";
        String type = params.getArgs().getType();
        if ("add".equalsIgnoreCase(type)) {
            name = "gaia部署任务节点";
        } else if ("delete".equalsIgnoreCase(type)) {
            name = "gaia卸载任务节点";
        }
        Iterator<String> iterator = userMap.keySet().iterator();
        int x = 400;
        int index = 1;
        int y = 50;
        int num = 0;
        int pre = 10;
        while (iterator.hasNext()) {
            String key = iterator.next();
            GeneralResult<TargetGroup> groupTarget = WorkerDataUtils.groupTargetByWorkerHost(userMap.get(key));
            List<List<WorkerHost>> linuxs = WorkerDataUtils.splitTarget(groupTarget.getData().getLinuxs());
            List<List<WorkerHost>> windows = WorkerDataUtils.splitTarget(groupTarget.getData().getWindows());
            for (List<WorkerHost> linux : linuxs) {
                JSONObject node = new JSONObject();
                node.put("name", name + "(" + pre + "" + index + ")");
                node.put("code", "task.maintain.gaia");
                node.put("moduleType", "task.maintain.gaia");
                NodeConfig config = new NodeConfig(UUID.randomUUID().toString(), num, x, y,
                        "script.png", "maintain-node");
                NodeControl control = new NodeControl(false, false, false, 3, 10,
                        null == params.getTimeout() ? 3600 : params.getTimeout());
                node.put("control", control);
                node.put("config", config);
                node.put("input",
                        getGaiaInput(params, target, linux));
                nodes.add(node);
                num++;
                y = y + 100;
                index++;
            }
            for (List<WorkerHost> window : windows) {
                JSONObject node = new JSONObject();
                node.put("name", name + "(" + pre + "" + index + ")");
                node.put("code", "task.maintain.gaia");
                node.put("moduleType", "task.maintain.gaia");
                NodeConfig config = new NodeConfig(UUID.randomUUID().toString(), num, x, y,
                        "script.png", "maintain-node");
                NodeControl control = new NodeControl(false, false, false, 3, 10,
                        null == params.getTimeout() ? 3600 : params.getTimeout());
                node.put("control", control);
                node.put("config", config);
                node.put("input",
                        getGaiaInput(params, target, window));
                nodes.add(node);
                num++;
                y = y + 100;
                index++;
            }
            y = 50;
            x = x + 250;
            index = 1;
            pre++;
        }
    }

    private static JSONObject getGaiaInput(DeployGaiaParams params, Target targetInfo, List<WorkerHost> hosts) {
        JSONObject input = new JSONObject();
        JSONObject fronts = new JSONObject();
        fronts.put("name", params.getName());
        fronts.put("args", JSONObject.parseObject(JSONObject.toJSONString(params.getArgs())));
        input.put("fronts", fronts);

        JSONObject target = new JSONObject();
        target.put("authenMode", targetInfo.getAuthenMode());
        target.put("sudo", targetInfo.getSudo());
        target.put("becomeUser", targetInfo.getBecomeUser());
        target.put("becomePass", targetInfo.getBecomePass());
        target.put("executeType", targetInfo.getExecuteType());
        // target.put("shareHostPool", targetInfo.getShareHostPool());
        target.put("identId", targetInfo.getIdentId());
        target.put("data", hosts);
        input.put("target", target);
        return input;
    }

}
