package com.bocloud.cop.service.utils;

import java.util.List;
import java.util.Map;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.TaskNode;
import com.bocloud.cop.model.NodeConfig;
import com.bocloud.cop.model.TaskEdge;
import com.bocloud.cop.model.TaskNodeBean;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.utils.ListTool;

/**
 * 任务节点帮助类
 */
public class NodeHelper {
    /**
     * 得到当前节点的最大数据
     *
     * @param nodeId
     * @param params
     * @return
     */
    public static boolean isCycle(String nodeId, Map<String, List<String>> params, int size) {
        List<String> nodes = params.get(nodeId);
        if (ListTool.isEmpty(nodes)) {
            return false;
        }
        int max = 1;
        return preNode(nodes, params, max, size);
    }

    /**
     * 得到节点之前的最大路径
     *
     * @param nodes
     * @param param
     * @param max
     * @return
     */
    private static boolean preNode(List<String> nodes, Map<String, List<String>> param, int max, int size) {
        List<String> preNodes = Lists.newArrayList();
        for (String node : nodes) {
            List<String> ids = param.get(node);
            if (null == ids) {
                continue;
            }
            for (String id : ids) {
                if (!preNodes.contains(id)) {
                    preNodes.add(id);
                }
            }
        }
        if (preNodes.isEmpty()) {
            return false;
        }
        max++;
        if (max > size) {
            return true;
        }
        return preNode(preNodes, param, max, size);
    }

    public static Map<String, String> getIngoreNodes(List<TaskNode> nodes) {
        Map<String, String> map = Maps.newHashMap();
        for (TaskNode node : nodes) {
            NodeConfig config = JSONObject.parseObject(node.getConfig(), NodeConfig.class);
            if (config.isIgnore()) {
                map.put(config.getId(), config.getPid());
            }
        }
        return map;
    }

    public static Map<String, String> getIngoreNodesForBean(List<TaskNodeBean> nodes) {
        Map<String, String> map = Maps.newHashMap();
        for (TaskNodeBean node : nodes) {
            NodeConfig config = node.getConfig();
            if (config.isIgnore()) {
                map.put(config.getId(), config.getPid());
            }
        }
        return map;
    }

    /**
     * 解析节点之间关系，节点对应的前节点
     *
     * @param edges
     * @param ingoreNodes
     * @return
     */
    public static Map<String, List<String>> getPreLinks(List<TaskEdge> edges, Map<String, String> ingoreNodes) {
        Map<String, List<String>> params = Maps.newHashMap();
        if (ListTool.isEmpty(edges)) {
            return params;
        }
        if (ingoreNodes.isEmpty()) {
            for (TaskEdge edge : edges) {
                put(params, edge.getTarget(), edge.getSource());
            }
            return params;
        }
        for (TaskEdge edge : edges) {
            if (edge.isIgnore()) {
                continue;
            }
            String source = edge.getSource();
            source = ingoreNodes.containsKey(source) ? ingoreNodes.get(source) : source;
            String target = edge.getTarget();
            target = ingoreNodes.containsKey(target) ? ingoreNodes.get(target) : target;
            put(params, target, source);
        }
        return params;
    }

    private static void put(Map<String, List<String>> params, String key, String value) {
        if (null == params.get(key)) {
            List<String> list = Lists.newArrayList();
            list.add(value);
            params.put(key, list);
        } else {
            params.get(key).add(value);
        }
    }

    /**
     * 解析节点之间关系，节点对应的后节点
     *
     * @param edges
     * @param ingoreNodes
     * @return
     */
    public static Map<String, List<String>> getNextLinks(List<TaskEdge> edges, Map<String, String> ingoreNodes) {
        Map<String, List<String>> params = Maps.newHashMap();
        if (ListTool.isEmpty(edges)) {
            return params;
        }
        if (ingoreNodes.isEmpty()) {
            for (TaskEdge edge : edges) {
                put(params, edge.getSource(), edge.getTarget());
            }
            return params;
        }
        for (TaskEdge edge : edges) {
            if (edge.isIgnore()) {
                continue;
            }
            String source = edge.getSource();
            source = ingoreNodes.containsKey(source) ? ingoreNodes.get(source) : source;
            String target = edge.getTarget();
            target = ingoreNodes.containsKey(target) ? ingoreNodes.get(target) : target;
            put(params, source, target);
        }
        return params;
    }

    public static Map<String, List<TaskNode>> getNextNodes(List<TaskEdge> edges, Map<String, String> ingoreNodes,
        List<TaskNode> nodes) {
        Map<String, List<TaskNode>> nextNodes = Maps.newHashMap();
        if (ListTool.isEmpty(edges)) {
            return nextNodes;
        }
        Map<String, TaskNode> nodeMap = Maps.newHashMap();
        for (TaskNode node : nodes) {
            nodeMap.put(node.getNodeId(), node);
        }
        for (TaskEdge edge : edges) {
            if (edge.isIgnore()) {
                continue;
            }
            String source = edge.getSource();
            source = ingoreNodes.containsKey(source) ? ingoreNodes.get(source) : source;
            String target = edge.getTarget();
            target = ingoreNodes.containsKey(target) ? ingoreNodes.get(target) : target;
            if (null == nextNodes.get(source)) {
                List<TaskNode> list = Lists.newArrayList();
                list.add(nodeMap.get(target));
                nextNodes.put(source, list);
            } else {
                nextNodes.get(source).add(nodeMap.get(target));
            }
        }
        return nextNodes;
    }

    /**
     * 解析节点之间关系，节点对应的出度
     *
     * @param edges
     * @return
     */
    public static Map<String, List<TaskEdge>> getNextEdges(List<TaskEdge> edges, Map<String, String> ingoreNodes) {
        Map<String, List<TaskEdge>> params = Maps.newHashMap();
        if (ListTool.isEmpty(edges)) {
            return params;
        }
        for (TaskEdge edge : edges) {
            if (edge.isIgnore()) {
                continue;
            }
            String source = edge.getSource();
            source = ingoreNodes.containsKey(source) ? ingoreNodes.get(source) : source;
            if (null == params.get(source)) {
                List<TaskEdge> list = Lists.newArrayList();
                list.add(edge);
                params.put(source, list);
            } else {
                params.get(source).add(edge);
            }
        }
        return params;
    }

}
