package com.bocloud.cop.service.utils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.Task;
import com.bocloud.cop.model.NodeConfig;
import com.bocloud.cop.model.NodeControl;
import com.bocloud.cop.model.TaskEdge;
import com.bocloud.cop.model.params.ScriptTaskParams;
import com.bocloud.cop.model.params.TaskArgs;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.encrypt.Encryptor;
import com.megatron.common.utils.IDFactory;
import com.megatron.common.utils.ListTool;
import org.apache.commons.lang3.StringUtils;
import org.apache.curator.shaded.com.google.common.collect.Lists;

import java.util.*;

public class TaskApiBuilder {

    /**
     * 构建通过playbook模板構建主机任务
     *
     * @param params
     * @return
     */
    public static Task getTemplateTask(ScriptTaskParams params) {
        JSONObject task = new JSONObject();
        task.put("code", "standard");
        String name = StringUtils.isEmpty(params.getTaskAlias()) ? "作业任务" : params.getTaskAlias();
        task.put("name", name);
        task.put("sceneId", 8);
        JSONArray nodes = new JSONArray();
        List<TaskEdge> edges = new ArrayList<>();
        // 配置任务节点信息
        getTempNodes(params, nodes, edges);
        JSONObject graph = new JSONObject();
        graph.put("nodes", nodes);
        graph.put("edges", edges);
        task.put("graph", graph);
        return JSONObject.parseObject(task.toJSONString(), Task.class);
    }

    private static void getTempNodes(ScriptTaskParams params, JSONArray nodes, List<TaskEdge> edges) {
        Map<String, List<JSONObject>> userMap = new HashMap<>();
        List<JSONObject> targets = params.getTargets();
        Encryptor encryptor = new AESEncryptor();
        for (JSONObject target : targets) {
            String key = target.getString("becomeUser");
            key = StringUtils.isEmpty(key) ? "nouser" : key;
            JSONObject host = new JSONObject();
            host.put("host", target.getString("host"));
            host.put("rawName", target.getString("host"));
            host.put("osName", target.getString("category"));
            host.put("segmentId", target.get("segmentId"));
            host.put("user", target.getString("user"));
            String pasd = target.getString("pasd");
            // 解密
            String decPas = encryptor.decrypt(pasd, null);
            if (StringUtils.isEmpty(decPas)) {
                // 解密失败，默认为明文密码，对密码进行加密处理
                pasd = encryptor.encrypt(pasd, null);
            }
            host.put("pasd", pasd);

            if (target.containsKey("free")) {
                host.put("free", target.getBooleanValue("free"));
            }
            if (target.containsKey("becomeUser")) {
                host.put("becomeUser", target.getString("becomeUser"));
            }
            if (target.containsKey("becomePass")) {
                String becomePass = target.getString("becomePass");
                // 解密
                String decBecomePas = encryptor.decrypt(becomePass, null);
                if (StringUtils.isEmpty(decBecomePas)) {
                    // 解密失败，默认为明文密码，对密码进行加密处理
                    becomePass = encryptor.encrypt(becomePass, null);
                }
                host.put("becomePass", becomePass);
            }
            if (target.containsKey("becomeMethod")) {
                host.put("becomeMethod", target.getString("becomeMethod"));
            }


            if (userMap.containsKey(key)) {
                List<JSONObject> list = userMap.get(key);
                list.add(host);
                userMap.put(key, list);
            } else {
                List<JSONObject> list = new ArrayList<>();
                list.add(host);
                userMap.put(key, list);
            }
        }
        Iterator<String> iterator = userMap.keySet().iterator();
        int x = 400;
        int index = 1;
        int y = 50;
        int pre = 10;
        int num = 0;
        while (iterator.hasNext()) {
            String key = iterator.next();
            // 根据网段拆分
            List<JSONObject> hosts = userMap.get(key);
            Map<Long, List<JSONObject>> hostMap = new HashMap<>();
            for (JSONObject host : hosts) {
                if (hostMap.containsKey(host.getLong("segmentId"))) {
                    hostMap.get(host.getLong("segmentId")).add(host);
                } else {
                    List<JSONObject> list = new ArrayList<>();
                    list.add(host);
                    hostMap.put(host.getLong("segmentId"), list);
                }
            }
            if (params.getArgs().size() == 1) {
                for (List<JSONObject> list : hostMap.values()) {
                    List<List<JSONObject>> partitions = Lists.partition(list, 50);
                    for (List<JSONObject> partition : partitions) {
                        TaskArgs taskArgs = params.getArgs().get(0);
                        JSONObject node = new JSONObject();
                        String name =
                                StringUtils.isEmpty(taskArgs.getRemark()) ? params.getTaskAlias() : taskArgs.getRemark();
                        node.put("name", name + "(" + index + ")");
                        node.put("code", "task.maintain.template");
                        node.put("moduleType", "task.maintain.template");
                        NodeConfig config = new NodeConfig(UUID.randomUUID().toString(), index - 1, x, y, "script.png",
                                "maintain-node");
                        NodeControl control = new NodeControl(false, false, false, 3, 10,
                                null == params.getTimeout() ? 3600 : params.getTimeout());
                        node.put("control", control);
                        node.put("config", config);
                        node.put("input",
                                getTempInput(params, taskArgs, "nouser".equalsIgnoreCase(key) ? null : key, partition));
                        nodes.add(node);
                        y = y + 100;
                        index++;
                    }
                    y = 50;
                    x = x + 250;
                    index = 1;
                    pre++;
                }
            } else {
                for (List<JSONObject> list : hostMap.values()) {
                    List<List<JSONObject>> partitions = Lists.partition(list, 50);
                    for (List<JSONObject> partition : partitions) {
                        JSONObject preNode = null;
                        for (TaskArgs args : params.getArgs()) {
                            JSONObject node = new JSONObject();
                            String name =
                                    StringUtils.isEmpty(args.getRemark()) ? params.getTaskAlias() : args.getRemark();
                            node.put("name", name + "(" + pre + "" + index + ")");
                            node.put("code", "task.maintain.template");
                            node.put("moduleType", "task.maintain.template");
                            /*   NodeConfig config = new NodeConfig(IDFactory.instance().uuid(), num, x, y,
                                "/static/img/graph/script.png", "170*34", "maintain-node");*/
                            NodeConfig config =
                                    new NodeConfig(IDFactory.instance().uuid(), num, x, y, "script.png", "maintain-node");
                            NodeControl control = new NodeControl(false, false, false, 3, 10,
                                    null == params.getTimeout() ? 3600 : params.getTimeout());
                            node.put("control", control);
                            node.put("config", config);
                            node.put("input", getTempInput(params, args, "nouser".equalsIgnoreCase(key) ? null : key,
                                    JSONObject.parseArray(JSONObject.toJSONString(partition), JSONObject.class)));
                            nodes.add(node);
                            num++;
                            if (null != preNode) {
                                edges.add(
                                        getEdge(num, ((NodeConfig) preNode.get("config")).getId(), config.getId(), false));
                                num++;
                            }
                            preNode = node;
                            y = y + 100;
                            index++;
                        }
                        y = 50;
                        x = x + 250;
                        index = 1;
                        pre++;
                    }
                }
            }
        }
    }

    private static TaskEdge getEdge(int index, String source, String target, boolean ignore) {
        if (ignore) {
            return new TaskEdge(UUID.randomUUID().toString(), index, "flow-polyline", source, null, target, null, false,
                    ignore);
        }
        return new TaskEdge(UUID.randomUUID().toString(), index, "flow-polyline", source, "3", target, "2", false,
                ignore);
    }

    private static JSONObject getTempInput(ScriptTaskParams params, TaskArgs args, String executeUser, List<JSONObject> hosts) {
        JSONObject input = new JSONObject();
        JSONObject fronts = new JSONObject();
        fronts.put("name", args.getName());
        fronts.put("args", JSONObject.parseObject(JSONObject.toJSONString(args.getArgs())));
        input.put("fronts", fronts);
        JSONObject target = new JSONObject();

        target.put("authenMode", StringUtils.isEmpty(params.getAuthenMode()) ? "SINGLE" : params.getAuthenMode());
        // ssh 连接方式
        target.put("executeType", StringUtils.isEmpty(params.getExecuteType()) ? "Remote" : params.getExecuteType());
        target.put("sudo", false);
        if (null == executeUser && !StringUtils.isEmpty(params.getBecomeUser())) {
            target.put("sudo", true);
            target.put("becomeUser", params.getBecomeUser());
            String becomePass = params.getBecomePass();
            if (StringUtils.isNotEmpty(becomePass)) {
                Encryptor encryptor = new AESEncryptor();
                // 解密
                String decPas = encryptor.decrypt(becomePass, null);
                if (StringUtils.isEmpty(decPas)) {
                    // 解密失败，默认为明文密码，对密码进行加密处理
                    becomePass = encryptor.encrypt(becomePass, null);
                }
                target.put("becomePass", becomePass);
            }
        }
        target.put("data", hosts);
        input.put("target", target);
        return input;
    }

    /**
     * 构建主机重启任务
     *
     * @param params
     * @return
     */
    public static Task getRebootTask(ScriptTaskParams params) {
        JSONObject task = new JSONObject();
        task.put("code", "standard");
        task.put("name", "主机重启作业");
        task.put("sceneId", 8);
        JSONArray nodes = new JSONArray();
        nodes.addAll(getRebootNodes(params));
        JSONObject graph = new JSONObject();
        graph.put("nodes", nodes);
        task.put("graph", graph);
        return JSONObject.parseObject(task.toJSONString(), Task.class);
    }

    private static List<JSONObject> getRebootNodes(ScriptTaskParams params) {
        List<JSONObject> nodes = new ArrayList<>();
        Map<String, List<JSONObject>> userMap = new HashMap<>();
        List<JSONObject> targets = params.getTargets();
        Encryptor encryptor = new AESEncryptor();
        for (JSONObject target : targets) {
            String key = target.getString("becomeUser");
            key = StringUtils.isEmpty(key) ? "nouser" : key;
            JSONObject host = new JSONObject();
            host.put("host", target.getString("host"));
            host.put("rawName", target.getString("host"));
            host.put("osName", target.getString("category"));
            host.put("segmentId", target.get("segmentId"));
            host.put("user", target.getString("user"));
            String pasd = target.getString("pasd");
            // 解密
            String decPas = encryptor.decrypt(pasd, null);
            if (StringUtils.isEmpty(decPas)) {
                // 解密失败，默认为明文密码，对密码进行加密处理
                pasd = encryptor.encrypt(pasd, null);
            }
            host.put("pasd", pasd);
            if (target.containsKey("free")) {
                host.put("free", target.getBooleanValue("free"));
            }
            if (target.containsKey("becomeUser")) {
                host.put("becomeUser", target.getString("becomeUser"));
            }
            if (target.containsKey("becomePass")) {
                String becomePass = target.getString("becomePass");
                // 解密
                String decBecomePas = encryptor.decrypt(becomePass, null);
                if (StringUtils.isEmpty(decBecomePas)) {
                    // 解密失败，默认为明文密码，对密码进行加密处理
                    becomePass = encryptor.encrypt(becomePass, null);
                }
                host.put("becomePass", becomePass);
            }
            if (target.containsKey("becomeMethod")) {
                host.put("becomeMethod", target.getString("becomeMethod"));
            }

            if (userMap.containsKey(key)) {
                List<JSONObject> list = userMap.get(key);
                list.add(host);
                userMap.put(key, list);
            } else {
                List<JSONObject> list = new ArrayList<>();
                list.add(host);
                userMap.put(key, list);
            }
        }
        Iterator<String> iterator = userMap.keySet().iterator();
        int x = 250;
        int index = 1;
        while (iterator.hasNext()) {
            String key = iterator.next();
            // 根据网段拆分
            List<JSONObject> hosts = userMap.get(key);
            Map<Long, List<JSONObject>> hostMap = new HashMap<>();
            for (JSONObject host : hosts) {
                if (hostMap.containsKey(host.getLong("segmentId"))) {
                    hostMap.get(host.getLong("segmentId")).add(host);
                } else {
                    List<JSONObject> list = new ArrayList<>();
                    list.add(host);
                    hostMap.put(host.getLong("segmentId"), list);
                }
            }
            for (List<JSONObject> list : hostMap.values()) {
                List<List<JSONObject>> partitions = Lists.partition(list, 50);
                for (List<JSONObject> partition : partitions) {
                    JSONObject node = new JSONObject();
                    node.put("name", "主机重启(" + index + ")");
                    node.put("code", "task.maintain.reboot");
                    node.put("moduleType", "task.maintain.reboot");
                    NodeConfig config =
                            new NodeConfig(UUID.randomUUID().toString(), index - 1, x, 250, "script.png", "maintain-node");
                    NodeControl control = new NodeControl(false, false, false, 3, 10,
                            null == params.getTimeout() ? 3600 : params.getTimeout());
                    node.put("control", control);
                    node.put("config", config);
                    node.put("input", getRebootInput(params, "nouser".equalsIgnoreCase(key) ? null : key, partition));
                    x = x + 200;
                    index++;
                    nodes.add(node);
                }
            }
        }
        return nodes;
    }

    private static JSONObject getRebootInput(ScriptTaskParams params, String executeUser, List<JSONObject> hosts) {
        JSONObject input = new JSONObject();
        JSONObject fronts = new JSONObject();
        fronts.put("rebootTimeout", 1800);
        input.put("fronts", fronts);
        JSONObject target = new JSONObject();
        target.put("authenMode", StringUtils.isEmpty(params.getAuthenMode()) ? "SINGLE" : params.getAuthenMode());
        target.put("executeType", StringUtils.isEmpty(params.getExecuteType()) ? "Remote" : params.getExecuteType());
        target.put("sudo", false);
        if (null == executeUser && !StringUtils.isEmpty(params.getBecomeUser())) {
            target.put("sudo", true);
            target.put("becomeUser", params.getBecomeUser());
            String becomePass = params.getBecomePass();
            if (StringUtils.isNotEmpty(becomePass)) {
                Encryptor encryptor = new AESEncryptor();
                // 解密
                String decPas = encryptor.decrypt(becomePass, null);
                if (StringUtils.isEmpty(decPas)) {
                    // 解密失败，默认为明文密码，对密码进行加密处理
                    becomePass = encryptor.encrypt(becomePass, null);
                }
                target.put("becomePass", becomePass);
            }
        }
        target.put("data", hosts);
        input.put("target", target);
        return input;
    }

    /**
     * 构建相关脚本类作业（ansible）
     *
     * @param params
     * @param scriptMap
     * @return
     */
    public static Task getTask(ScriptTaskParams params, Map<String, Object> scriptMap, String taskName) {
        JSONObject task = new JSONObject();
        task.put("code", "standard");
        String name = StringUtils.isEmpty(params.getTaskAlias()) ? "脚本执行作业" : params.getTaskAlias();
        task.put("name", (null != taskName ? params.getTaskAlias() + "_" + taskName : name));
        task.put("sceneId", 8);
        JSONArray nodes = new JSONArray();
        nodes.addAll(getScriptNodes(params, scriptMap));
        JSONObject graph = new JSONObject();
        graph.put("nodes", nodes);
        task.put("graph", graph);
        return JSONObject.parseObject(task.toJSONString(), Task.class);
    }

    private static List<JSONObject> getScriptNodes(ScriptTaskParams params, Map<String, Object> scriptMap) {
        List<JSONObject> nodes = new ArrayList<>();
        Map<String, List<JSONObject>> userMap = new HashMap<>();
        List<JSONObject> targets = params.getTargets();
        Encryptor encryptor = new AESEncryptor();
        for (JSONObject target : targets) {
            String key = target.getString("becomeUser");
            key = StringUtils.isEmpty(key) ? "nouser" : key;
            JSONObject host = new JSONObject();
            host.put("host", target.getString("host"));
            host.put("rawName", target.getString("host"));
            host.put("osName", target.getString("category"));
            host.put("segmentId", target.get("segmentId"));
            host.put("user", target.getString("user"));
            String pasd = target.getString("pasd");
            // 解密
            String decPas = encryptor.decrypt(pasd, null);
            if (StringUtils.isEmpty(decPas)) {
                // 解密失败，默认为明文密码，对密码进行加密处理
                pasd = encryptor.encrypt(pasd, null);
            }
            host.put("pasd", pasd);

            if (target.containsKey("free")) {
                host.put("free", target.getBooleanValue("free"));
            }
            if (target.containsKey("becomeUser")) {
                host.put("becomeUser", target.getString("becomeUser"));
            }
            if (target.containsKey("becomePass")) {
                String becomePass = target.getString("becomePass");
                // 解密
                String decBecomePas = encryptor.decrypt(becomePass, null);
                if (StringUtils.isEmpty(decBecomePas)) {
                    // 解密失败，默认为明文密码，对密码进行加密处理
                    becomePass = encryptor.encrypt(becomePass, null);
                }
                host.put("becomePass", becomePass);
            }
            if (target.containsKey("becomeMethod")) {
                host.put("becomeMethod", target.getString("becomeMethod"));
            }

            if (userMap.containsKey(key)) {
                List<JSONObject> list = userMap.get(key);
                list.add(host);
                userMap.put(key, list);
            } else {
                List<JSONObject> list = new ArrayList<>();
                list.add(host);
                userMap.put(key, list);
            }
        }
        Iterator<String> iterator = userMap.keySet().iterator();
        int x = 250;
        int index = 1;
        while (iterator.hasNext()) {
            String key = iterator.next();
            // 根据网段拆分
            List<JSONObject> hosts = userMap.get(key);
            Map<Long, List<JSONObject>> hostMap = new HashMap<>();
            for (JSONObject host : hosts) {
                if (hostMap.containsKey(host.getLong("segmentId"))) {
                    hostMap.get(host.getLong("segmentId")).add(host);
                } else {
                    List<JSONObject> list = new ArrayList<>();
                    list.add(host);
                    hostMap.put(host.getLong("segmentId"), list);
                }
            }
            for (List<JSONObject> list : hostMap.values()) {
                List<List<JSONObject>> partitions = Lists.partition(list, 50);
                for (List<JSONObject> partition : partitions) {
                    JSONObject node = new JSONObject();
                    String name = StringUtils.isEmpty(params.getTaskAlias()) ? "脚本执行" : params.getTaskAlias();
                    node.put("name", name + "(" + index + ")");
                    node.put("code", "task.maintain.script");
                    node.put("moduleType", "task.maintain.script");
                    NodeConfig config =
                            new NodeConfig(UUID.randomUUID().toString(), index - 1, x, 250, "script.png", "maintain-node");
                    NodeControl control = new NodeControl(false, false, false, 3, 10,
                            null == params.getTimeout() ? 3600 : params.getTimeout());
                    node.put("control", control);
                    node.put("config", config);
                    node.put("input",
                            getScriptInput(params, "nouser".equalsIgnoreCase(key) ? null : key, scriptMap, partition));
                    x = x + 200;
                    index++;
                    nodes.add(node);
                }
            }
        }
        return nodes;
    }

    private static JSONObject getScriptInput(ScriptTaskParams params, String executeUser, Map<String, Object> scriptMap,
                                             List<JSONObject> hosts) {
        JSONObject input = new JSONObject();
        JSONObject fronts = new JSONObject();
        fronts.put("async", params.getAsync());
        if (scriptMap.containsKey("scriptId")) {
            fronts.put("scriptId", scriptMap.get("scriptId"));
        }
        if (scriptMap.containsKey("name")) {
            fronts.put("name", scriptMap.get("name"));
        }
        if (scriptMap.containsKey("category")) {
            fronts.put("category", scriptMap.get("category"));
        }
        if (!ListTool.isEmpty(params.getParams())) {
            fronts.put("params", JSONObject.parseArray(JSONObject.toJSONString(params.getParams()), JSONObject.class));
        }
        if (null != params.getParam()) {
            fronts.put("param", params.getParam());
        }
        fronts.put("content", scriptMap.get("content"));
        input.put("fronts", fronts);
        JSONObject target = new JSONObject();
        target.put("authenMode", StringUtils.isEmpty(params.getAuthenMode()) ? "SINGLE" : params.getAuthenMode());
        target.put("executeType", StringUtils.isEmpty(params.getExecuteType()) ? "Remote" : params.getExecuteType());
        target.put("sudo", false);
        if (null == executeUser && !StringUtils.isEmpty(params.getBecomeUser())) {
            target.put("sudo", true);
            target.put("becomeUser", params.getBecomeUser());
            String becomePass = params.getBecomePass();
            if (StringUtils.isNotEmpty(becomePass)) {
                Encryptor encryptor = new AESEncryptor();
                // 解密
                String decPas = encryptor.decrypt(becomePass, null);
                if (StringUtils.isEmpty(decPas)) {
                    // 解密失败，默认为明文密码，对密码进行加密处理
                    becomePass = encryptor.encrypt(becomePass, null);
                }
                target.put("becomePass", becomePass);
            }
        }
        target.put("data", hosts);
        input.put("target", target);
        return input;
    }

}
