package com.bocloud.cop.service.utils.worker;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.amqp.rabbit.core.RabbitAdmin;

import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cop.constant.CommonConstant;
import com.bocloud.cop.model.result.WorkerHostResult;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.model.GeneralResult;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.model.RemoteService;
import com.megatron.framework.http.model.ServiceParam;

/**
 * 调用Worker服务工具类
 */
public class TaskWorkerHelper extends WorkerHelper {

    /**
     * 调用异步Worker服务并返回调用结果
     */
    public static GeneralResult<List<WorkerHostResult>> invoke(WorkerServiceFactory factory, String instance,
        Map<String, Object> params, RabbitAdmin rabbitAdmin, Long timeout) {
        WorkerRemoteService service =
            factory.build(BoCloudService.GAIA, instance, JOB_SUBMIT_URL, HttpMethod.OPERATE, HEADERS, params);
        return new AsyncWorkerCaller(service, params, rabbitAdmin, timeout).call();
    }

    /**
     * 调用同步Worker服务并返回调用结果
     */
    public static GeneralResult<?> invoke(String instance, Map<String, Object> params, boolean sync) {
        String url = "http://" + instance + JOB_SUBMIT_URL;
        RemoteService service = new RemoteService(Service.create(BoCloudService.GAIA.getName()),
            new ServiceParam(params, HEADERS), true, url, HttpMethod.POST);
        // 指定参数response的值为"sync"，Worker服务就会直接返回操作结果
        if (null == params) {
            params = new HashMap<String, Object>();
        }
        if (sync) {
            params.put(CommonConstant.RESPONSE, CommonConstant.WorkerResponseType.SYNC);
            service.getParam().setParams(params);
        }
        service.setDataFormat(PostDataFormat.RAW);
        return service.invoke();
    }
}
